/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.EventListener;

import nextapp.echo.event.ActionEvent;
import nextapp.echo.event.ActionListener;
import nextapp.echo.event.ChangeEvent;
import nextapp.echo.event.ChangeListener;
import nextapp.echo.event.ItemEvent;
import nextapp.echo.event.ItemListener;

/**
 * A base class from which various types of buttons (simple buttons, radio 
 * buttons, checkboxes, etc.) are derived.
 */
public abstract class AbstractButton extends Component 
implements ToolTipSupport {

    /** 
     * A style constant for the Horizontal Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";

    /** 
     * A style constant for the Horizontal Text Position property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_TEXT_POSITION = "horizontalTextPosition";

    /** 
     * A style constant for the Icon-Text Margin property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_ICON_TEXT_MARGIN = "iconTextMargin";

    /** 
     * A style constant for the Rollover Background property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_ROLLOVER_BACKGROUND = "rolloverBackground";

    /** 
     * A style constant for the Rollover Enabled property. 
     * Values of this key must be of type java.lang.Boolean.
     */
    public static final String STYLE_ROLLOVER_ENABLED = "rolloverEnabled";

    /** 
     * A style constant for the Rollover Font property. 
     * Values of this key must be of type nextapp.echo.Font.
     */
    public static final String STYLE_ROLLOVER_FONT = "rolloverFont";

    /** 
     * A style constant for the Rollover Foreground property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_ROLLOVER_FOREGROUND = "rolloverForeground";

    /** 
     * A style constant for the Vertical Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";

    /** 
     * A style constant for the Vertical Text Position property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_TEXT_POSITION = "verticalTextPosition";

    public static final String ACTION_COMMAND_CHANGED_PROPERTY = "actionCommand";
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String HORIZONTAL_TEXT_POSITION_CHANGED_PROPERTY = "horizontalTextPosition";
    public static final String ICON_CHANGED_PROPERTY = "icon";
    public static final String ICON_TEXT_MARGIN_CHANGED_PROPERTY = "iconTextMargin";
    public static final String MODEL_CHANGED_PROPERTY = "model";
    public static final String ROLLOVER_BACKGROUND_CHANGED_PROPERTY = "rolloverBackground";
    public static final String ROLLOVER_ENABLED_CHANGED_PROPERTY = "rolloverEnabled";
    public static final String ROLLOVER_FONT_CHANGED_PROPERTY = "rolloverFont";
    public static final String ROLLOVER_FOREGROUND_CHANGED_PROPERTY = "rolloverForeground";
    public static final String ROLLOVER_ICON_CHANGED_PROPERTY = "rolloverIcon";
    public static final String ROLLOVER_SELECTED_ICON_CHANGED_PROPERTY = "rolloverSelectedIcon";
    public static final String SELECTED_CHANGED_PROPERTY = "selected";
    public static final String SELECTED_ICON_CHANGED_PROPERTY = "selectedIcon";
    public static final String TEXT_CHANGED_PROPERTY = "text";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
    public static final String VERTICAL_TEXT_POSITION_CHANGED_PROPERTY = "verticalTextPosition";
    
    /**
     * An implementation of various listener interfaces that is registered
     * with the <code>ButtonModel</code> in order to forward its generated
     * events to listeners registered with the <code>AbstractButton</code>
     * itself.
     */
    private class EventForwarder 
    implements ActionListener, ChangeListener, ItemListener, Serializable {

        /**
         * @see nextapp.echo.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(ActionEvent modelEvent) {
            ActionEvent buttonEvent = new ActionEvent(AbstractButton.this, modelEvent.getActionCommand());
            fireActionPerformed(buttonEvent);
        }

        /**
         * @see nextapp.echo.event.ChangeListener#stateChanged(ChangeEvent)
         */
        public void stateChanged(ChangeEvent modelEvent) {
            fireStateChanged();
        }

        /**
         * @see nextapp.echo.event.ItemListener#itemStateChanged(ItemEvent)
         */
        public void itemStateChanged(ItemEvent modelEvent) {
            ItemEvent buttonEvent = new ItemEvent(AbstractButton.this, modelEvent.getItem(), modelEvent.getStateChange());
            fireItemStateChanged(buttonEvent);
        }
    }
       
    private Color rolloverForeground = null;
    private Color rolloverBackground = null;
    private Font rolloverFont = null;
    private ImageReference icon = null;
    private ImageReference rolloverIcon = null;
    private ImageReference selectedIcon = null;
    private ImageReference rolloverSelectedIcon = null;
    private boolean rolloverEnabled = false;
    private String text = null;
    private ButtonModel model;
    private int iconTextMargin = 3;
    private int horizontalAlignment = EchoConstants.LEFT;
    private int horizontalTextPosition = EchoConstants.RIGHT;
    private int verticalAlignment = EchoConstants.CENTER;
    private int verticalTextPosition = EchoConstants.CENTER;
    private String toolTipText = null;
    
    private EventForwarder eventForwarder = null;
    
    /**
     * Default constructor.
     */ 
    public AbstractButton() {
        super();
    }

    /**
     * Adds an <code>ActionListener</code> to the button.
     *
     * @param l The <code>ActionListener</code> to be added.
     */
    public void addActionListener(ActionListener l) {
        listenerList.addListener(ActionListener.class, l);
    }
    
    /**
     * Adds a <code>ChangeListener</code> to the button.
     *
     * @param l The <code>ChangeListener</code> to be added.
     */
    public void addChangeListener(ChangeListener l) {
        listenerList.addListener(ChangeListener.class, l);
    }
    
    /**
     * Adds a <code>ItemListener</code> to the button.
     *
     * @param l The <code>ItemListener</code> to be added.
     */
    public void addItemListener(ItemListener l) {
        listenerList.addListener(ItemListener.class, l);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_TEXT_POSITION)) {
            setHorizontalTextPosition(style.getIntegerAttribute(STYLE_HORIZONTAL_TEXT_POSITION));
        }
        if (style.hasAttribute(STYLE_ICON_TEXT_MARGIN)) {
            setIconTextMargin(style.getIntegerAttribute(STYLE_ICON_TEXT_MARGIN));
        }
        if (style.hasAttribute(STYLE_ROLLOVER_BACKGROUND)) {
            setRolloverBackground((Color) style.getAttribute(STYLE_ROLLOVER_BACKGROUND));
        }
        if (style.hasAttribute(STYLE_ROLLOVER_ENABLED)) {
            setRolloverEnabled(style.getBooleanAttribute(STYLE_ROLLOVER_ENABLED));
        }
        if (style.hasAttribute(STYLE_ROLLOVER_FONT)) {
            setRolloverFont((Font) style.getAttribute(STYLE_ROLLOVER_FONT));
        }
        if (style.hasAttribute(STYLE_ROLLOVER_FOREGROUND)) {
            setRolloverForeground((Color) style.getAttribute(STYLE_ROLLOVER_FOREGROUND));
        }
        if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
            setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_VERTICAL_TEXT_POSITION)) {
            setVerticalTextPosition(style.getIntegerAttribute(STYLE_VERTICAL_TEXT_POSITION));
        }
    }
    
    /**
     * Programmatically performs a click/activation of the button.
     */
    public void doAction() {
        model.doAction();
    }

    /**
     * Notifies all listeners that have registered for this event type.
     *
     * @param e The <code>ActionEvent</code> to send.
     */
    public void fireActionPerformed(ActionEvent e) {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ActionListener) listeners[index]).actionPerformed(e);
        }
    }
    
    /**
     * Notifies all listeners that have registered for this event type.
     *
     * @param e The <code>ItemEvent</code> to send.
     */
    public void fireItemStateChanged(ItemEvent e) {
        EventListener[] listeners = listenerList.getListeners(ItemListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ItemListener) listeners[index]).itemStateChanged(e);
        }
    }
    
    /**
     * Notifies all listeners that have registered for this event type.
     */
    public void fireStateChanged() {
        EventListener[] listeners = listenerList.getListeners(ChangeListener.class);
        ChangeEvent e = new ChangeEvent(this);
        for (int index = 0; index < listeners.length; ++index) {
            ((ChangeListener) listeners[index]).stateChanged(e);
        }
    }
    
    /**
     * Returns the action command for this button.
     *
     * @return The action command for this button.
     */
    public String getActionCommand() { 
        return model.getActionCommand();
    }
    
    /**
     * Returns the horizontal alignment of the text relative to the icon.
     *
     * @return The horizontal alignment of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }
    
    /**
     * Returns the horizontal position of the text relative to the icon.
     *
     * @return The horizontal position of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT (the default)</li>
     *         </ul>
     */
    public int getHorizontalTextPosition() {
        return horizontalTextPosition;
    }
    
    /**
     * Returns the icon.
     *
     * @return The icon.
     */
    public ImageReference getIcon() {
        return icon;
    }
    
    /**
     * Returns the size of the margin to be displayed between the icon and text 
     * The margin is only drawn if both icon and text are present.
     *
     * @return The size of the margin between the icon and text, in pixels.
     */
    public int getIconTextMargin() {
        return iconTextMargin;
    }
    
    /**
     * Returns the model that this button represents.
     *
     * @return The button's model.
     */
    public ButtonModel getModel() {
        return model;
    }
    
    /**
     * Returns the rollover background color of the button.
     *
     * @return The rollover background color.
     */
    public Color getRolloverBackground() {
        return rolloverBackground;
    }
    
    /**
     * Returns the rollover font of the button.
     *
     * @return The rollover font.
     */
    public Font getRolloverFont() {
        return rolloverFont;
    }
    
    /**
     * Returns the rollover foreground color of the button.
     *
     * @return The rollover foreground color.
     */
    public Color getRolloverForeground() {
        return rolloverForeground;
    }
    
    /**
     * Returns the rollover icon.
     *
     * @return The rollover icon.
     */
    public ImageReference getRolloverIcon() {
        return rolloverIcon;
    }
    
    /**
     * Returns the rollover selected icon.
     *
     * @return The rollover selected icon.
     */
    public ImageReference getRolloverSelectedIcon() {
        return rolloverSelectedIcon;
    }
    
    /**
     * Returns the selected icon.
     *
     * @return The selected icon.
     */
    public ImageReference getSelectedIcon() {
        return selectedIcon;
    }
    
    /**
     * Returns the text.
     *
     * @return The text.
     */    
    public String getText() {
        return text;
    }

    /**
     * @see nextapp.echo.ToolTipSupport#getToolTipText()
     */
    public String getToolTipText() {
        return toolTipText;
    }
    
    /**
     * Returns the vertical alignment of the text relative to the icon.
     *
     * @return The vertical alignment of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalAlignment() {
        return verticalAlignment;
    }
    
    /**
     * Returns the vertical position of the text relative to the icon.
     *
     * @return The vertical position of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalTextPosition() {
        return verticalTextPosition;
    }
    
    /**
     * Returns true if the button has one or more action listeners.
     *
     * @return True if the button has one or more action listeners.
     */
    public boolean hasActionListeners() {
        int listenerCount = listenerList.getListenerCount(ActionListener.class);
        if (listenerCount > 0) {
            return true;
        } else {
            if (model instanceof DefaultButtonModel) {
                // The listener count must be greater than one, not zero, in order to ignore 
                // the listener automatically registered by the component.
                return ((DefaultButtonModel) model).listenerList.getListenerCount(ActionListener.class) > 1;
            } else {
                return true;
            }
        }
    }
    
    /**
     * Returns whether rollovers are enabled.
     *
     * @return True if rollovers are enabled.
     */
    public boolean isRolloverEnabled() {
        return rolloverEnabled;
    }
    
    /**
     * Returns the button's selection state.
     *
     * @return True if the button is selected.
     */
    public boolean isSelected() {
        return model.isSelected();
    }
    
    /**
     * Removes an <code>ActionListener</code> from the button.
     *
     * @param l The <code>ActionListener</code> to be removed.
     */
    public void removeActionListener(ActionListener l) {
        listenerList.removeListener(ActionListener.class, l);
    }
    
    /**
     * Removes a <code>ChangeListener</code> from the button.
     *
     * @param l The <code>ChangeListener</code> to be removed.
     */
    public void removeChangeListener(ChangeListener l) {
        listenerList.removeListener(ChangeListener.class, l);
    }
    
    /**
     * Removes a <code>ItemListener</code> from the button.
     *
     * @param l The <code>ItemListener</code> to be removed.
     */
    public void removeItemListener(ItemListener l) {
        listenerList.removeListener(ItemListener.class, l);
    }
    
    /**
     * Sets the button's action command.
     *
     * @param newValue The new action command for this button.
     */
    public void setActionCommand(String newValue) {
        String oldValue = model.getActionCommand();
        model.setActionCommand(newValue);
        firePropertyChange(ACTION_COMMAND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the horizontal alignment of the text relative to the icon.
     *
     * @param newValue The horizontal alignment of the text relative to the icon,
     *        one of the following values:
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the horizontal position of the text relative to the icon.
     *
     * @param newValue The horizontal position of the text relative to the icon,
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.LEFT</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT (the default)</li>
     *        </ul>
     */
    public void setHorizontalTextPosition(int newValue) {
        int oldValue = horizontalTextPosition;
        horizontalTextPosition = newValue;
        firePropertyChange(HORIZONTAL_TEXT_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the default icon.
     *
     * @param newValue The default icon.
     */
    public void setIcon(ImageReference newValue) {
        ImageReference oldValue = icon;
        icon = newValue;
        firePropertyChange(ICON_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the size of the margin to be displayed between the icon and text.
     * The margin is only drawn if both icon and text are present.
     *
     * @param newValue The size of the margin between the icon and text, in 
     *        pixels.
     */
    public void setIconTextMargin(int newValue) {
        int oldValue = iconTextMargin;
        iconTextMargin = newValue;
        firePropertyChange(ICON_TEXT_MARGIN_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the model that this button represents.
     *
     * @param newValue The new model for this button.
     */
    public void setModel(ButtonModel newValue) {
        ButtonModel oldValue = model;
        
        model = newValue;
        
        if (oldValue != null) {
            oldValue.removeActionListener(eventForwarder);
            oldValue.removeChangeListener(eventForwarder);
            oldValue.removeItemListener(eventForwarder);
            eventForwarder = null;
        }
        
        if (newValue != null) {
            eventForwarder = new EventForwarder();
            newValue.addActionListener(eventForwarder);
            newValue.addChangeListener(eventForwarder);
            newValue.addItemListener(eventForwarder);
        }
    
        firePropertyChange(MODEL_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the rollover background.
     *
     * @param newValue The rollover background.
     */
    public void setRolloverBackground(Color newValue) {
        Color oldValue = rolloverBackground;
        rolloverBackground = newValue;
        firePropertyChange(ROLLOVER_BACKGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets whether rollovers are enabled.
     *
     * @param newValue True if rollovers should be enabled.
     */
    public void setRolloverEnabled(boolean newValue) {
        boolean oldValue = rolloverEnabled;
        rolloverEnabled = newValue;
        firePropertyChange(ROLLOVER_ENABLED_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the rollover font.
     *
     * @param newValue The rollover font.
     */
    public void setRolloverFont(Font newValue) {
        Font oldValue = rolloverFont;
        rolloverFont = newValue;
        firePropertyChange(ROLLOVER_FONT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the rollover foreground.
     *
     * @param newValue The rollover foreground.
     */
    public void setRolloverForeground(Color newValue) {
        Color oldValue = rolloverForeground;
        rolloverForeground = newValue;
        firePropertyChange(ROLLOVER_FOREGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the rollover icon.
     *
     * @param newValue The rollover icon.
     */
    public void setRolloverIcon(ImageReference newValue) {
        ImageReference oldValue = rolloverIcon;
        rolloverIcon = newValue;
        firePropertyChange(ROLLOVER_ICON_CHANGED_PROPERTY, oldValue, newValue);
    }
        
    /**
     * Sets the rollover selected icon.
     *
     * @param newValue The rollover selected icon.
     */
    public void setRolloverSelectedIcon(ImageReference newValue) {
        ImageReference oldValue = rolloverSelectedIcon;
        rolloverSelectedIcon = newValue;
        firePropertyChange(ROLLOVER_SELECTED_ICON_CHANGED_PROPERTY, oldValue, newValue);
    }
        
    /**
     * Sets the button's selection state.
     *
     * @param newValue The new selection state of this button.
     */
    public void setSelected(boolean newValue) {
        boolean oldValue = model.isSelected();
        model.setSelected(newValue);
        firePropertyChange(SELECTED_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the selected icon.
     *
     * @param newValue The selected icon.
     */
    public void setSelectedIcon(ImageReference newValue) {
        ImageReference oldValue = selectedIcon;
        selectedIcon = newValue;
        firePropertyChange(SELECTED_ICON_CHANGED_PROPERTY, oldValue, newValue);
    }
        
    /**
     * Sets the text.
     *
     * @param newValue The text.
     */
    public void setText(String newValue) {
        String oldValue = text;
        text = newValue;
        firePropertyChange(TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#setToolTipText(String)
     */
    public void setToolTipText(String newValue) {
        String oldValue = toolTipText;
        toolTipText = newValue;
        firePropertyChange(TOOL_TIP_TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical alignment of the text relative to the icon.
     *
     * @param newValue The vertical alignment of the text relative to the icon,
     *        one of the following values:
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalAlignment(int newValue) {
        int oldValue = verticalAlignment;
        verticalAlignment = newValue;
        firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical position of the text relative to the icon.
     *
     * @param newValue The vertical position of the text relative to the icon,
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalTextPosition(int newValue) {
        int oldValue = verticalTextPosition;
        verticalTextPosition = newValue;
        firePropertyChange(VERTICAL_TEXT_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }
}
