/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.EventListener;

import nextapp.echo.event.ListDataEvent;
import nextapp.echo.event.ListDataListener;

import nextapp.echo.event.EventListenerList;

/**
 * A base class from which ListModel implementations are derived.
 * This class provides event handling functionality so that components
 * may be redrawn when models change.
 */
public abstract class AbstractListModel 
implements ListModel, Serializable {

    /**
     * A storage facility for <code>EventListener</code>s.
     */
    protected EventListenerList listenerList = new EventListenerList();
 
    /**
     * Creates a new AbstractListModel.
     */
    public AbstractListModel() {
        super();
    }
 
    /**
     * @see nextapp.echo.ListModel#addListDataListener(ListDataListener)
     */
    public void addListDataListener(ListDataListener l) {
        listenerList.addListener(ListDataListener.class, l);
    }
    
    /**
     * AbstractListModel subclasses <strong>must</strong> call this method 
     * after one or elements are changed.
     */ 
    protected void fireContentsChanged(Object source, int index0, int index1) {
        ListDataEvent e = new ListDataEvent(source, ListDataEvent.CONTENTS_CHANGED, index0, index1);

        EventListener[] listeners = listenerList.getListeners(ListDataListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ListDataListener) listeners[index]).contentsChanged(e);
        }
    }

    /**
     * AbstractListModel subclasses <strong>must</strong> call this method 
     * after one or elements are added.
     */ 
    protected void fireIntervalAdded(Object source, int index0, int index1) {
        ListDataEvent e = new ListDataEvent(source, ListDataEvent.INTERVAL_ADDED, index0, index1);

        EventListener[] listeners = listenerList.getListeners(ListDataListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ListDataListener) listeners[index]).intervalAdded(e);
        }
    }
    
    /**
     * AbstractListModel subclasses <strong>must</strong> call this method 
     * after one or elements are removed.
     */ 
    protected void fireIntervalRemoved(Object source, int index0, int index1) {
        ListDataEvent e = new ListDataEvent(source, ListDataEvent.INTERVAL_REMOVED, index0, index1);

        EventListener[] listeners = listenerList.getListeners(ListDataListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ListDataListener) listeners[index]).intervalRemoved(e);
        }
    }

    /**
     * @see nextapp.echo.ListModel#removeListDataListener(ListDataListener)
     */
    public void removeListDataListener(ListDataListener l) {
        listenerList.removeListener(ListDataListener.class, l);
    }
}
