/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A base class from which all panes (e.g. ContainerPane and ContentPane) 
 * are derived.
 */
public abstract class AbstractPane extends Component {
    
    /** 
     * A style constant for the Resizable property. 
     * Values of this key must be of type java.lang.Boolean.
     */
    public static final String STYLE_RESIZABLE = "resizable";
     
    public static final String RESIZABLE_CHANGED_PROPERTY = "resizable";

    /**
     * @deprecated 
     */
    public static final int PROPORTIONAL = EchoConstants.UNDEFINED_SIZE;
    
    private boolean resizable = false;

    /**
     * Default constructor.
     */
    public AbstractPane() {
        super();
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_RESIZABLE)) {
            setResizable(style.getBooleanAttribute(STYLE_RESIZABLE));
        }
    }
    
    /**
     * Returns the height of the pane, in pixels.
     * Default implementation returns 
     * <code>getHeightExtent().getValue()</code> if
     * <code>getHeightExtent()</code> returns non-null and the extent's
     * <code>getUnits()</code> method returns <code>Extent.PX</code>.
     * Otherwise returns <code>EchoConstants.UNDEFINED_SIZE</code>.
     *
     * @return The height of the pane, in pixels.
     */
    public int getHeight() {
        Extent heightExtent = getHeightExtent();
        return (heightExtent != null && heightExtent.getUnits() == Extent.PX) 
                ? heightExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
    }
    
    /**
     * Returns the height of the pane as an <code>Extent</code>.
     * Default implementation returns null.
     * 
     * @return The height of the pane as an <code>Extent</code>.
     */
    public Extent getHeightExtent() {
        return null;
    }
    
    /**
     * Returns the width of the pane, in pixels.
     * Default implementation returns 
     * <code>getWidthExtent().getValue()</code> if
     * <code>getWidthExtent()</code> returns non-null and the extent's
     * <code>getUnits()</code> method returns <code>Extent.PX</code>.
     * Otherwise returns <code>EchoConstants.UNDEFINED_SIZE</code>.
     *
     * @return The width of the pane, in pixels.
     */
    public int getWidth() {
        Extent widthExtent = getWidthExtent();
        return (widthExtent != null && widthExtent.getUnits() == Extent.PX) 
                ? widthExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
    }
    
    /**
     * Returns the width of the pane as an <code>Extent</code>.
     * Default implementation returns null.
     * 
     * @return The width of the pane as an <code>Extent</code>.
     */
    public Extent getWidthExtent() {
        return null;
    }
    
    /**
     * Returns whether the pane is resizable.
     *
     * @return True if the pane is resizable.
     */
    public boolean isResizable() {
        return resizable;
    }
    
    /**
     * Sets whether the pane is resizable.
     *
     * @param newValue True if the pane should be resizable.
     */
    public void setResizable(boolean newValue) {
        boolean oldValue = resizable;
        resizable = newValue;
        firePropertyChange(RESIZABLE_CHANGED_PROPERTY, oldValue, newValue);
    }
}
