/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

/**
 * Describes a background image.
 */
public class BackgroundImageProperties
implements Serializable {

    /**
     * A constant for the Background Image Repeat property that indicates the
     * background image should be repeated both horizontally and vertically.
     */
    public static final int REPEAT = 0;

    /**
     * A constant for the Background Image Repeat property that indicates the
     * background image should not be repeated.
     */
    public static final int NO_REPEAT = 1;

    /**
     * A constant for the Background Image Repeat property that indicates the
     * background image should be repeated horizontally.
     */
    public static final int REPEAT_HORIZONTAL = 2;

    /**
     * A constant for the Background Image Repeat property that indicates the
     * background image should be repeated vertically.
     */
    public static final int REPEAT_VERTICAL = 3;

    /**
     * A constant for horizontal and vertical position units that indicates 
     * percent-based units.
     * 
     * @deprecated Use <code>Extent.PERCENT</code>.
     */
    public static final int PERCENT_UNITS = Extent.PERCENT;
    
    /**
     * A constant for horizontal and vertical position units that indicates 
     * pixel-based units.
     * 
     * @deprecated Use <code>Extent.PX</code>.
     */
    public static final int PIXEL_UNITS = Extent.PX;

    /** 
     * A constant for the attachment property that indicates that the 
     * background image should remain fixed and NOT scroll with content.
     */
    public static final int ATTACHMENT_FIXED = 0;
    
    /**
     * A constant for the attachment property that indicates that the
     * background image should scroll with content above it.
     */
    public static final int ATTACHMENT_SCROLL = 1;

    public static final String REPEAT_CHANGED_PROPERTY = "repeat";
    public static final String IMAGE_CHANGED_PROPERTY = "image";
    public static final String ATTACHMENT_CHANGED_PROPERTY = "image";
    public static final String HORIZONTAL_POSITION_CHANGED_PROPERTY = "horizontalPosition";
    public static final String VERTICAL_POSITION_CHANGED_PROPERTY = "verticalPosition";

    private ImageReference image = null;
    private int attachment;
    private int repeat;
    private Extent horizontalPositionExtent, verticalPositionExtent;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    
    /**
     * Creates a new background image properties object with the specified
     * background image.
     *
     * @param image The initial background image.
     */
    public BackgroundImageProperties(ImageReference image) {
        super();
        this.image = image;
    }
    
    /**
     * Adds a property change listener.
     *
     * @param l The listener to add.
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }
    
    /**
     * Returns the background image attachment policy.
     *
     * @return The background image attachment policy, one of the following
     *         values:
     *         <ul>
     *         <li>ATTACHMENT_FIXED</li>
     *         <li>ATTACHMENT_SCROLL</li>
     *         </ul>
     */
    public int getAttachment() {
        return attachment;
    }
    
    /**
     * Returns the horizontal position of the background image.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @return The horizontal position of the background image.
     */
    public int getHorizontalPosition() {
        Extent extent = getHorizontalPositionExtent();
        if (extent == null) {
            return 0;
        } else {
            return extent.getValue();
        }
    }
    
    /**
     * Returns the horizontal position of the background image as an 
     * <code>Extent</code>.
     * 
     * @return The horizontal position of the background image as an
     *         <code>Extent</code>.
     */
    public Extent getHorizontalPositionExtent() {
        return horizontalPositionExtent;
    }
    
    /**
     * Returns the units in which the horizontal position of the background is 
     * defined.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @return The units in which the horizontal position of the background is 
     *         defined, one of the units constants defined in the 
     *         <code>Extent</code> class.
     */
    public int getHorizontalPositionUnits() {
        Extent extent = getHorizontalPositionExtent();
        if (extent == null) {
            return PIXEL_UNITS;
        } else {
            return extent.getUnits();
        }
    }
    
    /**
     * Returns the background image.
     *
     * @return The background image.
     */
    public ImageReference getImage() {
        return image;
    }
    
    /**
     * Returns the background image repetition policy.
     *
     * @return The background image repetition policy, one of the following
     *         values:
     *         <ul>
     *         <li>REPEAT</li>
     *         <li>REPEAT_HORIZONTAL</li>
     *         <li>REPEAT_VERTICAL</li>
     *         <li>NO_REPEAT</li>
     *         </ul>
     */
    public int getRepeat() {
        return repeat;
    }
    
    /**
     * Returns the vertical position of the background image as an 
     * <code>Extent</code>.
     * 
     * @return The vertical position of the background image as an
     *         <code>Extent</code>.
     */
    public Extent getVerticalPositionExtent() {
        return verticalPositionExtent;
    }
    
    /**
     * Returns the vertical position of the background image.
     *
     * @return The vertical position of the background image.
     */
    public int getVerticalPosition() {
        Extent extent = getVerticalPositionExtent();
        if (extent == null) {
            return 0;
        } else {
            return extent.getValue();
        }
    }
    
    /**
     * Returns the units in which the vertical position of the background is 
     * defined.
     *
     * @return The units in which the vertical position of the background is 
     *         defined, one of the units constants defined in the 
     *         <code>Extent</code> class.
     */
    public int getVerticalPositionUnits() {
        Extent extent = getVerticalPositionExtent();
        if (extent == null) {
            return 0;
        } else {
            return extent.getUnits();
        }
    }
    
    /**
     * Removes a property change listener.
     *
     * @param l The listener to remove.
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }
    
    /**
     * Sets the background image attachment policy.
     *
     * @param newValue The new background image attachment policy, one of 
     *        the following values:
     *        <ul>
     *        <li>ATTACHMENT_FIXED</li>
     *        <li>ATTACHMENT_SCROLL</li>
     *        </ul>
     */
    public void setAttachment(int newValue) {
        int oldValue = attachment;
        attachment = newValue;
        pcs.firePropertyChange(ATTACHMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the horizontal position of the background image.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @param newValue The new horizontal position of the background image.
     */
    public void setHorizontalPosition(int newValue) {
        Extent extent = new Extent(newValue, getHorizontalPositionUnits());
        setHorizontalPositionExtent(extent);
    }
    
    /**
     * Sets the horizontal position of the background image as an 
     * <code>Extent</code>
     * 
     * @param newValue The new horizontal position of the background image.
     */
    public void setHorizontalPositionExtent(Extent newValue) {
        Extent oldValue = horizontalPositionExtent;
        horizontalPositionExtent = newValue;
        pcs.firePropertyChange(HORIZONTAL_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units in which the horizontal position of the background is 
     * defined.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @param newValue The new units in which the horizontal position of the 
     *        background is defined, one of the units constants defined in the 
     *        <code>Extent</code> class.
     */
    public void setHorizontalPositionUnits(int newValue) {
        Extent extent = new Extent(getHorizontalPosition(), newValue);
        setHorizontalPositionExtent(extent);
    }

    /**
     * Sets the background image.
     *
     * @param newValue The new background image.
     */
    public void setImage(ImageReference newValue) {
        ImageReference oldValue = image;
        image = newValue;
        pcs.firePropertyChange(IMAGE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the background image repetition policy.
     *
     * @param newValue The new background image repetition policy, one of 
     *         the following values:
     *         <ul>
     *         <li>REPEAT</li>
     *         <li>REPEAT_HORIZONTAL</li>
     *         <li>REPEAT_VERTICAL</li>
     *         <li>NO_REPEAT</li>
     *         </ul>
     */
    public void setRepeat(int newValue) {
        int oldValue = repeat;
        repeat = newValue;
        pcs.firePropertyChange(REPEAT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the vertical position of the background image.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @param newValue The new vertical position of the background image.
     */
    public void setVerticalPosition(int newValue) {
        Extent extent = new Extent(newValue, getVerticalPositionUnits());
        setVerticalPositionExtent(extent);
    }
    
    /**
     * Sets the vertical position of the background image as an 
     * <code>Extent</code>
     * 
     * @param newValue The new vertical position of the background image.
     */
    public void setVerticalPositionExtent(Extent newValue) {
        Extent oldValue = verticalPositionExtent;
        verticalPositionExtent = newValue;
        pcs.firePropertyChange(VERTICAL_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the units in which the vertical position of the background is 
     * defined.
     * Generally, the <code>Extent</code>-based positioning methods should
     * be used instead of this non-<code>Extent</code>-based positioning
     * method.
     *
     * @param newValue The new units in which the vertical position of the 
     *        background is defined, one of the units constants defined in the 
     *         <code>Extent</code> class.
     */
    public void setVerticalPositionUnits(int newValue) {
        Extent extent = new Extent(getVerticalPosition(), newValue);
        setVerticalPositionExtent(extent);
    }
}
