/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * A container of radio buttons which allows at most one radio button to be
 * selected.
 */
public class ButtonGroup 
implements Serializable {

    private Set buttons = null;
    private ButtonModel selection = null;

    /**
     * Creates a new ButtonGroup.
     */
    public ButtonGroup() {
        super();

        buttons = new HashSet();
    }

    /**
     * Adds the button to the group.  If the button is selected and is added
     * to a group that already has a button that is selected, the button will
     * be automatically deselected.
     *
     * @param button The button to add to the group.
     */
    public void add(AbstractButton button) {
        if (! buttons.contains(button)) {
            if (button.isSelected() && selection != button.getModel()) {
                button.setSelected(false);
            }
            buttons.add(button);
            button.getModel().setGroup(this);
        }
    }
    
    /**
     * Returns the number of buttons in the group.
     *
     * @return The number of buttons in the group.
     */ 
    public int getButtonCount() {
        return buttons.size();
    }

    /**
     * Returns all the buttons that are participating in this group.
     *
     * @return An iterator of buttons in this group.
     */
    public Iterator getElements() {
        return Collections.unmodifiableSet(buttons).iterator();
    }

    /**
     * Returns the selected button model.
     *
     * @return The selected button model.
     */
    public ButtonModel getSelection() {
        return selection;
    }
    
    /**
     * Returns the selected value for the button.
     *
     * @param buttonModel The button on which to test selection.
     * @return True if the specified button is selected.
     */ 
    public boolean isSelected(ButtonModel buttonModel) {
        return buttonModel.isSelected();
    }
    
    /**
     * Removes the button from the group.
     *
     * @param button The button to remove from the group.
     */
    public void remove(AbstractButton button) {
        if (buttons.contains(button)) {
            buttons.remove(button);
            button.getModel().setGroup(null);
        }
    }
    
    /**
     * Sets the selected value for the button.
     *
     * @param buttonModel The button whose state is to be changed.
     * @param selected The new state for the button.
     */
    public void setSelected(ButtonModel buttonModel, boolean selected) {
        // If a button in the group is already selected, deselect it.
        if (selected && selection != null && buttonModel != selection) {
            if (selection.isSelected()) {
                selection.setSelected(false);
            }
        }

        // Set the selected state of the button.
        // The ButtonModel and ButtonGroup will infinitely inform each
        // other about the update unless the we only update if state has
        // changed.
        if (buttonModel.isSelected() != selected) {
            buttonModel.setSelected(selected);
        }
        
        if (selected) {
            selection = buttonModel;
        }
    }
}
