/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package nextapp.echo;

/**
 * A highly configurable push button.
 */
public class CButton extends Button {
    
    /**
     * A border style that causes a simulated 3D border to be rendered, such 
     * that the button appears recessed.
     */
    public static final int BORDER_INSET = 2;
    
    /**
     * A border style that causes no border to be rendered.
     */
    public static final int BORDER_NONE = 0;
    
    /**
     * A border style that causes a simulated 3D border to be rendered, such 
     * that the button appears raised.
     */
    public static final int BORDER_OUTSET = 3;
    
    /**
     * A border style that causes a solid, monochrome border to be rendered in
     * in the specified color.
     */
    public static final int BORDER_SOLID = 1;
    
    /** 
     * A style constant for the Border Color property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_BORDER_COLOR = "borderColor";

    /** 
     * A style constant for the Border Size property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_SIZE = "borderSize";

    /** 
     * A style constant for the Border Style property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_STYLE = "borderStyle";

    /** 
     * A style constant for the Height Extent property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_HEIGHT_EXTENT = "heightExtent";

    /** 
     * A style constant for the Insets property. 
     * Values of this key must be of type nextapp.echo.Insets.
     */
    public static final String STYLE_INSETS = "insets";

    /** 
     * A style constant for the Pressed Border Color property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_PRESSED_BORDER_COLOR = "pressedBorderColor";

    /** 
     * A style constant for the Pressed Border Style property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_PRESSED_BORDER_STYLE = "pressedBorderStyle";
    
    /** 
     * A style constant for the Pressed Foreground property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_PRESSED_FOREGROUND = "pressedForeground";
    
    /** 
     * A style constant for the Pressed Background property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_PRESSED_BACKGROUND = "pressedBackground";
    
    /** 
     * A style constant for the Rollover Border Color property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_ROLLOVER_BORDER_COLOR = "rolloverBorderColor";
    
    /** 
     * A style constant for the Width Extent property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_WIDTH_EXTENT = "widthExtent";

    public static final String BORDER_COLOR_CHANGED_PROPERTY = "borderColor";
    public static final String BORDER_SIZE_CHANGED_PROPERTY = "borderSize";
    public static final String BORDER_STYLE_CHANGED_PROPERTY = "borderStyle";
    public static final String HEIGHT_EXTENT_CHANGED_PROPERTY = "heightExtent";
    public static final String INSETS_CHANGED_PROPERTY = "insets";
    public static final String PRESSED_BORDER_COLOR_CHANGED_PROPERTY = "pressedBorderColor";
    public static final String PRESSED_BORDER_STYLE_CHANGED_PROPERTY = "pressedBorderStyle";
    public static final String PRESSED_BACKGROUND_CHANGED_PROPERTY = "pressedBackground";
    public static final String PRESSED_FOREGROUND_CHANGED_PROPERTY = "pressedForeground";
    public static final String ROLLOVER_BORDER_COLOR_CHANGED_PROPERTY = "rolloverBorderColor";
    public static final String WIDTH_EXTENT_CHANGED_PROPERTY = "widthExtent";
    
    public static final Insets DEFAULT_INSETS = new Insets(5);
    
    private Color borderColor = Color.BLACK;
    private int borderSize = 1;
    private int borderStyle = BORDER_SOLID;
    private Insets insets = DEFAULT_INSETS;
    private Color pressedBorderColor;
    private int pressedBorderStyle;
    private Color pressedForeground;
    private Color pressedBackground;
    private Extent widthExtent;
    private Extent heightExtent;
    private Color rolloverBorderColor;

    /**
     * Creates a button with no text or icon.
     */
    public CButton() {
        this(null, null);
    }
    
    /**
     * Creates a button with an icon.
     *
     * @param icon The icon to be displayed in the button.
     */
    public CButton(ImageReference icon) {
        this(null, icon);
    }
    
    /**
     * Creates a button with text.
     *
     * @param text The text to be displayed in the button.
     */
    public CButton(String text) {
        this(text, null);
    }

    /**
     * Creates a button with text and an icon.
     *
     * @param text The text to be displayed in the button.
     * @param icon The icon to be displayed in the button.
     */
    public CButton(String text, ImageReference icon) {
        super();
        
        setModel(new DefaultButtonModel());
    
        setIcon(icon);
        setText(text);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(nextapp.echo.Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_BORDER_COLOR)) {
            setBorderColor((Color) style.getAttribute(STYLE_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_BORDER_SIZE)) {
            setBorderSize(style.getIntegerAttribute(STYLE_BORDER_SIZE));
        }
        if (style.hasAttribute(STYLE_BORDER_STYLE)) {
            setBorderStyle(style.getIntegerAttribute(STYLE_BORDER_STYLE));
        }
        if (style.hasAttribute(STYLE_HEIGHT_EXTENT)) {
            setHeightExtent((Extent) style.getAttribute(STYLE_HEIGHT_EXTENT));
        }
        if (style.hasAttribute(STYLE_INSETS)) {
            setInsets((Insets) style.getAttribute(STYLE_INSETS));
        }
        if (style.hasAttribute(STYLE_PRESSED_BACKGROUND)) {
            setPressedBackground((Color) style.getAttribute(STYLE_PRESSED_BACKGROUND));
        }
        if (style.hasAttribute(STYLE_PRESSED_BORDER_COLOR)) {
            setPressedBorderColor((Color) style.getAttribute(STYLE_PRESSED_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_PRESSED_BORDER_STYLE)) {
            setPressedBorderStyle(style.getIntegerAttribute(STYLE_PRESSED_BORDER_STYLE));
        }
        if (style.hasAttribute(STYLE_PRESSED_FOREGROUND)) {
            setPressedForeground((Color) style.getAttribute(STYLE_PRESSED_FOREGROUND));
        }
        if (style.hasAttribute(STYLE_ROLLOVER_BORDER_COLOR)) {
            setRolloverBorderColor((Color) style.getAttribute(STYLE_ROLLOVER_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_WIDTH_EXTENT)) {
            setWidthExtent((Extent) style.getAttribute(STYLE_WIDTH_EXTENT));
        }
    }

    /**
     * Returns the border color of the button.
     * 
     * @return The border color of the button.
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
    /**
     * Returns the size of the border.
     * 
     * @return The size of the border.
     */
    public int getBorderSize() {
        return borderSize;
    }
    
    /**
     * Returns the (non-pressed) border style.
     * 
     * @return The border style.
     */
    public int getBorderStyle() {
        return borderStyle;
    }
    
    /**
     * Returns the (minimum) height of the button.
     * 
     * @return The height of the button.
     */
    public Extent getHeightExtent() {
        return heightExtent;
    }
    
    /**
     * Returns the <code>Insets</code> of the button.
     * 
     * @return The <code>Insets</code> of the button.
     */
    public Insets getInsets() {
        return insets;
    }
    
    /**
     * Returns the pressed background color of the button.
     * 
     * @return The pressed background color of the button.
     */
    public Color getPressedBackground() {
        return pressedBackground;
    }
    
    /**
     * Returns the pressed border color of the button.
     * 
     * @return The pressed border color of the button.
     */
    public Color getPressedBorderColor() {
        return pressedBorderColor;
    }
    
    /**
     * Returns the pressed border style of the button.
     * 
     * @return The pressed border style of the button.
     */
    public int getPressedBorderStyle() {
        return pressedBorderStyle;
    }
    
    /**
     * Returns The pressed foreground of the button.
     * 
     * @return The pressed foreground of the button.
     */
    public Color getPressedForeground() {
        return pressedForeground;
    }
    
    /**
     * Returns The rollover border color of the button.
     * 
     * @return The rollover border color of the button.
     */
    public Color getRolloverBorderColor() {
        return rolloverBorderColor;
    }
    
    /**
     * Returns the (minimum) width of the button.
     * 
     * @return The width of the button.
     */
    public Extent getWidthExtent() {
        return widthExtent;
    }
    
    /**
     * Sets the border color of the button.
     * 
     * @param newValue The new border color of the button.
     */
    public void setBorderColor(Color newValue) {
        Color oldValue = borderColor;
        borderColor = newValue;
        firePropertyChange(BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the size of the border surrounding the button.
     * 
     * @param newValue the new border size, in pixels.
     */
    public void setBorderSize(int newValue) {
        int oldValue = borderSize;
        borderSize = newValue;
        firePropertyChange(BORDER_SIZE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the border style of the button.
     * 
     * @param newValue The new border style, one of the following
     *        values:
     *        <ul>
     *         <li>BORDER_NONE</li>
     *         <li>BORDER_SOLID</li>
     *         <li>BORDER_INSET</li>
     *         <li>BORDER_OUTSET</li>
     *        </ul>
     */
    public void setBorderStyle(int newValue) {
        int oldValue = borderStyle;
        borderStyle = newValue;
        firePropertyChange(BORDER_STYLE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the (minimum) height of the button.
     * 
     * @param newValue The new height of the button.
     */
    public void setHeightExtent(Extent newValue) {
        Extent oldValue = heightExtent;
        heightExtent = newValue;
        firePropertyChange(HEIGHT_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the insets of the button (distance between border edge and content
     * of button).
     * 
     * @param newValue The new <code>Insets</code> value.
     */
    public void setInsets(Insets newValue) {
        Insets oldValue = insets;
        insets = newValue;
        firePropertyChange(INSETS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the pressed background color.
     * 
     * @param newValue The new pressed background color.
     */
    public void setPressedBackground(Color newValue) {
        Color oldValue = pressedBackground;
        pressedBackground = newValue;
        firePropertyChange(PRESSED_BACKGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Set the pressed border color.
     * 
     * @param newValue The new pressed border color.
     */
    public void setPressedBorderColor(Color newValue) {
        Color oldValue = pressedBorderColor;
        pressedBorderColor = newValue;
        firePropertyChange(PRESSED_BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the pressed border style.
     * 
     * @param newValue The new pressed border style, one of the following
     *        values:
     *        <ul>
     *         <li>BORDER_NONE</li>
     *         <li>BORDER_SOLID</li>
     *         <li>BORDER_INSET</li>
     *         <li>BORDER_OUTSET</li>
     *        </ul>
     */
    public void setPressedBorderStyle(int newValue) {
        int oldValue = pressedBorderStyle;
        pressedBorderStyle = newValue;
        firePropertyChange(PRESSED_BORDER_STYLE_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the pressed foreground color.
     * 
     * @param newValue The new pressed foreground color.
     */
    public void setPressedForeground(Color newValue) {
        Color oldValue = pressedForeground;
        pressedForeground = newValue;
        firePropertyChange(PRESSED_FOREGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the rollover border color.
     * 
     * @param newValue The new rollover border color.
     */
    public void setRolloverBorderColor(Color newValue) {
        Color oldValue = rolloverBorderColor;
        rolloverBorderColor = newValue;
        firePropertyChange(ROLLOVER_BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the (minimum) width of the button.
     * 
     * @param newValue The new width of the button.
     */
    public void setWidthExtent(Extent newValue) {
        Extent oldValue = widthExtent;
        widthExtent = newValue;
        firePropertyChange(WIDTH_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
