/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package nextapp.echo;

/**
 * A layout container component which arranges its children in a single column.
 */
public class Column extends Component {
    
    /** 
     * A style constant for the Cell Height property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_CELL_HEIGHT = "cellHeight";
    
    /** 
     * A style constant for the Cell Spacing property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_CELL_SPACING = "cellSpacing";

    /** 
     * A style constant for the Horizontal Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";
    
    /** 
     * A style constant for the Vertical Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";
    
    /** 
     * A style constant for the Height Extent property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_HEIGHT_EXTENT = "heightExtent";
    
    /** 
     * A style constant for the Width Extent property. 
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_WIDTH_EXTENT = "widthExtent";

    public static final String CELL_HEIGHT_CHANGED_PROPERTY = "cellHeight";
    public static final String CELL_SPACING_CHANGED_PROPERTY = "cellSpacing";
    public static final String HEIGHT_EXTENT_CHANGED_PROPERTY = "heightExtent";
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
    public static final String WIDTH_EXTENT_CHANGED_PROPERTY = "widthExtent";
    
    private Extent cellHeight;
    private Extent cellSpacing;
    private Extent heightExtent;
    private int horizontalAlignment;
    private int verticalAlignment;
    private Extent widthExtent;
    
    /**
     * Creates a new <code>Column</code>.
     */
    public Column() {
        super();
    }

    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_CELL_HEIGHT)) {
            setCellHeight((Extent) style.getAttribute(STYLE_CELL_HEIGHT));
        }
        if (style.hasAttribute(STYLE_CELL_SPACING)) {
            setCellSpacing((Extent) style.getAttribute(STYLE_CELL_SPACING));
        }
        if (style.hasAttribute(STYLE_HEIGHT_EXTENT)) {
            setHeightExtent((Extent) style.getAttribute(STYLE_HEIGHT_EXTENT));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
            setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_WIDTH_EXTENT)) {
            setWidthExtent((Extent) style.getAttribute(STYLE_WIDTH_EXTENT));
        }
    }

    /**
     * Returns the height of a single cell of the container.
     * 
     * @return The height of a single cell of the container.
     */
    public Extent getCellHeight() {
        return cellHeight;
    }
    
    /**
     * Returns the spacing between cells.
     * 
     * @return The spacing between cells.
     */
    public Extent getCellSpacing() {
        return cellSpacing;
    }

    /**
     * Returns the overall height of the container.
     * 
     * @return The overall height of the container.
     */
    public Extent getHeightExtent() {
        return heightExtent;
    }
    
    /**
     * Returns the horizontal alignment of the contents of cells of this 
     * container.
     *
     * @return The horizontal alignment of the contents of cells of this 
     *         container, one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }

    /**
     * Returns the vertical alignment of the contents of cells of this 
     * container.
     *
     * @return The vertical alignment of the contents of cells of this 
     *         container, one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalAlignment() {
        return verticalAlignment;
    }
    
    /**
     * Returns the overall width of the container.
     * 
     * @return The overall width of the container.
     */
    public Extent getWidthExtent() {
        return widthExtent;
    }

    /**
     * Sets the height of a single cell of the container.
     * 
     * @param newValue The new height of a single cell of the container.
     */
    public void setCellHeight(Extent newValue) {
        Extent oldValue = cellHeight;
        cellHeight = newValue;
        firePropertyChange(CELL_HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the spacing between individual cells of the container.
     * 
     * @param newValue The new cell spacing.
     */
    public void setCellSpacing(Extent newValue) {
        Extent oldValue = cellSpacing;
        cellSpacing = newValue;
        firePropertyChange(CELL_SPACING_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the overall height of the container.
     * 
     * @param newValue The new overall height.
     */
    public void setHeightExtent(Extent newValue) {
        Extent oldValue = heightExtent;
        heightExtent = newValue;
        firePropertyChange(HEIGHT_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the horizontal alignment of the contents of cells of this 
     * container.
     *
     * @param newValue The horizontal alignment of the contents of cells of this 
     *        container, one of the following values:
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    
    /**
     * Sets the vertical alignment of the contents of cells of this 
     * container.
     *
     * @param newValue The vertical alignment of the contents of cells of this 
     *        container, one of the following values:
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalAlignment(int newValue) {
        int oldValue = verticalAlignment;
        verticalAlignment = newValue;
        firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the overall width of the container.
     * 
     * @param newValue The new overall width.
     */
    public void setWidthExtent(Extent newValue) {
        Extent oldValue = widthExtent;
        widthExtent = newValue;
        firePropertyChange(WIDTH_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
