/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.EventListener;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import nextapp.echo.event.EventListenerList;
import nextapp.echo.event.HierarchyEvent;
import nextapp.echo.event.HierarchyListener;

/**
 * A base class from which all Components are derived.
 */
public abstract class Component 
implements Serializable {

    private static final int CHILD_LIST_CAPACITY = 3;
    private static final Component[] EMPTY_COMPONENT_ARRAY = new Component[0];

    public static final String BACKGROUND_CHANGED_PROPERTY = "background";
    public static final String CHILDREN_CHANGED_PROPERTY = "children";
    public static final String ENABLED_CHANGED_PROPERTY = "enabled";
    public static final String FONT_CHANGED_PROPERTY = "font";
    public static final String FOREGROUND_CHANGED_PROPERTY = "foreground";
    public static final String IDENTIFIER_CHANGED_PROPERTY = "identifier";
    public static final String LOCALE_CHANGED_PROPERTY = "locale";
    public static final String MODEL_CHANGED_PROPERTY = "model";
    public static final String TAB_INDEX_CHANGED_PROPERTY = "tabIndex";
    public static final String UPDATE_PROPERTY = "update";
    public static final String VISIBLE_CHANGED_PROPERTY = "visible";
    
    private static final int FLAG_ENABLED = 0x1;
    private static final int FLAG_VISIBLE = 0x2;
    private static final int FLAG_REGISTERED = 0x4;
    private static final int FLAG_INITIALIZED = 0x8;
    
    private static final int FLAGS_TAB_INDEX = 0x7fff0000;

    /**
     * A style constant for the Background property.  
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_BACKGROUND = "background";

    /** 
     * A style constant for the Font property. 
     * Values of this key must be of type nextapp.echo.Font.
     */
    public static final String STYLE_FONT = "font";

    /**
     * A style constant for the Foreground property.  
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_FOREGROUND = "foreground";

    /** The propery change event dispatcher. */
    private PropertyChangeSupport propertyChangeSupport = null;
    
    /** A collection of references to child components. */
    private List children = null;
    
    /** The parent of the component. */
    private Component parent = null;
    
    /** The default background color of the component. */
    private Color background = null;
    
    /** The default font of the component. */
    private Font font = null;
    
    /** The default foreground color of the component. */
    private Color foreground = null;
    
    /** The default locale of the component. */
    Locale locale = null;

    /** Used to store any type of listener on this Component. */
    protected EventListenerList listenerList = null;
    
    /** The identifier of the component. */
    private Object identifier = null;
    
    /**
     * Boolean flags for this component, including enabled state, visibility, 
     * and registration.  Multiple booleans are wrapped in a single integer
     * to save memory, since many <code>Component</code>instances will be 
     * created.
     */
    private int flags = FLAG_ENABLED | FLAG_VISIBLE;
    
    /**
     * Default Component constructor.
     */
    public Component() {
        super();
        listenerList = new EventListenerList();
        propertyChangeSupport = new PropertyChangeSupport(this);
    }
    
    /**
     * Adds the specified Component at the end of this Component's children.
     *
     * @param c The child component to add.
     */ 
    public void add(Component c) {
        add(c, -1);
    }
    
    /**
     * Adds the specified Component as the nth child
     * All component add operations use this method to add components.
     * Components that require notification of all child additions should 
     * override this method (making sure to call the superclass' 
     * implementation).
     *
     * @param c The child component to add.
     * @param n The index at which to add the child component, or -1 to add the
     *          component at the end.
     */
    public void add(Component c, int n) {

        if (c.parent != null) {
            // Request component's current parent remove component.
            c.parent.remove(c);
        }
        
        // Lazy-create child collection if necessary.
        if (children == null) {
            children = new ArrayList(CHILD_LIST_CAPACITY);
        }

        // Connect child to parent.
        c.parent = this;
        if (n == -1) {
            children.add(c);
        } else {
            children.add(n, c);
        }

        // Determine change flags for generated HierarchyEvents.
        long changeFlags = HierarchyEvent.PARENT_CHANGED;
        if (isRegistered()) {
            c.setRegistered(true);
            
            changeFlags |= HierarchyEvent.REGISTRATION_CHANGED;
            if (c.isRecursivelyVisible()) {
                changeFlags |= HierarchyEvent.SHOWING_CHANGED;
            }
        }

        // Notify PropertyChangeListeners of change.
        firePropertyChange(CHILDREN_CHANGED_PROPERTY, null, c);
        
        // Notify HierarchyListeners of change.
        if (changeFlags != 0) {
            c.processHierarchyEvent(changeFlags, c, c.parent);
        }
    }
    
    /**
     * Adds a hierarchy listener to be notified when this component's
     * state in the hierarchy changes.
     * 
     * @param l The <code>HierarchyListener</code> to add.
     */
    public void addHierarchyListener(HierarchyListener l) {
        listenerList.addListener(HierarchyListener.class, l);
    }
    
    /**
     * Adds a property change listener to this component.
     *
     * @param l The listener to add.
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.addPropertyChangeListener(l);
    }
    
    /**
     * Sets stylistic properties of this component based on a 
     * <code>Style</code> object.  The version of this method in
     * <code>Component</code> will set foreground and background colors
     * and the font from the style if they are present.  Components
     * should override this method if they allow more properties to be
     * set from a style object, and should call super.applyStyle() if 
     * necessary.
     *
     * @param style The style object from which to retrieve properties
     *        for this <code>Component</code>.
     */
    public void applyStyle(Style style) {
        if (style.hasAttribute(STYLE_BACKGROUND)) {
            setBackground((Color) style.getAttribute(STYLE_BACKGROUND));
        }
        if (style.hasAttribute(STYLE_FONT)) {
            setFont((Font) style.getAttribute(STYLE_FONT));
        }
        if (style.hasAttribute(STYLE_FOREGROUND)) {
            setForeground((Color) style.getAttribute(STYLE_FOREGROUND));
        }
    }
    
    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, char oldValue, char newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, double oldValue, double newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, new Double(oldValue), new Double(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, float oldValue, float newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, new Float(oldValue), new Float(newValue));
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, int oldValue, int newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    /**
     * Reports a bound property change.
     *
     * @param propertyName The name of the changed property.
     * @param oldValue The previous value of the property.
     * @param newValue The present value of the property.
     */
    public void firePropertyChange(String propertyName, long oldValue, long newValue) {
        propertyChangeSupport.firePropertyChange(propertyName, new Long(oldValue), new Long(newValue));
    }
    
    /**
     * Returns the background color of the component.
     *
     * @return The background color of the component.
     */
    public Color getBackground() {
        return background;
    }
    
    /**
     * Gets the nth immediate child component.
     *
     * @param n The index of the component to retrieve.
     * @return The component at index n.
     * @throws IndexOutOfBoundsException If the index is invalid.
     */
    public Component getComponent(int n) {
        if (children == null) {
            throw new IndexOutOfBoundsException();
        }
        
        return (Component) children.get(n);
    }
    
    /**
     * Returns the number of immediate child components.
     *
     * @return The number of immediate child components.
     */
    public int getComponentCount() {
        if (children == null) {
            return 0;
        } else {
            return children.size();
        }
    }
    
    /**
     * Returns an array of all immediate child components.
     *
     * @return An array of all immediate child components.
     */
    public Component[] getComponents() {
        if (children == null) {
            return EMPTY_COMPONENT_ARRAY;
        } else {
            return (Component[]) children.toArray(new Component[children.size()]);
        }
    }
    
    /**
     * Returns the EchoInstance object to which this component belongs, or
     * null if it is not bound to any instance.
     *
     * @return The EchoInstance object to which this component belongs.
     */
    public EchoInstance getEchoInstance() {
        if (parent == null) {
            return EchoInstance.getEchoInstance();
        } else {
            return parent.getEchoInstance();
        }
    }

    /**
     * Returns the font.
     *
     * @return The font of the component.
     */
    public Font getFont() {
        return font;
    }
    
    /**
     * Returns the foreground of the component.
     *
     * @return The foreground of the component.
     */
    public Color getForeground() {
        return foreground;
    }

    /**
     * Returns the identifier for this component.   Every component may have
     * an identifier.  Identifiers are provided as a convenience to the
     * application developer, and are not used internally in any manner.
     *
     * @return The identifier for this component.
     */
    public Object getIdentifier() {
        return identifier;
    }
    
    /**
     * Returns the locale of the component.  If the component does not have a
     * locale, its ancestors will be queried for a locale.  If no ancestors
     * have locales set, the application instance's locale will be returned.
     * If the component is not registered, the active application instance's
     * will be returned.
     *
     * @return The locale for this component.
     */
    public Locale getLocale() {
        Locale locale;
    
        if (this.locale == null) {
            if (parent == null) {
                EchoInstance echoInstance = EchoInstance.getEchoInstance();
                if (echoInstance == null) {
                    locale = null;
                } else {
                    locale = echoInstance.getLocale();
                }
            } else {
                locale = parent.getLocale();
            }
        } else {
            locale = this.locale;
        }
        
        return locale;
    }
    
    /**
     * Returns the parent component.
     *
     * @return The parent component.
     */
    public Component getParent() {
        return parent;
    }
    
    /**
     * Returns the tab index of the component.
     * 
     * @return The tab index of the component.
     */
    public int getTabIndex() {
        return (flags & FLAGS_TAB_INDEX) >> 16;
    }
    
    /**
     * Returns the index of the specified component if it a child of this 
     * Component, or -1 if it is not.
     *
     * @param c The component to determine the index of.
     * @return The index of the specified component.
     */
    public int indexOf(Component c) {
        if (children == null) {
            return -1;
        } else {
            return children.indexOf(c);
        }
    }
    
    /**
     * Called exactly one time when a component is registered to a 
     * hierarchy for the first time. <br>
     * <br>
     * This method is intended to be overridden by applications that require
     * constructor-like functionality of a Component but can only perform
     * such functions after the component has been registered to a hierarchy.
     * This is often necessary as application-specific and locale information
     * are not available until registration occurs.<br>
     * <br>
     * Developers overriding this method should ensure that 
     * <code>super.init()</code> is also invoked.
     */
    protected void init() { }

    /**
     * Determines if this component is an ancestor of the specified
     * component.
     * 
     * @param c The component to test for ancestry.
     * @return true If this component is an ancestor of the specified 
     *         component.
     */
    public boolean isAncestorOf(Component c) {
        while (c != null && c != this) {
            c = c.parent;
        }
        return c == this;
    }
        
    /**
     * Returns true if the component is enabled.
     *
     * @return True if the component is enabled.
     */
    public boolean isEnabled() {
        return (FLAG_ENABLED & flags) != 0;
    }
    
    /**
     * Returns true if the component is focused.
     * 
     * @return True if the component is focused.
     */
    public boolean isFocused() {
        EchoInstance instance = getEchoInstance();
        return instance != null && instance.getFocusedComponent() == this;
    }
    
    /**
     * Returns true if the component and all of its ancestors are visible.
     * 
     * @return True if the component and all of its ancestors are visible.
     */
    private boolean isRecursivelyVisible() {
        return isVisible() && (parent == null || parent.isRecursivelyVisible());
    }
    
    /**
     * Returns true if the component is registered to an application.
     * 
     * @return True if the component is registered to an application.
     */
    public final boolean isRegistered() {
        return (FLAG_REGISTERED & flags) != 0;
    }
    
    /**
     * Returns true if the component is registered and all of its ancestors 
     * are visible.
     * 
     * @return True if the component is registered and all of its ancestors
     *         are visible.
     */
    public final boolean isShowing() {
        return isRegistered() && isRecursivelyVisible();
    }
    
    /**
     * Returns the component's visibility state.
     *
     * @return The component's visibility state.
     */
    public final boolean isVisible() {
        return (FLAG_VISIBLE & flags) != 0;
    }
    
    /**
     * Notifies descendant components of a hierarchy event.
     * 
     * @param changeFlags The hierarchy event flags used to describe the 
     *        changes.
     * @param changed The component whose state in the hierarchy changed.
     * @param changedParent The parent (or in some cases, former parent) of 
     *        the component whose state in the hierarchy changed.
     */
    void processHierarchyEvent(long changeFlags, Component changed, Component changedParent) {
        Component[] children = getComponents();        

        // Invoke init() method if registered and uninitialized.
        if (isRegistered() && (flags & FLAG_INITIALIZED) == 0) {
            flags |= FLAG_INITIALIZED;
            init();
        }
        
        // Fire HierarchyEvent to HierarchyListeners of this component.
        EventListener[] listeners = listenerList.getListeners(HierarchyListener.class);
        if (listeners.length > 0) {
            HierarchyEvent e = new HierarchyEvent(this, changeFlags, changed, changedParent);
            for (int index = 0; index < listeners.length; ++index) {
                ((HierarchyListener) listeners[index]).hierarchyChanged(e);
            }
        }
        
        // Recursively call processHierarchyEvent() on children.
        for (int index = 0; index < children.length; ++index) {
            children[index].processHierarchyEvent(changeFlags, changed, changedParent);
        }
    }
    
    /**
     * Removes the specified child component.
     * All component remove operations use this method to remove components.
     * Components that require notification of all child removals should 
     * override this method (making sure to call the superclass' 
     * implementation).
     * 
     * @param c The child component to remove.
     */
    public void remove(Component c) {

        if (children == null || !children.contains(c)) {
            // Do-nothing if component is not a child.
            return;
        }

        // Determine change flags for HierarchyEvents.
        long changeFlags = HierarchyEvent.PARENT_CHANGED;
        if (isRegistered()) {
            c.setRegistered(false);
            
            changeFlags |= HierarchyEvent.REGISTRATION_CHANGED;
            if (c.isRecursivelyVisible()) {
                changeFlags |= HierarchyEvent.SHOWING_CHANGED;
            }
        }
        
        // Store reference to current parent for use in HierarchyEvents.
        Component oldParent = c.parent;
        
        // Dissolve references between parent and child.
        children.remove(c);
        c.parent = null;
        
        // Notify PropertyChangeListeners of change.
        firePropertyChange(CHILDREN_CHANGED_PROPERTY, c, null);

        // Notify HierarchyListeners of change.
        c.processHierarchyEvent(changeFlags, c, oldParent);
    }
    
    /**
     * Removes the component at the given index.
     *
     * @param n The index of the child component to remove.
     * @throws IndexOutOfBoundsException If the index is not valid.
     */
    public void remove(int n) {
        if (children == null) {
            throw new IndexOutOfBoundsException();
        }
        remove(getComponent(n));
    }
    
    /**
     * Removes all child Components.
     */
    public void removeAll() {
        if (children != null) {
            while (children.size() > 0) {
                Component c = (Component) children.get(children.size() - 1);
                remove(c);
            }
            children = null;
        }
    }

    /**
     * Removes a hierarchy listener from being notified when this 
     * component's state in the hierarchy changes.
     * 
     * @param l The <code>HierarchyListener</code> to remove.
     */
    public void removeHierarchyListener(HierarchyListener l) {
        listenerList.removeListener(HierarchyListener.class, l);
    }
    
    /**
     * Removes a property change listener from this component.
     *
     * @param l The listener to be removed.
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.removePropertyChangeListener(l);
    }
    
    /**
     * Sets the background color of the component.
     *
     * @param newValue The new background color of the component.
     */
    public void setBackground(Color newValue) {
        Color oldValue = background;
        background = newValue;
        firePropertyChange(BACKGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets whether the component is enabled.
     *
     * @param newValue True if the component is enabled.
     */
    public void setEnabled(boolean newValue) {
        boolean oldValue = (FLAG_ENABLED & flags) != 0;
        if (oldValue != newValue) {
            flags ^= FLAG_ENABLED;
        }
        firePropertyChange(ENABLED_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets whether the component is focused.
     * 
     * @param newValue True if the component is to be focused.
     */
    public void setFocused(boolean newValue) {
        EchoInstance instance = getEchoInstance();
        if (instance == null) {
            return;
        }
        instance.setFocusedComponent(this);
    }

    /**
     * Sets the font of the component.
     *
     * @param newValue The new font of the component.
     */
    public void setFont(Font newValue) {
        Font oldValue = font;
        font = newValue;
        firePropertyChange(FONT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the foreground color of the component.
     *
     * @param newValue The new foreground color of the component.
     */
    public void setForeground(Color newValue) {
        Color oldValue = foreground;
        foreground = newValue;
        firePropertyChange(FOREGROUND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the identifier for this component.   Every component may have
     * an identifier.  Identifiers are provided as a convenience to the
     * application developer, and are not used internally in any manner.
     *
     * @param newValue The new identifier for this component.
     */
    public void setIdentifier(Object newValue) {
        Object oldValue = identifier;
        identifier = newValue;
        firePropertyChange(IDENTIFIER_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the locale of the component.
     *
     * @param newValue The new locale for the component.
     */
    public void setLocale(Locale newValue) {
        Locale oldValue = locale;
        locale = newValue;
        firePropertyChange(LOCALE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the registration state of the component and children beneath it
     * within the hierarchy.
     * 
     * @param registered The new registration state.
     */
    void setRegistered(boolean registered) {
        if (registered) {
            flags |= FLAG_REGISTERED;
        } else {
            flags = flags & ~FLAG_REGISTERED;
        }
        if (children != null) {
            Iterator it = children.iterator();
            while (it.hasNext()) {
                ((Component) it.next()).setRegistered(registered);
            }
        }
    }
    
    /**
     * Sets the tab index of the component.
     * 
     * @param newValue the new tab index of the component.
     */
    public void setTabIndex(int newValue) {
        int oldValue = getTabIndex();
        newValue &= 0x7fff;
        flags = flags & ((~FLAGS_TAB_INDEX)) | (newValue << 16);
        firePropertyChange(TAB_INDEX_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the component visible or invisible.
     *
     * @param newValue The new visibility state of the component.
     */
    public void setVisible(boolean newValue) {
        boolean oldValue = (FLAG_VISIBLE & flags) != 0;
        if (oldValue != newValue) {
            flags ^= FLAG_VISIBLE;
            firePropertyChange(VISIBLE_CHANGED_PROPERTY, oldValue, newValue);
            if (isRegistered()) {
                processHierarchyEvent(HierarchyEvent.SHOWING_CHANGED, this, parent);
            }
        }
    }
    
    /**
     * Forces the component to be re-rendered immediately.
     * Fires a property change event indicating that any/all properties
     * of the Component changed, forcing the container to re-render it.
     */
    public void update() {
        firePropertyChange(null, null, null);
    }

    /**
     * Validates this component.
     * This method will be called by the application container before the
     * component is rendered.  This method should be overridden if the 
     * component needs to be updated before being rendered.  Validation 
     * occurs from the top of the component hierarchy down.  Overriding
     * methods should take care to first call super.validate() to ensure the 
     * superclass is validated as well.<br>
     * <br>
     * If components above this one in the hierarchy are modified by an
     * overriding implementation of this method, those components WILL NOT
     * be automatically revalidated.  Thus, performing such operations in
     * <code>validate()</code> is not inadvisable.
     */
    public void validate() { }
}
