/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A container used to lay out panes within a window. <br>
 * <br>
 * Only <code>AbstractPane</code>s, e.g. <code>ContainerPane</code>s,
 * <code>ContentPane</code>s, and <code>HttpPane</code>s may be added as 
 * children to a <code>ContainerPane</code>.  Adding components that do not 
 * subclass <code>AbstractPane</code> will result in undocumented behavior.<br>
 * <br>
 * <code>ContainerPane</code>s may only be added as children to 
 * <code>Window</code>s and other <code>ContainerPane</code>s.
 */
public class ContainerPane extends AbstractPane {

    public static final String CONTENT_CHANGED_PROPERTY = "content";
    public static final String ORIENTATION_CHANGED_PROPERTY = "orientation";
    public static final String PREFERRED_HEIGHT_CHANGED_PROPERTY = "preferredHeight";
    public static final String PREFERRED_HEIGHT_EXTENT_CHANGED_PROPERTY = "preferredHeightExtent";
    public static final String PREFERRED_WIDTH_CHANGED_PROPERTY = "preferredWidth";
    public static final String PREFERRED_WIDTH_EXTENT_CHANGED_PROPERTY = "preferredWidthExtent";
    
    /** 
     * A style constant for the Orientation property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_ORIENTATION = "orientation";

    /** 
     * Describes a horizontally oriented pane whose child panes are laid out 
     * left to right. 
     */
    public static final int ORIENTATION_HORIZONTAL = 0;
    
    /** 
     * Describes a vertically oriented pane whose child panes are laid out 
     * top to bottom. 
     */
    public static final int ORIENTATION_VERTICAL = 1;
    
    private int orientation;
    
    private Extent preferredHeightExtent = null;
    private Extent preferredWidthExtent = null;

    /**
     * Creates an empty container pane with a horizontal orientation.
     */
    public ContainerPane() {
        this(ORIENTATION_HORIZONTAL);
    }
    
    /**
     * Creates an empty container pane with the specified orientation.
     *
     * @param orientation The initial orientation of the container pane, one 
     *        of the following values:
     *        <ul>
     *        <li>ORIENTATION_HORIZONTAL (the default)</li>
     *        <li>ORIENTATION_VERTICAL</li>
     *        </ul>
     */
    public ContainerPane(int orientation) {
        super();
        this.orientation = orientation;
    }
    
    /**
     * Adds a subpane at the end of this container pane.
     *
     * @param pane The subpane to add.
     */
    public void add(AbstractPane pane) {
        super.add(pane);
        firePropertyChange(CONTENT_CHANGED_PROPERTY, null, pane);
    }
    
    /**
     * Adds a subpane at a specified position.
     *
     * @param pane The subpane to add.
     * @param index The position of the subpane.
     */
    public void add(AbstractPane pane, int index) {
        super.add(pane, index);
        firePropertyChange(CONTENT_CHANGED_PROPERTY, null, pane);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_ORIENTATION)) {
            setOrientation(style.getIntegerAttribute(STYLE_ORIENTATION));
        }
    }
    
    /**
     * Calculates the height of the pane.
     * If a value is provided for the preferred height of the pane, it is
     * returned.  Otherwise the height is calculated based on the 
     * contents of the pane.  In the case of a horizontally oriented container
     * pane, the height is determined to be the height of the tallest child 
     * pane.  In the case of a vertically oriented container pane, the height
     * is determined to be the sum of the individual heights of the child panes.
     *
     * @return The calculated height of the pane.
     * @see nextapp.echo.AbstractPane#getHeightExtent()
     */    
    public Extent getHeightExtent() {
        if (preferredHeightExtent == null) {
            int childCount = getComponentCount();
            if (childCount == 0) {
                return null;
            } else {
                AbstractPane pane = null;
                Extent calculatedHeight = null;

                if (orientation == ORIENTATION_HORIZONTAL) {
                    // Children are horizontally oriented: sum heights to determine height.
                    for (int index = 0; index < childCount; ++index) {
                        pane = (AbstractPane) getComponent(index);
                        Extent paneHeight = pane.getHeightExtent();
                        if (paneHeight == null || paneHeight.isPercentage()) {
                            // Height of tested pane unknown or is a percentage: cannot calculate overall height.
                            return null;
                        } else {
                            if (calculatedHeight == null) {
                                // First height analyzed: store in calculatedHeight.
                                calculatedHeight = pane.getHeightExtent();
                            } else {
                                // Second or later height analyzed: sum with calculatedHeight.
                                calculatedHeight = Extent.add(calculatedHeight, paneHeight);
                                if (calculatedHeight == null) {
                                    // Extent.add() was unable to calculate: cannot calculate overall height.
                                    return null;
                                }
                            }
                        }
                    }
                    return calculatedHeight;
                } else {
                    // Children are vertically oriented:  
                    for (int index = 0; index < childCount; ++index) {
                        pane = (AbstractPane) getComponent(index);
                        Extent paneHeight = pane.getHeightExtent();
                        if (paneHeight == null || paneHeight.isPercentage()) {
                            return null;
                        } else {
                            if (calculatedHeight == null) {
                                calculatedHeight = paneHeight;
                            } else {
                                if (!paneHeight.isComparableTo(calculatedHeight)) {
                                    return null;
                                } else {
                                    if (paneHeight.compareTo(calculatedHeight) > 0) {
                                        calculatedHeight = paneHeight;
                                    }
                                }
                            }
                        }
                    }
                    return calculatedHeight;
                }
            }
        } else {
            return preferredHeightExtent;
        }
    }
    
    /**
     * Returns the orientation of the pane.
     *
     * @return The orientation of the pane, one of the following values:
     *         <ul>
     *         <li>ORIENTATION_HORIZONTAL (the default)</li>
     *         <li>ORIENTATION_VERTICAL</li>
     *         </ul>
     */
    public int getOrientation() {
         return orientation;
    }
    
    /**
     * Returns the preferred width of the pane in pixels.
     *
     * @return The preferred width of the pane in pixels, or
     *         <code>EchoConstants.UNDEFINED_SIZE</code> if the width of the  
     *         pane is not in pixels.
     * @deprecated Use <code>getPreferredHeightExtent()</code>.
     */
    public int getPreferredHeight() {
        Extent extent = getPreferredHeightExtent();
        return (extent != null && extent.getUnits() == Extent.PX) ? extent.getValue() : EchoConstants.UNDEFINED_SIZE;
    }
    
    /**
     * Returns the preferred height of the pane.
     * 
     * @return The preferred height of the pane. 
     */
    public Extent getPreferredHeightExtent() {
        return preferredHeightExtent;
    }
    
    /**
     * Returns the preferred width of the pane in pixels.
     *
     * @return The preferred width of the pane in pixels, or
     *         <code>EchoConstants.UNDEFINED_SIZE</code> if the width of the  
     *         pane is not in pixels.
     * @deprecated Use <code>getPreferredWidthExtent()</code>.
     */    
    public int getPreferredWidth() {
        Extent extent = getPreferredWidthExtent();
        return (extent != null && extent.getUnits() == Extent.PX) ? extent.getValue() : EchoConstants.UNDEFINED_SIZE;
    }
    
    /**
     * Returns the preferred width of the pane.
     * 
     * @return The preferred width of the pane.
     */
    public Extent getPreferredWidthExtent() {
        return preferredWidthExtent;
    }
    
    /**
     * Calculates the width of the pane.
     * If a value is provided for the preferred width of the pane, it is
     * returned.  Otherwise the width is calculated based on the 
     * contents of the pane.  In the case of a vertically oriented container
     * pane, the width is determined to be the width of the widest child 
     * pane.  In the case of a horizontally oriented container pane, the width
     * is determined to be the sum of the individual widths of the child panes.
     *
     * @return The calculated width of the pane.
     * @see nextapp.echo.AbstractPane#getWidthExtent()
     */    
    public Extent getWidthExtent() {
        if (preferredWidthExtent == null) {
            int childCount = getComponentCount();
            if (childCount == 0) {
                return null;
            } else {
                AbstractPane pane = null;
                Extent calculatedWidth = null;

                if (orientation == ORIENTATION_HORIZONTAL) {
                    // Children are horizontally oriented: find greatest width.
                    for (int index = 0; index < childCount; ++index) {
                        pane = (AbstractPane) getComponent(index);
                        Extent paneWidth = pane.getWidthExtent();
                        if (paneWidth == null || paneWidth.isPercentage()) {
                            return null;
                        } else {
                            if (calculatedWidth == null) {
                                calculatedWidth = paneWidth;
                            } else {
                                if (!paneWidth.isComparableTo(calculatedWidth)) {
                                    return null;
                                } else {
                                    if (paneWidth.compareTo(calculatedWidth) > 0) {
                                        calculatedWidth = paneWidth;
                                    }
                                }
                            }
                        }
                    }
                    return calculatedWidth;
                } else {
                    // Children are vertically oriented: sum widths to determine width.
                    for (int index = 0; index < childCount; ++index) {
                        pane = (AbstractPane) getComponent(index);
                        Extent paneWidth = pane.getWidthExtent();
                        if (paneWidth == null || paneWidth.isPercentage()) {
                            // Width of tested pane unknown or is a percentage: cannot calculate overall width.
                            return null;
                        } else {
                            if (calculatedWidth == null) {
                                // First width analyzed: store in calculatedWidth.
                                calculatedWidth = pane.getWidthExtent();
                            } else {
                                // Second or later width analyzed: sum with calculatedWidth.
                                calculatedWidth = Extent.add(calculatedWidth, paneWidth);
                                if (calculatedWidth == null) {
                                    // Extent.add() was unable to calculate: cannot calculate overall width.
                                    return null;
                                }
                            }
                        }
                    }
                    return calculatedWidth;
                }
            }
        } else {
            return preferredWidthExtent;
        }
    }
    
    /**
     * Removes a subpane.
     *
     * @param pane The pane to remove.
     */
    public void remove(AbstractPane pane) {
        super.remove(pane);
        firePropertyChange(CONTENT_CHANGED_PROPERTY, pane, null);
    }
    
    /**
     * Sets the preferred height (in pixels) of the container.
     *
     * @param newValue The preferred height of the container.  If the preferred
     *        height is set to <code>EchoConstants.UNDEFINED_SIZE</code>, the 
     *        height will be determined based on the heights of the container's
     *        contents.
     * @deprecated Use <code>setPreferredHeightExtent()</code>.
     */
    public void setPreferredHeight(int newValue) {
        int oldValue = getPreferredHeight();
        setPreferredHeightExtent(newValue == EchoConstants.UNDEFINED_SIZE ? null : new Extent(newValue, Extent.PX));
        firePropertyChange(PREFERRED_HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the preferred height of the container.
     *
     * @param newValue The new preferred height of the container.
     */
    public void setPreferredHeightExtent(Extent newValue) {
        Extent oldValue = preferredHeightExtent;
        preferredHeightExtent = newValue;
        firePropertyChange(PREFERRED_HEIGHT_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the orientation of the pane.
     *
     * @param newValue The orientation of the pane, one of the following values:
     *        <ul>
     *        <li>ORIENTATION_HORIZONTAL (the default)</li>
     *        <li>ORIENTATION_VERTICAL</li>
     *        </ul>
     */
    public void setOrientation(int newValue) {
        int oldValue = orientation;
        orientation = newValue;
        firePropertyChange(ORIENTATION_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the preferred width of the container.
     *
     * @param newValue The preferred width of the container.  If the preferred
     *        width is set to <code>PROPORTIONAL</code>, the width will be
     *        determined based on the widths of the container's contents.
     * @deprecated Use <code>setPreferredWidthExtent()</code>.
     */
    public void setPreferredWidth(int newValue) {
        int oldValue = getPreferredWidth();
        setPreferredWidthExtent(newValue == EchoConstants.UNDEFINED_SIZE ? null : new Extent(newValue, Extent.PX));
        firePropertyChange(PREFERRED_WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the preferred width of the container.
     * 
     * @param newValue The preferred width of the container.
     */
    public void setPreferredWidthExtent(Extent newValue) {
        Extent oldValue = preferredWidthExtent;
        preferredWidthExtent = newValue;
        firePropertyChange(PREFERRED_WIDTH_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}    
