/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;

/**
 * A pane to which content components may be added. <br>
 * <br>
 * <code>ContentPane</code>s may only be added as children to 
 * <code>Window</code>s and <code>ContainerPane</code>s.
 */
public class ContentPane extends AbstractPane {

    /** 
     * A style constant for the Background Image property. 
     * Values of this key must be of type nextapp.echo.ImageReference.
     */
    public static final String STYLE_BACKGROUND_IMAGE = "backgroundImage";

    /** 
     * A style constant for the Background Image Properties property. 
     * Values of this key must be of type 
     * nextapp.echo.BackgroundImageProperties
     */
    public static final String STYLE_BACKGROUND_IMAGE_PROPERTIES = "backgroundImageProperties";

    /** 
     * A style constant for the Height property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HEIGHT = "height";
    
    /**
     * A style constant for the Height Extent property.
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_HEIGHT_EXTENT = "heightExtent";

    /** 
     * A style constant for the Horizontal Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";

    /** 
     * A style constant for the Insets property. 
     * Values of this key must be of type nextapp.echo.Insets.
     */
    public static final String STYLE_INSETS = "insets";

    /** 
     * A style constant for the Scroll Bar Policy property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_SCROLL_BAR_POLICY = "scrollBarPolicy";

    /**
     * A style constant for the Vertical Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";

    /** 
     * A style constant for the Width property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH = "width";
    
    /**
     * A style constant for the Width Extent property.
     * Values of this key must be of type nextapp.echo.Extent.
     */
    public static final String STYLE_WIDTH_EXTENT = "widthExtent";

    public static final String BACKGROUND_IMAGE_PROPERTIES_CHANGED_PROPERTY = "backgroundImageProperties";
    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String HEIGHT_EXTENT_CHANGED_PROPERTY = "heightExtent";
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String INSETS_CHANGED_PROPERTY = "insets";
    public static final String SCROLL_BAR_POLICY_CHANGED_PROPERTY = "scrollBarPolicy";
    public static final String SCROLL_BAR_X_CHANGED_PROPERTY = "scrollBarX";
    public static final String SCROLL_BAR_Y_CHANGED_PROPERTY = "scrollBarY";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_EXTENT_CHANGED_PROPERTY = "widthExtent";

    /**
     * A scroll bar policy that will cause scroll bars to be visible if they 
     * are necessary, and invisible if they are not.
     */
    public static final int SCROLL_BAR_AUTO = 0;
    
    /**
     * A scroll bar policy that will cause scroll bars to always appear, 
     * without regard for whether they are required.
     */
    public static final int SCROLL_BAR_ALWAYS = 1;
    
    /**
     * A scroll bar policy that will cause scroll bars never to appear,
     * without regard for whether they are required.
     */
    public static final int SCROLL_BAR_NEVER = 2;
    
    /**
     * A default Insets object that provides 10-pixel margins on the top, 
     * bottom, left, and right sides of the content pane.
     */
    public static final Insets DEFAULT_INSETS = new Insets(10);

    private Insets insets = DEFAULT_INSETS;
    private int scrollBarPolicy = SCROLL_BAR_AUTO;
    private int scrollBarX = 0;
    private int scrollBarY = 0;
    private int horizontalAlignment = EchoConstants.LEFT;
    private int verticalAlignment = EchoConstants.TOP;
    private BackgroundImageProperties backgroundImageProperties = null;
    private PropertyChangeForwarder propertyChangeForwarder = new PropertyChangeForwarder();
    private Extent widthExtent = null;
    private Extent heightExtent = null;
    
    private class PropertyChangeForwarder 
    implements PropertyChangeListener, Serializable {
    
        public void propertyChange(PropertyChangeEvent e) {
            if (e.getSource() == backgroundImageProperties) {
                firePropertyChange(BACKGROUND_IMAGE_PROPERTIES_CHANGED_PROPERTY, null, null);
            }
        }
    }

    /**
     * Creates a content pane.
     */
    public ContentPane() {
        super();
        setForeground(EchoInstance.DEFAULT_FOREGROUND);
        setBackground(EchoInstance.DEFAULT_BACKGROUND);
        setFont(EchoInstance.DEFAULT_FONT);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_BACKGROUND_IMAGE)) {
            setBackgroundImage((ImageReference) style.getAttribute(STYLE_BACKGROUND_IMAGE));
        }
        if (style.hasAttribute(STYLE_BACKGROUND_IMAGE_PROPERTIES)) {
            setBackgroundImageProperties((BackgroundImageProperties) style.getAttribute(STYLE_BACKGROUND_IMAGE_PROPERTIES));
        }
        if (style.hasAttribute(STYLE_HEIGHT)) {
            setHeight(style.getIntegerAttribute(STYLE_HEIGHT));
        }
        if (style.hasAttribute(STYLE_HEIGHT_EXTENT)) {
            setHeightExtent((Extent) style.getAttribute(STYLE_HEIGHT_EXTENT));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_INSETS)) {
            setInsets((Insets) style.getAttribute(STYLE_INSETS));
        }
        if (style.hasAttribute(STYLE_SCROLL_BAR_POLICY)) {
            setScrollBarPolicy(style.getIntegerAttribute(STYLE_SCROLL_BAR_POLICY));
        }
        if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
            setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));
        }
        if (style.hasAttribute(STYLE_WIDTH_EXTENT)) {
            setWidthExtent((Extent) style.getAttribute(STYLE_WIDTH_EXTENT));
        }
    }
    
    /**
     * Returns the background image.
     *
     * @return The background image.
     */
    public ImageReference getBackgroundImage() {
        if (backgroundImageProperties == null) {
            return null;
        } else {
            return backgroundImageProperties.getImage();
        }
    }
    
    /**
     * Returns the background image properties for this content pane.
     *
     * @return The background image properties for this content pane.
     */
    public BackgroundImageProperties getBackgroundImageProperties() {
        return backgroundImageProperties;
    }
    
    /** 
     * Returns the horizontal alignment of the contents of this pane.
     *
     * @return The horizontal alignment of the contents of this pane,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }
    
    /**
     * Returns the height of the pane.
     * 
     * @return The height of the pane.
     */
    public Extent getHeightExtent() {
        return heightExtent;
    }
    
    /** 
     * Returns the insets (margins) of the pane.
     *
     * @return The insets of the pane.
     */
    public Insets getInsets() {
        return insets;
    }
    
    /**
     * Returns the width of the pane.
     * 
     * @return The width of the pane.
     */
    public Extent getWidthExtent() {
        return widthExtent;
    }
    
    /**
     * Returns the scroll bar policy.
     *
     * @return The scroll bar policy, which should be one of the following:
     *         <ul>
     *         <li>SCROLL_BAR_ALWAYS
     *         <li>SCROLL_BAR_AUTO
     *         <li>SCROLL_BAR_NEVER
     *         </ul>
     */
    public int getScrollBarPolicy() {
        return scrollBarPolicy;
    }
    
    /**
     * Returns the position of the horizontal scroll bar, in pixels offset from 
     * the left of the pane.
     *
     * @return The position of the horizontal scroll bar.
     */
    public int getScrollBarX() {
        return scrollBarX;
    }
    
    /**
     * Returns the position of the vertical scroll bar, in pixels offset from 
     * the top the pane.
     *
     * @return The position of the vertical scroll bar.
     */
    public int getScrollBarY() {
        return scrollBarY;
    }

    /** 
     * Returns the vertical alignment of the contents of this pane.
     *
     * @return The vertical alignment of the contents of this pane, 
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalAlignment() {
        return verticalAlignment;
    }

    /**
     * Sets the background image.
     * 
     * @param newValue Then new background image.
     */
    public void setBackgroundImage(ImageReference newValue) {
        if (backgroundImageProperties == null) {
            setBackgroundImageProperties(new BackgroundImageProperties(newValue));
        } else {
            backgroundImageProperties.setImage(newValue);
        }
    }
    
    /**
     * Sets the background image properties for this content pane.
     *
     * @param newValue The new background image properties for this content 
     *        pane.  Any previously set background image information is 
     *        overridden by calling this method.
     */
    public void setBackgroundImageProperties(BackgroundImageProperties newValue) {
        BackgroundImageProperties oldValue = backgroundImageProperties;
        if (oldValue != null) {
            oldValue.removePropertyChangeListener(propertyChangeForwarder);
        }
        if (newValue != null) {
            newValue.addPropertyChangeListener(propertyChangeForwarder);
        }
        backgroundImageProperties = newValue;
        firePropertyChange(BACKGROUND_IMAGE_PROPERTIES_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the height of the pane in pixels.
     *
     * @param newValue The new height of the pane in pixels.
     */
    public void setHeight(int newValue) {
        Extent heightExtent = getHeightExtent();
        int oldValue = (heightExtent != null && heightExtent.getUnits() == Extent.PX) 
                ? heightExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
        setHeightExtent(new Extent(newValue, Extent.PX));
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the height of the pane.
     * 
     * @param newValue the new height of the pane.
     */
    public void setHeightExtent(Extent newValue) {
        Extent oldValue = heightExtent;
        heightExtent = newValue;
        firePropertyChange(HEIGHT_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the horizontal alignment of the contents of this pane.
     *
     * @param newValue The horizontal alignment of the contents of this pane, 
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the insets (margins) of the pane.
     *
     * @param newValue An insets object that specifies the size of each margin 
     *        of this pane.
     */
    public void setInsets(Insets newValue) {
        Insets oldValue = insets;
        insets = newValue;
        firePropertyChange(INSETS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the scroll bar policy.
     *
     * @param newValue The new scroll bar policy, one of the following:
     *        <ul>
     *        <li>SCROLL_BAR_ALWAYS
     *        <li>SCROLL_BAR_AUTO
     *        <li>SCROLL_BAR_NEVER
     *        </ul>
     */
    public void setScrollBarPolicy(int newValue) {
        int oldValue = scrollBarPolicy;
        scrollBarPolicy = newValue;
        firePropertyChange(SCROLL_BAR_POLICY_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the position of the horizontal scroll bar, in pixels offset from 
     * the left of the pane.
     *
     * @param newValue The new position of the horizontal scroll bar, in pixels.
     */
    public void setScrollBarX(int newValue) {
        int oldValue = scrollBarX;
        scrollBarX = newValue;
        firePropertyChange(SCROLL_BAR_X_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the position of the vertical scroll bar, in pixels offset from 
     * the top of the pane.
     *
     * @param newValue The new position of the vertical scroll bar, in pixels.
     */
    public void setScrollBarY(int newValue) {
        int oldValue = scrollBarY;
        scrollBarY = newValue;
        firePropertyChange(SCROLL_BAR_Y_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the vertical alignment of the contents of this pane.
     *
     * @param newValue The vertical alignment of the contents of this pane, 
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalAlignment(int newValue) {
        int oldValue = verticalAlignment;
        verticalAlignment = newValue;
        firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the pane in pixels.
     *
     * @param newValue The new width of the pane in pixels.
     */
    public void setWidth(int newValue) {
        Extent widthExtent = getWidthExtent();
        int oldValue = (widthExtent != null && widthExtent.getUnits() == Extent.PX) 
                ? widthExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
        setWidthExtent(new Extent(newValue, Extent.PX));
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the pane.
     * 
     * @param newValue The new width of the pane.
     */
    public void setWidthExtent(Extent newValue) {
        Extent oldValue = widthExtent;
        widthExtent = newValue;
        firePropertyChange(WIDTH_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
