/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;

/**
 * An two-dimensional immutable integer (x, y) coordinate.
 */ 
public class Coordinate 
implements Comparable, Serializable {

    private int x;
    private int y;

    /** 
     * Creates a new coordinate representing the position (x,y) in 2D space.
     *
     * @param x The coordinate x position.
     * @param y The coordinate y position.
     */
    public Coordinate(int x, int y) {
        super();

        this.x = x;
        this.y = y;
    }

    /**
     * Compares this coordinate to another.
     *
     * @param o A <code>Coordinate</code> to compare this coordinate to.
     * @return A negative integer if this coordinate falls above or to the
     *         left of the test object and a positive integer if this 
     *         coordinate falls below or to the right of it.
     *
     * @see java.lang.Comparable#compareTo(Object)
     */
    public int compareTo(Object o) {
        Coordinate that = (Coordinate) o;
        int order;

        if (this.getY() == that.getY()) {
            order = this.getX() - that.getX();
        } else {
            order = (this.getY() - that.getY()) * 0x10000 + this.getX() - that.getX();
        }

        return order;
    }

    /**
     * Returns true if this coordinate is equivalent to the provided object.
     *
     * @param o The object to test this coordinate for equality with.
     * @return True if this coordinate is equivalent to the provided object.
     * 
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object o) {
        boolean equal;

        if (o != null && o instanceof Coordinate) {
            Coordinate that = (Coordinate) o;
            equal = (this.getX() == that.getX()) && (this.getY() == that.getY());
        } else {
            equal = false;
        }

        return equal;
    }
    
    /**
     * Returns a hash code value for this coordinate.
     *
     * @return A hash code value for this coordinate.
     * 
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return getX() ^ (0 - getY());
    }

    /**
     * Returns the x position of this coordinate.
     *
     * @return The x position of this coordinate.
     */
    public int getX() {
        return x;
    }

    /**
     * Returns the y position of this coordinate.
     *
     * @return The y position of this coordinate.
     */
    public int getY() {
        return y;
    }
}

