/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.EventListener;

import nextapp.echo.event.ActionEvent;
import nextapp.echo.event.ActionListener;
import nextapp.echo.event.ChangeEvent;
import nextapp.echo.event.ChangeListener;
import nextapp.echo.event.EventListenerList;
import nextapp.echo.event.ItemEvent;
import nextapp.echo.event.ItemListener;

/**
 * The default implementation of a button's data model.
 */
public class DefaultButtonModel 
implements ButtonModel, Serializable {

    protected EventListenerList listenerList = null;  
    private String actionCommand = null;
    private ButtonGroup group = null;
    
    /**
     * Creates a new default button model.
     */
    public DefaultButtonModel() {
        super();
        
        listenerList = new EventListenerList();
    }
    
    /**
     * @see nextapp.echo.ButtonModel#addActionListener(ActionListener)
     */
    public void addActionListener(ActionListener l) {
        listenerList.addListener(ActionListener.class, l);
    }
    
    /**
     * @see nextapp.echo.ButtonModel#addChangeListener(ChangeListener)
     */
    public void addChangeListener(ChangeListener l) {
        listenerList.addListener(ChangeListener.class, l);
    }
    
    /**
     * @see nextapp.echo.ButtonModel#addItemListener(ItemListener)
     */
    public void addItemListener(ItemListener l) {
        listenerList.addListener(ItemListener.class, l);
    }
    
    /**
     * Processes a user action.
     */
    public void doAction() {
        fireActionPerformed(new ActionEvent(this, getActionCommand()));
    }

    /**
     * Notifies all listeners that have registered for this event type.
     *
     * @param e The <code>ActionEvent</code> to send.
     */
    public void fireActionPerformed(ActionEvent e) {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ActionListener) listeners[index]).actionPerformed(e);
        }
    }
    
    /**
     * Notifies all listeners that have registered for this event type.
     *
     * @param e The <code>ItemEvent</code> to send.
     */
    public void fireItemStateChanged(ItemEvent e) {
        EventListener[] listeners = listenerList.getListeners(ItemListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ItemListener) listeners[index]).itemStateChanged(e);
        }
    }
    
    /**
     * Notifies all listeners that have registered for this event type.
     */
    public void fireStateChanged() {
        EventListener[] listeners = listenerList.getListeners(ChangeListener.class);
        ChangeEvent e = new ChangeEvent(this);
        for (int index = 0; index < listeners.length; ++index) {
            ((ChangeListener) listeners[index]).stateChanged(e);
        }
    }
    
    /**
     * @see nextapp.echo.ButtonModel#getActionCommand()
     */
    public String getActionCommand() { 
        return actionCommand;
    }
    
    /**
     * Returns the <code>ButtonGroup</code> to which this button belongs, if 
     * any.
     *
     * @return The group which contains this button.
     */
    public ButtonGroup getGroup() {
        return group;
    }
    
    /**
     * @see nextapp.echo.ButtonModel#isSelected()
     */
    public boolean isSelected() {
        return false;
    }

    /**
     * @see nextapp.echo.ButtonModel#removeActionListener(ActionListener)
     */
    public void removeActionListener(ActionListener l) {
        listenerList.removeListener(ActionListener.class, l);
    }
    
    /**
     * @see nextapp.echo.ButtonModel#removeChangeListener(ChangeListener)
     */
    public void removeChangeListener(ChangeListener l) {
        listenerList.removeListener(ChangeListener.class, l);
    }
    
    /**
     * @see nextapp.echo.ButtonModel#removeItemListener(ItemListener)
     */
    public void removeItemListener(ItemListener l) {
        listenerList.removeListener(ItemListener.class, l);
    }
    
    /**
     * @see nextapp.echo.ButtonModel#setActionCommand(String)
     */
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }

    /**
     * @see nextapp.echo.ButtonModel#setSelected(boolean)
     */
    public void setSelected(boolean selected) { }
    
    /**
     * @see nextapp.echo.ButtonModel#setGroup(ButtonGroup)
     */
    public void setGroup(ButtonGroup group) {
        this.group = group;
    }
}
