/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.BitSet;
import java.util.EventListener;

import nextapp.echo.event.EventListenerList;
import nextapp.echo.event.ListSelectionEvent;
import nextapp.echo.event.ListSelectionListener;

/**
 * The default implementation of ListSelectionModel, the model used by
 * ListBoxes.
 */
public class DefaultListSelectionModel 
implements ListSelectionModel, Serializable {

    private EventListenerList listenerList = new EventListenerList();
    private int selectionMode = MULTIPLE_SELECTION;
    private BitSet selection = new BitSet();
    private int minSelectedIndex = -1;
    
    public static final String SELECTION_MODE_CHANGED_PROPERTY = "selectionMode";
    
    /**
     * Adds a <code>ListSelectionListener</code> to the selection model.
     *
     * @param l The <code>ListSelectionListener</code> to be added.
     */
    public void addListSelectionListener(ListSelectionListener l) {
        listenerList.addListener(ListSelectionListener.class, l);
    }
    
    /**
     * Deselects all items.
     */
    public void clearSelection() {
        selection = new BitSet();
        minSelectedIndex = -1;
        
        fireValueChanged();
    }
    
    /**
     * Notifies <code>ListSelectionListener</code>s that the state of this 
     * <code>ListSelectionModel</code> has changed.
     */
    protected void fireValueChanged() {
        ListSelectionEvent e = new ListSelectionEvent(this);

        EventListener[] listeners = listenerList.getListeners(ListSelectionListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ListSelectionListener) listeners[index]).valueChanged(e);
        }
    }
    
    /**
     * Returns the maximum selected index.
     *
     * @return the maximum selected index.
     */
    public int getMaxSelectedIndex() {
        return selection.length() - 1;
    }
    
    /**
     * Returns the maximum selected index.
     *
     * @return the maximum selected index.
     */
    public int getMinSelectedIndex() {
        return minSelectedIndex;
    }
    
    /**
     * Returns the selection mode.  
     * 
     * @return The selection mode, one of the following values:
     *         <ul>
     *         <li>ListSelectionModel.SINGLE_SELECTION: only one list element 
     *         may be selected.</li>
     *         <li>ListSelectionModel.MULTIPLE_SELECTION: multiple list elements
     *         may be selected.</li>
     *         </ul>
     */
    public int getSelectionMode() {
        return selectionMode;
    }
    
    /**
     * Determines whether an index is selected.
     *
     * @param index The index to test for selection.
     * @return True if the given index is selected.
     */
    public boolean isSelectedIndex(int index) {
        return selection.get(index);
    }
    
    /**
     * Determines if any items are selected.
     *
     * @return True if no items are selected.
     */
    public boolean isSelectionEmpty() {
        return selection.length() == 0;
    }
    
    /**
     * Removes a <code>ListSelectionListener</code> from the selection model.
     *
     * @param l The <code>ListSelectionListener</code> to be removed.
     */
    public void removeListSelectionListener(ListSelectionListener l) {
        listenerList.removeListener(ListSelectionListener.class, l);
    }

    /**
     * Sets the selection state of the given index.
     *
     * @param index The index to select.
     * @param newValue The new selection state of the item at the given index.
     */
    public void setSelectedIndex(int index, boolean newValue) {
        boolean oldValue = isSelectedIndex(index);
    
        if (newValue ^ oldValue) {
            if (newValue) {
                if (selectionMode == SINGLE_SELECTION && getMinSelectedIndex() != -1) {
                    setSelectedIndex(getMinSelectedIndex(), false);
                }
                selection.set(index);
                if (index < minSelectedIndex || minSelectedIndex == -1) {
                    minSelectedIndex = index;
                }
            } else {
                selection.clear(index);
                if (index == minSelectedIndex) {
                    // Minimum selected index has been unselected, find new minimum selected index.
                    int max = getMaxSelectedIndex();
                    minSelectedIndex = -1;
                    for (int i = 0; i <= max; ++i) {
                        if (selection.get(i)) {
                            minSelectedIndex = index;
                            break;
                        }
                    }
                }
            }
            
            fireValueChanged();
        }
    }

    /**
     * Sets the selection mode.  
     * 
     * @param selectionMode The selection mode, one of the following values:
     *        <ul>
     *        <li>ListSelectionModel.SINGLE_SELECTION: only one list element 
     *        may be selected.</li>
     *        <li>ListSelectionModel.MULTIPLE_SELECTION: multiple list elements
     *        may be selected.</li>
     *        </ul>
     */
    public void setSelectionMode(int selectionMode) {
        if (selectionMode != MULTIPLE_SELECTION && this.selectionMode == MULTIPLE_SELECTION) {
            // deselect all but first selected element.
            int maxSelectedIndex = getMaxSelectedIndex();
            for (int i = minSelectedIndex + 1; i <= maxSelectedIndex; ++i) {
                setSelectedIndex(i, false);
            }
        }
    
        this.selectionMode = selectionMode;
    }
}
