/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.util.ArrayList;
import java.util.List;

/**
 * The default implementation of SelectFieldModel, the model used
 * for SelectFields.
 */
public class DefaultSelectFieldModel extends AbstractSelectFieldModel {

    private List items = new ArrayList();

    /** 
     * Creates a empty new DefaultSelectFieldModel.
     */
    public DefaultSelectFieldModel() {
        this(new Object[0]);
    }
    
    /**
     * Creates a new DefaultSelectFieldModel that contains an array of
     * items.
     *
     * @param itemArray An array of the initial items to be displayed in the
     *        SelectField.
     */
    public DefaultSelectFieldModel(Object[] itemArray) {
        super();
        
        for (int index = 0; index < itemArray.length; ++index) {
            items.add(itemArray[index]);
        }
    }
    
    /**
     * Adds an item at the end of the model.
     *
     * @param item The item to add.
     */
    public void add(Object item) {
        items.add(item); 

        int index = items.size() - 1;
    
        fireContentsChanged(this, index, index); 
    }
    
    /**
     * Inserts an item into the model at the specified index.
     *
     * @param index The index at which to insert the item.
     * @param item The item to insert.
     */
    public void add(int index, Object item) {
        items.add(index, item);

        fireContentsChanged(this, index, index); 
    }
    
    /**
     * Removes all items from the model.
     */
    public void clear() {
        items.clear();

        fireContentsChanged(this, -1, -1); 
    }
    
    /**
     * Returns the element at the specified index.
     *
     * @param index The index of the element to return.
     * @return The element at the specified index.
     */
    public Object get(int index) {
        return items.get(index);
    }

    /**
     * Returns the index of the specified item within the model.
     *
     * @param item An item in the model.
     * @return The index of the item within the model.
     */
    public int indexOf(Object item) {
        return items.indexOf(item);
    }
    
    /**
     * Removes the item at the specified index from the model.
     *
     * @param index The index of the item to remove.
     */
    public void remove(int index) {
        items.remove(index);

        fireContentsChanged(this, index, index); 
    }

    /**
     * Removes the specified item from the model.
     *
     * @param item The item to remove from the model.
     */
    public void remove(Object item) {
        int index = items.indexOf(item);
    
        items.remove(item);

        fireContentsChanged(this, index, index); 
    }

    /**
     * Returns the number of items in the model.
     *
     * @return The number of items in the model.
     */
    public int size() {
        return items.size();
    }
}
