/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

/**
 * A single user-instance of an Echo application.
 */
public abstract class EchoInstance 
implements Serializable {

    /** 
     * Holds a thread local reference to the active EchoInstance for
     * easy retrieval.
     */ 
    private static final ThreadLocal activeInstance = new InheritableThreadLocal();

    /** The default background color (white) */
    public static final Color DEFAULT_BACKGROUND = Color.WHITE;
    
    /** The default font (10 point sans-serif) */
    public static final Font DEFAULT_FONT = new Font(Font.SANS_SERIF, 0, 10);

    /** The default foreground color (black) */
    public static final Color DEFAULT_FOREGROUND = Color.BLACK;
    
    /** The default window height (100 pixels). */
    public static final int DEFAULT_WINDOW_HEIGHT = 100;

    /** The default window width (100 pixels). */
    public static final int DEFAULT_WINDOW_WIDTH = 100;

    /** The name and version of the Echo API in use. */
    public static final String ID_STRING = "NextApp Echo v1.1.4";

    public static final String FOCUSED_COMPONENT_CHANGED_PROPERTY = "focusedComponent";
    public static final String LOCALE_CHANGED_PROPERTY = "locale";
    public static final String WINDOWS_CHANGED_PROPERTY = "windows";
    
    /**
     * Returns a reference to the <code>EchoInstance</code> that is relevant 
     * to the current thread.
     * 
     * @return The relevant <code>EchoInstance</code>.
     */
    public static final EchoInstance getEchoInstance() {
        return (EchoInstance) activeInstance.get();
    }
    
    /**
     * An mutable map of application attributes.
     */
    private Map attributes = null;
    
    /**
     * The set of all windows registered with the application.
     */
    private Set windows = null;
    
    /**
     * Fires property change events for the instance object.
     */
    private PropertyChangeSupport propertyChangeSupport = null;
    
    /**
     * The default locale of the application.
     */
    private Locale locale = null;
    
    /**
     * The presently focused component.
     */
    private WeakReference focusedComponent = null;
    
    /**
     * Creates a new instance of this Echo application.
     */
    public EchoInstance() {
        super();

        windows = new HashSet();
        propertyChangeSupport = new PropertyChangeSupport(this);
    }

    /**
     * Adds a property change listener to this instance.
     *
     * @param l The listener to be added.
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.addPropertyChangeListener(l);
    }

    /**
     * Adds a new window to the application, causing it to then be displayed.
     * If the window is already registered with the application (i.e. it was
     * previously added and not removed), then this method has no effects.
     *
     * @param w A new window to be added to the application and displayed.
     */
    public void addWindow(Window w) {
        if (windows.contains(w)) {
            // Attempting to re-add an already added window has no effect.
            return;
        }
    
        // Set the EchoInstance of the window.
        w.setEchoInstance(this);
        
        // Set width and height of window to default if they are not set.
        if (w.getWidth() == EchoConstants.UNDEFINED_SIZE) {
            w.setWidth(DEFAULT_WINDOW_WIDTH);
        }
        if (w.getHeight() == EchoConstants.UNDEFINED_SIZE) {
            w.setHeight(DEFAULT_WINDOW_HEIGHT);
        }
        
        // Add window to internal list of windows.
        windows.add(w);

        // Notify the window it has been opened.
        w.fireWindowOpened();
        
        // Notify listeners that a new window has been opened.
        propertyChangeSupport.firePropertyChange(WINDOWS_CHANGED_PROPERTY, null, w);

        w.notifyRegistrationChange(true);
    }
    
    /**
     * Invoked by the application container when the application instance
     * is to be destroyed.
     */
    public final void doExit() {
        while (!windows.isEmpty()) {
            removeWindow((Window)windows.iterator().next());
        }    	
        exit();
    }
    
    /**
     * Invoked by the application container to initialize the application.
     * Returns the first window of the application.
     *
     * @return The first window of the application.
     */
    public final Window doInit() {
        // Create first window.
        Window firstWindow = init();
        
        // Add window to list of windows. 
        windows.add(firstWindow);
        
        // Set window EchoInstance.
        firstWindow.setEchoInstance(this);
        
        firstWindow.notifyRegistrationChange(true);
        
        // Perform initial validation of application.
        doValidation();
        
        return firstWindow;
    }
    
    /**
     * Validates all components in all windows registered with the application.
     */
    public final void doValidation() {
        for (Iterator it = windows.iterator(); it.hasNext();) {
            doValidation((Window) it.next());
        }
    }
    
    /**
     * Validates a single component and then recursively validates its 
     * children.
     *
     * @param c The component to be validated.
     */
    private void doValidation(Component c) {
        c.validate();
        int size = c.getComponentCount();
        for (int index = 0; index < size; ++index) {
            doValidation(c.getComponent(index));
        }
    }
    
    /**
     * Notifies the application that it has reached the end of its life,
     * and should clean up any resources if required.   This method should be
     * overridden by applications that require end-of-life resource clean-up.
     */
    public void exit() { }

    /**
     * Returns the value of an attribute.
     *
     * @param key The key (name) of the attribute.
     * @return Object The value of the attribute represented by the
     *         provided key.
     */
    public Object getAttribute(String key) {
        Object attribute;
        
        if (attributes == null) {
            attribute = null;
        } else {
            attribute = attributes.get(key);
        }
        
        return attribute;
    }
    
    /**
     * Returns the presently focused component, if known.
     * 
     * @return The presently focused component, if known.
     */
    public Component getFocusedComponent() {
        if (focusedComponent == null) {
            return null;
        } else {
            return (Component) focusedComponent.get();
        }
    }
    
    /**
     * Returns the application instance's default locale.
     *
     * @return The application instance's default locale.
     */
    public Locale getLocale() {
        return locale;
    }
    
    /**
     * Returns an array of all windows registered with the application 
     * instance.
     *
     * @return An array of all windows registered with the application 
     * instance.
     */
    public Window[] getWindows() {
        return (Window[]) windows.toArray(new Window[windows.size()]);
    }
    
    /**
     * Invoked to initialize the application, returning the primary window.
     * The returned window must be visible.
     *
     * @return The primary window of the application.
     */
    protected abstract Window init();

    /**
     * Marks this EchoInstance as active or inactive for the current thread.
     * 
     * @param active True if the EchoInstance should be marked active for the
     *        current thread, false if it should be marked inactive.
     */
    public void markActive(boolean active) {
        if (active) {
            activeInstance.set(this);
        } else {
            activeInstance.set(null);
        }
    }
    
    /**
     * Removes a property change listener from this instance.
     *
     * @param l The listener to be removed.
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        propertyChangeSupport.removePropertyChangeListener(l);
    }

    /**
     * Removes a window from the application, causing it to then be closed.
     * If the window is not registered with the application (e.g. if it was
     * disposed earlier or never added), then this method has no effects.
     *
     * @param w A window that was previously added to the application that now
     *          should be removed and closed.
     */
    public void removeWindow(Window w) {
        if (!windows.contains(w)) {
            // Window is not part of this EchoInstance: do nothing.
            return;
        }
        w.setEchoInstance(null);
        windows.remove(w);
        w.fireWindowClosed();
        propertyChangeSupport.firePropertyChange(WINDOWS_CHANGED_PROPERTY, w, null);
        w.notifyRegistrationChange(false);
    }
    
    /**
     * Sets an attribute value.  Your application should rarely (if ever)
     * call this method.  Attributes should NOT be used to hold application
     * data.
     *
     * @param key The key (name) of the attribute.
     * @param value The new value of the attribute.
     */
    public void setAttribute(String key, Object value) {
        if (attributes == null) {
            attributes = new HashMap();
        }
        attributes.put(key, value);
    }
    
    /**
     * Sets the presently focused component.
     * 
     * @param newValue The component to be focused.
     */
    public void setFocusedComponent(Component newValue) {
        Component oldValue = getFocusedComponent();
        if (newValue == null) {
            focusedComponent = null;
        } else {
            focusedComponent = new WeakReference(newValue);
        }
        propertyChangeSupport.firePropertyChange(FOCUSED_COMPONENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the application instance's default locale.
     * The locale will be automatically set based on information from the 
     * client during initialization if the application has not yet set it.
     *
     * @param newValue The new default locale.
     */
    public void setLocale(Locale newValue) {
        Locale oldValue = locale;
        locale = newValue;
        propertyChangeSupport.firePropertyChange(LOCALE_CHANGED_PROPERTY, oldValue, newValue);
    }
}
