/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A component used to add space between other components.
 */
public class Filler extends Component {

    /**
     * A value used in the width and height attributes used by Filler
     * objects that do not wish to separate components in a particular
     * dimension.
     */
    public static final int NO_DIMENSION = -1;
    
    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String WIDTH_CHANGED_PROPERTY = "width";

    private int width;
    private int height;
    
    /**
     * Returns a new horizontal strut filler component of the given width.
     *
     * @param width The width of the filler strut, in pixels.
     */
    public static Filler createHorizontalStrut(int width) {
        return new Filler(width, NO_DIMENSION);
    }
    
    /**
     * Returns a new vertical strut filler component of the given height.
     *
     * @param height The height of the filler strut in pixels.
     */
    public static Filler createVerticalStrut(int height) {
        return new Filler(NO_DIMENSION, height);
    }
    
    /**
     * Creates a new filler component of zero dimension.
     */
    public Filler() {
        this(0, 0);
    }
    
    /**
     * Creates a new filler component of the given width and height.
     *
     * @param width The width of the filler strut, in pixels.
     * @param height The height of the filler strut in pixels.
     */
    public Filler(int width, int height) {
        super();
        
        this.width = width;
        this.height = height;
    }
    
    /**
     * Returns the height of the filler.
     *
     * @return The height of the filler.
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * Returns the width of the filler.
     *
     * @return The width of the filler.
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Sets the height of the filler.
     * 
     * @param newValue The new height of the filler.
     */
    public void setHeight(int newValue) {
        int oldValue = height;
        height = newValue;
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the filler.
     * 
     * @param newValue The new width of the filler.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
}
