/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.Arrays;

/**
 * A representation of a text font.  Once created, a font is immutable.
 */
public class Font 
implements Serializable {

    /**
     * A style value indicating no text attributes.
     */
    public static final int PLAIN = 0;
    
    /**
     * A style value indicating bold.
     */
    public static final int BOLD = 1;

    /**
     * A style value indicating bold.
     */
    public static final int ITALIC = 2;

    /**
     * A style value indicating underline.
     */
    public static final int UNDERLINE = 4;
    
    /**
     * A style value indicating overline.
     */
    public static final int OVERLINE = 8;
    
    /**
     * A style value indicating line-through.
     */
    public static final int LINE_THROUGH = 16;
    
    public static final String[] ARIAL = {"Arial", "Helvetica", "Sans-Serif"};
    public static final String[] HELVETICA = {"Helvetica",  "Sans-Serif"};
    public static final String[] MONOSPACE = {"Monospace"};
    public static final String[] SANS_SERIF = {"Sans-Serif"};
    public static final String[] SERIF = {"Serif"};
    public static final String[] TIMES = {"Times", "Serif"};
    public static final String[] TIMES_NEW_ROMAN = {"Times New Roman", "Times Roman", "Times", "Serif"};
    public static final String[] TIMES_ROMAN = {"Times Roman", "Times", "Serif"};
    public static final String[] VERDANA = {"Verdana", "Arial", "Helvetica", "Sans-Serif"};

    private String[] names;
    private Extent sizeExtent;
    private int style;

    /**
     * Creates a font with a single typeface, the given style attributes 
     * and size.  Use of this constructor is not recommended, as it does not
     * provide for alternative fonts in the event the client does not support
     * the application's ideal font choice.
     *
     * @param name The font name.  This font name must be available on the 
     *        client computer or the use of this font may not have any effect.
     * @param style The style of the font, one or more of the following values:
     *        <ul>
     *        <li>PLAIN</li>
     *        <li>BOLD</li>
     *        <li>ITALIC</li>
     *        <li>UNDERLINE</li>
     *        </ul>
     *        If it is necessary create a font with multiple style attributes,
     *        they should be bitwise-ORed together, using an expression such as
     *        <code>BOLD | UNDERLINE</code>.
     * @param size The point size of the font.
     */
    public Font(String name, int style, int size) {
        this(new String[]{name}, style, size);
    }

    /**
     * Creates a font with a primary and alternative typefaces, the given 
     * style attributes and size.
     *
     * @param names An array of font names.  The client will use the font with
     *        the lowest index in the array that it supports to display text.
     *        It is recommended that an application provide either "Sans-Serif",
     *        "Serif", or "Monospace" as the last element of this array, as 
     *        these fonts will be supported by all clients.
     * @param style The style of the font, one or more of the following values:
     *        <ul>
     *        <li>PLAIN</li>
     *        <li>BOLD</li>
     *        <li>ITALIC</li>
     *        <li>UNDERLINE</li>
     *        </ul>
     *        If it is necessary create a font with multiple style attributes,
     *        they should be bitwise-ORed together, using an expression such as
     *        <code>BOLD | UNDERLINE</code>.
     * @param size The point size of the font.
     */
    public Font(String[] names, int style, int size) {
        this(names, style, new Extent(size, Extent.PT));
    }
    
    /**
     * Creates a font with a primary and alternative typefaces, the given 
     * style attributes and size.
     *
     * @param names An array of font names.  The client will use the font with
     *        the lowest index in the array that it supports to display text.
     *        It is recommended that an application provide either "Sans-Serif",
     *        "Serif", or "Monospace" as the last element of this array, as 
     *        these fonts will be supported by all clients.
     * @param style The style of the font, one or more of the following values:
     *        <ul>
     *        <li>PLAIN</li>
     *        <li>BOLD</li>
     *        <li>ITALIC</li>
     *        <li>UNDERLINE</li>
     *        </ul>
     *        If it is necessary create a font with multiple style attributes,
     *        they should be bitwise-ORed together, using an expression such as
     *        <code>BOLD | UNDERLINE</code>.
     * @param sizeExtent The size of the font as a <code>Extent</code>.
     */
    public Font(String[] names, int style, Extent sizeExtent) {
        super();
        
        if (names == null || names.length == 0 || names[0] == null) {
            this.names = EchoInstance.DEFAULT_FONT.getNames();
        } else {
            this.names = names;
        }
        this.style = style;
        this.sizeExtent = sizeExtent;
    }
    
    /** 
     * Returns true if this font is equivalent to the provided object.
     *
     * @return True if this font is equivalent to the provided object.
     */
    public boolean equals(Object o) {
        boolean equal;
        if (this == o) {
            equal = true;
        } else if (o instanceof Font) {
            Font that = (Font) o;
            boolean sizeEqual = this.sizeExtent == that.sizeExtent 
                    || (this.sizeExtent != null && this.sizeExtent.equals(that.sizeExtent));
            equal = sizeEqual && that.style == this.style && Arrays.equals(this.names, that.names);
        } else {
            equal = false;
        }
        return equal;
    }
    
    /**
     * Returns the names of acceptable typefaces of this font, in 
     * preferential order.
     */
    public String[] getNames() {
        String[] namesCopy;
        if (names == null) {
            namesCopy = null;
        } else {
            namesCopy = new String[names.length];
            System.arraycopy(names, 0, namesCopy, 0, names.length);
        }
        return namesCopy;
    }
    
    /**
     * Returns the size of the font, in points.
     *
     * @return The size of the font, in points.
     */
    public int getSize() {
        return sizeExtent.toPoint();
    }
    
    /**
     * Returns the size of the font.
     * 
     * @return The size of the font.
     */
    public Extent getSizeExtent() {
        return sizeExtent;
    }
    
    /**
     * Returns a hash code value for this font.
     *
     * @return A hash code value for this font.
     */
    public int hashCode() {
        int hashCode = 0;
        if (names != null && names.length > 0) {
            hashCode += names[0].hashCode();
        }
        hashCode ^= (sizeExtent.getValue() + style * 0x100);
        return hashCode;
    }
    
    /**
     * Returns whether the font is bold.
     *
     * @return True if the font is bold.
     */
    public boolean isBold() {
        return (style & BOLD) != 0;
    }
    
    /**
     * Returns whether the font is italicized.
     *
     * @return True if the font is italicized.
     */
    public boolean isItalic() {
        return (style & ITALIC) != 0;
    }
    
    /**
     * Returns whether the font has line-through enabled.
     *
     * @return True if the font has line-through enabled.
     */
    public boolean isLineThrough() {
        return (style & LINE_THROUGH) != 0;
    }
    
    /**
     * Returns whether the font has an overline.
     *
     * @return True if the font has an overline.
     */
    public boolean isOverline() {
        return (style & OVERLINE) != 0;
    }
    
    /**
     * Returns whether the font is underlined.
     *
     * @return True if the font is underlined.
     */
    public boolean isUnderline() {
        return (style & UNDERLINE) != 0;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer out = new StringBuffer("Font: ");
        for (int index = 0; index < names.length; ++index) {
            out.append(names[index]);
            if (index < names.length) {
                out.append(", ");
            }
        }
        if (isBold()) {
            out.append(" Bold");
        }
        if (isItalic()) {
            out.append(" Italic");
        }
        if (isLineThrough()) {
            out.append(" LineThrough");
        }
        if (isOverline()) {
            out.append(" Overline");
        }
        if (isUnderline()) {
            out.append(" Underline");
        }
        out.append(" ");
        out.append(sizeExtent);
        return out.toString();
    }
}
