/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import nextapp.echo.util.DynamicIntegerArray;

/**
 * A component used to lay out child components in a grid.
 */
public class Grid extends Component {

    /**
     * A container component representing a single cell of a grid.
     */
    public static class Cell extends Component {

        /**
         * A flow value that indicates that a cell should attempt to flow
         * into vacant cells below it.
         */
        public static final int FLOW_DOWN  = 0x2;

        /**
         * A flow value that indicates that a cell should attempt to flow
         * into vacant cells to its left.
         */
        public static final int FLOW_LEFT  = 0x4;
    
        /**
         * A flow value that indicates that a cell should attempt to flow
         * into vacant cells to its right.
         */
        public static final int FLOW_RIGHT = 0x1;

        /**
         * A flow value that indicates that a cell should attempt to flow
         * into vacant cells above it.
         */
        public static final int FLOW_UP    = 0x8;
        
        /**
         * A style constant for the Background Image property.
         * Values of this key must be of type nextapp.echo.ImageReference.
         */
        public static final String STYLE_BACKGROUND_IMAGE = "backgroundImage";
    
        /** 
         * A style constant for the Horizontal Alignment property. 
         * Values of this key must be of type java.lang.Integer.
         */
        public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";

        /** 
         * A style constant for the Insets property. 
         * Values of this key must be of type nextapp.echo.Insets.
         */
        public static final String STYLE_INSETS = "insets";
        
        /**
         * A style constant for the Line Wrap property.
         * Values of this key must be of type java.lang.Boolean.
         */
        public static final String STYLE_LINE_WRAP = "lineWrap";

        /**
         * A style constant for the Vertical Alignment property. 
         * Values of this key must be of type java.lang.Integer.
         */
        public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";

        public static final String BACKGROUND_IMAGE_CHANGED_PROPERTY = "backgroundImage";
        public static final String FLOW_CHANGED_PROPERTY = "flowChanged";
        public static final String FLOW_PRIORITY_CHANGED_PROPERTY = "flowPriorityChanged";
        public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
        public static final String LINE_WRAP_CHANGED_PROPERTY = "lineWrap";
        public static final String INSETS_CHANGED_PROPERTY = "insets";
        public static final String POSITION_CHANGED_PROPERTY = "position";
        public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
        
        private ImageReference backgroundImage = null;
        private int columnSpan = 1;
        private int flow = 0;
        private int flowPriority = 0;
        private int horizontalAlignment = EchoConstants.LEFT;
        private Insets insets = null;
        private Coordinate position = null;
        private int rowSpan = 1;
        private int verticalAlignment = EchoConstants.TOP;
        private boolean lineWrap = true;
    
        /**
         * Creates a new empty Cell.
         */
        public Cell() {
            this(1, 1);
        }
        
        /**
         * Creates a new Cell containing the given Component.
         *
         * @param c The component that this cell should contain.
         */
        public Cell(Component c) {
            this(1, 1);
            this.add(c);
        }
        
        /**
         * Creates a new Cell with a column and row span.
         *
         * @param columnSpan The number of columns this cell should span.
         * @param rowSpan The number of rows this cell should span.
         */
        public Cell(int columnSpan, int rowSpan) {
            super();
            this.columnSpan = columnSpan;
            this.rowSpan = rowSpan;
        }
        
        /**
         * Creates a new Cell containing a Component with a column and row 
         * span.
         *
         * @param columnSpan The number of columns this cell should span.
         * @param rowSpan The number of rows this cell should span.
         * @param c The component that this cell should contain.
         */
        public Cell(int columnSpan, int rowSpan, Component c) {
            this(columnSpan, rowSpan);
            this.add(c);
        }
        
        /**
         * @see nextapp.echo.Component#applyStyle(Style)
         */
        public void applyStyle(Style style) {
            super.applyStyle(style);
            if (style.hasAttribute(STYLE_BACKGROUND_IMAGE)) {
                setBackgroundImage((ImageReference) style.getAttribute(STYLE_BACKGROUND_IMAGE));
            }
            if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
                setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
            }
            if (style.hasAttribute(STYLE_INSETS)) {
                setInsets((Insets) style.getAttribute(STYLE_INSETS));
            }
            if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
                setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
            }
        }
        
        /**
         * Returns the background image of the cell.
         *
         * @return The background image of the cell.
         */
        public ImageReference getBackgroundImage() {
            return backgroundImage;
        }
        
        /**
         * Returns the horizontal coordinate of the cell.
         *
         * @return The horizontal coordinate of the cell.
         */
        public final int getColumn() {
            if (position == null) {
                return -1;
            } else {
                return position.getX();
            }
        }
        
        /**
         * Returns the number of columns that this cell will span.
         *
         * @return The number of columns that this cell will span.
         */
        public final int getColumnSpan() {
            return columnSpan;
        }
        
        /** 
         * Returns the directions that this cell will flow into empty cells 
         * around it.  A bitmask will be returned indicating whether the cell
         * will occupy cells above, below, to the left, and/or to the right of
         * it if they are empty.
         *
         * @return A bitmask indicating which directions the cell will flow,
         *         which may consist of the following values:
         *         <ul>
         *         <li><code>FLOW_UP</code> Indicates the cell will flow 
         *          up.</li>
         *         <li><code>FLOW_LEFT</code> Indicates the cell will flow 
         *         left.</li>
         *         <li><code>FLOW_RIGHT</code> Indicates the cell will flow 
         *         right.</li>
         *         <li><code>FLOW_DOWN</code>Indicates the cell will flow 
         *         down.</li>
         *         </ul>
         */
        public int getFlow() {
            return flow;
        }
        
        /**
         * Returns the priority with which this cell should attempt to flow.
         * When grids are being rendered, cells with higher flow priority 
         * values will be given the opportunity to flow into neighboring cells
         * before those with lower values.  By default all cells have a flow
         * priority of zero.
         *
         * @return The flow priority value of this cell.
         */
        public int getFlowPriority() {
            return flowPriority;
        }
        
        /** 
         * Returns the horizontal alignment of the contents of this panel.
         *
         * @return The horizontal alignment of the contents of this panel,
         *         one of the following values:
         *         <ul>
         *         <li>EchoConstants.LEFT (the default)</li>
         *         <li>EchoConstants.CENTER</li>
         *         <li>EchoConstants.RIGHT</li>
         *         </ul>
         */
        public int getHorizontalAlignment() {
            return horizontalAlignment;
        }
        
        /**
         * Returns the insets of the cell. 
         *
         * @return The insets of the cell.  A null value indicates that the
         *         internal margins of the cell are those provided in the
         *         &quot;CellMargin&quot; property of the grid.
         */
        public Insets getInsets() {
            return insets;
        }
        
        /**
         * Returns whether lines of text should be wrapped if necessary.
         *
         * @return True if lines of text should be wrapped if necessary
         */
        public boolean getLineWrap() {
            return lineWrap;
        }
        
        /**
         * Returns the coordinate position of the cell within the grid.
         * 
         * @return The coordinate position of the cell within the grid.
         */
        public Coordinate getPosition() {
        	return position;
        }
        
        /**
         * Returns the vertical coordinate of the cell.
         *
         * @return The vertical coordinate of the cell.
         */
        public final int getRow() {
            if (position == null) {
                return -1;
            } else {
                return position.getY();
            }
        }

        /**
         * Returns the number of rows that this cell will span.
         *
         * @return The number of rows that this cell will span.
         */
        public final int getRowSpan() {
            return rowSpan;
        }
        
        /** 
         * Returns the vertical alignment of the contents of this panel.
         *
         * @return The vertical alignment of the contents of this panel, 
         *         one of the following values:
         *         <ul>
         *         <li>EchoConstants.TOP (the default)</li>
         *         <li>EchoConstants.CENTER</li>
         *         <li>EchoConstants.BOTTOM</li>
         *         </ul>
         */
        public int getVerticalAlignment() {
            return verticalAlignment;
        }

        /**
         * Sets the background image of the cell.
         *
         * @param newValue the new background image for the cell.
         */
        public void setBackgroundImage(ImageReference newValue) {
            ImageReference oldValue = backgroundImage;
            this.backgroundImage = newValue;
            this.firePropertyChange(BACKGROUND_IMAGE_CHANGED_PROPERTY, oldValue, newValue);
        }
        
        /**
         * Sets the number of columns that this cell will span.
         *
         * @param newValue The number of columns that this cell will span.
         */
        public final void setColumnSpan(int newValue) {
            int oldValue = columnSpan;
            this.columnSpan = newValue;
            this.firePropertyChange("columnSpan", oldValue, newValue);
        }
        
        /** 
         * Sets the directions that this cell will flow into empty cells 
         * around it.  The bitmask contains information about whether the cell
         * will occupy cells above, below, to the left, and/or to the right of
         * it if they are empty.
         *
         * @param newValue bitmask indicating which directions the cell will 
         *         flow, which may consist of the following values:
         *         <ul>
         *         <li><code>FLOW_UP</code> Indicates the cell will flow 
         *          up.</li>
         *         <li><code>FLOW_LEFT</code> Indicates the cell will flow 
         *         left.</li>
         *         <li><code>FLOW_RIGHT</code> Indicates the cell will flow 
         *         right.</li>
         *         <li><code>FLOW_DOWN</code>Indicates the cell will flow 
         *         down.</li>
         *         </ul>
         */
        public void setFlow(int newValue) {
            int oldValue = flow;
            this.flow = newValue;
            this.firePropertyChange(FLOW_CHANGED_PROPERTY, oldValue, newValue);
        }

        /**
         * Sets the priority with which this cell should attempt to flow.
         * When grids are being rendered, cells with higher flow priority 
         * values will be given the opportunity to flow into neighboring cells
         * before those with lower values.  By default all cells have a flow
         * priority of zero.
         *
         * @param newValue The new flow priority value of this cell.
         */
        public void setFlowPriority(int newValue) {
            int oldValue = flowPriority;
            this.flowPriority = newValue;
            this.firePropertyChange(FLOW_PRIORITY_CHANGED_PROPERTY, oldValue, newValue);
        }
        
        /**
         * Sets the horizontal alignment of the contents of this panel.
         *
         * @param newValue The horizontal alignment of the contents of this panel, 
         *        one of the following values.
         *        <ul>
         *        <li>EchoConstants.LEFT (the default)</li>
         *        <li>EchoConstants.CENTER</li>
         *        <li>EchoConstants.RIGHT</li>
         *        </ul>
         */
        public void setHorizontalAlignment(int newValue) {
            int oldValue = horizontalAlignment;
            horizontalAlignment = newValue;
            firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
        }
        
        /**
         * Sets the insets of the cell. 
         *
         * @param newValue The new insets of the cell.  A null value indicates 
         *         that the internal margins of the cell are those provided in 
         *         the &quot;CellMargin&quot; property of the grid.
         */
        public void setInsets(Insets newValue) {
            Insets oldValue = insets;
            insets = newValue;
            firePropertyChange(INSETS_CHANGED_PROPERTY, oldValue, newValue);
        }

        /**
         * Sets whether lines of text should be wrapped if necessary.
         *
         * @param newValue True if lines of text should be wrapped if 
         *        necessary.
         */
        public void setLineWrap(boolean newValue) {
            boolean oldValue = lineWrap;
            lineWrap = newValue;
            firePropertyChange(LINE_WRAP_CHANGED_PROPERTY, oldValue, newValue);
        }
        
        /**
         * Sets the coordinate position of this cell in the grid.
         * If a cell is currently located at the specified position in the grid,
         * it will be removed.
         * 
         * @param newValue The new position for the cell.
         */
        public void setPosition(Coordinate newValue) {
            Component parentComponent = getParent();
            Grid parentGrid = parentComponent instanceof Grid ? (Grid) parentComponent : null;
            if (parentGrid != null) {
                parentGrid.removePositionData(this);
            }
            Coordinate oldValue = position;
        	position = newValue;
            if (parentGrid != null) {
                parentGrid.addPositionData(this);
            }
        	firePropertyChange(POSITION_CHANGED_PROPERTY, oldValue, newValue);
        }
        
        /**
         * Sets the number of rows that this cell will span.
         *
         * @param newValue The number of rows that this cell will span.
         */
        public final void setRowSpan(int newValue) {
            int oldValue = rowSpan;
            this.rowSpan = newValue;
            this.firePropertyChange("rowSpan", oldValue, newValue);
        }
    
        /** 
         * Sets the vertical alignment of the contents of this panel.
         *
         * @param newValue The vertical alignment of the contents of this panel, 
         *        one of the following values.
         *        <ul>
         *        <li>EchoConstants.TOP (the default)</li>
         *        <li>EchoConstants.CENTER</li>
         *        <li>EchoConstants.BOTTOM</li>
         *        </ul>
         */
        public void setVerticalAlignment(int newValue) {
            int oldValue = verticalAlignment;
            verticalAlignment = newValue;
            firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
        }
    }
    
    /**
     * A utility class used to layout cells at validation/rendering.
     * Provides a dynamic two dimensional array to represent the grid
     * and capabilities to query regions for cells and set regions as
     * containing cells.  This class is used to determine conflicts
     * between cells with explicity set positions as well as to lay
     * out cells with automatically determined positions.
     * It is also used to calculate the size of the grid.
     */
    private static class RenderingWorkspace {
        
        private static final String FILLER = "filler"; 
        private List rows = new ArrayList();
        
        /**
         * Returns the number of columns.
         *
         * @return The number of columns.
         */
        public int getColumnCount() {
            int maxColumns = 0;
            Iterator it = rows.iterator();
            while (it.hasNext()) {
                List row = (List) it.next();
                if (row != null && row.size() > maxColumns) {
                    maxColumns = row.size();
                }
            }
            return maxColumns;
        }
        
        /**
         * Returns the number of rows.
         * 
         * @return The number of rows.
         */
        public int getRowCount() {
            return rows.size();
        }
        
        /**
         * Determines if the specified region is empty.
         * 
         * @param x1 The left coordinate.
         * @param y1 The top coordinate.
         * @param x2 The right coordiante.
         * @param y2 The bottom coordinate.
         * @return True if no cells occupy any of the specified region.
         */
        public boolean isRegionEmpty(int x1, int y1, int x2, int y2) {
            if (y2 >= getRowCount()) {
                y2 = getRowCount() - 1;
            }
            for (int y = y1; y <= y2; ++y) {
                List row = (List) rows.get(y);
                if (row != null) {
                    int rowSize = row.size();
                    for (int x = x1; x <= x2 && x < rowSize; ++x) {
                        if (row.get(x) != null) {
                            return false;
                        }
                    }
                }
            }
            return true;
        }
        
        /**
         * Marks the specified region as containing a cell.
         * 
         * @param x1 The left coordinate.
         * @param y1 The top coordinate.
         * @param x2 The right coordiante.
         * @param y2 The bottom coordinate.
         */
        public void markRegionFilled(int x1, int y1, int x2, int y2) {
            // Add nulls to rows list such that capacity is adequate.
            int ySize = rows.size();
            for (int y = ySize; y <= y2; ++y) {
                rows.add(null);
            }
            
            for (int y = y1; y <= y2; ++y) {
                // Retrieve row / create and store if necessary.
                List row = (List) rows.get(y);
                if (row == null) {
                    row = new ArrayList();
                    rows.set(y, row);
                }
                
                // Add nulls to row list such that capacity is adequate.
                int xSize = row.size();
                for (int x = xSize; x <= x2; ++x) {
                    row.add(null);
                }
                
                // Add filler elements to row.
                for (int x = x1; x <= x2; ++x) {
                    row.set(x, FILLER);
                }
            }
        }
    }
    
    /**
     * An exception that is thrown when grid cells overlap (due to their 
     * column/row spans).
     */
    public class GridCellOverlapException 
    extends IllegalStateException {

        /**
         * Creates a new <code>GridCellOverlapException</code>.
         */
        protected GridCellOverlapException(String cause) {
            super(cause);
        }
    }

    /**
     * A constant value for the <code>CellLayoutOrientation</code>
     * property.  Specifies that cells should be layed out horizontally
     * until all columns are used and then vertically.
     */
    public static final int CELL_LAYOUT_ORIENTATION_HORIZONTAL = 0;

    /**
     * A constant value for the <code>CellLayoutOrientation</code>
     * property.  Specifies that cells should be layed out vertically
     * until all rows are used and then horizontally.
     */
    public static final int CELL_LAYOUT_ORIENTATION_VERTICAL = 1;
    
    /**
     * A constant value for the <code>WidthUnits</code> and 
     * <code>HeightUnits</code> properties.
     * This value specifies that pixel-based units are used
     * in defining the individual column or row sizes and the 
     * overall width and height of the grid.
     */
    public static final int PIXEL_UNITS = 1;

    /**
     * A constant value for the <code>WidthUnits</code> and 
     * <code>HeightUnits</code> properties.
     * This value specifies that percent-based units are used
     * in defining the individual column or row sizes, but pixel based units 
     * are used in defining the overall width or height of the grid.
     */
    public static final int PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS = 3;

    /**
     * A constant value for the <code>WidthUnits</code> and 
     * <code>HeightUnits</code> properties.
     * This value specifies that percent-based are used in defining both the
     * individual column or row sizes and the overall width or height of the 
     * grid.
     */
    public static final int PERCENT_UNITS = 2;
    
    /** 
     * A style constant for the Border Color property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_COLOR = "borderColor";

    /** 
     * A style constant for the Border Size property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_SIZE = "borderSize";

    /** 
     * A style constant for the Cell Margin property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_CELL_MARGIN = "cellMargin";

    /** 
     * A style constant for the setting column widths. 
     * Values of this key must be an array of integer primitives.
     */
    public static final String STYLE_COLUMN_WIDTHS = "columnWidths";

    /** 
     * A style constant for the Height property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HEIGHT = "height";

    /** 
     * A style constant for the Height Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HEIGHT_UNITS = "heightUnits";

    /** 
     * A style constant for the setting row heights. 
     * Values of this key must be an array of integer primitives.
     */
    public static final String STYLE_ROW_HEIGHTS = "rowHeights";

    /** 
     * A style constant for the Width property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH = "width";

    /** 
     * A style constant for the Width Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH_UNITS = "widthUnits";
    
    /**
     * A constant used to specify an unknown grid cell width or height.
     *
     * @deprecated Use EchoConstants.UNDEFINED_SIZE.
     */
    public static final int UNKNOWN_SIZE = EchoConstants.UNDEFINED_SIZE;
    
    public static final String BORDER_COLOR_CHANGED_PROPERTY = "borderColor";
    public static final String BORDER_SIZE_CHANGED_PROPERTY = "borderSize";
    public static final String CELL_MARGIN_CHANGED_PROPERTY = "cellMargin";
    public static final String CELL_LAYOUT_ORIENTATION_CHANGED_PROPERTY = "cellLayoutOrientation";
    public static final String COLUMN_DELETED_PROPERTY = "columnDeleted";
    public static final String COLUMN_INSERTED_PROPERTY = "columnInserted";
    public static final String COLUMN_WIDTH_CHANGED_PROPERTY = "columnWidth";
    public static final String COLUMNS_CHANGED_PROPERTY = "columns";
    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String HEIGHT_UNITS_CHANGED_PROPERTY = "heightUnits";
    public static final String ROW_DELETED_PROPERTY = "rowDeleted";
    public static final String ROW_HEIGHT_CHANGED_PROPERTY = "rowHeight";
    public static final String ROW_INSERTED_PROPERTY = "rowInserted";
    public static final String ROWS_CHANGED_PROPERTY = "rows";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_UNITS_CHANGED_PROPERTY = "widthUnits";

    private SortedMap explicitPositionedCells = new TreeMap();
    private Map autoPositionedCellPositions;
    private Color borderColor = null;
    private int borderSize = 0;
    private int cellMargin = 0;
    private int columns = -1;
    private DynamicIntegerArray columnWidths = new DynamicIntegerArray(EchoConstants.UNDEFINED_SIZE);
    private int height = EchoConstants.UNDEFINED_SIZE;
    private int heightUnits = PIXEL_UNITS;
    private DynamicIntegerArray rowHeights = new DynamicIntegerArray(EchoConstants.UNDEFINED_SIZE);
    private int rows = -1;
    private boolean valid = false;
    private int width = EchoConstants.UNDEFINED_SIZE;
    private int widthUnits = PIXEL_UNITS;
    private int cellLayoutOrientation = CELL_LAYOUT_ORIENTATION_HORIZONTAL;
    private Dimension size = null;

    /**
     * Creates a new Grid.
     */
    public Grid() {
        super();
    }
    
    /**
     * Throws an <code>UnsupportedOperationException</code> if component is 
     * not a cell.
     * 
     * @see nextapp.echo.Component#add(Component, int)
     */
    public void add(Component c, int index) {
        if (!(c instanceof Cell)) {
        	throw new IllegalArgumentException("Cannot add component to grid that is not a "
                    + "derivative of nextapp.echo.Grid$Cell.");
        }
        Cell cell = (Cell) c;
        removePositionData(cell);
        super.add(cell, -1);
        addPositionData(cell);
    }
    
    /**
     * Adds a cell at a specific coordinate.
     *
     * @param position The coordinate position of the cell.
     * @param cell The cell to be added.
     */
    public void add(Coordinate position, Cell cell) {
    	cell.setPosition(position);
        add(cell, -1);
    }

    /**
     * Adds a cell at a specific coordinate.
     *
     * @param column The column at which to add the component.
     * @param row The row at which to add the component.
     * @param cell The cell to be added.
     */
    public void add(int column, int row, Cell cell) {
        add(new Coordinate(column, row), cell);
    }
    
    /**
     * Creates a new Cell containing only the given component and adds it at 
     * the specified position.
     *
     * @param column The column at which to add the component.
     * @param row The row at which to add the component.
     * @param component The component to be added.
     *
     * @return The <code>Cell</code> object that was added to the grid at the
     *         specified coordinate, containing the specified component.
     *         This value is useful if the application needs to set additional
     *         properties on the cell.
     */
    public Cell add(int column, int row, Component component) {
        Cell cell = new Cell(component);
        add(column, row, cell);
        return cell;
    }
    
    /**
     * Adds the specified cell into the internal map of cell positions or to
     * the set of unpositioned cells.
     * 
     * @param cell The cell to be added.
     */
    private void addPositionData(Cell cell) {
        invalidate();
        if (cell.getPosition() != null) {
            Cell oldCell = (Cell) explicitPositionedCells.put(cell.getPosition(), cell);
            if (oldCell != null && oldCell != cell) {
                remove(oldCell);
            }
        }
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_BORDER_COLOR)) {
            setBorderColor((Color) style.getAttribute(STYLE_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_BORDER_SIZE)) {
            setBorderSize(style.getIntegerAttribute(STYLE_BORDER_SIZE));
        }
        if (style.hasAttribute(STYLE_CELL_MARGIN)) {
            setCellMargin(style.getIntegerAttribute(STYLE_CELL_MARGIN));
        }
        if (style.hasAttribute(STYLE_COLUMN_WIDTHS)) {
            int[] widths = (int[]) style.getAttribute(STYLE_COLUMN_WIDTHS);
            for (int index = 0; index < widths.length; ++index) {
                setColumnWidth(index, widths[index]);
            }
        }
        if (style.hasAttribute(STYLE_HEIGHT)) {
            setHeight(style.getIntegerAttribute(STYLE_HEIGHT));
        }
        if (style.hasAttribute(STYLE_HEIGHT_UNITS)) {
            setHeightUnits(style.getIntegerAttribute(STYLE_HEIGHT_UNITS));
        }
        if (style.hasAttribute(STYLE_ROW_HEIGHTS)) {
            int[] heights = (int[]) style.getAttribute(STYLE_ROW_HEIGHTS);
            for (int index = 0; index < heights.length; ++index) {
                setRowHeight(index, heights[index]);
            }
        }
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));
        }
        if (style.hasAttribute(STYLE_WIDTH_UNITS)) {
            setWidthUnits(style.getIntegerAttribute(STYLE_WIDTH_UNITS));
        }
    }
    
    /**
     * Remove a column from the grid.
     *
     * @param column The column to remove from the grid.
     */
    public void deleteColumn(int column) {
        Cell cell;
        int cellColumn;
        int cellRow;
        List removedCells = new ArrayList();
        List relocatedCells = new ArrayList();
        
        for (Iterator it = explicitPositionedCells.values().iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            if (cell.getColumn() < column) {
                // Cell is in a column less than the deleted column.
                if (cell.getColumn() + cell.columnSpan - 1 >= column) {
                    // Column spans across deleted column, reduce its span.
                    cell.columnSpan--;
                }
            } else if (cell.getColumn() == column) {
                // Cell is on deleted column: if it spans multiple columns, reduce its column span, otherwise, delete it.
                if (cell.columnSpan > 1) {
                    cell.columnSpan--;
                } else {
                    removedCells.add(cell);
                }
            } else {
                // Cell is after deleted column: move it up one column.
                relocatedCells.add(cell);
            }
        }
        
        // Remove cell that are being permanently removed.
        for (Iterator it = removedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }

        // Remove cell that are being relocated.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }
        
        // Add relocating cells at new locations.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            cellColumn = cell.position.getX();
            cellRow = cell.position.getY();
            add(cellColumn - 1, cellRow, cell);
        }
        
        firePropertyChange(COLUMN_DELETED_PROPERTY, null, null);
    }
    
    /**
     * Remove a row from the grid.
     *
     * @param row The row to remove from the grid.
     */
    public void deleteRow(int row) {
        Cell cell;
        int cellColumn;
        int cellRow;
        List removedCells = new ArrayList();
        List relocatedCells = new ArrayList();
        
        for (Iterator it = explicitPositionedCells.values().iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            if (cell.getRow() < row) {
                // Cell is on a row less than the deleted row.
                if (cell.getRow() + cell.rowSpan - 1 >= row) {
                    // Row spans across deleted row, reduce its span.
                    cell.rowSpan--;
                }
            } else if (cell.getRow() == row) {
                // Cell is on deleted row: if it spans multiple rows, reduce its row span, otherwise, delete it.
                if (cell.rowSpan > 1) {
                    cell.rowSpan--;
                } else {
                    removedCells.add(cell);
                }
            } else {
                // Cell is after deleted row: move it up one row.
                relocatedCells.add(cell);
            }
        }
        
        // Remove cell that are being permanently removed.
        for (Iterator it = removedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }

        // Remove cell that are being relocated.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }
        
        // Add relocating cells at new locations.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            cellColumn = cell.position.getX();
            cellRow = cell.position.getY();
            add(cellColumn, cellRow - 1, cell);
        }
        
        firePropertyChange(ROW_DELETED_PROPERTY, null, null);
    }
    
    /**
     * Returns the Cell at the specified position
     *
     * @param position The coordinate of the cell.
     * @return The cell at the specified position.  If no cell
     *         was found, null is returned.
     */
    public Cell get(Coordinate position) {
        return (Cell) explicitPositionedCells.get(position);
    }
    
    /**
     * Returns the Cell at the specified position
     *
     * @param column The column number of the cell to retrieve.
     * @param row The row number of the cell to retrieve.
     * @return The cell at the specified column and row position.  If no cell
     *         was found, null is returned.
     */
    public Cell get(int column, int row) {
        return get(new Coordinate(column, row));
    }
    
    /**
     * Returns the border color.
     * 
     * @return The border color.
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
    /**
     * Returns the border size. 
     *
     * @return The border size in pixels.
     */
    public int getBorderSize() {
        return borderSize;
    }
    
    /**
     * Returns the cell layout orientation.  The cell layout orientation 
     * defines the path in which automatically positioned cells will be 
     * placed within the grid.
     * 
     * @return The cell layout orientation, one of the following values:
     *         <ul>
     *          <li>CELL_LAYOUT_ORIENTATION_HORIZONTAL</li>
     *          <li>CELL_LAYOUT_ORIENTATION_VERTICAL</li>
     *         </ul>
     */
    public int getCellLayoutOrientation() {
        return cellLayoutOrientation;
    }
    
    /**
     * Returns the cell margin.  The default cell margin is 3 pixels.
     *
     * @return The cell margin in pixels.
     */
    public int getCellMargin() {
        return cellMargin;
    }
    
    /**
     * Returns the position of a specific cell in the grid.
     * 
     * @param cell The cell whose position is to be returned.
     * @return The position of the cell.
     */
    public Coordinate getCellPosition(Cell cell) {
        Coordinate position = cell.getPosition();
        if (position == null) {
            position = (Coordinate) autoPositionedCellPositions.get(cell);
        }
        return position;
    }
    
    /**
     * Returns the programmatically-defined allowed number of columns. 
     *
     * @return The programmatically-defined allowed number of columns.
     */
    public int getColumns() {
        return columns;
    }
    
    /**
     * Returns the width of a grid column.  If the width is not 
     * defined, <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     * The units for this value may be pixels or percentiles, based on the
     * value of the &quot;WidthUnits&quot; property.
     *
     * @param column The column index.
     * @return The width of the grid column.
     */
    public int getColumnWidth(int column) {
        return columnWidths.get(column);
    }
    
    /**
     * Returns the overall height of the grid.  This property is not relevant
     * when row heights are being measured in pixels and all rows have a 
     * height value, or if the sum of all row heights (and margins) is 
     * greater than the height value.
     *
     * @return The height of the grid.  If the height is not defined, 
     *         <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * Returns the units (pixel or percent) in which column heights and overall 
     * grid height are measured.
     *
     * @return The unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public int getHeightUnits() {
        return heightUnits;
    }
    
    /**
     * Returns the height of a grid row.  If the height is not 
     * defined, <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     * The units for this value may be pixels or percentiles, based on the
     * value of the &quot;HeightUnits&quot; property.
     *
     * @param row The row index.
     * @return The height of the grid row.
     */
    public int getRowHeight(int row) {
        return rowHeights.get(row);
    }
    
    /** 
     * Returns the size of the grid.
     *
     * @return The size of the grid in cells as a <code>Dimension</code>.
     */
    public Dimension getSize() {
        validate();
        return size;
    }
    
    /**
     * Returns the programmatically-defined allowed number of rows. 
     *
     * @return The programmatically-defined allowed number of rows.
     */
    public int getRows() {
        return rows;
    }
    
    /**
     * Returns the overall width of the grid.  This property is not relevant
     * when column widths are being measured in pixels and all columns have a 
     * width value, or if the sum of all columns widths (and margins) is 
     * greater than the width value.
     *
     * @return The width of the grid.  If the width is not defined, 
     *         <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     */
    public int getWidth() {
         return width;
    }
    
    /**
     * Returns the units (pixel or percent) in which column widths and overall 
     * grid width are measured.
     *
     * @return The unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public int getWidthUnits() {
        return widthUnits;
    }
    
    /**
     * Inserts a column into the grid.
     *
     * @param column The index of the inserted column.
     */
    public void insertColumn(int column) {
        Cell cell;
        int cellColumn;
        int cellRow;
        List relocatedCells = new ArrayList();
        
        for (Iterator it = explicitPositionedCells.values().iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            if (cell.getColumn() < column) {
                // Cell is on a column less than the inserted column.
                if (cell.getColumn() + cell.columnSpan > column) {
                    // Column spans across inserted column, increase its span.
                    cell.columnSpan++;
                }
            } else {
                // Cell is after inserted column: move it down one column.
                relocatedCells.add(cell);
            }
        }
        
        // Remove cell that are being relocated.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }
        
        // Add relocating cells at new locations.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            cellColumn = cell.position.getX();
            cellRow = cell.position.getY();
            add(cellColumn + 1, cellRow, cell);
        }
        
        firePropertyChange(ROW_DELETED_PROPERTY, null, null);
    }
    
    /**
     * Inserts a row into the grid.
     *
     * @param row The index of the inserted row.
     */
    public void insertRow(int row) {
        Cell cell;
        int cellColumn;
        int cellRow;
        List relocatedCells = new ArrayList();
        
        for (Iterator it = explicitPositionedCells.values().iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            if (cell.getRow() < row) {
                // Cell is on a row less than the inserted row.
                if (cell.getRow() + cell.rowSpan > row) {
                    // Row spans across inserted row, increase its span.
                    cell.rowSpan++;
                }
            } else {
                // Cell is after inserted row: move it down one row.
                relocatedCells.add(cell);
            }
        }
        
        // Remove cell that are being relocated.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            remove((Cell) it.next());
        }
        
        // Add relocating cells at new locations.
        for (Iterator it = relocatedCells.iterator(); it.hasNext();) {
            cell = (Cell) it.next();
            cellColumn = cell.position.getX();
            cellRow = cell.position.getY();
            add(cellColumn, cellRow + 1, cell);
        }
        
        firePropertyChange(ROW_DELETED_PROPERTY, null, null);
    }
    
    public void invalidate() {
        autoPositionedCellPositions = null;
        valid = false;
    }
    
    /**
     * Returns a sequential (column then row) iterator over all cells in 
     * the grid.
     *
     * @return An iterator over all cells in the grid.
     */
    public Iterator iterator() {
        validate();
        SortedMap cells = new TreeMap(explicitPositionedCells);
        Iterator it = autoPositionedCellPositions.keySet().iterator();
        while (it.hasNext()) {
            Cell cell = (Cell) it.next();
            Coordinate coordinate = (Coordinate) autoPositionedCellPositions.get(cell);
            cells.put(coordinate, cell);
        }
        return cells.values().iterator();
    }
    
	/**
     * Removes the specified Cell.
     *
     * @param component The cell to be removed.
     * @see nextapp.echo.Component#remove(nextapp.echo.Component)
     */
    public void remove(Component component) {
        Cell cell = (Cell) component;
        removePositionData(cell);
        super.remove(cell);
    }
    
    /**
     * Removes and returns the cell at the specified position.
     *
     * @param position The coordinate position of the cell to remove.
     * @return The cell that was removed.  If no cell was found
     *         at the specified coordinates, null is returned.
     */
    public Cell remove(Coordinate position) {
        Cell cell = get(position);
        if (cell != null) {
            remove(cell);
        }
        return cell;
    }
    
    /**
     * Removes and returns the cell at the specified position.
     *
     * @param column The column number of the cell to remove.
     * @param row The row number of the cell to remove.
     * @return The cell that was removed.  If no cell was found
     *         at the specified coordinates, null is returned.
     */
    public Cell remove(int column, int row) {
    	return remove(new Coordinate(column, row));
    }
    
    /**
     * Removes the specified cell from the internal map of cell positions or
     * from the set of unpositioned cells.
     * 
     * @param cell The cell to be removed. 
     */
    private void removePositionData(Cell cell) {
        invalidate();
        if (cell.getPosition() != null) {
            explicitPositionedCells.remove(cell.getPosition());
        }
    }
    
    /**
     * Prepares cells for rendering by rendering peer.
     * Creates <code>autoPositionedCellPositions</code> map.
     * Ensures that no cells overlap.
     */
    private void renderCells() {
        Component[] components = getComponents();
        autoPositionedCellPositions = new HashMap();
        RenderingWorkspace workspace = new RenderingWorkspace();

        // Lay out explicity positioned cells.
        Iterator it = explicitPositionedCells.values().iterator();
        while (it.hasNext()) {
            Cell cell = (Cell) it.next();
            Coordinate position = cell.getPosition();
            int x2 = position.getX() + cell.getColumnSpan() - 1;
            int y2 = position.getY() + cell.getRowSpan() - 1;
            if (!workspace.isRegionEmpty(position.getX(), position.getY(), x2, y2)) {
                throw new GridCellOverlapException("Grid region (" + position.getX() + ", " + position.getY() 
                        + ")-(" + x2 + "," + y2 + ") is occupied by multiple explicity positioned cells.");
            }
            workspace.markRegionFilled(position.getX(), position.getY(), x2, y2);
        }
        
        // Lay out automatically positioned cells.
        int cursorX = 0, cursorY = 0;
        for (int index = 0; index < components.length; ++index) {
            Cell cell = (Cell) components[index];
            if (cell.getPosition() == null) {
                if (getCellLayoutOrientation() == CELL_LAYOUT_ORIENTATION_VERTICAL) {
                    // Verify that no cell is vertically too large to fit.
                    if (getRows() > 0 && cell.getRowSpan() > getRows()) {
                        throw new GridCellOverlapException("Cell row span larger than grid row size.");
                    }

                    boolean positioned = false;
                    while (!positioned) {
                        int x2 = cursorX + cell.getColumnSpan() - 1;
                        int y2 = cursorY + cell.getRowSpan() - 1;
                        if ((getRows() <= 0 || y2 < getRows()) && workspace.isRegionEmpty(cursorX, cursorY, x2, y2)) {
                            // Available region found.
                            workspace.markRegionFilled(cursorX, cursorY, x2, y2);
                            autoPositionedCellPositions.put(cell, new Coordinate(cursorX, cursorY));
                            positioned = true;
                        }
                        
                        // Move cursor down, then return and right.
                        if (getRows() <= 0 || cursorY < getRows() - 1) {
                            ++cursorY;
                        } else {
                            cursorY = 0;
                            ++cursorX;
                        }
                    }
                } else {
                    // Verify that no cell is horizontally too large to fit.
                    if (getColumns() > 0 && cell.getColumnSpan() > getColumns()) {
                        throw new GridCellOverlapException("Cell column span larger than grid column size.");
                    }

                    boolean positioned = false;
                    while (!positioned) {
                        int x2 = cursorX + cell.getColumnSpan() - 1;
                        int y2 = cursorY + cell.getRowSpan() - 1;
                        if ((getColumns() <= 0 || x2 < getColumns()) && workspace.isRegionEmpty(cursorX, cursorY, x2, y2)) {
                            // Available region found.
                            workspace.markRegionFilled(cursorX, cursorY, x2, y2);
                            autoPositionedCellPositions.put(cell, new Coordinate(cursorX, cursorY));
                            positioned = true;
                        }
                        
                        // Move cursor right, then return and down.
                        if (getColumns() <= 0 || cursorX < getColumns() - 1) {
                            ++cursorX;
                        } else {
                            cursorX = 0;
                            ++cursorY;
                        }
                    }
                }
            }
        }
        
        // Store calculated size of grid.
        size = new Dimension(workspace.getColumnCount(), workspace.getRowCount());
    }

    /** 
     * Sets the border color of the grid.
     *
     * @param newValue The border color of the grid.
     */
    public void setBorderColor(Color newValue) {
        Color oldValue = borderColor;
        borderColor = newValue;
        firePropertyChange(BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the size of the grid's border.
     *
     * @param newValue The size of the grid's border, in pixels.
     */
    public void setBorderSize(int newValue) {
        int oldValue = borderSize;
        borderSize = newValue;
        firePropertyChange(BORDER_SIZE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the cell layout orientation.  The cell layout orientation 
     * defines the path in which automatically positioned cells will be 
     * placed within the grid.
     * 
     * @param newValue The new cell layout orientation, one of the 
     *        following values:
     *        <ul>
     *         <li>CELL_LAYOUT_ORIENTATION_HORIZONTAL</li>
     *         <li>CELL_LAYOUT_ORIENTATION_VERTICAL</li>
     *        </ul>
     */
    public void setCellLayoutOrientation(int newValue) {
        int oldValue = cellLayoutOrientation;
        cellLayoutOrientation = newValue;
        firePropertyChange(CELL_LAYOUT_ORIENTATION_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the cell interior margin.  The default size is 3 pixels
     *
     * @param newValue The cell margin, in pixels.
     */
    public void setCellMargin(int newValue) {
        invalidate();
        int oldValue = cellMargin;
        cellMargin = newValue;
        firePropertyChange(CELL_MARGIN_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the programmatically defined allowed number of columns.
     * 
     * @param newValue The allowed number of columns in the grid.
     */
    public void setColumns(int newValue) {
        int oldValue = columns;
        columns = newValue;
        firePropertyChange(COLUMNS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of a column of the grid.
     *
     * @param column The column number.
     * @param newValue The new width of the specified column.
     */
    public void setColumnWidth(int column, int newValue) {
        columnWidths.set(column, newValue);
        firePropertyChange(COLUMN_WIDTH_CHANGED_PROPERTY, null, null);
    }
    
    /**
     * Sets the overall height of the grid.  Setting this property will have no
     * effect in the event that all rows have a height value, or if the sum
     * of all row heights (and margins) is greater than the height value.
     *
     * @param newValue The new height of the grid.   To set the height to be
     *        undefined, use the <code>EchoConstants.UNDEFINED_SIZE</code> 
     *        constant.
     */
    public void setHeight(int newValue) {
        int oldValue = height;
        height = newValue;
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units (pixel or percent) in which row heights and overall grid 
     * height are measured.
     *
     * @param newValue The new unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setHeightUnits(int newValue) {
        int oldValue = heightUnits;
        heightUnits = newValue;
        firePropertyChange(HEIGHT_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the height of a row of the grid.
     *
     * @param row The row number.
     * @param newValue The new height of the specified row.
     */
    public void setRowHeight(int row, int newValue) {
        rowHeights.set(row, newValue);
        firePropertyChange(ROW_HEIGHT_CHANGED_PROPERTY, null, null);
    }
    
    /**
     * Sets the programmatically defined allowed number of rows.
     * 
     * @param newValue The allowed number of rows in the grid.
     */
    public void setRows(int newValue) {
        int oldValue = rows;
        rows = newValue;
        firePropertyChange(ROWS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the overall width of the grid.  Setting this property will have no
     * effect in the event that all columns have a width value, or if the sum
     * of all columns widths (and margins) is greater than the width value.
     *
     * @param newValue The new width of the grid.  To set the width to be
     *        undefined, use the <code>EchoConstants.UNDEFINED_SIZE</code> 
     *        constant.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units (pixel or percent) in which column widths and overall 
     * grid width are measured.
     *
     * @param newValue The new unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setWidthUnits(int newValue) {
        int oldValue = widthUnits;
        widthUnits = newValue;
        firePropertyChange(WIDTH_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Overrides Component.validate(), to ensure that no cells of the grid 
     * overlap.  If any cells overlap, a <code>GridCellOverlapException</code>
     * is thrown.
     */
    public void validate() {
        if (valid) {
            return;
        }
        renderCells();
        valid = true;
    }
}
