/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A pane that displays content retrieved from an HTTP request.
 */
public class HttpPane extends AbstractPane {
    
    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String HEIGHT_EXTENT_CHANGED_PROPERTY = "heightExtent";
    public static final String URI_CHANGED_PROPERTY = "uri";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_EXTENT_CHANGED_PROPERTY = "widthExtent";
    
    private String uri;
    private Extent widthExtent = null;
    private Extent heightExtent = null;
    
    /**
     * Creates a new HTTP Pane.
     */
    public HttpPane() {
        this(null);
    }

    /**
     * Creates a new HTTP Pane containing the document at the given URI.
     *
     * @param uri The URI to load in the pane.
     */
    public HttpPane(String uri) {
        super();
        
        this.uri = uri;
    }
    
    /**
     * Returns the height of the pane.
     * 
     * @return The height of the pane.
     */
    public Extent getHeightExtent() {
        return heightExtent;
    }
    
    /**
     * Returns the URI of the HttpPane.  If the user has clicked on links 
     * within the document in the HttpPane, this URI may not be correct.
     *
     * @return The URI of the HttpPane.
     */
    public String getUri() {
        return uri;
    }
    
    /**
     * Returns the width of the pane.
     * 
     * @return The width of the pane.
     */
    public Extent getWidthExtent() {
        return widthExtent;
    }

    /**
     * Sets the height of the pane in pixels.
     *
     * @param newValue The new height of the pane in pixels.
     */
    public void setHeight(int newValue) {
        Extent heightExtent = getHeightExtent();
        int oldValue = (heightExtent != null && heightExtent.getUnits() == Extent.PX) 
                ? heightExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
        setHeightExtent(new Extent(newValue, Extent.PX));
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the height of the pane.
     * 
     * @param newValue the new height of the pane.
     */
    public void setHeightExtent(Extent newValue) {
        Extent oldValue = heightExtent;
        heightExtent = newValue;
        firePropertyChange(HEIGHT_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the URI of the pane's content.
     *
     * @param newValue The URI of the pane's content.
     */
    public void setUri(String newValue) {
        String oldValue = uri;
        uri = newValue;
        firePropertyChange(URI_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the pane in pixels.
     *
     * @param newValue The new width of the pane in pixels.
     */
    public void setWidth(int newValue) {
        Extent widthExtent = getWidthExtent();
        int oldValue = (widthExtent != null && widthExtent.getUnits() == Extent.PX) 
                ? widthExtent.getValue() : EchoConstants.UNDEFINED_SIZE;
        setWidthExtent(new Extent(newValue, Extent.PX));
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the pane.
     * 
     * @param newValue The new width of the pane.
     */
    public void setWidthExtent(Extent newValue) {
        Extent oldValue = widthExtent;
        widthExtent = newValue;
        firePropertyChange(WIDTH_EXTENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
