/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;

/**
 * Contains information about a component's margins.
 * Insets may be used with components such as <code>Panel</code>s, 
 * <code>Grid.Cell</code>s and <code>ContentPane</code>s.
 */
public class Insets 
implements Serializable {

    private int top;
    private int bottom;
    private int left;
    private int right;
    
    /**
     * Creates a new Insets object with the given margin sizes (in pixels).
     *
     * @param left The size of the left margin.
     * @param top The size of the top margin.
     * @param right The size of the right margin.
     * @param bottom The size of the bottom margin.
     */
    public Insets(int left, int top, int right, int bottom) {
        super();
        
        this.left = left;
        this.top = top;
        this.right = right;
        this.bottom = bottom;
    }
    
    /**
     * Creates a new Insets object, defining all margins to be the provided 
     * value.
     *
     * @param size The margin size.
     */
    public Insets(int size) {
        this(size, size, size, size);
    }
    
    /**
     * Creates a new Insets object by defining values for the horizontal and
     * vertical margins.
     *
     * @param horizontal The size of the horizontal (left and right) margins.
     * @param vertical The size of the vertical (top and bottom) margins.
     */
    public Insets(int horizontal, int vertical) {
        this(horizontal, vertical, horizontal, vertical);
    }
    
    /**
     * Returns the size of the bottom margin.
     *
     * @return The size of the bottom margin.
     */
    public int getBottom() {
        return bottom;
    }
    
    /**
     * Returns the size of the left margin.
     *
     * @return The size of the left margin.
     */
    public int getLeft() {
        return left;
    }
    
    /**
     * Returns the size of the right margin.
     *
     * @return The size of the right margin.
     */
    public int getRight() {
        return right;
    }

    /**
     * Returns the size of the top margin.
     *
     * @return The size of the top margin.
     */
    public int getTop() {
        return top;
    }
}
