/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A display area for text, an image, or both.
 */
public class Label extends Component {
    
    /** 
     * A style constant for the Horizontal Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";

    /** 
     * A style constant for the Horizontal Text Position property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_TEXT_POSITION = "horizontalTextPosition";

    /** 
     * A style constant for the Icon-Text Margin property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_ICON_TEXT_MARGIN = "iconTextMargin";

    /** 
     * A style constant for the Vertical Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";

    /** 
     * A style constant for the Vertical Text Position property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_TEXT_POSITION = "verticalTextPosition";
    
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String HORIZONTAL_TEXT_POSITION_CHANGED_PROPERTY = "horizontalTextPosition";
    public static final String ICON_CHANGED_PROPERTY = "icon";
    public static final String ICON_TEXT_MARGIN_CHANGED_PROPERTY = "iconTextMargin";
    public static final String TEXT_CHANGED_PROPERTY = "text";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";
    public static final String VERTICAL_TEXT_POSITION_CHANGED_PROPERTY = "verticalTextPosition";

    private String text = null;
    private ImageReference icon = null;
    private int iconTextMargin = 3;
    private int horizontalAlignment = EchoConstants.LEFT;
    private int horizontalTextPosition = EchoConstants.RIGHT;
    private int verticalAlignment = EchoConstants.CENTER;
    private int verticalTextPosition = EchoConstants.CENTER;

    /**
     * Creates a label with no text or icon.
     */
    public Label() {
        this(null, null);
    }
    
    /**
     * Creates a label with text.
     *
     * @param text The text to be displayed.
     */
    public Label(String text) {
        this(text, null);
    }
    
    /**
     * Creates a label with an icon.
     *
     * @param icon The icon to be displayed.
     */
    public Label(ImageReference icon) {
        this(null, icon);
    }

    /**
     * Creates a label with text and an icon.
     *
     * @param text The text to be displayed.
     * @param icon The icon to be displayed.
     */
    public Label(String text, ImageReference icon) {
        super();
    
        setIcon(icon);
        setText(text);
    }

    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_TEXT_POSITION)) {
            setHorizontalTextPosition(style.getIntegerAttribute(STYLE_HORIZONTAL_TEXT_POSITION));
        }
        if (style.hasAttribute(STYLE_ICON_TEXT_MARGIN)) {
            setIconTextMargin(style.getIntegerAttribute(STYLE_ICON_TEXT_MARGIN));
        }
        if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
            setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_VERTICAL_TEXT_POSITION)) {
            setVerticalTextPosition(style.getIntegerAttribute(STYLE_VERTICAL_TEXT_POSITION));
        }
    }

    /**
     * Returns the horizontal alignment of the text relative to the icon.
     *
     * @return The horizontal alignment of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }
    
    /**
     * Returns the horizontal position of the text relative to the icon.
     *
     * @return The horizontal position of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT (the default)</li>
     *         </ul>
     */
    public int getHorizontalTextPosition() {
        return horizontalTextPosition;
    }
    
    /**
     * Returns the icon.
     *
     * @return The icon.
     */
    public ImageReference getIcon() {
        return icon;
    }

    /**
     * Returns the size of the margin to be displayed between the icon and text 
     * The margin is only drawn if both icon and text are present.
     *
     * @return The size of the margin between the icon and text, in pixels.
     */
    public int getIconTextMargin() {
        return iconTextMargin;
    }

    /**
     * Returns the text.
     *
     * @return The text.
     */    
    public String getText() {
        return text;
    }
    
    /**
     * Returns the vertical alignment of the text relative to the icon.
     *
     * @return The vertical alignment of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalAlignment() {
        return verticalAlignment;
    }
    
    /**
     * Returns the vertical position of the text relative to the icon.
     *
     * @return The vertical position of the text relative to the icon,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalTextPosition() {
        return verticalTextPosition;
    }
    
    /**
     * Sets the horizontal alignment of the text relative to the icon.
     *
     * @param newValue The horizontal alignment of the text relative to the icon,
     *        one of the following values:
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the horizontal position of the text relative to the icon.
     *
     * @param newValue The horizontal position of the text relative to the icon,
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.LEFT</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT (the default)</li>
     *        </ul>
     */
    public void setHorizontalTextPosition(int newValue) {
        int oldValue = horizontalTextPosition;
        horizontalTextPosition = newValue;
        firePropertyChange(HORIZONTAL_TEXT_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the icon to be displayed.
     *
     * @param newValue The icon to be displayed.
     */
    public void setIcon(ImageReference newValue) {
        ImageReference oldValue = icon;
        icon = newValue;
        firePropertyChange(ICON_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the size of the margin to be displayed between the icon and text.
     * The margin is only drawn if both icon and text are present.
     *
     * @param newValue The size of the margin between the icon and text, in 
     *        pixels.
     */
    public void setIconTextMargin(int newValue) {
        int oldValue = iconTextMargin;
        iconTextMargin = newValue;
        firePropertyChange(ICON_TEXT_MARGIN_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the text to be displayed.
     *
     * @param newValue The text to be displayed.
     */
    public void setText(String newValue) {
        String oldValue = text;
        text = newValue;
        firePropertyChange(TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical alignment of the text relative to the icon.
     *
     * @param newValue The vertical alignment of the text relative to the icon,
     *        one of the following values:
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalAlignment(int newValue) {
        int oldValue = verticalAlignment;
        verticalAlignment = newValue;
        firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical position of the text relative to the icon.
     *
     * @param newValue The vertical position of the text relative to the icon,
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalTextPosition(int newValue) {
        int oldValue = verticalTextPosition;
        verticalTextPosition = newValue;
        firePropertyChange(VERTICAL_TEXT_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }
}
