/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;

import nextapp.echo.event.ListDataEvent;
import nextapp.echo.event.ListDataListener;
import nextapp.echo.event.ListSelectionEvent;
import nextapp.echo.event.ListSelectionListener;

/**
 * A component that allows the selection of one or more displayed items.
 */
public class ListBox extends Component
implements ToolTipSupport {
    
    /**
     * A constant for units-defining properties that represent pixel-based 
     * units.
     */
    public static final int PIXEL_UNITS = 0;

    /**
     * A constant for units-defining properties that represent percent-based 
     * units.
     */
    public static final int PERCENT_UNITS = 2;
    
    /** 
     * A style constant for the Width property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH = "width";

    /** 
     * A style constant for the Width Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH_UNITS = "widthUnits";

    /** 
     * A style constant for the Visible Row Count property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VISIBLE_ROW_COUNT = "visibleRowCount";

    public static final String CELL_RENDERER_CHANGED_PROPERTY = "cellRenderer";
    public static final String LIST_DATA_CHANGED_PROPERTY = "listData";
    public static final String LIST_SELECTION_CHANGED_PROPERTY = "listSelection";
    public static final String SELECTION_MODEL_CHANGED_PROPERTY = "selectionModel";
    public static final String VISIBLE_ROW_COUNT_CHANGED_PROPERTY = "visibleRowCount";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_UNITS_CHANGED_PROPERTY = "widthUnits";

    /**
     * Local handler for list selection events.
     */
    private class ListSelectionHandler
    implements ListSelectionListener, Serializable {

        public void valueChanged(ListSelectionEvent e) {
            firePropertyChange(LIST_SELECTION_CHANGED_PROPERTY, null, null);
        }
    }
    
    /**
     * Local handler for list data events.
     */
    private class ListDataHandler 
    implements ListDataListener, Serializable {

        public void contentsChanged(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }

        public void intervalAdded(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }

        public void intervalRemoved(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }
    }

    private ListCellRenderer cellRenderer = DefaultListCellRenderer.DEFAULT_LIST_CELL_RENDERER;
    private ListModel model;
    private ListSelectionModel selectionModel;
    private int visibleRowCount = 8;
    private ListSelectionListener selectionListener = new ListSelectionHandler();
    private ListDataListener dataListener = new ListDataHandler();
    private String toolTipText;
    private int width = -1;
    private int widthUnits = PIXEL_UNITS;

    /**
     * Creates an empty ListBox. 
     */
    public ListBox() {
        this(new DefaultListModel());
    }
    
    /** 
     * Creates a ListBox with the given data model.
     *
     * @param model The ListModel for this ListBox.
     */
    public ListBox(ListModel model) {
        super();

        this.model = model;
        model.addListDataListener(dataListener);
        
        selectionModel = new DefaultListSelectionModel();
        selectionModel.addListSelectionListener(selectionListener);
    }
    
    /**
     * Creates a ListBox with a default model populated with the 
     * given array of elements.
     *
     * @param elements An array of elements that will initially populate
     *        this ListBox.
     */
    public ListBox(Object[] elements) {
        this();
    
        DefaultListModel model = (DefaultListModel) getModel();
        for (int index = 0; index < elements.length; ++index) {
            model.add(elements[index]);
        }
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_VISIBLE_ROW_COUNT)) {
            setVisibleRowCount(style.getIntegerAttribute(STYLE_VISIBLE_ROW_COUNT));
        }
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));   
        }
        if (style.hasAttribute(STYLE_WIDTH_UNITS)) {
            setWidthUnits(style.getIntegerAttribute(STYLE_WIDTH_UNITS));   
        }
    }
    
    /**
     * Deselects all items.
     */
    public void clearSelection() {
        selectionModel.clearSelection();
    }
    
    /**
     * Returns the renderer for items.
     * 
     * @return The renderer for items.
     */
    public ListCellRenderer getCellRenderer() {
        return cellRenderer;
    }
    
    /**
     * Returns the maximum selected index.
     *
     * @return the maximum selected index.
     */
    public int getMaxSelectedIndex() {
        return selectionModel.getMaxSelectedIndex();
    }
        
    /**
     * Returns the minimum selected index.
     *
     * @return The minimum selected index.
     */
    public int getMinSelectedIndex() {
        return selectionModel.getMinSelectedIndex();
    }
        
    /**
     * Returns the model.
     *
     * @return The model.
     */
    public ListModel getModel() {
        return model;
    }
    
    /**
     * Returns the selected index.  This method is used for ListBoxes where the
     * selectionMode property is set to ListSelectionModel.SINGLE_SELECT.  In
     * the event multiple selection is enabled, this method will return the 
     * lowest selected index.
     *
     * @return The selected index.
     */
    public int getSelectedIndex() {
        return getMinSelectedIndex();
    }
    
    /**
     * Returns the selected item.  This method is used for ListBoxes where the
     * selectionMode property is set to ListSelectionModel.SINGLE_SELECT.  In
     * the event multiple selection is enabled, this method will return the 
     * item at the lowest selected index.
     *
     * @return The selected item.
     */
    public Object getSelectedValue() {
        int selectedIndex = getMinSelectedIndex();
        Object value;
        
        if (selectedIndex == -1) {
            value = null;
        } else {
            value = model.get(getMinSelectedIndex());
        }

        return value;
    }
    
    /**
     * Returns all selected indices.
     *
     * @return An array of all selected indices.
     */
    public int[] getSelectedIndices() {

        int min = getMinSelectedIndex();
        if (min == -1) {
            // No indices are selected, return empty array.
            return new int[0];
        }
        
        int selectionCount = 0;
        int max = getMaxSelectedIndex();
        int size = model.size();
        if (max >= size - 1) {
            max = size - 1;
        }

        for (int index = min; index <= max; ++index) {
            if (isSelectedIndex(index)) {
                ++selectionCount;
            }
        }

        int[] selectedIndices = new int[selectionCount];
        selectionCount = 0;
        for (int index = min; index <= max; ++index) {
            if (isSelectedIndex(index)) {
                selectedIndices[selectionCount] = index;
                ++selectionCount;
            }
        }
        
        return selectedIndices;
    }
    
    /**
     * Returns all selected items.
     *
     * @return An array of all the selected items.
     */
    public Object[] getSelectedValues() {
        int min = getMinSelectedIndex();

        if (min == -1) {
            // No values are selected, return empty array.
            return new Object[0];
        }
        
        int selectionCount = 0;
        int max = getMaxSelectedIndex();
        int size = model.size();
        if (max >= size - 1) {
            max = size - 1;
        }

        for (int index = min; index <= max; ++index) {
            if (isSelectedIndex(index)) {
                ++selectionCount;
            }
        }

        Object[] selectedValues = new Object[selectionCount];
        selectionCount = 0;
        for (int index = min; index <= max; ++index) {
            if (isSelectedIndex(index)) {
                selectedValues[selectionCount] = model.get(index);
                ++selectionCount;
            }
        }
        
        return selectedValues;
    }
    
    /**
     * Returns the selection mode.  
     * 
     * @return The selection mode, one of the following values:
     *         <ul>
     *         <li>ListSelectionModel.SINGLE_SELECTION: only one list element 
     *         may be selected.</li>
     *         <li>ListSelectionModel.MULTIPLE_SELECTION: multiple list elements
     *         may be selected.</li>
     *         </ul>
     */
    public int getSelectionMode() {
        return selectionModel.getSelectionMode();
    }
   
    /**
     * Returns the selection model.
     *
     * @return The selection model.
     */
    public ListSelectionModel getSelectionModel() {
        return selectionModel;
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#getToolTipText()
     */
    public String getToolTipText() {
        return toolTipText;
    }
    
    /**
     * Returns the number of rows that will be simultaneously displayed.
     *
     * @return The number of rows that will be simultaneously displayed.
     */
    public int getVisibleRowCount() {
        return visibleRowCount;
    }
    
    /**
     * Returns the width of the <code>ListBox</code>.
     * 
     * @return The width of the <code>ListBox</code>.
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Returns the width units of the <code>ListBox</code>.
     * 
     * @return The width units of the <code>ListBox</code>,
     *         one of the following values:
     *         <ul>
     *         <li><code>PIXEL_UNITS</code> (the default)</li>
     *         <li><code>PERCENT_UNITS</code></li>
     *         </ul>
     */
    public int getWidthUnits() {
        return widthUnits;
    }
    
    /**
     * Determines whether an index is selected.
     *
     * @param index The index to test for selection.
     * @return True if the given index is selected.
     */
    public boolean isSelectedIndex(int index) {
        return selectionModel.isSelectedIndex(index);
    }
    
    /**
     * Sets the renderer for items.
     * 
     * @param newValue The new renderer for items.
     */
    public void setCellRenderer(ListCellRenderer newValue) {
        ListCellRenderer oldValue = cellRenderer;
        cellRenderer = newValue;
        firePropertyChange(CELL_RENDERER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the model.
     *
     * @param newValue The new model.
     */
    public void setModel(ListModel newValue) {
        ListModel oldValue = model;
        model = newValue;
        
        oldValue.removeListDataListener(dataListener);
        newValue.addListDataListener(dataListener);
        
        firePropertyChange(MODEL_CHANGED_PROPERTY, null, null);
    }
    
    /**
     * Selects only the given index.
     *
     * @param index The index to select.
     */
    public void setSelectedIndex(int index) {
        selectionModel.clearSelection();
        selectionModel.setSelectedIndex(index, true);    
    }
    
    /**
     * Sets the selection state of the given index.
     *
     * @param index The index to select.
     */
    public void setSelectedIndex(int index, boolean selected) {
        selectionModel.setSelectedIndex(index, selected);    
    }
    
    /**
     * Selects an array of indices.
     *
     * @param indices The indices to be selected.
     */
    public void setSelectedIndices(int[] indices) {
        selectionModel.clearSelection();
        for (int i = 0; i < indices.length; ++i) {
            selectionModel.setSelectedIndex(indices[i], true);
        }
    }
    
    /**
     * Sets the selection mode.  
     * 
     * @param newValue The selection mode, one of the following values:
     *        <ul>
     *        <li>ListSelectionModel.SINGLE_SELECTION: only one list element 
     *        may be selected.</li>
     *        <li>ListSelectionModel.MULTIPLE_SELECTION: multiple list elements
     *        may be selected.</li>
     *        </ul>
     */
    public void setSelectionMode(int newValue) {
        selectionModel.setSelectionMode(newValue);
    }
    
    /**
     * Sets the selection model.
     *
     * @param newValue The new selection model.
     */
    public void setSelectionModel(ListSelectionModel newValue) {
        ListSelectionModel oldValue = selectionModel;
        selectionModel = newValue;
        
        oldValue.removeListSelectionListener(selectionListener);
        newValue.addListSelectionListener(selectionListener);
        
        firePropertyChange(SELECTION_MODEL_CHANGED_PROPERTY, null, null);
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#setToolTipText(String)
     */
    public void setToolTipText(String newValue) {
        String oldValue = toolTipText;
        toolTipText = newValue;
        firePropertyChange(TOOL_TIP_TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the number of rows that will be simultaneously displayed.
     * The default value of this property is 8.
     *
     * @param newValue The number of rows that will be simultaneously displayed.
     */
    public void setVisibleRowCount(int newValue) {
        int oldValue = visibleRowCount;
        visibleRowCount = newValue;
        firePropertyChange(VISIBLE_ROW_COUNT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the width of the <code>ListBox</code>.
     * 
     * @param newValue The new width.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width units of the <code>ListBox</code>.
     * 
     * @param newValue The new width units, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setWidthUnits(int newValue) {
        int oldValue = widthUnits;
        widthUnits = newValue;
        firePropertyChange(WIDTH_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
}
