/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

/**
 * A rectangular container.
 */
public class Panel extends Component {

    /** 
     * A style constant for the Horizontal Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";

    /** 
     * A style constant for the Insets property. 
     * Values of this key must be of type nextapp.echo.Insets.
     */
    public static final String STYLE_INSETS = "insets";

    /**
     * A style constant for the Vertical Alignment property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_ALIGNMENT = "verticalAlignment";

    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String INSETS_CHANGED_PROPERTY = "insets";
    public static final String VERTICAL_ALIGNMENT_CHANGED_PROPERTY = "verticalAlignment";

    /**
     * A default insets object used for panels, that has zero-pixel margins.
     */
    public static final Insets DEFAULT_INSETS = new Insets(0);
    
    private int horizontalAlignment = EchoConstants.LEFT;
    private int verticalAlignment = EchoConstants.TOP;
    private Insets insets = DEFAULT_INSETS;

    /**
     * Creates a new Panel.
     */
    public Panel() {
        super();
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_INSETS)) {
            setInsets((Insets) style.getAttribute(STYLE_INSETS));
        }
        if (style.hasAttribute(STYLE_VERTICAL_ALIGNMENT)) {
            setVerticalAlignment(style.getIntegerAttribute(STYLE_VERTICAL_ALIGNMENT));
        }
    }
    
    /** 
     * Returns the horizontal alignment of the contents of this panel.
     *
     * @return The horizontal alignment of the contents of this panel,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }
    
    /** 
     * Returns the insets (margins) of the panel.
     *
     * @return The insets of the panel.
     */
    public Insets getInsets() {
        return insets;
    }
    
    /** 
     * Returns the vertical alignment of the contents of this panel.
     *
     * @return The vertical alignment of the contents of this panel, 
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalAlignment() {
        return verticalAlignment;
    }
    
    /** 
     * Sets the horizontal alignment of the contents of this panel.
     *
     * @param newValue The horizontal alignment of the contents of this panel, 
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the insets (margins) of the pane.
     *
     * @param newValue An insets object that specifies the size of each margin 
     *        of this pane.
     */
    public void setInsets(Insets newValue) {
        Insets oldValue = insets;
        insets = newValue;
        firePropertyChange(INSETS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the vertical alignment of the contents of this panel.
     *
     * @param newValue The vertical alignment of the contents of this panel, 
     *        one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalAlignment(int newValue) {
        int oldValue = verticalAlignment;
        verticalAlignment = newValue;
        firePropertyChange(VERTICAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
