/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import nextapp.echo.event.ImageUpdateListener;

/**
 * A reference to an image that can be retrieved as a resource.
 */
public class ResourceImageReference 
implements ImageReference, Serializable {

    private static final int BUFFER_SIZE = 4096;
    
    private static final Map extContentType = new HashMap();
    static {
        extContentType.put("gif",  "image/gif");
        extContentType.put("png",  "image/png");
        extContentType.put("jpeg", "image/jpeg");
        extContentType.put("jpg",  "image/jpg");
        extContentType.put("bmp",  "image/bmp");
    }
    
    /**
     * Automatically determines the content type based on the name of a 
     * resource.
     */
    private static final String getContentType(String resourceName) {
        String contentType;
    
        // Determine content type.
        int extensionDelimiterPosition = resourceName.lastIndexOf(".");
        if (extensionDelimiterPosition == -1) {
            throw new IllegalArgumentException("Invalid file extension (resource has no extension: " + resourceName + ")");
        } else {
            String extension = resourceName.substring(extensionDelimiterPosition + 1).toLowerCase();
            contentType = (String) extContentType.get(extension);
            if (contentType == null) {
                throw new IllegalArgumentException("Invalid file extension (no matching content type: " + resourceName + ")");
            }
        }
        
        return contentType;
    }

    private String contentType;
    private String resourceName;
    private int width;
    private int height;
    
    /**
     * Creates a resource image reference retrieved from the provided resource 
     * name.  The content type of the image will be determined based on the 
     * extension of the specified resource name.
     */
    public ResourceImageReference(String resourceName) {
        this(resourceName, getContentType(resourceName), SIZE_UNKNOWN, SIZE_UNKNOWN);
    }

    /**
     * Creates a resource image reference retrieved from the provided resource 
     * name with the specified content type.
     */
    public ResourceImageReference(String resourceName, String contentType) {
        this(resourceName, contentType, SIZE_UNKNOWN, SIZE_UNKNOWN);
    }

    /**
     * Creates a resource image reference retrieved from the provided resource 
     * name with the specified width and height.  The content type of the image
     * will be determined based on the extension of the specified resource name.
     */
    public ResourceImageReference(String resourceName, int width, int height) {
        this(resourceName, getContentType(resourceName), width, height);
    }

    /**
     * Creates a resource image reference retrieved from the provided resource 
     * name with the specified content type, width and height.
     */
    public ResourceImageReference(String resourceName, String contentType, int width, int height) {
        super();
        
        this.resourceName = resourceName;
        this.contentType = contentType;
        this.width = width;
        this.height = height;
    }

    /**
     * Not supported.
     * 
     * @see nextapp.echo.ImageReference#addImageUpdateListener(ImageUpdateListener)
     */
    public void addImageUpdateListener(ImageUpdateListener l) { }
    
    /**
     * @see nextapp.echo.StreamImageReference#getContentType()
     */
    public String getContentType() {
        return contentType;
    }
    
    /**
     * @see nextapp.echo.ImageReference#getHeight()
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * @see nextapp.echo.ImageReference#getWidth()
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Not supported.
     * 
     * @see nextapp.echo.ImageReference#removeImageUpdateListener(ImageUpdateListener)
     */
    public void removeImageUpdateListener(ImageUpdateListener l) { }
    
    /**
     * @see nextapp.echo.StreamImageReference#render(OutputStream)
     */
    public void render(OutputStream out) 
    throws IOException {
        InputStream in = null;
        byte[] buffer = new byte[BUFFER_SIZE];
        int bytesRead = 0;
        
        try {
            in = ResourceImageReference.class.getResourceAsStream(resourceName);
            if (in == null) {
                throw new IllegalArgumentException("Specified resource does not exist: " + resourceName + ".");
            }
            do {
                bytesRead = in.read(buffer);
                if (bytesRead > 0) {
                    out.write(buffer, 0, bytesRead);
                }
            } while (bytesRead > 0);
        } finally {
            if (in != null) { try { in.close(); } catch (IOException ex) { } } 
        }
    }

    /**
     * Not supported.
     * 
     * @see nextapp.echo.ImageReference#update()
     */
    public void update() { }
}
