/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.EventListener;

import nextapp.echo.event.ActionEvent;
import nextapp.echo.event.ActionListener;
import nextapp.echo.event.ItemEvent;
import nextapp.echo.event.ItemListener;
import nextapp.echo.event.ListDataEvent;
import nextapp.echo.event.ListDataListener;

/**
 * A component that is used to select one item from a drop-down list of items.
 */
public class SelectField extends Component 
implements ToolTipSupport {

    /**
     * A constant for units-defining properties that represent pixel-based 
     * units.
     */
    public static final int PIXEL_UNITS = 0;

    /**
     * A constant for units-defining properties that represent percent-based 
     * units.
     */
    public static final int PERCENT_UNITS = 2;
    
    /** 
     * A style constant for the Width property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH = "width";

    /** 
     * A style constant for the Width Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH_UNITS = "widthUnits";

    public static final String CELL_RENDERER_CHANGED_PROPERTY = "cellRenderer";
    public static final String LIST_DATA_CHANGED_PROPERTY = "listData";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_UNITS_CHANGED_PROPERTY = "widthUnits";
    
    /**
     * Local handler of ListDataEvents.
     */
    private class ListDataHandler 
    implements ListDataListener, Serializable {
    
        /**
         * @see nextapp.echo.event.ListDataListener#contentsChanged(ListDataEvent)
         */
        public void contentsChanged(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }

        /**
         * @see nextapp.echo.event.ListDataListener#intervalAdded(ListDataEvent)
         */
        public void intervalAdded(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }

        /**
         * @see nextapp.echo.event.ListDataListener#intervalRemoved(ListDataEvent)
         */
        public void intervalRemoved(ListDataEvent e) {
            firePropertyChange(LIST_DATA_CHANGED_PROPERTY, null, null);
        }
    }
 
    private ListDataListener dataListener = new ListDataHandler();
    private ListCellRenderer cellRenderer = DefaultListCellRenderer.DEFAULT_LIST_CELL_RENDERER;
    private SelectFieldModel model;
    private String actionCommand;
    private String toolTipText;
    private int width = -1;
    private int widthUnits = PIXEL_UNITS;

    /**
     * Creates a new, empty <code>SelectField</code>.
     */
    public SelectField() {
        this(new DefaultSelectFieldModel());
    }
    
    /**
     * Creates a new <code>SelectField</code> that will initially contain the
     * provided array of items.
     *
     * @param items The items the <code>SelectField</code> will initially 
     *        contain.
     */
    public SelectField(Object[] items) {
        this(new DefaultSelectFieldModel(items));
    }

    /**
     * Creates a new <code>SelectField</code> with the provided model.
     *
     * @param model The model for the <code>SelectField</code>.
     */
    public SelectField(SelectFieldModel model) {
        super();
        
        if (model == null) {
            throw new IllegalArgumentException("Null model is not allowed.");
        }
        
        this.model = model;
        if (model.getSelectedItem() == null && model.size() > 0) {
            model.setSelectedItem(model.get(0));
        }
        
        model.addListDataListener(dataListener);
    }
    
    /**
     * Adds an <code>ActionListener</code>.  The <code>ActionListener</code> 
     * will be notified immediately when an item changes state.
     *
     * @param l The <code>ActionListener</code> to be added.
     */
    public void addActionListener(ActionListener l) {
        listenerList.addListener(ActionListener.class, l);
    }
    
    /**
     * Adds an <code>ItemListener</code>.
     *
     * @param l The <code>ItemListener</code> to be added.
     */
    public void addItemListener(ItemListener l) {
        listenerList.addListener(ItemListener.class, l);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(nextapp.echo.Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));   
        }
        if (style.hasAttribute(STYLE_WIDTH_UNITS)) {
            setWidthUnits(style.getIntegerAttribute(STYLE_WIDTH_UNITS));   
        }
    }
    
    /**
     * Notifies all listeners that an item has been selected.
     */
    public void fireActionEvent() {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        ActionEvent e = null;
        for (int index = 0; index < listeners.length; ++index) {
            if (e == null) {
                e = new ActionEvent(this, actionCommand);
            } 
            ((ActionListener) listeners[index]).actionPerformed(e);
        }
    }
    
    /**
     * Notifies all listeners with the specified <code>ItemEvent</code>.
     *
     * @param e The <code>ItemEvent</code> of which to notify the listeners.
     */
    protected void fireItemStateChanged(ItemEvent e) {
        EventListener[] listeners = listenerList.getListeners(ItemListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ItemListener) listeners[index]).itemStateChanged(e);
        }
    }
    
    /**
     * Returns the action command of the <code>SelectField</code>.
     *
     * @return The action command.
     */
    public String getActionCommand() {
        return actionCommand;
    }
    
    /**
     * Returns the renderer for items.
     * 
     * @return The renderer for items.
     */
    public ListCellRenderer getCellRenderer() {
        return cellRenderer;
    }
    
    /**
     * Returns the model of the <code>SelectField</code>.
     *
     * @return The model of the <code>SelectField</code>.
     */
    public SelectFieldModel getModel() {
        return model;
    }
    
    /**
     * Returns the currently selected item.
     *
     * @return The currently selected item.
     */
    public Object getSelectedItem() {
        return model.getSelectedItem();
    }
    
    /**
     * Returns the index of the currently selected item.
     *
     * @return The index of the currently selected item.
     */
    public int getSelectedIndex() {
        Object selectedItem = getSelectedItem();
        if (selectedItem == null) {
            // should not occur.
            return -1;
        }
        int size = model.size();
        for (int index = 0; index < size; ++index) {
            if (model.get(index).equals(selectedItem)) {
                return index;
            }
        }
        // should not occur.
        return -1;
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#getToolTipText()
     */
    public String getToolTipText() {
        return toolTipText;
    }
    
    /**
     * Returns the width of the <code>SelectField</code>.
     * 
     * @return The width of the <code>SelectField</code>.
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Returns the width units of the <code>SelectField</code>.
     * 
     * @return The width units of the <code>SelectField</code>,
     *         one of the following values:
     *         <ul>
     *         <li><code>PIXEL_UNITS</code> (the default)</li>
     *         <li><code>PERCENT_UNITS</code></li>
     *         </ul>
     */
    public int getWidthUnits() {
        return widthUnits;
    }
    
    /**
     * Returns true if the select field has one or more action listeners.
     *
     * @return True if the select field has one or more action listeners.
     */
    public boolean hasActionListeners() {
        return listenerList.getListenerCount(ActionListener.class) > 0;
    }
    
    /**
     * Removes an <code>ActionListener</code>. 
     *
     * @param l The <code>ActionListener</code> to be removed.
     */
    public void removeActionListener(ActionListener l) {
        listenerList.removeListener(ActionListener.class, l);
    }
    
    /**
     * Removes an <code>ItemListener</code>.
     *
     * @param l The <code>ItemListener</code> to be removed.
     */
    public void removeItemListener(ItemListener l) {
        listenerList.removeListener(ItemListener.class, l);
    }
    
    /**
     * Sets the action command of the <code>SelectField</code>.
     *
     * @param actionCommand The new action command.
     */
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }
    
    /**
     * Sets the renderer for items.
     * 
     * @param newValue The new renderer for items.
     */
    public void setCellRenderer(ListCellRenderer newValue) {
        ListCellRenderer oldValue = cellRenderer;
        cellRenderer = newValue;
        firePropertyChange(CELL_RENDERER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the model of the <code>SelectField</code>.
     *
     * @param newValue The new model for the <code>SelectField</code>.
     */
    public void setModel(SelectFieldModel newValue) {
        SelectFieldModel oldValue = model;
        model = newValue;
        
        oldValue.removeListDataListener(dataListener);
        newValue.addListDataListener(dataListener);
        
        firePropertyChange(MODEL_CHANGED_PROPERTY, null, null);
    }
    
    /**
     * Sets the selected index.
     *
     * @param index The new selected index.
     */
    public void setSelectedIndex(int index) {
        if (index >= model.size()) {
            throw new IllegalArgumentException("Specified index " + index + " is out of range.");
        }
        
        Object oldValue = model.getSelectedItem();
        fireItemStateChanged(new ItemEvent(this, oldValue, ItemEvent.DESELECTED));
    
        model.setSelectedItem(model.get(index));
        
        Object newValue = model.get(index);
        fireItemStateChanged(new ItemEvent(this, newValue, ItemEvent.SELECTED));
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#setToolTipText(String)
     */
    public void setToolTipText(String newValue) {
        String oldValue = toolTipText;
        toolTipText = newValue;
        firePropertyChange(TOOL_TIP_TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the <code>SelectField</code>.
     * 
     * @param newValue The new width.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width units of the <code>SelectField</code>.
     * 
     * @param newValue The new width units, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setWidthUnits(int newValue) {
        int oldValue = widthUnits;
        widthUnits = newValue;
        firePropertyChange(WIDTH_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
}
