/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.util.EventListener;

import nextapp.echo.event.ChangeEvent;
import nextapp.echo.event.ChangeListener;

/**
 * A stacked pane resembles a stack of cards in concept.  While panes may be 
 * added and removed from a StackedPane, only the top pane is visible.
 * A StackedPane is useful when an application needs to offer a user interface
 * where users can navigate down and then back up a hierarchy.
 */
public class StackedPane extends ContainerPane {
    
    /**
     * Adds a listener to be notified of change events.  The listener will
     * be notified when the visible pane has changed.
     *
     * @param l The <code>ChangeListener</code> to be added.
     */
    public void addChangeListener(ChangeListener l) {
        listenerList.addListener(ChangeListener.class, l);
    }
    
    /**
     * Closes the top pane.
     *
     * @return The pane that was closed.
     * @throws IllegalStateException If called when no child panes exist in 
     *         the <code>StackedPane</code>.
     */
    public AbstractPane close() {
        AbstractPane oldTopPane = getVisiblePane();
        if (oldTopPane == null) {
            throw new IllegalStateException("StackedPane does not have any children.");
        }
        super.remove(oldTopPane);
        AbstractPane newTopPane = getVisiblePane();
        if (newTopPane != null) {
            newTopPane.setVisible(true);
        }
        fireVisiblePaneChanged();
        return oldTopPane;
    }
    
    /**
     * Closes the specified pane.
     *
     * @param pane The pane to be closed.
     */
    public void close(AbstractPane pane) {
        int index = super.indexOf(pane);
        if (index == -1) {
            throw new IllegalStateException("StackedPane does not contain specified pane.");
        }
        int count = getComponentCount();
        if (index == count - 1) {
            // top pane.
            close();
        } else {
            // obscured pane.
            super.remove(pane);
        }
    }

    /**
     * Closes panes until the specified pane is again visible.
     *
     * @param pane The pane that should be made visible.
     */
    public void closeUntilVisible(AbstractPane pane) {
        int visibleIndex = indexOf(pane);
        if (visibleIndex == -1) {
            throw new IllegalArgumentException("The provided pane is not present in this StackedPane.");
        }
        int count = getComponentCount();
        for (int index = count - 1; index > visibleIndex; --index) {
            super.remove(index);
        }
        getVisiblePane().setVisible(true);
        fireVisiblePaneChanged();
    }
    
    /**
     * Notifies <code>ChangeListener</code>s that the visible pane has changed.
     */
    private void fireVisiblePaneChanged() {
        ChangeEvent e = new ChangeEvent(this);
        EventListener[] listeners = listenerList.getListeners(ChangeListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((ChangeListener) listeners[index]).stateChanged(e);
        }
    }
    
    /**
     * Returns the pane at the specified index.
     *
     * @param index The index of the pane.
     * @return The pane at the specified index.
     */
    public AbstractPane getPane(int index) {
        return (AbstractPane) getComponent(index);
    }
    
    /**
     * Returns the currently visible (top) pane.
     * 
     * @return the currently visible (top) pane.
     */
    public AbstractPane getVisiblePane() {
        int count = getComponentCount();
        if (count == 0) {
            return null;
        } else {
            return getPane(count - 1);
        }
    }
    
    /**
     * Inserts the pane into the stacked pane.  The pane will be made visible 
     * if the specified index is one greater than that the previously currently
     * pane.
     *
     * @param index The index at which to insert the pane.
     * @param pane The pane to be inserted.
     */
    public void insert(int index, AbstractPane pane) {
        verifyAddition(pane);
        int count = getComponentCount();
        if (index == count) {
            open(pane);
        } else if (index > count) {
            throw new IndexOutOfBoundsException(Integer.toString(index));
        } else {
            pane.setVisible(false);
            super.add(pane, index);
        }
    }
    
    /**
     * Opens a new pane.
     *
     * @param pane The pane to open.
     */
    public void open(AbstractPane pane) {
        verifyAddition(pane);
        AbstractPane visiblePane = getVisiblePane();
        if (visiblePane != null) {
            visiblePane.setVisible(false);
        }
        super.add(pane);
        fireVisiblePaneChanged();
    }
    
    /**
     * Replaces the currently visible pane with the provided pane.
     *
     * @param pane The new visible pane.
     */
    public void replaceVisiblePane(AbstractPane pane) {
        verifyAddition(pane);
        close();
        open(pane);
    }
    
    /**
     * Returns the number of panes in this StackedPane.  This number includes 
     * both the obscured panes and the open pane.
     *
     * @return The number of panes in this StackedPane.
     */ 
    public int size() {
        return getComponentCount();
    }
    
    /**
     * Removes a listener from being notified of change events.
     *
     * @param l The <code>ChangeListener</code> to be added.
     */
    public void removeChangeListener(ChangeListener l) {
        listenerList.removeListener(ChangeListener.class, l);
    }

    /**
     * Verifies pane addition operation to ensure panes are not added
     * multiple times.
     * 
     * @param pane The pane to verify for addition.
     */
    private void verifyAddition(AbstractPane pane) {
        if (super.indexOf(pane) != -1) {
            throw new IllegalStateException("Specified pane already present in StackedPane.");
        }
    }
}
