/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * A representation of stylistic properties that may be applied to 
 * <code>Component</code>s.
 */
public class Style 
implements Serializable {

    private boolean immutable = false;
    private Map attributes = new HashMap();

    /**
     * Creates a new style object.
     */
    public Style() {
        super();
    }

    /**
     * Returns the attribute associate with the given key as an object.
     *
     * @param key The name of the attribute to return.
     * @return The value of the attribute.
     */
    public Object getAttribute(String key) {
        return attributes.get(key);
    }
    
    /**
     * Returns the attribute associated with the given key as a primitive
     * boolean value.  The key must map to a <code>Boolean</code> object
     * for this method to execute successfully.
     *
     * @param key The name of the attribute to return.
     * @return The value of the attribute.
     * @throws <code>NullPointerException</code> if the key is not present.
     * @throws <code>ClassCastException</code> if the attribute value is not a
     *         <code>Boolean</code> object.
     */
    public boolean getBooleanAttribute(String key) {
        Object o = attributes.get(key);
        boolean value;
        
        if (o == null) {
            throw new NullPointerException("Value is null.");
        } else if (o instanceof Boolean) {
            value = ((Boolean) o).booleanValue();
        } else {
            throw new ClassCastException("Value is not a Boolean.");
        }
        
        return value;
    }
    
    /**
     * Returns the attribute associated with the given key as a primitive
     * integer value.  The key must map to a <code>Integer</code> object
     * for this method to execute successfully.
     *
     * @param key The name of the attribute to return.
     * @return The value of the attribute.
     * @throws <code>NullPointerException</code> if the key is not present.
     * @throws <code>ClassCastException</code> if the attribute value is not 
     *         an <code>Integer</code> object.
     */
    public int getIntegerAttribute(String key) {
        Object o = attributes.get(key);
        int value;
        
        if (o == null) {
            throw new NullPointerException("Value is null.");
        } else if (o instanceof Integer) {
            value = ((Integer) o).intValue();
        } else {
            throw new ClassCastException("Value is not an Integer.");
        }
        
        return value;
    }
    
    /**
     * Returns the attribute associated with the given key as a string.
     * This method is identical to <code>getAttribute()</code> with the
     * exception that it casts the attribute to a <code>String</code>.
     * If the key maps to an object other than a <code>String</code>,
     * a <code>ClassCastException</code> will be thrown.
     *
     * @param key The name of the attribute to return.
     * @return The value of the attribute.
     * @throws <code>ClassCastException<code> if the attribute value is not a
     *         <code>String</code>.
     */
    public String getStringAttribute(String key) {
        return (String) attributes.get(key);
    }
    
    /**
     * Returns true if the specified key is associated with an attribute.
     *
     * @param key The key to analyze.
     * @return True if the key maps to an attribute.
     */
    public boolean hasAttribute(String key) {
        return attributes.containsKey(key);
    }
    
    /** 
     * Returns true if this object has been made immutable.
     *
     * @return True if this object has been made immutable.
     */
    public boolean isImmutable() {
        return immutable;
    }
    
    /**
     * Sets an attribute.  If the attribute already exists, it will be
     * overwritten.  The attribute will be stored as a <code>Boolean</code> 
     * object.
     *
     * @param key The key to be set.
     * @param value The new value of the key.
     * @throws IllegalStateException if the style is immutable.
     */
    public void setAttribute(String key, boolean value) {
        setAttribute(key, new Boolean(value));
    }
    
    /**
     * Sets an attribute.  If the attribute already exists, it will be
     * overwritten. The attribute will be stored as an <code>Integer</code>
     * object.
     *
     * @param key The key to be set.
     * @param value The new value of the key.
     * @throws IllegalStateException if the style is immutable.
     */
    public void setAttribute(String key, int value) {
        setAttribute(key, new Integer(value));
    }
    
    /**
     * Sets an attribute.  If the attribute already exists, it will be
     * overwritten.
     *
     * @param key The key to be set.
     * @param value The new value of the key.
     * @throws IllegalStateException if the style is immutable.
     */
    public void setAttribute(String key, Object value) {
        if (immutable) {
            throw new IllegalStateException("Attempt to set an attribute on an immutable style.");
        }
    
        attributes.put(key, value);
    }
    
    /**
     * Sets this Style to be immutable.  Any future attempts to set attributes
     * will throw <code>IllegalStateException</code>s.
     */
    public void setImmutable() {
        immutable = true;
    }
}
