/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * Contributed by Phillip Baird
 */

package nextapp.echo;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import nextapp.echo.event.TableColumnModelListener;
import nextapp.echo.event.TableColumnModelEvent;
import nextapp.echo.event.TableModelListener;
import nextapp.echo.event.TableModelEvent;
import nextapp.echo.table.*;
import nextapp.echo.util.DynamicIntegerArray;

/**
 * A component used to display data in a tabular format.
 *
 * @see nextapp.echo.table
 */
public class Table extends Component {

    /**
     * A constant used in the <code>setWidthUnits()</code> and 
     * <code>setHeightUnits()</code> methods.
     * This value specifies that pixel-based units are used
     * in defining the individual column or row sizes and the 
     * overall width and height of the table.
     */
    public static final int PIXEL_UNITS = 1;

    /**
     * A constant used in the <code>setWidthUnits()</code> and 
     * <code>setHeightUnits()</code> methods.
     * This value specifies that percent-based units are used
     * in defining the individual column or row sizes, but pixel based units 
     * are used in defining the overall width or height of the table.
     */
    public static final int PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS = 3;

    /**
     * A constant used in the <code>setWidthUnits()</code> and 
     * <code>setHeightUnits()</code> methods.
     * This value specifies that percent-based are used in defining both the
     * individual column or row sizes and the overall width or height of the 
     * table.
     */
    public static final int PERCENT_UNITS = 2;

    /** 
     * A style constant for the Border Color property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_COLOR = "borderColor";

    /** 
     * A style constant for the Border Size property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_SIZE = "borderSize";

    /** 
     * A style constant for the Cell Margin property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_CELL_MARGIN = "cellMargin";
    
    /** 
     * A style constant for the setting column widths. 
     * Values of this key must be an array of integer primitives.
     */
    public static final String STYLE_COLUMN_WIDTHS = "columnWidths";

    /** 
     * A style constant for the Height property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HEIGHT = "height";

    /** 
     * A style constant for the Height Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HEIGHT_UNITS = "heightUnits";

    /** 
     * A style constant for the setting row heights. 
     * Values of this key must be an array of integer primitives.
     */
    public static final String STYLE_ROW_HEIGHTS = "rowHeights";

    /** 
     * A style constant for the Width property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH = "width";

    /** 
     * A style constant for the Width Units property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_WIDTH_UNITS = "widthUnits";

    /**
     * The default renderer for table cells. 
     */
    public static final TableCellRenderer DEFAULT_TABLE_CELL_RENDERER = new DefaultTableCellRenderer();

    // Property names for property change events.
    public static final String AUTO_CREATE_COLUMNS_FROM_MODEL_CHANGED_PROPERTY = "autoCreateColumnsFromModel";
    public static final String BORDER_COLOR_CHANGED_PROPERTY = "borderColor";
    public static final String BORDER_SIZE_CHANGED_PROPERTY = "borderSize";
    public static final String CELL_MARGIN_CHANGED_PROPERTY = "cellMargin";
    public static final String TABLE_HEADER_CHANGED_PROPERTY = "tableHeader";
    public static final String TABLE_COLUMN_MODEL_CHANGED_PROPERTY = "tableColumnModel";
    public static final String ROW_HEIGHT_CHANGED_PROPERTY = "rowHeight";
    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String HEIGHT_UNITS_CHANGED_PROPERTY = "heightUnits";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String WIDTH_UNITS_CHANGED_PROPERTY = "widthUnits";
    
    private boolean autoCreateColumnsFromModel = false;
    private Color borderColor = Color.BLACK;
    private int borderSize = 1;
    private int cellMargin = 3;
    private TableHeader tableHeader = null;
    private Map componentPositionMap = new HashMap();
    private Map defaultRenderers = new HashMap(3);
    private TableModel model;
    private TableModelListener modelListener = new TableModelHandler();
    private TableColumnModel columnModel = new DefaultTableColumnModel();
    private TableColumnModelListener columnModelListener = new TableColumnModelHandler();
    private DynamicIntegerArray rowHeights = new DynamicIntegerArray(EchoConstants.UNDEFINED_SIZE);
    private int width = EchoConstants.UNDEFINED_SIZE;
    private int widthUnits = PIXEL_UNITS;
    private int height = EchoConstants.UNDEFINED_SIZE;
    private int heightUnits = PIXEL_UNITS;
    private boolean valid;
    private Container renderedCells;
    private Container headerContainer;

    /**
     * A local handler for TableModelEvents.
     */
    private class TableModelHandler 
    implements Serializable, TableModelListener {

        /**
         * @see nextapp.echo.event.TableModelListener#tableChanged(TableModelEvent)
         */
        public void tableChanged(TableModelEvent e) {
            invalidate();

            if (e == null || e.getType() == TableModelEvent.STRUCTURE_CHANGED) {
                updateColumnModel();
            }
        }
    }
    
    /**
     * A local handler for TableColumnModelEvents.
     */
    private class TableColumnModelHandler 
    implements Serializable, TableColumnModelListener {
    
        /**
         * @see nextapp.echo.event.TableColumnModelListener#columnAdded(TableColumnModelEvent)
         */
        public void columnAdded(TableColumnModelEvent e) {
            invalidate();
        }

        /**
         * @see nextapp.echo.event.TableColumnModelListener#columnMoved(TableColumnModelEvent)
         */
        public void columnMoved(TableColumnModelEvent e) {
            invalidate();
        }

        /**
         * @see nextapp.echo.event.TableColumnModelListener#columnRemoved(TableColumnModelEvent)
         */
        public void columnRemoved(TableColumnModelEvent e) {
            invalidate();
        }
    }
    
    /**
     * Creates a new table, generating an empty default table model.
     */
    public Table() {
        this(new DefaultTableModel());
    }
    
    /**
     * Creates a new table, generating an empty default table model of the 
     * given dimensions.
     *
     * @param columns The initial number of columns in the table.
     * @param rows The initial number of rows in the table.
     */
    public Table(int columns, int rows) {
        this(new DefaultTableModel(columns, rows));
    }
    
    /** 
     * Creates a Table using the supplied TableModel.
     *
     * @param model A TableModel containing the table's data.
     */
    public Table(TableModel model) {
        this(model, null);
    }
    
    /** 
     * Creates a Table using the supplied TableModel.
     *
     * @param model A TableModel containing the table's data.
     */
    public Table(TableModel model, TableColumnModel columnModel) {
        super();
        
        renderedCells = new Container();
        add(renderedCells);
        headerContainer = new Container();
        add(headerContainer);
                
        if (columnModel == null) {
            setColumnModel(new DefaultTableColumnModel());
            setAutoCreateColumnsFromModel(true);
        } else {
            setColumnModel(columnModel);
        }
        setModel(model);
        
        setTableHeader(new TableHeader());
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_BORDER_COLOR)) {
            setBorderColor((Color) style.getAttribute(STYLE_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_BORDER_SIZE)) {
            setBorderSize(style.getIntegerAttribute(STYLE_BORDER_SIZE));
        }
        if (style.hasAttribute(STYLE_CELL_MARGIN)) {
            setCellMargin(style.getIntegerAttribute(STYLE_CELL_MARGIN));
        }
        if (style.hasAttribute(STYLE_COLUMN_WIDTHS)) {
            int[] widths = (int[]) style.getAttribute(STYLE_COLUMN_WIDTHS);
            for (int index = 0; index < widths.length; ++index) {
                getColumnModel().getColumn(index).setWidth(widths[index]);
            }
        }
        if (style.hasAttribute(STYLE_HEIGHT)) {
            setHeight(style.getIntegerAttribute(STYLE_HEIGHT));
        }
        if (style.hasAttribute(STYLE_HEIGHT_UNITS)) {
            setHeightUnits(style.getIntegerAttribute(STYLE_HEIGHT_UNITS));
        }
        if (style.hasAttribute(STYLE_ROW_HEIGHTS)) {
            int[] heights = (int[]) style.getAttribute(STYLE_ROW_HEIGHTS);
            for (int index = 0; index < heights.length; ++index) {
                setRowHeight(index, heights[index]);
            }
        }
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));
        }
        if (style.hasAttribute(STYLE_WIDTH_UNITS)) {
            setWidthUnits(style.getIntegerAttribute(STYLE_WIDTH_UNITS));
        }
    }
    
    /**
     * Creates a <code>TableColumnModel</code> based on the present
     * TableModel.  This method is invoked automatically when the table
     * model's structure changes if the <code>AutoCreateColumnsFromModel</code>
     * flag is set.
     */
    public void createDefaultColumnsFromModel() {
        if (model != null) {
            while (columnModel.getColumnCount() > 0) {
                columnModel.removeColumn(columnModel.getColumn(0));
            }
            
            int columnCount = model.getColumnCount();
            for (int index = 0; index < columnCount; ++index) {
                columnModel.addColumn(new TableColumn(index));
            }
        }
    }

    /**
     * Lays out rows and columns.
     */
    private void doLayout() {
        valid = true;
        
        renderedCells.removeAll();
        componentPositionMap.clear();
        
        doLayoutDataRows();
    }
    
    /**
     * Retrieves cell components from the model and adds them to
     * the Table Component.
     */
    private void doLayoutDataRows() {
        int columns = getColumnCount();
        int rows = getRowCount();
        TableCellRenderer renderer;
                
        for (int row = 0; row < rows; row++) {
            for (int column = 0; column < columns; column++) {
                TableColumn tableColumn = columnModel.getColumn(column);
                Object value = model.getValueAt(tableColumn.getModelIndex(), row);
                renderer = null;
                if (tableColumn != null) {
                    renderer = tableColumn.getCellRenderer();
                }
                if (renderer == null) {
                    renderer = getDefaultRenderer(model.getColumnClass(tableColumn.getModelIndex()));
                }

                Component cell = renderer.getTableCellRendererComponent(this, value, tableColumn.getModelIndex(), row);
                if (cell != null) {             
                    renderedCells.add(cell);
                    componentPositionMap.put(new Coordinate(column, row), cell);
                }
            }
        }
    }

    /**
     * Returns the border color.
     * 
     * @return The border color.
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
    /**
     * Returns the border size.     
     *
     * @return The border size in pixels.
     */
    public int getBorderSize() {
        return borderSize;
    }
    
    /**
     * Returns the cell margin.  The default cell margin is 3 pixels.
     *
     * @return The cell margin in pixels.
     */
    public int getCellMargin() {
        return cellMargin;
    }
    
    /**
     * Returns the number of columns in the table.
     *
     * @return The number of columns in the table.
     */
    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    /** 
     * Returns the table's column model.
     *
     * @return the table's column model.
     */
    public TableColumnModel getColumnModel() {
        return columnModel;
    }
    
    /**
     * Returns the contents of the table cell at the specified coordinate.
     * This method should be used only by the rendering peer.
     * This method will not return correct information prior to balidation.
     *
     * @deprecated See getRenderedComponent(Coordinate).
     */
    public Component getComponent(Coordinate coordinate) {
        return getRenderedComponent(coordinate);
    }
    
    /**
     * Returns the contents of the table cell at the specified coordinate.
     * This method should be used only by the rendering peer.
     * This method will not return correct information prior to validation.
     *
     * @param coordinate The coordinate of the rendered table cell component 
     *        to return.
     * @return The component located at the given coordinate.
     */
    public Component getRenderedComponent(Coordinate coordinate) {
        if (coordinate.getY() == -1) {
            if (tableHeader != null) {
                return tableHeader.getRenderedComponent(coordinate.getX());
            } else {
                return null;
            }
        } else {
            return (Component) componentPositionMap.get(coordinate);
        }
    }
    
    /**
     * Returns the TableCellRenderer that is used for the specified column.
     *
     * @param columnClass The class for which to return the default 
     *        TableCellRenderer.
     * @return The TableCellRenderer that will be used to render cells whose
     *         values are of the type specified in columnClass.
     */
    public TableCellRenderer getDefaultRenderer(Class columnClass) {
    
        TableCellRenderer renderer = null;
        
        while (renderer == null && columnClass != null) {
            renderer = (TableCellRenderer) defaultRenderers.get(columnClass);
            if (renderer == null) {
                columnClass = columnClass.getSuperclass();
            }
        }
        
        if (renderer == null) {
            renderer = DEFAULT_TABLE_CELL_RENDERER;
        }
        
        return renderer;
    }
        
    /**
     * Returns the overall height of the table.  This property is not relevant
     * when row heights are being measured in pixels and all rows have a 
     * height value, or if the sum of all row heights (and margins) is 
     * greater than the height value.
     *
     * @return The height of the table.  If the height is not defined, 
     *         <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * Returns the units (pixel or percent) in which column heights and overall 
     * table height are measured.
     *
     * @return The unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public int getHeightUnits() {
        return heightUnits;
    }
    
    /**
     * Returns the table model.
     *
     * @return The table model.
     */
    public TableModel getModel() {
        return model;
    }
    
    /**
     * Returns the number of rows in the table.
     *
     * @return The number of rows in the table.
     */
    public int getRowCount() {
        return model.getRowCount();
    }
    
    /**
     * Returns the height of a table row.  If the height is not 
     * defined, <code>EchoConstants.UNDEFINED_SIZE</code> is returned.
     * The units for this value may be pixels or percentiles, based on the
     * value of the &quot;HeightUnits&quot; property.
     *
     * @param row The row index.
     * @return The height of the table row.
     */
    public int getRowHeight(int row) {
        if (row == -1) {
            return tableHeader.getHeight();
        } else {
            return rowHeights.get(row);
        }
    }
    
    /**
     * Returns the TableHeader object which is responsible for rendering the
     * table's header.
     *
     * @return The TableHeader used for rendering the header.
     */
    public TableHeader getTableHeader() {
        return tableHeader;
    }
    
    /**
     * Returns the overall width of the table.  This property is not relevant
     * when column widths are being measured in pixels and all columns have a 
     * width value, or if the sum of all columns widths (and margins) is 
     * greater than the width value.
     *
     * @return The width of the table.  If the width is not defined, 
     *         <code>UNKNOWN_SIZE</code> is returned.
     */
    public int getWidth() {
         return width;
    }

    /**
     * Returns the units (pixel or percent) in which column widths and overall 
     * table width are measured.
     *
     * @return The unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public int getWidthUnits() {
        return widthUnits;
    }
    
    /**
     * Marks the table as needing to be re-rendered.
     */
    public void invalidate() {
        valid = false;
    }
    
    /**
     * Returns true if the <code>TableColumnModel</code> will be directly
     * derived from the <code>TableModel</code>.  If this flag is set,
     * changes to the <code>TableModel</code> will cause the
     * <code>TableColumnModel</code> to be recreated.  This flag is
     * automatically set to true when the <code>Table</code> is created if a 
     * <code>TableColumnModel</code> is no specified in the constructor.
     *
     * @return True if the <code>TableColumnModel</code> will be directly
     *         derived from the <code>TableModel</code>.
     */
    public boolean isAutoCreateColumnsFromModel() {
        return autoCreateColumnsFromModel;
    }
        
    /**
     * Sets whether the <code>TableColumnModel</code> will be directly
     * derived from the <code>TableModel</code>.  If this flag is set,
     * changes to the <code>TableModel</code> will cause the
     * <code>TableColumnModel</code> to be recreated.  This flag is
     * automatically set to true when the <code>Table</code> is created if a 
     * <code>TableColumnModel</code> is no specified in the constructor.
     *
     * @param newValue True if the <code>TableColumnModel</code> should be 
     *        directly derived from the <code>TableModel</code>.
     */
    public void setAutoCreateColumnsFromModel(boolean newValue) {
        boolean oldValue = autoCreateColumnsFromModel;
        autoCreateColumnsFromModel = newValue;
        
        if (!oldValue && newValue) {
            createDefaultColumnsFromModel();
        }
        
        firePropertyChange(AUTO_CREATE_COLUMNS_FROM_MODEL_CHANGED_PROPERTY, oldValue, newValue);
    }

    /** 
     * Sets the border color of the table.
     *
     * @param newValue The border color of the table.
     */
    public void setBorderColor(Color newValue) {
        Color oldValue = borderColor;
        borderColor = newValue;
        firePropertyChange(BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the size of the table's border.
     *
     * @param newValue The size of the table's border, in pixels.
     */
    public void setBorderSize(int newValue) {
        int oldValue = borderSize;
        borderSize = newValue;
        firePropertyChange(BORDER_SIZE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the cell interior margin.  The default size is 3 pixels
     *
     * @param newValue The cell margin, in pixels.
     */
    public void setCellMargin(int newValue) {
        int oldValue = cellMargin;
        cellMargin = newValue;
        firePropertyChange(CELL_MARGIN_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the column model for this table.
     *
     * @param newValue The new column model for this table.
     */
    public void setColumnModel(TableColumnModel newValue) {
        invalidate();
    
        if (columnModel == null) {
            throw new IllegalArgumentException("A non-null column model is required.");
        }
        
        TableColumnModel oldValue = columnModel;
        if (oldValue != null) {
            // column model is initially null.
            oldValue.removeColumnModelListener(columnModelListener);
        }
        columnModel = newValue;
        newValue.addColumnModelListener(columnModelListener);
        
        firePropertyChange(TABLE_COLUMN_MODEL_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the default renderer that will be used for the given column class.
     * If the provided renderer is null, removes the default renderer setting 
     * for the given column class.
     *
     * @param columnClass The class whose default renderer is to be updated.
     * @param defaultRenderer The new default renderer for the specified column
     *        class.
     */
    public void setDefaultRenderer(Class columnClass, TableCellRenderer defaultRenderer) {
        invalidate();

        if (defaultRenderer == null) {
            defaultRenderers.remove(columnClass);
        } else {
            defaultRenderers.put(columnClass, defaultRenderer);
        }
    }

    /**
     * Sets the overall height of the table.  Setting this property will have no
     * effect in the event that all rows have a height value, or if the sum
     * of all row heights (and margins) is greater than the height value.
     *
     * @param newValue The new height of the table.   To set the height to be
     *        undefined, use the <code>EchoConstants.UNDEFINED_SIZE</code> 
     *        constant.
     */
    public void setHeight(int newValue) {
        int oldValue = height;
        height = newValue;
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units (pixel or percent) in which row heights and overall table 
     * height are measured.
     *
     * @param newValue The new unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setHeightUnits(int newValue) {
        int oldValue = heightUnits;
        heightUnits = newValue;
        firePropertyChange(HEIGHT_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the TableModel the Table is visualizing.
     *
     * @param newValue The new TableModel.
     */
    public void setModel(TableModel newValue) {
        invalidate();

        if (newValue == null) {
            throw new IllegalArgumentException("A non-null model is required.");
        }
    
        TableModel oldValue = model;
        if (oldValue != null) {
            // model is initially null.
            oldValue.removeTableModelListener(modelListener);
        }
        newValue.addTableModelListener(modelListener);
        model = newValue;
        
        updateColumnModel();
        firePropertyChange(MODEL_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the height of a row of the table.
     *
     * @param row The row number.
     * @param newValue The new height of the specified row.
     */
    public void setRowHeight(int row, int newValue) {
        if (row == -1) {
            tableHeader.setHeight(newValue);
        } else {
            rowHeights.set(row, newValue);
            firePropertyChange(ROW_HEIGHT_CHANGED_PROPERTY, null, null);
        }
    }
    
    /**
     * Sets the TableHeader the table uses to render the table's header.
     *
     * @param newValue The new TableHeader.
     */
    public void setTableHeader(TableHeader newValue) {
        invalidate();
        
        TableHeader oldValue = tableHeader;
        tableHeader = newValue;
        
        if (oldValue != null) {
            if (oldValue.getTable() == this) {
                oldValue.setTable(null);
            }
        }
        if (newValue != null) {
            if (newValue.getTable() != this) {
                newValue.setTable(this);
            }
        }

        headerContainer.removeAll();
        headerContainer.add(tableHeader);
        
        firePropertyChange(TABLE_HEADER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the overall width of the table.  Setting this property will have no
     * effect in the event that all columns have a width value, or if the sum
     * of all columns widths (and margins) is greater than the width value.
     *
     * @param newValue The new width of the table.  To set the width to be
     *        undefined, use the <code>EchoConstants.UNDEFINED_SIZE</code> 
     *        constant.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units (pixel or percent) in which column widths and overall 
     * table width are measured.
     *
     * @param newValue The new unit measurement, one of the following values:
     *        <ul>
     *        <li><code>PIXEL_UNITS</code> (the default)</li>
     *        <li><code>PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS</code></li>
     *        <li><code>PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setWidthUnits(int newValue) {
        int oldValue = widthUnits;
        widthUnits = newValue;
        firePropertyChange(WIDTH_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Updates the associated column model in response to the model having 
     * changed or having been replaced.  This method will invoke 
     * createDefaultColumnsFromModel() if necessary.
     */
    private void updateColumnModel() {
        if (isAutoCreateColumnsFromModel()) {
            createDefaultColumnsFromModel();
        }
    }
    
    /**
     * Updates the <code>Table</code> component to reflect its model.
     * 
     * @see nextapp.echo.Component#validate()
     */
    public void validate() {
        while (!valid) {
            doLayout();
        }
    }
}
