/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import nextapp.echo.text.Document;
import nextapp.echo.text.StringDocument;
import nextapp.echo.text.TextComponent;

/**
 * A multiple line text-entry field.
 */
public class TextArea extends TextComponent {

    private static final int DEFAULT_COLUMNS = 20;
    private static final int DEFAULT_ROWS = 4;
    
    public static final String COLUMNS_CHANGED_PROPERTY = "columns";
    public static final String COLUMN_UNITS_CHANGED_PROPERTY = "columnUnits";
    public static final String LINE_WRAP_CHANGED_PROPERTY = "lineWrap";
    public static final String ROWS_CHANGED_PROPERTY = "rows";
    public static final String ROW_UNITS_CHANGED_PROPERTY = "rowUnits";
        
    private int columns = -1;
    private int rows = -1;
    private int columnUnits = CHARACTER_UNITS;
    private int rowUnits = CHARACTER_UNITS;
    private boolean lineWrap = true;
    
    /**
     * Creates a new TextArea.
     */ 
    public TextArea() {
        this(null, DEFAULT_COLUMNS, DEFAULT_ROWS);
    }
    
    /**
     * Creates a new TextArea with the given text.
     *
     * @param text The initial text in the text field.
     */ 
    public TextArea(String text) {
        this(text, DEFAULT_COLUMNS, DEFAULT_ROWS);
    }
    
    /**
     * Creates a new TextArea of the given size.
     *
     * @param columns The number of columns to display.
     * @param rows The number of rows to display.
     */ 
    public TextArea(int columns, int rows) {
        this(null, columns, rows);
    }

    /**
     * Creates a new TextArea with the given text and size.
     *
     * @param text The initial text in the text field.
     * @param columns The number of columns to display.
     * @param rows The number of rows to display.
     */ 
    public TextArea(String text, int columns, int rows) {
        this(new StringDocument(), text, columns, rows);        
    }
    
    /**
     * Creates a new TextArea with the given text and size.
     *
     * @param document The model for this text field.
     */ 
    public TextArea(Document document) {
        this(document, null, DEFAULT_COLUMNS, DEFAULT_ROWS);
    }
    
    /**
     * Creates a new TextArea with the given text and size.
     *
     * @param document The model for this text field.
     * @param text The initial text in the text field.
     * @param columns The number of columns to display.
     * @param rows The number of rows to display.
     */ 
    public TextArea(Document document, String text, int columns, int rows) {
        super();
        
        setDocument(document);
        if (text != null) {
            setText(text);
        }
        setColumns(columns);
        setRows(rows);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_COLUMNS)) {
            setColumns(style.getIntegerAttribute(STYLE_COLUMNS));
        }
        if (style.hasAttribute(STYLE_ROWS)) {
            setRows(style.getIntegerAttribute(STYLE_ROWS));
        }
        if (style.hasAttribute(STYLE_LINE_WRAP)) {
            setLineWrap(style.getBooleanAttribute(STYLE_LINE_WRAP));
        }
    }
    
    /**
     * Returns the number of columns in the TextArea.
     *
     * @return The number of columns to display.
     */
    public int getColumns() {
        return columns;
    }
    
    /**
     * Returns the units of the columns property.
     * 
     * @return The units of the columns property, one of the
     *         following values:
     *         <ul>
     *         <li><code>TextArea.CHARACTER_UNITS</code> (the default)</li>
     *         <li><code>TextArea.PIXEL_UNITS</code></li>
     *         <li><code>TextArea.PERCENT_UNITS</code></li>
     *         </ul>
     */
    public int getColumnUnits() {
        return columnUnits;
    }
    
    /**
     * Returns whether lines should wrap in the text area.
     *
     * @return True if lines should wrap in the text area.
     */
    public boolean getLineWrap() {
        return lineWrap;
    }
    
    /**
     * Returns the number of rows in the TextArea.
     *
     * @return The number of rows to display.
     */
    public int getRows() {
        return rows;
    }

    /**
     * Returns the units of the rows property.
     * 
     * @return The units of the rows property, one of the
     *         following values:
     *         <ul>
     *         <li><code>TextArea.CHARACTER_UNITS</code> (the default)</li>
     *         <li><code>TextArea.PIXEL_UNITS</code></li>
     *         <li><code>TextArea.PERCENT_UNITS</code></li>
     *         </ul>
     */
    public int getRowUnits() {
        return rowUnits;
    }
    
    /**
     * Sets the number of columns in the TextArea.
     *
     * @param newValue The number of columns to display.
     */ 
    public void setColumns(int newValue) {
        int oldValue = columns;
        columns = newValue;
        firePropertyChange(COLUMNS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the units for the columns property.
     * 
     * @param newValue The new units for the columns property, one of the
     *        following values:
     *        <ul>
     *        <li><code>TextArea.CHARACTER_UNITS</code> (the default)</li>
     *        <li><code>TextArea.PIXEL_UNITS</code></li>
     *        <li><code>TextArea.PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setColumnUnits(int newValue) {
        int oldValue = columnUnits;
        columnUnits = newValue;
        firePropertyChange(COLUMN_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets whether lines should wrap in the text area.
     *
     * @param newValue True if lines should wrap in the text area.
     */
    public void setLineWrap(boolean newValue) {
        boolean oldValue = lineWrap;
        lineWrap = newValue;
        firePropertyChange(LINE_WRAP_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the number of rows in the TextArea.
     *
     * @param newValue The number of rows to display.
     */
    public void setRows(int newValue) {
        int oldValue = rows;
        rows = newValue;
        firePropertyChange(ROWS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the units for the rows property.
     * 
     * @param newValue The new units for the rows property, one of the
     *        following values:
     *        <ul>
     *        <li><code>TextArea.CHARACTER_UNITS</code> (the default)</li>
     *        <li><code>TextArea.PIXEL_UNITS</code></li>
     *        <li><code>TextArea.PERCENT_UNITS</code></li>
     *        </ul>
     */
    public void setRowUnits(int newValue) {
        int oldValue = rowUnits;
        rowUnits = newValue;
        firePropertyChange(ROW_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
}
