/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.util.EventListener;

import nextapp.echo.event.ActionEvent;
import nextapp.echo.event.ActionListener;
import nextapp.echo.text.Document;
import nextapp.echo.text.StringDocument;
import nextapp.echo.text.TextComponent;

/** 
 * A single line text-entry field.
 */
public class TextField extends TextComponent {

    /**
     * A style constant for the Maximum Length property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_MAXIMUM_LENGTH = "maximumLength";

    private static final int DEFAULT_COLUMNS = 20;
    
    public static final String ACTION_COMMAND_CHANGED_PROPERTY = "actionCommand";
    public static final String COLUMNS_CHANGED_PROPERTY = "columns";
    public static final String COLUMN_UNITS_CHANGED_PROPERTY = "columnUnits";
    public static final String MAXIMUM_LENGTH_CHANGED_PROPERTY = "maximumLength";
    
    private String actionCommand = null;
    private int columns = -1;
    private int columnUnits = TextComponent.CHARACTER_UNITS;
    private int maximumLength = -1;
    
    /**
     * Creates a new text field.
     */
    public TextField() {
        this(null, DEFAULT_COLUMNS);
    }
    
    /**
     * Creates a new text field with the given text.
     *
     * @param text The initial text in the text field.
     */
    public TextField(String text) {
        this(text, DEFAULT_COLUMNS);
    }
    
    /**
     * Creates a new text field with the given number of columns.
     *
     * @param columns The number of columns that will be displayed in the text
     *        field.
     */
    public TextField(int columns) {
        this(null, columns);
    }
    
    /**
     * Creates a new text field with the given text and number of columns.
     *
     * @param text The initial text in the text field.
     * @param columns The number of columns that will be displayed in the text
     *        field.
     */
    public TextField(String text, int columns) {
        this(new StringDocument(), text, columns);
    }
    
    /**
     * Creates a new text field with the given document model, text and number 
     * of columns.
     *
     * @param document The model for this text field.
     */
    public TextField(Document document) {
        this(document, null, DEFAULT_COLUMNS);
    }

    /**
     * Creates a new text field with the given document model, text and number 
     * of columns.
     *
     * @param document The model for this text field.
     * @param text The initial text in the text field.
     * @param columns The number of columns that will be displayed in the text
     *        field.
     */
    public TextField(Document document, String text, int columns) { 
        super();
        
        setDocument(document);
        setColumns(columns);
        if (text != null) {
            setText(text);
        }
    }
    
    /**
     * Adds an <code>ActionListener</code> to the text field.
     *
     * @param l The <code>ActionListener</code> to be added.
     */
    public void addActionListener(ActionListener l) {
        listenerList.addListener(ActionListener.class, l);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_COLUMNS)) {
            setColumns(style.getIntegerAttribute(STYLE_COLUMNS));
        }
        if (style.hasAttribute(STYLE_MAXIMUM_LENGTH)) {
            setMaximumLength(style.getIntegerAttribute(STYLE_MAXIMUM_LENGTH));
        }
    }
    
    /**
     * Fires an action event to all listeners.
     */
    public void fireActionEvent() {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        ActionEvent e = null;
        for (int index = 0; index < listeners.length; ++index) {
            if (e == null) {
                e = new ActionEvent(this, actionCommand);
            } 
            ((ActionListener) listeners[index]).actionPerformed(e);
        }
    }
    
    /**
     * Returns the action command for this textField.
     *
     * @return The action command for this textField.
     */
    public String getActionCommand() { 
        return actionCommand;
    }
    
    /**
     * Returns the columns of the text field.
     *
     * @return The number of columns that will be displayed in the text field.
     */ 
    public int getColumns() {
        return columns;
    }

    /**
     * Returns the units of the columns property.
     * 
     * @return The units of the columns property, one of the
     *        following values:
     *        <ul>
     *        <li><code>TextArea.UNITS_CHARACTER</code> (the default)</li>
     *        <li><code>TextArea.UNITS_PIXEL</code></li>
     *        <li><code>TextArea.UNITS_PERCENT</code></li>
     *        </ul>
     */
    public int getColumnUnits() {
        return columnUnits;
    }
    
    /**
     * Returns the maximum number of characters that may be entered into the
     * text field.  A value of -1 imposes no limitation on maximum length.
     * This value imposes no limitations on the document.
     *
     * @return The maximum number of characters that may be entered into the
     *         text field.
     */
    public int getMaximumLength() {
        return maximumLength;
    }
    
    /**
     * Returns true if the text field has one or more action listeners.
     *
     * @return True if the text field has one or more action listeners.
     */
    public boolean hasActionListeners() {
        return listenerList.getListenerCount(ActionListener.class) > 0;
    }
    
    /**
     * Removes an <code>ActionListener</code> from the text field.
     *
     * @param l The <code>ActionListener</code> to be removed.
     */
    public void removeActionListener(ActionListener l) {
        listenerList.removeListener(ActionListener.class, l);
    }

    /**
     * Sets the text field's action command.
     *
     * @param newValue The new action command for this text field.
     */
    public void setActionCommand(String newValue) {
        String oldValue = actionCommand;
        actionCommand = newValue;
        firePropertyChange(ACTION_COMMAND_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the columns of the text field.
     *
     * @param newValue The number of columns that will be displayed in the 
     *        text field.
     */
    public void setColumns(int newValue) {
        int oldValue = columns;
        columns = newValue;
        firePropertyChange(COLUMNS_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the units for the columns property.
     * 
     * @param newValue The new units for the columns property, one of the
     *        following values:
     *        <ul>
     *        <li><code>TextArea.UNITS_CHARACTER</code> (the default)</li>
     *        <li><code>TextArea.UNITS_PIXEL</code></li>
     *        <li><code>TextArea.UNITS_PERCENT</code></li>
     *        </ul>
     */
    public void setColumnUnits(int newValue) {
        int oldValue = columnUnits;
        columnUnits = newValue;
        firePropertyChange(COLUMN_UNITS_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the maximum number of characters that may be entered into the
     * text field.  A value of -1 imposes no limitation on maximum length.
     * This value will not impose any limitations on the document.
     *
     * @param newValue The maximum length.
     */
    public void setMaximumLength(int newValue) {
        int oldValue = maximumLength;
        maximumLength = newValue;
        firePropertyChange(MAXIMUM_LENGTH_CHANGED_PROPERTY, oldValue, newValue);
    }
}
