/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import nextapp.echo.event.ItemEvent;

/**
 * A base class for stateful buttons, such as radio buttons and checkboxes.
 */
public abstract class ToggleButton extends AbstractButton {
    
    /** 
     * A style constant for the Horizontal State Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_STATE_ALIGNMENT = "horizontalStateAlignment";

    /** 
     * A style constant for the Horizontal State Position property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_STATE_POSITION = "horizontalStatePosition";

    /** 
     * A style constant for the State Margin property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_STATE_MARGIN = "stateMargin";

    /** 
     * A style constant for the Vertical State Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_STATE_ALIGNMENT = "verticalStateAlignment";

    /** 
     * A style constant for the Vertical State Position property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_VERTICAL_STATE_POSITION = "verticalStatePosition";

    public static final String HORIZONTAL_STATE_ALIGNMENT_CHANGED_PROPERTY = "horizontalStateAlignment";
    public static final String HORIZONTAL_STATE_POSITION_CHANGED_PROPERTY = "horizontalStatePosition";
    public static final String STATE_MARGIN_CHANGED_PROPERTY = "stateMargin";
    public static final String VERTICAL_STATE_ALIGNMENT_CHANGED_PROPERTY = "verticalStateAlignment";
    public static final String VERTICAL_STATE_POSITION_CHANGED_PROPERTY = "verticalStatePosition";
    
    /**
     * A <code>ButtonModel</code> for buttons that may be selected.
     */
    public static class ToggleButtonModel extends DefaultButtonModel {

        private boolean selected = false;
        
        /**
         * Creates a new toggle button model.
         */
        public ToggleButtonModel() {
            super();
        }
        
        /**
         * Returns the button's selection state.
         *
         * @return The toggle button's selection state.
         */
        public boolean isSelected() {
            return selected;
        }

        /**
         * Sets the toggle button's selection state.
         *
         * @param selected True if the button should be selected.
         */
        public void setSelected(boolean selected) {
            this.selected = selected;

            if (getGroup() != null) {
                getGroup().setSelected(this, selected);
            }
            
            this.fireItemStateChanged(new ItemEvent(this, this, selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED));
            this.fireStateChanged();
        }
    }
    
    private int horizontalStateAlignment = EchoConstants.CENTER;
    private int horizontalStatePosition = EchoConstants.LEFT;
    private int verticalStateAlignment = EchoConstants.CENTER;
    private int verticalStatePosition = EchoConstants.CENTER;
    private int stateMargin = 3;

    /**
     * Creates an initially unselected toggle button with no text or image.
     */
    public ToggleButton() {
        this(null, null, false);
    }
    
    /**
     * Creates an initially unselected toggle button with the specified text.
     *
     * @param text The text to be displayed in the button.
     */
    public ToggleButton(String text) {
        this(text, null, false);
    }
    
    /**
     * Creates a toggle button with the specified text and selection state.
     *
     * @param text The text to be displayed in the button.
     * @param selected The selection state of the toggle button.
     */
    public ToggleButton(String text, boolean selected) {
        this(text, null, selected);
    }
    
    /**
     * Creates a initially unselected toggle button with the specified icon.
     *
     * @param icon The icon to be displayed in the button.
     */
    public ToggleButton(ImageReference icon) {
        this(null, icon, false);
    }

    /**
     * Creates a toggle button with the specified icon and selection state.
     *
     * @param icon The icon to be displayed in the button.
     * @param selected The selection state of the toggle button.
     */
    public ToggleButton(ImageReference icon, boolean selected) {
        this(null, icon, selected);
    }

    /**
     * Creates a toggle button with the specified text, icon and selection state.
     *
     * @param text The text to be displayed in the button.
     * @param icon The icon to be displayed in the button.
     * @param selected The selection state of the toggle button.
     */
    public ToggleButton(String text, ImageReference icon, boolean selected) {
        super();
        
        setModel(new ToggleButtonModel());
    
        setIcon(icon);
        setText(text);
        setSelected(selected);
    }

    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_HORIZONTAL_STATE_ALIGNMENT)) {
            setHorizontalStateAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_STATE_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_STATE_POSITION)) {
            setHorizontalStatePosition(style.getIntegerAttribute(STYLE_HORIZONTAL_STATE_POSITION));
        }
        if (style.hasAttribute(STYLE_STATE_MARGIN)) {
            setStateMargin(style.getIntegerAttribute(STYLE_STATE_MARGIN));
        }
        if (style.hasAttribute(STYLE_VERTICAL_STATE_ALIGNMENT)) {
            setVerticalStateAlignment(style.getIntegerAttribute(STYLE_VERTICAL_STATE_ALIGNMENT));
        }
        if (style.hasAttribute(STYLE_VERTICAL_STATE_POSITION)) {
            setVerticalStatePosition(style.getIntegerAttribute(STYLE_VERTICAL_STATE_POSITION));
        }
    }

    /**
     * Returns the horizontal alignment of the state indicator (check box or 
     * radio button) relative to the button.
     *
     * @return The horizontal alignment of the state indicator relative to the 
     *         button, one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalStateAlignment() {
        return horizontalStateAlignment;
    }

    /**
     * Returns the horizontal position of the state indicator (check box or 
     * radio button) relative to the button.
     *
     * @return The horizontal position of the state indicator relative to the 
     *         button, one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalStatePosition() {
        return horizontalStatePosition;
    }
    
    /**
     * Returns the size of the margin to be displayed between the state 
     * indicator of the toggle button and its icon and text.
     *
     * @return The size of the margin between the state indicator and 
     *         the toggle button's icon and text, in pixels.
     */
    public int getStateMargin() {
        return stateMargin;
    }
    
    /**
     * Returns the vertical alignment of the state indicator (check box or 
     * radio button) relative to the button.
     *
     * @return The alignment position of the state indicator relative to the 
     *         button, one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalStateAlignment() {
        return verticalStateAlignment;
    }
    
    /**
     * Returns the vertical position of the state indicator (check box or 
     * radio button) relative to the button.
     *
     * @return The vertical position of the state indicator relative to the 
     *         button, one of the following values:
     *         <ul>
     *         <li>EchoConstants.TOP</li>
     *         <li>EchoConstants.CENTER (the default)</li>
     *         <li>EchoConstants.BOTTOM</li>
     *         </ul>
     */
    public int getVerticalStatePosition() {
        return verticalStatePosition;
    }
    
    /**
     * Sets the horizontal alignment of the state indicator 
     * relative to the button.
     *
     * @param newValue The horizontal alignment of the state indicator 
     *        relative to the button, one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setHorizontalStateAlignment(int newValue) {
        int oldValue = horizontalStateAlignment;
        horizontalStateAlignment = newValue;
        firePropertyChange(HORIZONTAL_STATE_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the horizontal position of the state indicator 
     * relative to the button.
     *
     * @param newValue The horizontal position of the state indicator 
     *        relative to the button, one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setHorizontalStatePosition(int newValue) {
        int oldValue = horizontalStatePosition;
        horizontalStatePosition = newValue;
        firePropertyChange(HORIZONTAL_STATE_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the size of the margin to be displayed between the state indicator 
     * of the toggle button and its icon and text.
     *
     * @param newValue The size of the margin between the state indicator and 
     *        the toggle button's icon and text, in pixels.
     */
    public void setStateMargin(int newValue) {
        int oldValue = stateMargin;
        stateMargin = newValue;
        firePropertyChange(STATE_MARGIN_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical alignment of the state indicator 
     * relative to the button.
     *
     * @param newValue The vertical alignment of the state indicator 
     *        relative to the button, one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalStateAlignment(int newValue) {
        int oldValue = verticalStateAlignment;
        verticalStateAlignment = newValue;
        firePropertyChange(VERTICAL_STATE_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the vertical position of the state indicator 
     * relative to the button.
     *
     * @param newValue The vertical position of the state indicator 
     *        relative to the button, one of the following values.
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER (the default)</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    public void setVerticalStatePosition(int newValue) {
        int oldValue = verticalStatePosition;
        verticalStatePosition = newValue;
        firePropertyChange(VERTICAL_STATE_POSITION_CHANGED_PROPERTY, oldValue, newValue);
    }
}
