/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo;

import java.util.EventListener;
import java.util.Locale;

import nextapp.echo.event.HierarchyEvent;
import nextapp.echo.event.WindowEvent;
import nextapp.echo.event.WindowListener;

/**
 * A top-level window.
 */
public class Window extends Component {

    /** A style constant for the DefaultCloseOperation property. */
    public static final String STYLE_DEFAULT_CLOSE_OPERATION = "defaultCloseOperation";

    /** A style constant for the Height property. */
    public static final String STYLE_HEIGHT = "height";
    
    /** A style constant for the Width property. */
    public static final String STYLE_WIDTH = "width";
    
    /**
     * A constant used to specify an unknown grid cell width or height.
     *
     * @deprecated Use EchoConstants.UNDEFINED_SIZE.
     */
    public static final int SIZE_UNKNOWN = EchoConstants.UNDEFINED_SIZE;
    
    /**
     * A value for the DefaultCloseOperation property that indicates the 
     * window should not take any action when closed.  If a user clicks
     * the close button on a window and its <code>WindowListener</codes>s
     * take no action, the window will remain open.
     */
    public static final int DO_NOTHING_ON_CLOSE = 1;
    
    /**
     * A value for the DefaultCloseOperation property that indicates the
     * window should be hidden when a user clicks its close button.
     * This DefaultCloseOperation is equivalent to calling 
     * Window.setVisible(false).
     */
    public static final int HIDE_ON_CLOSE = 2;
    
    /**
     * A value for the DefaultCloseOperation property that indicates the
     * window should be disposed when a user clicks its close button.
     * This DefaultCloseOperation is equivalent to calling Window.dispose().
     */
    public static final int DISPOSE_ON_CLOSE = 3;

    public static final String HEIGHT_CHANGED_PROPERTY = "height";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    public static final String TITLE_CHANGED_PROPERTY = "title";
    public static final String Z_INDEX_CHANGED_PROPERTY = "zIndex";

    private String title = null;
    private int width = EchoConstants.UNDEFINED_SIZE;
    private int height = EchoConstants.UNDEFINED_SIZE;
    private EchoInstance echoInstance = null;
    private int defaultCloseOperation = DO_NOTHING_ON_CLOSE;

    /** 
     * Creates a Window.
     */
    public Window() {
        super();
    }
    
    /**
     * Creates a Window with the specified title.
     *
     * @param title The window title.
     */
    public Window(String title) {
        super();
        this.title = title;
    }
    
    /**
     * @see nextapp.echo.Component#add(Component, int)
     */
    public void add(Component c, int index) {
        if (!(c instanceof AbstractPane)) {
            throw new IllegalArgumentException("Windows may only have AbstractPanes as children.");
        }
        if (getComponentCount() > 0) {
            throw new IllegalStateException("Windows may only have a single child component.");
        }
        super.add(c, index);
    }
    
    /**
     * Adds a <code>WindowListener</code> to the window.
     *
     * @param l The <code>WindowListener</code> to be added.
     */
    public void addWindowListener(WindowListener l) {
        listenerList.addListener(WindowListener.class, l);
    }
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_DEFAULT_CLOSE_OPERATION)) {
            setDefaultCloseOperation(style.getIntegerAttribute(STYLE_DEFAULT_CLOSE_OPERATION));
        }
        if (style.hasAttribute(STYLE_HEIGHT)) {
            setHeight(style.getIntegerAttribute(STYLE_HEIGHT));
        }
        if (style.hasAttribute(STYLE_WIDTH)) {
            setWidth(style.getIntegerAttribute(STYLE_WIDTH));
        }
    }
    
    /**
     * Destroys the window.  Identical to calling 
     * EchoInstance.removeWindow().
     */
    public void dispose() {
        if (echoInstance != null) {
            echoInstance.removeWindow(this);
        }
    }
    
    /**
     * Notifies <code>WindowListener</code>s that the window has been closed.
     */
    void fireWindowClosed() {
        WindowEvent e = new WindowEvent(this);
        EventListener[] listeners = listenerList.getListeners(WindowListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((WindowListener) listeners[index]).windowClosed(e);
        }
    }
    
    /**
     * Notifies <code>WindowListener</code>s that the window has been 
     * requested to close.
     */
    private void fireWindowClosing() {
        WindowEvent e = new WindowEvent(this);
        EventListener[] listeners = listenerList.getListeners(WindowListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((WindowListener) listeners[index]).windowClosing(e);
        }
    }
    
    /**
     * Notifies <code>WindowListener</code>s that the window has been opened.
     */
    void fireWindowOpened() {
        WindowEvent e = new WindowEvent(this);
        EventListener[] listeners = listenerList.getListeners(WindowListener.class);
        for (int index = 0; index < listeners.length; ++index) {
            ((WindowListener) listeners[index]).windowOpened(e);
        }
    }
    
    /**
     * Returns the content of this Window.
     *
     * @return A ContentPane or ContainerPane that is the content of this 
     *         window.
     */
    public final AbstractPane getContent() {
        if (getComponentCount() > 0) {
            return (AbstractPane) getComponent(0);
        } else {
            return null;
        }
    }
    
    /**
     * Returns the EchoInstance object to which this window belongs.
     *
     * @return The EchoInstance object to which this window belongs.
     */
    public EchoInstance getEchoInstance() {
        if (echoInstance == null) {
            return EchoInstance.getEchoInstance();
        } else {
            return echoInstance;
        }
    }

    /**
     * Returns the height of the window.
     *
     * @return The inside height of the window, in pixels.
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * Overrides Component.getLocale() such that the applciation's locale is
     * returned if this Window's locale is not set.
     *
     * @return The locale for this component, or if it has none, the 
     *         application's locale.
     */
    public Locale getLocale() {
        if (locale == null) {
            if (echoInstance == null) {
                EchoInstance activeEchoInstance = EchoInstance.getEchoInstance();
                if (activeEchoInstance == null) {
                    return null;
                } else {
                    return activeEchoInstance.getLocale();
                }
            } else {
                return echoInstance.getLocale();
            }
        } else {
            return locale;
        } 
    }
    
    /**
     * Returns the operation that will happen by default if the user closes the 
     * window.
     *
     * @return The default close operation, one of the following values:
     *         <ul>
     *         <li><code>DO_NOTHING_ON_CLOSE</code>: Do not dispose or hide the
     *             window.  Instead, registered window listeners should handle 
     *             the closing of the window if necessary in the 
     *             windowClosing() method.</li>
     *         <li><code>HIDE_ON_CLOSE</code>: Hide the window.</li>
     *         <li><code>DISPOSE_ON_CLOSE</code>: Dispose of the window.</li>
     *         </ul>
     */
    public int getDefaultCloseOperation() {
        return defaultCloseOperation;
    }
    
    /**
     * Returns the window title.
     *
     * @return The title of the window.
     */
    public String getTitle() {
        return title;
    }
    
    /**
     * Returns the width of the window.
     *
     * @return The inside width of the window, in pixels.
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Internally invoked when the Window is registered to or unregistered 
     * from an application.
     * 
     * @param registered The new registration state.
     */
    void notifyRegistrationChange(boolean registered) {
        setRegistered(registered);
        long changeFlags = HierarchyEvent.REGISTRATION_CHANGED;
        if (isVisible()) {
            changeFlags |= HierarchyEvent.SHOWING_CHANGED;
        }
        processHierarchyEvent(changeFlags, this, null);
    }
        
    /**
     * Brings the window to the back, such that it is obscured by any other 
     * browser windows.
     */
    public void toBack() {
        firePropertyChange(Z_INDEX_CHANGED_PROPERTY, 0, -1);
    }
    
    /**
     * Brings the window to the front, such that it is not obscured by any other 
     * browser windows.
     */
    public void toFront() {
        firePropertyChange(Z_INDEX_CHANGED_PROPERTY, 0, 1);
    }
    
    /**
     * Removes a <code>WindowListener</code> from the window.
     *
     * @param l The <code>WindowListener</code> to be removed.
     */
    public void removeWindowListener(WindowListener l) {
        listenerList.removeListener(WindowListener.class, l);
    }
    
    /**
     * Sets the content of the window.
     *
     * @param newValue a ContainerPane or ContentPane that holds content.
     */
    public final void setContent(AbstractPane newValue) {
        removeAll();
        add(newValue);
    }
    
    /**
     * Sets the operation that will happen by default if the user closes the 
     * window.
     *
     * @param defaultCloseOperation The default close operation, one of the
     *        following values:
     *        <ul>
     *        <li><code>DO_NOTHING_ON_CLOSE</code>: Do not dispose or hide the 
     *            window.  Instead, registered window listeners should handle 
     *            the closing of the window if necessary in the 
     *            windowClosing() method.</li>
     *        <li><code>HIDE_ON_CLOSE</code>: Hide the window.</li>
     *        <li><code>DISPOSE_ON_CLOSE</code>: Dispose of the window.</li>
     *        </ul>
     */
    public void setDefaultCloseOperation(int defaultCloseOperation) {
        this.defaultCloseOperation = defaultCloseOperation;
    }
    
    /**
     * Sets the window's associated Echo instance.  This is called only by 
     * the Echo instance itself.
     */
    void setEchoInstance(EchoInstance echoInstance) {
        if (this.echoInstance != null && echoInstance != null) {
            throw new IllegalStateException("Window is already registered to an Echo instance.");
        }
        this.echoInstance = echoInstance;
    }
    
    /**
     * Sets the height of the window.
     *
     * @param newValue The inside height of the window, in pixels.
     */
    public void setHeight(int newValue) {
        int oldValue = height;
        height = newValue;
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the window title.
     *
     * @param newValue The title of the window.
     */
    public void setTitle(String newValue) {
        String oldValue = title;
        title = newValue;
        firePropertyChange(TITLE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the width of the window.
     *
     * @param newValue The inside width of the window, in pixels.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Informs the window that the user has attempted to close it.
     */    
    public void userClose() {
        fireWindowClosing();

        switch (getDefaultCloseOperation()) {
        case Window.HIDE_ON_CLOSE:
            setVisible(false);
            break;
        case Window.DISPOSE_ON_CLOSE:
            dispose();
            break;
        } 
    }
}
