/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.event;

import java.util.EventObject;

import nextapp.echo.Component;

/**
 * An event describing a change to the component hierarchy.
 */
public class HierarchyEvent extends EventObject {
    
    public static final int SHOWING_CHANGED = 0x1;
    public static final int PARENT_CHANGED = 0x2;
    public static final int REGISTRATION_CHANGED = 0x4;

    private long changeFlags;
    private Component changed;
    private Component changedParent;
    private Component component;
    
    /**
     * Creates a new <code>HierarchyEvent</code>
     * 
     * @param source The generator of the event.
     */
    public HierarchyEvent(Component source, long changeFlags, Component changed, Component changedParent) {
        super(source);
        this.component = source;
        this.changeFlags = changeFlags;
        this.changed = changed;
        this.changedParent = changedParent;
    }
    
    /**
     * Returns a bitmask describing the change to the heirarchy.
     * 
     * @return A bitmask describing the change to the hierarchy, consisting
     *         of the following bits:
     *         <ul>
     *         <li>SHOWING_CHANGED</li>
     *         <li>PARENT_CHANGED</li>
     *         <li>REGISTRATION_CHANGED</li>
     *         </ul>
     */
    public long getChangeFlags() {
        return changeFlags;
    }
    
    /**
     * Returns the component whose state within the hierarchy changed.
     * 
     * @return The component whose state within the hierarchy changed.
     */
    public Component getChanged() {
        return changed;
    }
    
    /**
     * Returns the parent (or former parent) of the component whose state
     * within the hierarchy changed.
     * 
     * @return The parent (or former parent) of the component whose state
     *         within the hierarchy changed.
     */
    public Component getChangedParent() {
        return changedParent;
    }
    
    /**
     * Returns the component which generated this event.
     * 
     * @return The component which generated this event.
     */
    public Component getComponent() {
        return component;
    }

    /**
     * Returns true if the <code>Component</code>'s parent has changed.
     * 
     * @return True if the <code>Component</code>'s parent has changed.
     */
    public boolean isParentChanged() {
        return (changeFlags & PARENT_CHANGED) == PARENT_CHANGED;
    }

    /**
     * Returns true if the <code>Component</code>'s registration state has 
     *         changed.
     * 
     * @return True if the <code>Component</code>'s registration state has 
     *         changed.
     */
    public boolean isRegistrationChanged() {
        return (changeFlags & REGISTRATION_CHANGED) == REGISTRATION_CHANGED;
    }
    
    /**
     * Returns true if the <code>Component</code>'s showing state has 
     *         changed.
     * 
     * @return True if the <code>Component</code>'s showing state has 
     *         changed.
     */
    public boolean isShowingChanged() {
        return (changeFlags & SHOWING_CHANGED) == SHOWING_CHANGED;
    }
}
