/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.table;

import java.io.Serializable;
import java.util.EventListener;

import nextapp.echo.event.EventListenerList;
import nextapp.echo.event.TableModelEvent;
import nextapp.echo.event.TableModelListener;

/**
 * An abstract implementation of a table model.  This class provides listener 
 * management and convenience methods for notifying listeners of events.  
 * It provides generic implementations of getColumnClass() (which always returns
 * Object.class) and getColumnName() (which returns "spreadsheet-style" column 
 * names).  It also provides an empty implementation of setValueAt().<br>
 * <br>
 * This class should be extended when you desire to create a TableModel which
 * is best served by a custom representation of its internal data.
 *
 * @see DefaultTableModel
 */
public abstract class AbstractTableModel 
implements Serializable, TableModel {

    /**
     * A listener management facility.
     */
    protected EventListenerList listenerList = new EventListenerList();
    
    /** 
     * Default constructor.
     */
    public AbstractTableModel() {
        super();
    }
    
    /**
     * Adds a listener that will be notified of changes to the table model.
     *
     * @param l The listener to add.
     */
    public void addTableModelListener(TableModelListener l) {
        listenerList.addListener(TableModelListener.class, l);
    }
    
    /**
     * Notifies listeners that the contents of the cell at the specified 
     * coordinate were changed.
     *
     * @param column The column value of the cell that changed.
     * @param row The row value of the cell that changed.
     */
    public void fireTableCellUpdated(int column, int row) {
        fireTableChanged(new TableModelEvent(this, row, row, column, TableModelEvent.UPDATE));
    }
    
    /**
     * Notifies listeners that the content of the table (possibly including the
     * number of rows) was changed, but that the table's structure has remained
     * intact.
     */
    public void fireTableDataChanged() {
        fireTableChanged(new TableModelEvent(this));
    }
    
    /**
     * Notifies listeners that rows the rows from firstRow to LastRow were 
     * deleted.
     *
     * @param firstRow The first row that was deleted.
     * @param lastRow The last row that was deleted.
     */
    public void fireTableRowsDeleted(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, TableModelEvent.ALL_COLUMNS, firstRow, lastRow, TableModelEvent.DELETE));
    }
    
    /**
     * Notifies listeners that rows the rows from firstRow to LastRow were 
     * inserted.
     *
     * @param firstRow The first row of the new inserted rows.
     * @param lastRow The last row of the new inserted rows.
     */
    public void fireTableRowsInserted(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, TableModelEvent.ALL_COLUMNS, firstRow, lastRow, TableModelEvent.INSERT));
    }
    
    /**
     * Notifies listeners that data found in the specified rows was updated.
     *
     * @param firstRow The first row that was updated.
     * @param lastRow The last row that was updated.
     */
    public void fireTableRowsUpdated(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, TableModelEvent.ALL_COLUMNS, firstRow, lastRow, TableModelEvent.UPDATE));
    }
    
    /**
     * Notifies all listeners that all data in the table may have changed, 
     * including the size and structure of the table.
     */
    public void fireTableStructureChanged() {
        fireTableChanged(new TableModelEvent(this, TableModelEvent.ALL_COLUMNS, TableModelEvent.HEADER_ROW,
                TableModelEvent.HEADER_ROW, TableModelEvent.STRUCTURE_CHANGED));
    }
    
    /**
     * Notifies all TableModelListeners of the specified event.
     *
     * @param e The event to send to the listeners.
     */
    public void fireTableChanged(TableModelEvent e) {
        EventListener[] listeners = listenerList.getListeners(TableModelListener.class);
        
        for (int index = 0; index < listeners.length; ++index) {
            ((TableModelListener) listeners[index]).tableChanged(e);
        }
    }

    /**
     * Returns Object.class.
     */
    public Class getColumnClass(int column) {
        return Object.class;
    }

    /**
     * Returns the value found at the given coordinate within the table.
     * This method returns a column name using the same notation as a
     * spreadsheet (A, B, C ... Y, Z, AA, AB, AC ...)
     *
     * @param column The column of the value to return.
     */
    public String getColumnName(int column) {
        StringBuffer sb = new StringBuffer();
        int value = column;
        do {
            int digit = value % 26;
            sb.insert(0, (char) ('A' + digit));
            value = value / 26 - 1;
        } while (value >= 0);
        
        return sb.toString();
    }
    
    /**
     * Removes a listener from being notified of changes to the table model.
     *
     * @param l The listener to remove.
     */
    public void removeTableModelListener(TableModelListener l) {
        listenerList.removeListener(TableModelListener.class, l);
    }
}
