/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.table;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * The default TableModel implementation.
 */
public class DefaultTableModel extends AbstractTableModel 
implements Serializable {

    private List rows;
    private List columnNames;
    
    /**
     * Creates a new table model of 0x0 size.
     */
    public DefaultTableModel() {
        super();
        
        columnNames = new ArrayList();
        rows = new ArrayList();
    }
    
    /**
     * Creates a new table model of the specified dimension.
     *
     * @param columns The number of columns in the table model.
     * @param rows The number of rows in the table model.
     */
    public DefaultTableModel(int columns, int rows) {
        this();
        
        setRowCount(rows);
        setColumnCount(columns);
    }
    
    /**
     * Creates a new Table Model with the specified data and column names.
     *
     * @param data A two dimensional array containing the table data.
     *        The first index divides the array into rows.
     *        The second index specifies columns.
     * @param names An array of column names.
     */
    public DefaultTableModel(Object[][] data, Object[] names) {
        super();
        
        if (data == null) {
            columnNames = new ArrayList();
            rows = new ArrayList();
        } else {
            ArrayList rowList;
            int height = data.length;
            int width = 0;
            if (height > 0 && data[0] != null) {
                width = data[0].length;
            }

            // Add column names
            columnNames = new ArrayList(width);
            for (int column = 0; column < width; ++column) {
                columnNames.add(names[column]);
            }
            
            // Add table data
            rows = new ArrayList(height);
            for (int row = 0; row < height; ++row) {
                if (width != 0) {
                    rowList = new ArrayList(width);
                    for (int column = 0; column < width; ++column) {
                        rowList.add(data[row][column]);
                    }
                    rows.add(rowList);
                }
            }
        }
    }
    
    /**
     * Deletes the specified row.
     *
     * @param row The row to delete.
     */
    public void deleteRow(int row) {
        rows.remove(row);
        fireTableRowsDeleted(row, row);
    }

    /**
     * Returns the number of columns in the table.
     *
     * @return The number of columns in the table.
     */
    public int getColumnCount() {
        return columnNames.size();
    }

    /**
     * Returns the name of the specified column number.
     *
     * @param column The column number whose name is to be returned.
     * @return The name of the specified column number.
     */
    public String getColumnName(int column) {
        String name;
        
        if (column < columnNames.size()) {
            name = (String) columnNames.get(column);
        } else {
            name = "";
        }
        
        return name;
    }
    
    /**
     * Returns the number of rows in the table.
     *
     * @return The number of rows in the table.
     */
    public int getRowCount() {
        return rows.size();
    }
    
    /**
     * Returns the contents of the table at the specified position.
     *
     * @param row The row index of the desired value.
     * @param column The column index of the desired value.
     * @return The value of the specified cell.
     */
    public Object getValueAt(int column, int row) {
        Object value;
        List rowList;
    
        if (row < rows.size()) {
            if (column < columnNames.size()) {
                rowList = (List) rows.get(row);
                if (rowList == null) {
                    value = null;
                } else {
                    value = rowList.get(column);
                }
            } else {
                throw new ArrayIndexOutOfBoundsException("Table column " + column + " does not exist.");
            }
        } else {
            throw new ArrayIndexOutOfBoundsException("Table row " + row + " does not exist.");
        }
        
        return value;
    }
    
    /**
     * Inserts a row with the provided data.
     *
     * @param row The index of the inserted row.
     * @param rowData The data for the new.
     */
    public void insertRow(int row, Object[] rowData) {
        int maxIndex = rowData.length > columnNames.size() ? columnNames.size() : rowData.length;
        List rowList = new ArrayList(columnNames.size());
    
        for (int index = 0; index < maxIndex; ++index) {
            rowList.add(rowData[index]);
        }

        rows.add(row, rowList);
        
        fireTableRowsInserted(row, row);
    }
    
    /**
     * Sets the number of columns in the table.
     * Empty columns will be added at the end of the table if the new column 
     * count exceeds the number of existing columns.  Existing columns will be
     * hidden if the number of existing columns exceeds the new column count.
     *
     * @param newValue The new number of columns in the table.
     */
    public void setColumnCount(int newValue) {
        while (columnNames.size() > newValue) {
            columnNames.remove(columnNames.size() - 1);
        }
        
        while (columnNames.size() < newValue) {
            columnNames.add(null);
        }
    }

    /**
     * Sets the number of rows in the table.
     * Empty rows will be added at the end of the table if the new row 
     * count exceeds the number of existing rows.  Existing rows will be
     * hidden if the number of existing rows exceeds the new row count.
     *
     * @param newValue The new number of rows in the table.
     */
    public void setRowCount(int newValue) {
        // Remove excess rows
        while (rows.size() > newValue) {
            rows.remove(rows.size() - 1);
        }
        
        while (rows.size() < newValue) {
            rows.add(null);
        }
    }

    /**
     * Sets the contents of the table at the specified coordinate.
     *
     * @param newValue The new value for the given cell.
     * @param column The column at which to place the value.
     * @param row The row at which to place the value.
     */
    public void setValueAt(Object newValue, int column, int row) {
        if (rows.size() < row || columnNames.size() < column) {
            throw new ArrayIndexOutOfBoundsException("Table coordinate (" + column + ", " + row + ") does not exist");
        }

        List rowList = (List) rows.get(row);
        if (rowList == null && newValue != null) {
            rowList = new ArrayList(columnNames.size());
            rows.set(row, rowList);
        }
        
        while (rowList.size() <= column) {
            rowList.add(null);
        }
        
        rowList.set(column, newValue);
        
        fireTableCellUpdated(column, row);
    }
}
