/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * Contributed by Phillip Baird
 */

package nextapp.echo.table;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

import nextapp.echo.EchoConstants;

/**
 * Describes a single table column.
 */
public class TableColumn 
implements Serializable {

    public static final String CELL_RENDERER_CHANGED_PROPERTY = "cellRenderer";
    public static final String HEADER_RENDERER_CHANGED_PROPERTY = "headerRenderer";
    public static final String HEADER_VALUE_CHANGED_PROPERTY = "headerValue";
    public static final String IDENTIFIER_CHANGED_PROPERTY = "identifier";
    public static final String MODEL_INDEX_CHANGED_PROPERTY = "modelIndex";
    public static final String WIDTH_CHANGED_PROPERTY = "width";
    
    private int width;
    private TableCellRenderer cellRenderer;
    private TableCellRenderer headerRenderer;
    private Object headerValue;
    private int modelIndex;
    private Object identifier;
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    
    /**
     * This constructor is necessary for serialization and should otherwise not
     * be used.
     */
    public TableColumn() {
        this(0, EchoConstants.UNDEFINED_SIZE, null, null);
    }
    
    /**
     * Creates a TableColumn with the specified model index, undefined width,
     * and undefined cell and header renderers.
     *
     * @param modelIndex The column index in the model from which this column
     *        should display data.
     */
    public TableColumn(int modelIndex) {
        this(modelIndex, EchoConstants.UNDEFINED_SIZE, null, null);
    }
    
    /**
     * Creates a TableColumn with the specified model index and width, 
     * and undefined cell and header renderers.
     * 
     * @param modelIndex The column index in the model from which this column
     *        should display data.
     * @param width The width of the column.
     */
    public TableColumn(int modelIndex, int width) {
        this(modelIndex, width, null, null);
    }

    /**
     * Creates a TableColumn with the specified model index, width, 
     * and cell and header renderers.
     * 
     * @param modelIndex The column index in the model from which this column
     *        should display data.
     * @param width The width of the column.
     * @param cellRenderer The cell renderer to use for rendering 
     *        values in this column.  This parameter may be null if desired.
     * @param headerRenderer The cell renderer to use for rendering header 
     *        values in this column.  This parameter may be null if desired.
     */
    public TableColumn(int modelIndex, int width, TableCellRenderer cellRenderer, TableCellRenderer headerRenderer) {        
        super();
        
        this.modelIndex = modelIndex;
        this.width = width;
        setCellRenderer(cellRenderer);
        setHeaderRenderer(headerRenderer);
    }
    
    /**
     * Adds a property change listener to the column.
     *
     * @param l The listener to add.
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        pcs.addPropertyChangeListener(l);
    }
    
    /** 
     * Returns the cell renderer for this column.  The cell renderer
     * will produce a component for each row of this column based
     * on the value in that row.
     *
     * @return The cell renderer for this column.
     */
    public TableCellRenderer getCellRenderer() {
        return cellRenderer;
    }
    
    /**
     * Returns the header cell renderer for this column.  The header cell
     * renderer will produce a component for the header cell of this
     * column based on the header value.
     *
     * @return The header cell renderer for this column.
     */
    public TableCellRenderer getHeaderRenderer() {
        return headerRenderer;
    }
    
    /** 
     * Returns the header value for this column.  The header value is the 
     * object that will be provided to the header renderer to produce
     * a component that will be used as the table header for this column.
     *
     * @return The header value for this column.
     */
    public Object getHeaderValue() {
        return headerValue;
    }
    
    /**
     * Returns the identifier for this column.   Each table column may have
     * an identifier.  Identifiers are provided as a convenience to the
     * application developer, and are neither used nor required by the
     * Table component.
     *
     * @return The identifier for this column.
     */
    public Object getIdentifier() {
        return identifier;
    }
    
    /**
     * Returns the index of this column in the model.  This value is 
     * independent of the <code>TableColumn</code>'s. position within the
     * table.
     *
     * @return The index of this column in the model.
     */
    public int getModelIndex() {
        return modelIndex;
    }
    
    /** 
     * Returns the width of this column, in pixels.
     *
     * @return The width of this column, in pixels.  
     *         Negative values indicate an undefined width.
     */
    public int getWidth() {
        return width;
    }
    
    /**
     * Removes a property change listener from the column.
     *
     * @param l The listener to remove.
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        pcs.removePropertyChangeListener(l);
    }
    
    /** 
     * Sets the cell renderer for this column.  The cell renderer
     * will produce a component for each row of this column based
     * on the value in that row.
     *
     * @param newValue The new cell renderer for this column.
     */
    public void setCellRenderer(TableCellRenderer newValue) {
        TableCellRenderer oldValue = cellRenderer;
        cellRenderer = newValue;
        pcs.firePropertyChange(CELL_RENDERER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the header cell renderer for this column.  The header cell
     * renderer will produce a component for the header cell of this
     * column based on the header value.
     *
     * @param newValue The new header renderer for this column.
     */
    public void setHeaderRenderer(TableCellRenderer newValue) {
        TableCellRenderer oldValue = headerRenderer;
        headerRenderer = newValue;
        pcs.firePropertyChange(HEADER_RENDERER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the header value for this column.  The header value is the 
     * object that will be provided to the header renderer to produce
     * a component that will be used as the table header for this column.
     *
     * @param newValue The new header value for this column.
     */
    public void setHeaderValue(Object newValue) {
        Object oldValue = headerValue;
        headerValue = newValue;
        pcs.firePropertyChange(HEADER_VALUE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the identifier for this column.  Each table column may have
     * an identifier.  Identifiers are provided as a convenience to the
     * application developer, and are neither used nor required by the
     * Table component.
     *
     * @param newValue The new identifier for this column.
     */
    public void setIdentifer(Object newValue) {
        Object oldValue = identifier;
        identifier = newValue;
        pcs.firePropertyChange(IDENTIFIER_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the index of the column in the <code>TableModel</code> which
     * this <code>TableColumn</code> object represents.  This value is
     * independent of the column's position within the column model, such that
     * columns may be displayed in an arbitrary order.
     *
     * @param newValue The new model index of this column.
     */
    public void setModelIndex(int newValue) {
        int oldValue = modelIndex;
        modelIndex = newValue;
        pcs.firePropertyChange(MODEL_INDEX_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /** 
     * Sets the width of the column.
     *
     * @param newValue The width of this column.  A negative value indicates
     *        an undefined width.
     */
    public void setWidth(int newValue) {
        int oldValue = width;
        width = newValue;
        pcs.firePropertyChange(WIDTH_CHANGED_PROPERTY, oldValue, newValue);
    }
}
