/*
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.table;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import nextapp.echo.Component;
import nextapp.echo.Container;
import nextapp.echo.Label;
import nextapp.echo.Table;
import nextapp.echo.event.TableColumnModelEvent;
import nextapp.echo.event.TableColumnModelListener;
import nextapp.echo.event.TableModelEvent;
import nextapp.echo.event.TableModelListener;

/**
 * A representation of a table's header.
 */
public class TableHeader extends Component implements Serializable {

    public static final String HEIGHT_CHANGED_PROPERTY = "height";

    /**
     * A default renderer for table header cells.
     */
    public static class DefaultHeaderRenderer
    implements Serializable, TableCellRenderer {

    /**
	 * @see nextapp.echo.table.TableCellRenderer#getTableCellRendererComponent(nextapp.echo.Table,
	 *      java.lang.Object, int, int)
	 */
        public Component getTableCellRendererComponent(Table table, Object value, int column, int row) { Label label;

            if (value == null) {
                label = null;
            } else {
                label = new Label(value.toString());
            }

            return label;
        }
    }

    /**
     * A local handler for TableColumnModelEvents.
     */
    private class TableColumnModelHandler
    implements Serializable, TableColumnModelListener {

        /**
	 * @see nextapp.echo.event.TableColumnModelListener#columnAdded(TableColumnModelEvent)
	 */
        public void columnAdded(TableColumnModelEvent e) {
            invalidate();
        }

        /**
	 * @see nextapp.echo.event.TableColumnModelListener#columnMoved(TableColumnModelEvent)
	 */
        public void columnMoved(TableColumnModelEvent e) {
            invalidate();
        }

    /**
	 * @see nextapp.echo.event.TableColumnModelListener#columnRemoved(TableColumnModelEvent)
	 */
        public void columnRemoved(TableColumnModelEvent e) {
            invalidate();
        }
    }

    /**
     * A local handler for TableModelEvents.
     */
    private class TableModelHandler
    implements Serializable, TableModelListener {

    /**
	 * @see nextapp.echo.event.TableModelListener#tableChanged(TableModelEvent)
	 */
        public void tableChanged(TableModelEvent e) {
            invalidate();
        }
    }

    private TableCellRenderer defaultRenderer = new DefaultHeaderRenderer();
    private TableColumnModelListener columnModelListener = new TableColumnModelHandler();
    private TableModelListener modelListener = new TableModelHandler();
    private Table table = null;
    private Container renderedCells;
    private List renderedCellsList = new ArrayList();
    private boolean valid = false;
    private int height;

    /**
     * Creates a table header.
     */
    public TableHeader() {
        super();
        renderedCells = new Container();
        add(renderedCells);
    }

    /**
     * Retrieves column header components from the model and adds them to the
     * Table Component.
     */
    private void doLayout() {
        valid = true;

        renderedCells.removeAll();
        renderedCellsList.clear();

        TableColumnModel columnModel = table.getColumnModel();
        TableModel model = table.getModel();
        int columns = columnModel.getColumnCount();
        TableCellRenderer renderer;

        for (int column = 0; column < columns; column++) {
            TableColumn tableColumn = columnModel.getColumn(column);
            Object value;

            value = tableColumn.getHeaderValue();
            if (value == null) {
                value = model.getColumnName(column);
            }

            renderer = tableColumn.getHeaderRenderer();
            if (renderer == null) {
                renderer = defaultRenderer;
            }

            Component cell = null;
            if (renderer != null) {
                cell = renderer.getTableCellRendererComponent(table, value, tableColumn.getModelIndex(), -1);
                if (cell != null) {
                    renderedCells.add(cell);
                }
            }
            renderedCellsList.add(cell);
        }
    }

    /**
     * Returns the default cell renderer.
     * 
     * @return The default cell renderer.
     */
    public TableCellRenderer getDefaultRenderer() {
        return defaultRenderer;
    }

    /**
     * Returns the height of the table header.
     * 
     * @return The height of the table header. The units of this value are
     *         specified by the underlying table's <code>HeightUnits property.</code>
     */
    public int getHeight() {
        return height;
    }
    
    /**
     * Returns the contents of the header cell at the specified column.
     * This method should be used only by the rendering peer.
     * This method will not return correct information prior to validation.
     *
     * @param column The column of the rendered header cell component 
     *        to return.
     * @return The header component.
     */
    public Component getRenderedComponent(int column) {
        return (Component) renderedCellsList.get(column);
    }

    /**
     * Returns the table component with which this TableHeader is associated.
     * 
     * @return The table component with which this TableHeader is associated.
     */
    public Table getTable() {
        return table;
    }

    /**
     * Marks the table header as needing to be re-rendered.
     */
    public void invalidate() {
        valid = false;
    }

    /**
     * Sets the default cell renderer that will be used if a column does not
     * have an assigned renderer.
     * 
     * @param defaultRenderer
     *                The new default renderer.
     */
    public void setDefaultRenderer(TableCellRenderer defaultRenderer) {
        this.defaultRenderer = defaultRenderer;
        invalidate();
    }

    /**
     * Sets the table component with which this TableHeader is associated.
     * 
     * @param newValue
     *                The table component with which this TableHeader is
     *                associated.
     */
    public void setTable(Table newValue) {
        Table oldValue = table;
        table = newValue;
        if (oldValue != null) {
            oldValue.getColumnModel().removeColumnModelListener(columnModelListener);
            oldValue.getModel().removeTableModelListener(modelListener);
            if (oldValue.getTableHeader() == this) {
                oldValue.setTableHeader(null);
            }
        }
        if (newValue != null) {
            newValue.getColumnModel().addColumnModelListener(columnModelListener);
            newValue.getModel().addTableModelListener(modelListener);
            if (newValue.getTableHeader() != this) {
                newValue.setTableHeader(this);
            }
        }
        invalidate();
    }

    /**
     * Sets the height of the table header.
     * 
     * @param newValue The new height of the table header. The units of this
     *        value are specified by the underlying table's 
     *        <code>HeightUnits property.</code>
     */
    public void setHeight(int newValue) {
        int oldValue = height;
        height = newValue;
        firePropertyChange(HEIGHT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Updates the <code>Table</code> component to reflect its model.
     * 
     * @see nextapp.echo.Component#validate()
     */
    public void validate() {
        super.validate();
        while (!valid) {
            doLayout();
        }
    }
}
