/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.text;

import java.io.Serializable;

import nextapp.echo.Color;
import nextapp.echo.Component;
import nextapp.echo.EchoConstants;
import nextapp.echo.Style;
import nextapp.echo.ToolTipSupport;
import nextapp.echo.event.DocumentEvent;
import nextapp.echo.event.DocumentListener;

/**
 * An abstract Text component whose data is represented in a document model.
 */
public abstract class TextComponent extends Component
implements ToolTipSupport {

    /** 
     * A style constant for the Border Color property. 
     * Values of this key must be of type nextapp.echo.Color.
     */
    public static final String STYLE_BORDER_COLOR = "borderColor";

    /** 
     * A style constant for the Border Size property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_SIZE = "borderSize";

    /** 
     * A style constant for the Border Style property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_BORDER_STYLE = "borderStyle";

    /**
     * A style constant for the Columns property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_COLUMNS = "columns";

    /**
     * A style constant for the Horizontal Alignment property.
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_HORIZONTAL_ALIGNMENT = "horizontalAlignment";
    
    /**
     * A style constant for the Rows property. 
     * Values of this key must be of type java.lang.Integer.
     */
    public static final String STYLE_ROWS = "rows";
    
    public static final int CHARACTER_UNITS = 0;
    public static final int PIXEL_UNITS = 1;
    public static final int PERCENT_UNITS = 2;

    /**
     * A style constant for the Line Wrap property. 
     * Values of this key must be of type java.lang.Boolean.
     */
    public static final String STYLE_LINE_WRAP = "lineWrap";

    public static final String BORDER_COLOR_CHANGED_PROPERTY = "borderColor";
    public static final String BORDER_SIZE_CHANGED_PROPERTY = "borderSize";
    public static final String BORDER_STYLE_CHANGED_PROPERTY = "borderStyle";
    public static final String DOCUMENT_CHANGED_PROPERTY = "document";
    public static final String EDITABLE_CHANGED_PROPERTY = "editable";
    public static final String HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY = "horizontalAlignment";
    public static final String TEXT_CHANGED_PROPERTY = "text";
    
    /** 
     * A border style that causes the client to render the text component
     * with its default style. 
     */
    public static final int BORDER_DEFAULT = -1;
    
    /**
     * A border style that causes no border to be rendered.
     */
    public static final int BORDER_NONE = 0;
    
    /**
     * A border style that causes a solid, monochrome border to be rendered in
     * in the specified color.
     */
    public static final int BORDER_SOLID = 1;
    
    /**
     * A border style that causes a simulated 3D border to be rendered, such 
     * that the text component appears recessed.
     */
    public static final int BORDER_INSET = 2;
    
    /**
     * A border style that causes a simulated 3D border to be rendered, such 
     * that the text component appears raised.
     */
    public static final int BORDER_OUTSET = 3;

    /**
     * A handler for document events.
     */
    private class DocumentHandler 
    implements DocumentListener, Serializable {
       
        /**
         * @see nextapp.echo.event.DocumentListener#documentUpdate(DocumentEvent)
         */
        public void documentUpdate(DocumentEvent e) {
            update();
        }
    }

    private Color borderColor = null;
    private int borderSize = -1;
    private int borderStyle = BORDER_DEFAULT;
    private Document document = null;
    private DocumentListener documentHandler = new DocumentHandler();
    private boolean editable = true;
    private String toolTipText;
    private int horizontalAlignment = EchoConstants.LEFT;
    
    /**
     * @see nextapp.echo.Component#applyStyle(Style)
     */
    public void applyStyle(Style style) {
        super.applyStyle(style);
        if (style.hasAttribute(STYLE_BORDER_COLOR)) {
            setBorderColor((Color) style.getAttribute(STYLE_BORDER_COLOR));
        }
        if (style.hasAttribute(STYLE_BORDER_SIZE)) {
            setBorderSize(style.getIntegerAttribute(STYLE_BORDER_SIZE));
        }
        if (style.hasAttribute(STYLE_BORDER_STYLE)) {
            setBorderStyle(style.getIntegerAttribute(STYLE_BORDER_STYLE));
        }
        if (style.hasAttribute(STYLE_HORIZONTAL_ALIGNMENT)) {
            setHorizontalAlignment(style.getIntegerAttribute(STYLE_HORIZONTAL_ALIGNMENT));
        }
    }
    
    /**
     * Returns the border color.
     *
     * @return The border color.
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
    /**
     * Returns the border size.
     *
     * @return The border size, in pixels.
     */
    public int getBorderSize() {
        return borderSize;
    }
    
    /**
     * Returns the border style.
     *
     * @return The border style, one of the following values:
     *         <ul>      
     *         <li>BORDER_DEFAULT - The client's default style (the default).</li>
     *         <li>BORDER_SOLID - A solid single     color border.</li>
     *         <li>BORDER_INSET - A recessed 3D appearance.</li>
     *         <li>BORDER_OUTSET - A raised 3D appearance.</li>
     *         </ul>
     */
    public int getBorderStyle() {
        return borderStyle;
    }
    
    /**
     * Returns the document model.
     *
     * @return The document model.
     */
    public Document getDocument() {
        return document;
    }
    
    /**
     * Returns the horizontal alignment of the text.
     *
     * @return The horizontal alignment of the text,
     *         one of the following values:
     *         <ul>
     *         <li>EchoConstants.LEFT (the default)</li>
     *         <li>EchoConstants.CENTER</li>
     *         <li>EchoConstants.RIGHT</li>
     *         </ul>
     */
    public int getHorizontalAlignment() {
        return horizontalAlignment;
    }
    
    /**
     * Returns the text contained in the document model.  If this text
     * component's document model is null, throws a NullPointerException.
     *
     * @return The text contained in the document model.
     */
    public String getText() {
        return document.getText();
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#getToolTipText()
     */
    public String getToolTipText() {
        return toolTipText;
    }
    
    /**
     * Returns whether the text component's is editable.
     *
     * @return True if the text component is editable.
     */
    public boolean isEditable() {
        return editable;
    }

    /**
     * Sets the border color.
     *
     * @param newValue The new border color.
     */
    public void setBorderColor(Color newValue) {
        Color oldValue = borderColor;
        borderColor = newValue;
        firePropertyChange(BORDER_COLOR_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the border size.
     *
     * @param newValue The new border size, in pixels.
     */
    public void setBorderSize(int newValue) {
        int oldValue = borderSize;
        borderSize = newValue;
        firePropertyChange(BORDER_SIZE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the border style.
     *
     * @param newValue The new border style, one of the following values:
     *        <ul>      
     *        <li>BORDER_DEFAULT - The client's default style (the default).</li>
     *        <li>BORDER_SOLID - A solid single color border.</li>
     *        <li>BORDER_INSET - A recessed 3D appearance.</li>
     *        <li>BORDER_OUTSET - A raised 3D appearance.</li>
     *        </ul>
     */
    public void setBorderStyle(int newValue) {
        int oldValue = borderStyle;
        borderStyle = newValue;
        firePropertyChange(BORDER_STYLE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the document model.
     *
     * @param newValue The new document model.
     */
    public void setDocument(Document newValue) {
        Document oldValue = document;
        if (oldValue != null) {
            oldValue.removeDocumentListener(documentHandler);
        }
        document = newValue;
        if (newValue != null) {
            newValue.addDocumentListener(documentHandler);
        }
        firePropertyChange(DOCUMENT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets whether the text component is editable.
     *
     * @param newValue True if the text component's value may be changed.
     */
    public void setEditable(boolean newValue) {
        boolean oldValue = editable;
        editable = newValue;
        firePropertyChange(EDITABLE_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * Sets the horizontal alignment of the text.
     *
     * @param newValue The horizontal alignment of the text,
     *        one of the following values:
     *        <ul>
     *        <li>EchoConstants.LEFT (the default)</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     */
    public void setHorizontalAlignment(int newValue) {
        int oldValue = horizontalAlignment;
        horizontalAlignment = newValue;
        firePropertyChange(HORIZONTAL_ALIGNMENT_CHANGED_PROPERTY, oldValue, newValue);
    }

    /**
     * Sets the text contained in the document model.  If this text 
     * component's document model is null, throws a NullPointerException.
     *
     * @param newValue The text to replace the text in the document model.
     */
    public void setText(String newValue) {
        String oldValue = document.getText();
        document.setText(newValue);
        firePropertyChange(TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }
    
    /**
     * @see nextapp.echo.ToolTipSupport#setToolTipText(String)
     */
    public void setToolTipText(String newValue) {
        String oldValue = toolTipText;
        toolTipText = newValue;
        firePropertyChange(TOOL_TIP_TEXT_CHANGED_PROPERTY, oldValue, newValue);
    }
}
