/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echo.util;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * A limited implementation of a variable-length array of integers.
 * <br><br>
 * Because the get() method of this object returns a primitive integer
 * type, the user must specify a value that should be returned in the
 * even no value is contained at the specified index when it is called.
 * This value is permanently set at instantiation.<br>
 * <br>
 * This class is VERY inefficient when used to store only a few values
 * at high indices.  This object will allocate an array with containers
 * at values from 0 to the highest set index.<br>
 * <br>
 * <b>This class is a supporting class for the Echo framework.  It is not
 * intended for use by applications.</b>
 */
public class DynamicIntegerArray
implements Serializable {

    private List data = null;
    private int nullValue;

    /**
     * Creates a new DynamicIntegerArray with a &quot;null value&quot; of 0.
     */
    public DynamicIntegerArray() {
        this(0);
    }

    /**
     * Creates a new DynamicIntegerArray with the specified &quot;null 
     * value&quot;.
     *
     * @param nullValue The value that is used to indicate null values.
     */
    public DynamicIntegerArray(int nullValue) {
        super();
        this.nullValue = nullValue;
    }

    /**
     * Returns true if the array contains a value at the specified index.
     *
     * @param index The array index to analyze.
     * @return True if the array contains a value at the specified index.
     */
    public boolean contains(int index) {
        return data != null && index < data.size() && data.get(index) != null;
    }
    
    /**
     * Returns the value in the array at the specified index.
     *
     * @param index The index of the value to be returned.
     * @return The value in the array at the specified index.  If no value
     *         exists at the specified index, -1 is returned.
     */
    public int get(int index) {
        if (data == null) {
            return nullValue;
        } else if (index < data.size()) {
            Integer integer = (Integer) data.get(index);
            if (integer == null) {
                return nullValue;
            } else {
                return integer.intValue();
            }
        } else {
            return nullValue;
        }
    }
    
    /**
     * Sets the value at the specified index to the given value.
     *
     * @param index The index in the array to be modified.
     * @param value The new value to be placed at the specified index.
     */
    public void set(int index, int value) {
        if (data == null) {
            data = new ArrayList();
        }
        Integer integer = new Integer(value);
        while (index >= data.size()) {
            data.add(null);
        }
        data.set(index, integer);
    }
}
