/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.Iterator;
import java.util.MissingResourceException;

import nextapp.echoservlet.util.PropertyMap;

/**
 * Configuration properties for an application.
 */
public class ApplicationProperties 
implements Serializable {

    private static final String DEFAULT_PROPERTIES_FILE = "nextapp.echoservlet.EchoServer";
    
    public static final String STARTUP_WINDOW_TITLE = "startupWindowTitle";
    public static final String STARTUP_WINDOW_TEXT = "startupWindowText";
    public static final String STARTUP_NO_SCRIPT_ERROR_TEXT = "startupNoScriptErrorText";
    public static final String SPLASH_SCREEN_URI = "splashScreenUri";
    public static final String STARTUP_DELAY = "startupDelay";
    public static final String DEFAULT_PARAMETER_ENCODING = "defaultParameterEncoding";
    public static final String FIX_PARAMETER_ENCODING = "fixParameterEncoding";
    public static final String DEBUG_OPTIONS_ENABLED = "debugOptionsEnabled";
    public static final String DEBUG_VERBOSE_LOGGING = "debugVerboseLoggingEnabled";
    public static final String DEBUG_INTERACTIVE_CONTROLLER = "debugInteractiveControllerEnabled";
    public static final String DEBUG_PERFORMANCE_LOGGING = "debugPerformanceLoggingEnabled";
    public static final String DISPLAY_CLIENT_ERROR_FEEDBACK = "displayClientErrorFeedback";
    public static final String CLIENT_ERROR_MESSAGE_RESOURCE = "clientErrorMessageResource";
    public static final String DISABLE_ONUNLOAD = "disableOnunload";
    public static final String DISABLE_BACK_BUTTON_DISABLE_SUPPORT = "disableBackButtonDisableSupport";
    
    private PropertyMap properties;
    
    /**
     * Creates a new <code>ApplicationProperties</code> object for the 
     * specified class name.  The <code>ApplicationProperties</code> object
     * will attempt to retrieve a correspondingly named .properties file for
     * the class located in the same package and with the same name as the 
     * class.
     *
     * @param className The fully qualified name of the class for which the
     *        <code>ApplicationProperties</code> object is being created.
     */
    ApplicationProperties(String className) {
        super();

        properties = new PropertyMap(DEFAULT_PROPERTIES_FILE);
        try {
            PropertyMap customProperties = new PropertyMap(className);
            Iterator it = customProperties.keySet().iterator();
            while (it.hasNext()) {
                Object o = it.next();
                properties.put(o, customProperties.get(o));
            }
        } catch (MissingResourceException ex) { }
    }

    /**
     * Returns the value of the specified property as a String.
     *
     * @param key The key of the property to be returned.
     * @return The value of the property as a String.
     */
    public String get(String key) {
        return properties.getString(key);
    }
    
    /**
     * Returns the value of the specified property as a boolean.
     *
     * @param key The key of the property to be returned.
     * @return The value of the property as a boolean.
     */
    public boolean getBoolean(String key) {
        return properties.getBoolean(key);
    }
    
    /**
     * Returns the value of the specified property as an integer.
     *
     * @param key The key of the property to be returned.
     * @return The value of the property as an integer.
     */
    public int getInteger(String key) {
        return properties.getInteger(key);
    }
    
    /**
     * Sets a String property value.
     * 
     * @param key The key of the property to be set.
     * @param value The new value of the property.
     */
    public void set(String key, String value) {
        properties.put(key, value);
    }
    
    /**
     * Sets a boolean property value.
     * 
     * @param key The key of the property to be set.
     * @param value The new value of the property.
     */
    public void setBoolean(String key, boolean value) {
        properties.putBoolean(key, value);
    }
    
    /**
     * Sets an integer property value.
     * 
     * @param key The key of the property to be set.
     * @param value The new value of the property.
     */
    public void setInteger(String key, int value) {
        properties.putInteger(key, value);
    }
}
