/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/**
 * General-purpose constants and class methods used in the generation 
 * of HTML and JavaScript code for the client browser.
 */
public class ClientObjects {

    /**
     * The standard prefix that is prepended to all client-side variables and 
     * method names.  Appending this standard prefix is necessary--as browsers 
     * continually expand their built-in JavaScript capabilities--to avoid 
     * method conflicts.  This does have a negative effect on the aesthetics of
     * JavaScript code, but it has proven to be very necessary such that Echo 
     * doesn't have to be updated every time a browser company issues a 
     * service pack.
     */
    public static final String STD_PREFIX = "E_";
    
    /**
     * The standard prefix prepended to component Ids.
     */
    public static final String COMPONENT_ID_PREFIX = STD_PREFIX + "C_";
    
    /**
     * The standard prefix prepended to image names.
     */
    public static final String IMAGE_NAME_PREFIX = STD_PREFIX + "IMG_";
    
    /**
     * The standard prefix prepended to Ids.
     */
    public static final String PARAMETER_ID = STD_PREFIX + "id";

    /**
     * The name of the JavaScript Controller frame.
     */
    public static final String CONTROLLER_FRAME_NAME = STD_PREFIX + "cf";
    
    /**
     * Returns the JavaScript name of the component identified by the given Id.
     *
     * @param id The Id of the ComponentPeer.
     * @return The component's JavaScript name.
     */
    public static String getObjectName(Id id) {
        return COMPONENT_ID_PREFIX + id;
    }
    
    /**
     * Returns the JavaScript name of the image identified by the given Id.
     *
     * @param id The Id of the image.
     * @return The image's JavaScript name.
     */
    public static String getImageName(Id id) {
        return IMAGE_NAME_PREFIX + id;
    }
    
    /**
     * Returns the URI of the specified service in relation to the specified 
     * Connection.
     */
    public static String getServiceUri(Connection conn, Service service) {
        if (service instanceof CacheableService) {
            return conn.getApplicationUri() + "?" + PARAMETER_ID + "=" + service.getId() 
                    + "&E_z=" + conn.getCacheIdPrefix() + ((CacheableService) service).getVersion();
        } else {
            return conn.getApplicationUri() + "?" + PARAMETER_ID + "=" + service.getId()
                    + "&E_z=" + conn.getCacheIdPrefix() + System.currentTimeMillis();
        }
    }
    
    /**
     * Adds a hidden parameter field to the form containing the service Id.
     *
     * @param conn The connection to which the rendered HTML document that
     *             contains the specified form <code>Element</code> will be 
     *             written.
     * @param service The service that requires a hidden parameter field.
     * @param form The form that will contain the hidden parameter field.
     */
    public static void addServiceIdFormElement(Connection conn, Service service, Element form) {
        Element input = new Element(ElementNames.INPUT, false);
        input.addAttribute("type", "hidden");
        input.addAttribute("name", PARAMETER_ID);
        input.addAttribute("value", service.getId().toString());
        form.add(input);
    }
    
    /**
     * Returns a query parameter used in a URI specifying a service.
     *
     * @param service The service which is to be requested.
     * @return A parameter usable in a URI to specify the given service.
     *         The parameter will be returned as a string, such as "e_id=25ae".
     */
    static String getServiceUriQuery(Service service) {
        return PARAMETER_ID + "=" + service.getId();
    }
    
    /** Non-instantiable class. */
    private ClientObjects() { }
}
