/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import nextapp.echoservlet.util.Version;

/**
 * A repository of information gathered from a user's client browser.
 * This object contains such information as the width and height of 
 * the user's screen and his/her browser make and version.
 */
public final class ClientProperties 
implements Serializable {

    /**
     * An app name substring that identifies Netscape browsers.
     */
    private static final String NETSCAPE = "Netscape";
    
    /** 
     * An app name substring that identifies Microsoft Internet 
     * Explorer browsers. 
     */ 
    private static final String MSIE = "Microsoft Internet Explorer";
    
    /** 
     * An app name substring that identifies Opera browsers. 
     */ 
    private static final String OPERA = "Opera";
    
    /**
     * The property name referencing the client's accepted response content 
     * types.
     */
    public static final String ACCEPT = "accept";
    
    /**
     * The property name referencing the client's accepted character sets.
     */
    public static final String ACCEPT_CHARSET = "acceptCharSet";
    
    /**
     * The property name referencing the client's accepted character encodings.
     */
    public static final String ACCEPT_ENCODING = "acceptEncoding";
    
    /**
     * The property name referencing the client's accepted languages.
     */
    public static final String ACCEPT_LANGUAGE = "acceptLanguage";
    
    /** 
     * The property name referencing the browser's application name. 
     */
    public static final String APP_NAME = "appName";

    /** 
     * The property name referencing the browser's version string. 
     */
    public static final String APP_VERSION = "appVersion";
    
    /**
     * The property name referencing whether the browser will allow the
     * back button to be effectively disabled by placing a history.go(1)
     * method call in the controller frame.
     */
    public static final String BACK_BUTTON_DISABLE_SUPPORT = "backButtonDisableSupport";

    /** 
     * The property name referencing the browser's &quot;chrome 
     * height&quot;. 
     */
    public static final String CHROME_HEIGHT = "chromeHeight";

    /** 
     * The property name referencing the browser's &quot;chrome 
     * width&quot;. 
     */
    public static final String CHROME_WIDTH = "chromeWidth";

    /** 
     * The property name referencing the client machine's video color 
     * depth. 
     */
    public static final String COLOR_DEPTH = "colorDepth";

    /** 
     * The property name referencing a boolean indicator as to whether the
     * browser supports dynamic modification of CSS styles.
     */
    public static final String CSS_MODIFICATION_SUPPORT = "cssModificationSupport";
    
    /**
     * The property name referencing a boolean indicator as to whether the
     * browser properly supports setting alignment properties on tables cells.
     * This property exists because Mozilla 1.0-1.2b (and possibly higher) and 
     * Opera 6.0 (and possibly higher) do not respect table cell alignment
     * settings when the cell contains another table.  Therefore alignment
     * attributes will be rendered directly in table cell elements instead of 
     * in CSS when such support is lacking.
     */
    public static final String CSS_TABLE_CELL_ALIGNMENT_SUPPORT = "cssTableCellAlignmentSupport";
        
    /** 
     * The property name referencing a boolean indicator as to whether the
     * browser supports exception handling through try/catch blocks.
     */
    public static final String EXCEPTION_SUPPORT = "exceptionSupport";
        
    /**
     * The property name referencing the hostname the client is contacting.
     */
    public static final String HOST = "host";
    
    /** 
     * The property name referencing the height of the initial browser 
     * window.
     */
    public static final String INITIAL_WINDOW_HEIGHT = "initialWindowHeight";

    /** 
     * The property name referencing the width of the initial browser 
     * window.
     */
    public static final String INITIAL_WINDOW_WIDTH = "initialWindowWidth";

    /** 
     * The property name referencing whether the initial browser window
     * had a menu bar.
     */
    public static final String INITIAL_WINDOW_HAS_MENU_BAR = "initialWindowHasMenuBar";
    
    /** 
     * The property name referencing whether the initial browser window
     * had a status bar.
     */
    public static final String INITIAL_WINDOW_HAS_STATUS_BAR = "initialWindowHasStatusBar";
    
    /** 
     * The property name referencing whether the initial browser window
     * had a tool bar.
     */
    public static final String INITIAL_WINDOW_HAS_TOOL_BAR = "initialWindowHasToolBar";

    /**
     * The property name referencing the browser's highest supported
     * JavaScript version.
     */
    public static final String JS_VERSION = "jsVersion";
    
    /**
     * The property name referencing the URI that referred the client browser
     * to the Echo application.  Note that the actual HTTP header is spelled
     * "referer", but was misspelled in the HTTP specification.
     */
    public static final String REFERRER = "referrer";
    
    /**
     * The property name referencing the client machine's IP Address
     */
    public static final String REMOTE_ADDR = "remoteAddr";

    /**
     * The property name referencing the client machine's fully qualified host 
     * name.
     */
    public static final String REMOTE_HOST = "remoteHost";

    /** 
     * The property name referencing the client machine's vertical screen 
     * resolution.
     */
    public static final String SCREEN_HEIGHT = "screenHeight";

    /** 
     * The property name referencing the client machine's horizontal screen 
     * resolution.
     */
    public static final String SCREEN_WIDTH = "screenWidth";
    
    /** 
     * The property name referencing the client machine's time zone offset.
     * This property indicates the difference in minutes between GMT (UTC) and
     * the client's time.
     */
    public static final String TIME_ZONE_OFFSET = "timeZoneOffset";

    /**
     * The property name referencing the client machine's user agent.
     */
    public static final String USER_AGENT = "userAgent";

    /** A property value representing boolean true. */
    public static final String TRUE = "1";

    /** A property value representing boolean false. */
    public static final String FALSE = "0";

    /**
     * A mapping of property names to property values.
     */
    private Map properties = new HashMap();
    
    /**
     * Creates a new client properties object.  The connection must wrap 
     * the HttpRequest containing the form data submitted by the client 
     * detection page.
     *
     * @param conn A connection that wraps the HttpRequest containing the form
     *        data submitted by the client detection page.
     */
    ClientProperties(Connection conn) {
        super();
        
        // Store properties that will be further analyzed locally.
        String userAgent = conn.getRequest().getHeader("User-Agent");
        String appVersion = conn.getParameter(ClientObjects.STD_PREFIX + APP_VERSION);
        String appName = conn.getParameter(ClientObjects.STD_PREFIX + APP_NAME);

        boolean isNetscape4 = NETSCAPE.equals(appName) && 
                Version.meetsRequirement("4", appVersion) && ! Version.meetsRequirement("5", appVersion);
        boolean isOpera = userAgent != null && userAgent.indexOf(OPERA) != -1;
        boolean isInternetExplorer = MSIE.equals(appName) || userAgent.indexOf("MSIE") != -1;
        
        properties.put(REMOTE_ADDR,     conn.getRequest().getRemoteAddr());
        properties.put(REMOTE_HOST,     conn.getRequest().getRemoteHost());
        properties.put(ACCEPT,          conn.getRequest().getHeader("Accept"));
        properties.put(ACCEPT_CHARSET,  conn.getRequest().getHeader("Accept-Charset"));
        properties.put(ACCEPT_ENCODING, conn.getRequest().getHeader("Accept-Encoding"));
        properties.put(ACCEPT_LANGUAGE, conn.getRequest().getHeader("Accept-Language"));
        properties.put(HOST,            conn.getRequest().getHeader("Host"));
        properties.put(REFERRER,        conn.getRequest().getHeader("Referer"));
        properties.put(USER_AGENT,      userAgent);
        
        properties.put(APP_NAME, appName);
        properties.put(APP_VERSION, appVersion);
        properties.put(CHROME_HEIGHT, conn.getParameter(ClientObjects.STD_PREFIX + CHROME_HEIGHT));
        properties.put(CHROME_WIDTH, conn.getParameter(ClientObjects.STD_PREFIX + CHROME_WIDTH));
        properties.put(COLOR_DEPTH, conn.getParameter(ClientObjects.STD_PREFIX + COLOR_DEPTH));
        properties.put(SCREEN_HEIGHT, conn.getParameter(ClientObjects.STD_PREFIX + SCREEN_HEIGHT));
        properties.put(SCREEN_WIDTH, conn.getParameter(ClientObjects.STD_PREFIX + SCREEN_WIDTH));
        properties.put(INITIAL_WINDOW_HAS_MENU_BAR, conn.getParameter(ClientObjects.STD_PREFIX + INITIAL_WINDOW_HAS_MENU_BAR));
        properties.put(INITIAL_WINDOW_HAS_STATUS_BAR, conn.getParameter(ClientObjects.STD_PREFIX + INITIAL_WINDOW_HAS_STATUS_BAR));
        properties.put(INITIAL_WINDOW_HAS_TOOL_BAR, conn.getParameter(ClientObjects.STD_PREFIX + INITIAL_WINDOW_HAS_TOOL_BAR));
        properties.put(INITIAL_WINDOW_HEIGHT, conn.getParameter(ClientObjects.STD_PREFIX + INITIAL_WINDOW_HEIGHT));
        properties.put(INITIAL_WINDOW_WIDTH,  conn.getParameter(ClientObjects.STD_PREFIX + INITIAL_WINDOW_WIDTH));
        properties.put(JS_VERSION, conn.getParameter(ClientObjects.STD_PREFIX + JS_VERSION));
        properties.put(TIME_ZONE_OFFSET, conn.getParameter(ClientObjects.STD_PREFIX + TIME_ZONE_OFFSET));

        // If browser version = Netscape 4.x, assume JavaScript exceptions are not supported, otherwise assume they are.
        properties.put(EXCEPTION_SUPPORT, isNetscape4 ? FALSE : TRUE);
        
        // Only MSIE and Netscape4 seem to obey CSS table alignment attributes when tables are inside of other tables.
        properties.put(CSS_TABLE_CELL_ALIGNMENT_SUPPORT, (!isOpera && (isInternetExplorer || isNetscape4)) ? TRUE : FALSE);
        
        // Opera does not like history.foward() (tested w/ 7.23).
        properties.put(BACK_BUTTON_DISABLE_SUPPORT, isOpera ? FALSE : TRUE);
        
        // NS4, Opera do not support CSS modification (e.g. as used by Button renderers).
        properties.put(CSS_MODIFICATION_SUPPORT, (isNetscape4 || isOpera) ? FALSE : TRUE);
    }
    
    /**
     * Returns the specified client property.
     *
     * @param name The name of the property.  Use the constants provided in 
     *        this class.
     * @return The value of the property.
     */
    public String get(String name) {
        return (String) properties.get(name);
    }
    
    /**
     * Returns the specified client property as a boolean value.  True is 
     * returned in the event that the string value of the property is equal
     * to "1".
     *
     * @param name The name of the property.  Use the constants provided in
     *        this class.
     * @return The boolean value of the property.
     */
    public boolean getBoolean(String name) {
        return TRUE == properties.get(name);
    }
    
    /**
     * Outputs all data held in this ClientProperties object.  
     * Useful only for debugging or logging purposes.
     *
     * @return A string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        String key;
        
        for (Iterator it = properties.keySet().iterator(); it.hasNext();) {
             key = (String) it.next();
             sb.append(key);
             sb.append("=");
             sb.append(properties.get(key));
             if (it.hasNext()) {
                 sb.append(", ");
             }
        }
        
        return sb.toString();
    }
}
