/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.PrintWriter;
import java.io.Serializable;

import nextapp.echo.AbstractPane;
import nextapp.echo.Component;
import nextapp.echo.EchoInstance;
import nextapp.echo.Window;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.util.ContentType;

/**
 * A debugging utility which displays a hierarchal view of the component 
 * hierarchy.
 */
final class ComponentHierarchyViewer
implements Serializable, Service {

    private static final Id ID = new Id(ClientObjects.STD_PREFIX + "chview");

    /**
	 * @see nextapp.echoservlet.Service#getId()
	 */
	public Id getId() {
        return ID;
    }

    /**
     * Renders information about a particular component, and then recursively 
     * calls itself on the component's children.
     */
    private void renderComponentData(Connection conn, Element parent, Component component) {
    
        Class echoClass = component.getClass();
        while (! echoClass.getName().startsWith("nextapp.echo.")) {
            echoClass = echoClass.getSuperclass();
        }
    
        parent.addHtml("<b>" + echoClass.getName().substring("nextapp.echo.".length()) + "</b>");
        parent.addText("(" + component.getClass().getName() + ")");
        parent.addText(component.isVisible() ? "[visible]" : "[hidden]");

        try {
            ComponentPeer ui = conn.getInstancePeer().getPeer(component);
            parent.addText(" [peerId=" + ui.getId() + "]");
            if (ui instanceof PaneUI) {
                parent.addHtml("<br>");
                Element link = new Element(ElementNames.A);
                link.setWhitespaceRelevant(true);
                String uri = ((PaneUI) ui).getUri(conn);
                link.addAttribute("href", uri);
                link.addText(uri);
                parent.add(link);
                if (ui instanceof ContainerPaneUI) {
                    renderContainerPaneUIData(conn, parent, (ContainerPaneUI) ui);
                }
            }
        } catch (EchoServletException ex) {
            parent.addText(" [unregistered]");
        }


        Component[] children = component.getComponents();
        if (children.length > 0) {
            Element ul = new Element("ul");
            parent.add(ul);
        
            for (int index = 0; index < children.length; ++index) {
                Element li = new Element("li");
                renderComponentData(conn, li, children[index]);
                ul.add(li);
            }
        }
    }
    
    /**
     * Renders special data pertinent to container panes.
     */
    private void renderContainerPaneUIData(Connection conn, Element parent, ContainerPaneUI ui) {
        PaneUI forwardingUI = ui.getForwardingPane();
        if (ui != forwardingUI) {
            parent.addHtml("<br>");
            parent.addText("Forwarding to: " + forwardingUI.getId());
        }
    }

    /**
	 * @see nextapp.echoservlet.Service#service(nextapp.echoservlet.Connection)
	 */
	public void service(Connection conn) {
        HtmlDocument doc = HtmlDocument.create(conn);
        
        Element title = new Element(ElementNames.TITLE);
        title.addText("Echo Component Hierarchy Viewer");
        doc.getHeadElement().add(title);
        
        InstancePeer instancePeer = conn.getInstancePeer();
        EchoInstance instance = instancePeer.getInstance();
        
        Element windowList = new Element("ul");
        Window[] registeredWindows = instance.getWindows();
        for (int index = 0; index < registeredWindows.length; ++index) {
            StringBuffer windowDescription = new StringBuffer("Window");
            try {
                WindowUI windowUI = (WindowUI) instancePeer.getPeer(registeredWindows[index]);
                windowDescription.append(" peer id=" + windowUI.getId());
            } catch (EchoServletException ex) {
                windowDescription.append(" no peer");
            }
            windowDescription.append(registeredWindows[index].isVisible() ? " [visible]" : " [hidden]");
            windowDescription.append(" title=\"");
            windowDescription.append(registeredWindows[index].getTitle());
            windowDescription.append("\"");
            
            AbstractPane pane = registeredWindows[index].getContent();
            
            Element window = new Element("li");
            window.addText(windowDescription.toString());

            Element ul = new Element("ul");
            Element li = new Element("li");
            ul.add(li);
            window.add(ul);

            if (pane == null) {
                li.addText("empty");
            } else {
                renderComponentData(conn, li, pane);
            }

            windowList.add(window);
        }
        doc.getBodyElement().add(windowList);

        // render
        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }
}
