/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nextapp.echo.Color;
import nextapp.echo.Component;
import nextapp.echo.Font;
import nextapp.echoservlet.html.Element;

/** 
 * The base class from which all component peers are derived.
 */
public abstract class ComponentPeer
implements Serializable {

    /** The Id of the component peer. */
    private Id id = null;
    
    /** The <code>Component</code> that this peer represents. */
    private Component component = null;
    
    /** The <code>InstancePeer</code> of the application. */
    private InstancePeer instancePeer = null;
    
    /** A container of this peers ancillary services. */
    private Set ancillaryServices = null;
    
    /**
     * Creates a new <code>ComponentPeer</code>.
     */
    public ComponentPeer() {
        super();
    }
    
    /**
     * Adds an ancillary service to this <code>ComponentPeer</code>.  
     * Ancillary services will be registered for the life of the 
     * <code>ComponentPeer</code> and automatically removed when it is
     * unregistered.  An ancillary service should be used by ONLY one
     * peer, as the peer will register and unregister it from
     * the <code>ServiceRegistry</code> as required.
     *
     * @param service The ancillary service to be added.
     */
    public void addAncillaryService(Service service) {
        if (ancillaryServices == null) {
            ancillaryServices = new HashSet();
        }
        
        ancillaryServices.add(service);
        if (instancePeer != null) {
            instancePeer.getServiceRegistry().add(service, getAncillaryServiceAlias());
        }
    }
    
    /**
     * Returns a unique identifier for an ancillary service.
     *
     * @return The alias that should be used for the ancillary service.
     */
    private String getAncillaryServiceAlias() {
        return "_Ancillary." + getId();
    }
    
    /**
     * Returns the background color in which the component will be
     * rendered.  The color will be determined by examining the represented 
     * component and then its ancestors until a background color can be 
     * determined.
     *
     * @return The background color in which the component will be
     *         rendered.
     */
    public Color getBackground() {
        Component testComponent = component;
        Color background = null;
        
        while (testComponent != null && background == null) {
            background = testComponent.getBackground();
            testComponent = testComponent.getParent();
        }
        
        return background;
    }
    
    public Id generateId() {
        return instancePeer.generateId();
    }
    
    /**
     * Returns the number of visible child components contained in
     * this component.
     *
     * @return The number of visible child components.
     */
    protected int getChildCount() {
        int count = 0;
        Component[] components = component.getComponents();
        for (int index = 0; index < components.length; ++index) {
            if (components[index].isVisible()) {
                ++count;
            }
        }
        return count;
    }
    
    /**
     * Returns the peers for all of the referenced component's visible 
     * children.
     *
     * @return The peers of all visible child components of this peer's
     *         represented component.
     */
    protected ComponentPeer[] getChildren() {
        // Determine visible component count
        Component[] components = component.getComponents();
        int visibleComponentCount = 0;
        for (int index = 0; index < components.length; ++index) {
            if (components[index].isVisible()) {
                ++visibleComponentCount;
            }
        }
        
        // Create array of visible component peers.
        int componentIndex = 0;
        int peerIndex = 0;
        ComponentPeer[] peers = new ComponentPeer[visibleComponentCount];
        while (peerIndex < peers.length) {
            if (components[componentIndex].isVisible()) {
                peers[peerIndex] = instancePeer.getPeer(components[componentIndex]);
                ++peerIndex;
            }
            ++componentIndex;
        }
        
        return peers;
    }
    
    /**
     * Returns the component that is handled by this peer.
     *
     * @return The component that is handled by this peer.
     */
    public Component getComponent() {
        return component;
    }
    
    public String getFocusedElementId() {
        return getId().toString();
    }
    
    /**
     * Returns the font in which the component will be
     * rendered.  The font will be determined by examining the represented 
     * component and then its ancestors until a font can be 
     * determined.
     *
     * @return The font in which the component will be rendered.
     */
    public Font getFont() {
        Component testComponent = component;
        Font font = null;
        
        while (testComponent != null && font == null) {
            font = testComponent.getFont();
            testComponent = testComponent.getParent();
        }
        
        return font;
    }
    
    /**
     * Returns the foreground color in which the component will be
     * rendered.  The color will be determined by examining the represented 
     * component and then its ancestors until a foreground color can be 
     * determined.
     *
     * @return The foreground color in which the component will be
     *         rendered.
     */
    public Color getForeground() {
        Component testComponent = component;
        Color foreground = null;
        
        while (testComponent != null && foreground == null) {
            foreground = testComponent.getForeground();
            testComponent = testComponent.getParent();
        }
        
        return foreground;
    }
    
    /**
     * Returns the <code>Id</code> by which this peer is identified on the 
     * client.
     *
     * @return The <code>Id</code> by which this peer is identified on the 
     *         client.
     */
    public Id getId() {
        return id;
    }
    
    /**
     * Returns the <code>InstancePeer<code> with which this peer is associated.
     * If the peer is unregistered, null will be returned.
     *
     * @return The <code>InstancePeer<code> with which this peer is associated.
     */
    public InstancePeer getInstancePeer() {
        return instancePeer;
    }
    
    /**
     * Returns the peer of the associated component's component.
     *
     * @return The peer of the associated component's component.
     */
    protected ComponentPeer getParent() {
        Component parent = component.getParent();
        ComponentPeer parentPeer;
        
        if (parent == null) {
            parentPeer = null;
        } else {
            parentPeer = instancePeer.getPeer(parent);
        }

        return parentPeer;
    }
    
    /**
     * Returns the peer of the specified component.
     *
     * @param component The component whose peer is to be returned.
     * @return The peer of the specified component.
     */
    protected ComponentPeer getPeer(Component component) {
        return instancePeer.getPeer(component);
    }
    
    /**
     * Returns the tab index of the component, recursively querying parent
     * components for the information if the tab index of a component is
     * not set.
     * 
     * @return The tab index with which the component should be rendered.
     */
    public int getTabIndex() {
        Component testComponent = component;
        int tabIndex = testComponent.getTabIndex();
        while (tabIndex == 0) {
            testComponent = testComponent.getParent();
            if (testComponent == null) {
                return 0; 
            }
            tabIndex = testComponent.getTabIndex();
            if (tabIndex != 0) {
                return tabIndex;
            }
        }
        return component.getTabIndex();
    }
    
    /**
     * Requests that the component be redrawn on the client.
     */
    public void redraw() {
        if (instancePeer != null) {
            instancePeer.redraw(this);
        }
    }

    /**
     * Registers all ancillary services used by this component peer
     * with the application container's service registry.
     */
    void registerAncillaryServices() {
        if (ancillaryServices != null) {
            Iterator it = ancillaryServices.iterator();
            ServiceRegistry services = instancePeer.getServiceRegistry();
            String alias = getAncillaryServiceAlias();

            while (it.hasNext()) {
                services.add((Service) it.next(), alias);
            }
        }
    }
    
    /**
     * Called when the component is registered.
     * This method should be overridden if necessary.
     */
    protected void registered() { }
    
    /**
     * Removes an ancillary service from this <code>ComponentPeer</code>.  
     * Ancillary services will be registered for the life of the 
     * <code>ComponentPeer</code> and automatically removed when it is
     * unregistered.  An ancillary service should be used by ONLY one
     * peer, as the peer will register and unregister it from
     * the <code>ServiceRegistry</code> as required.
     *
     * @param service The ancillary service to be removed.
     */
    public void removeAncillaryService(Service service) {
        if (ancillaryServices != null) {
            ancillaryServices.remove(service);
            if (instancePeer != null) {
                instancePeer.getServiceRegistry().remove(service, getAncillaryServiceAlias());
            }
            if (ancillaryServices.size() == 0) {
                ancillaryServices = null;
            }
        }
    }
    
    /**
     * A method that should be overridden for the ComponentPeer to produce 
     * HTML output.
     *
     * @param rc A rendering context provided by the <code>Connection</code>.
     * @param parent The element that will contain content rendered by this
     *        <code>ComponentPeer</code>.
     */
    public void render(RenderingContext rc, Element parent) { }

    /**
     * Sets the component with which the peer will be associated.
     * This method is only called once, by the 
     * <code>ComponentPeerFactory</code> object.
     *
     * @param component The <code>Component</code> that this peer represents.
     */
    void setComponent(Component component) {
        this.component = component;
    }
    
    /**
     * Sets the <code>Id</code> of the component.  This method is only called 
     * once, by the <code>ComponentPeerFactory</code>.
     *
     * @param id The <code>Id</code> of the peer.
     */
    void setId(Id id) {
        this.id = id;
    }
    
    /**
     * Sets the instance (peer) with which the peer will be associated.
     *
     * @param instancePeer The <code>InstancePeer</code> with which this 
     *        <code>ComponentPeer</code> will be associated.
     */
    void setInstancePeer(InstancePeer instancePeer) {
        this.instancePeer = instancePeer;
    }
    
    /**
     * Unregisters all ancillary services used by this component peer
     * from the application container's service registry.
     */
    void unregisterAncillaryServices() {
        if (ancillaryServices != null) {
            Iterator it = ancillaryServices.iterator();
            ServiceRegistry services = instancePeer.getServiceRegistry();
            String alias = getAncillaryServiceAlias();

            while (it.hasNext()) {
                services.remove((Service) it.next(), alias);
            }
        }
    }
    
    /**
     * Called when the component is unregistered.
     * This method should be overridden if necessary.
     */
    protected void unregistered() { }
}
