/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import nextapp.echo.Component;

import nextapp.echoservlet.util.PeerFactory;

/**
 * Creates <code>ComponentPeer</code> objects for <code>Components</code>.
 */
final class ComponentPeerFactory {

    /**
     * The <code>PeerFactory</code> used to create new 
     * <code>ComponentPeer</code> instances.
     */
    private static final PeerFactory factory;
    
    static {
        try {
            factory = new PeerFactory("nextapp.echoservlet.ComponentPeerFactory");
        } catch (PeerFactory.PeerFactoryException ex) {
            throw new EchoServletException("Unable to load default component peer bindings resource file.", ex);
        }
    }
    
    /**
     * Returns a new peer for the specified component.  The peer must be
     * registered before it may be used.
     * 
     * @param c The <code>Component</code> for which a peer is to be created.
     * @return A <code>ComponentPeer</code> bound to the specified 
     *         <code>Component</code>.
     */
    static ComponentPeer createComponentPeer(Component c) {
        ComponentPeer peer;
        try {
            peer = (ComponentPeer) factory.createPeer(c);
        } catch (PeerFactory.PeerFactoryException ex) {
            // Because the Component class has a peer (ComponentUI), this exception should never occur.
            throw new EchoServletException("Unable to create ComponentPeer class for Component: \"" 
                    + c.getClass().getName() + "\".", ex);
        }
        peer.setComponent(c);
        return peer;
    }
    
    /**
     * Adds a single <code>Component</code>-to-<code>ComponentPeer</code>
     * association.
     * 
     * @param className The fully qualified name of the <code>Component</code> 
     *        class.
     * @param peerClassName The fully qualified name of the
     *        <code>ComponentPeer</code> class.
     */
    static void addAssociation(String className, String peerClassName) {
        factory.addAssociation(className, peerClassName);
    }
    
    /**
     * Adds additional <code>Component</code>-to-<code>ComponentPeer</code>
     * associations.
     *
     * @param propertiesFile The name of a properties file containing 
     *        additional associations.
     */
    static void addAssociations(String propertiesFile) {
        try {
            factory.addAssociations(propertiesFile);
        } catch (PeerFactory.PeerFactoryException ex) {
            throw new EchoServletException("Unable to load associations file: \"" + propertiesFile + "\".", ex);
        }
    }
    
    /** Non-instantiable class. */
    private ComponentPeerFactory() { }
}
