/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import nextapp.echo.Color;
import nextapp.echo.EchoConstants;
import nextapp.echo.Font;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Style;
import nextapp.echoservlet.util.Delimiter;

/**
 * A CSS style handler for a <code>ComponentPeer</code>. <br>
 * <br>
 * This class has an unusual design as it is intended to minimize the number of
 * CSS style definitions generated for a given HTML document.  Minimizing the 
 * number of styles is necessary as, if an independent style were to be 
 * generated for each and every component, more complex documents would have a
 * ridiculous quantity of CSS code.<br>
 * <br>
 * To create a <code>ComponentPeer</code> that will render a component using a 
 * CSS style generated by <code>ComponentStyle</code> you must follow this
 * procedure:
 * <ol>
 * <li>Create a <code>ComponentStyle</code> for your 
 * <code>ComponentPeer</code>:
 * <ul><li>
 * <code>ComponentStyle style = 
 * ComponentStyle.forComponent(this);</code></li></ul></li>
 * <li>Add additional attributes to the style if necessary:
 * <ul><li>
 * <code>style.addAttribute(&quot;attribute-name&quot;, 
 * &quot;attribute-value&quot;);</code></li></ul></li>
 * <li>Add any element types that will use the style:
 * <ul><li>
 * <code>style.addElementType(ElementNames.SPAN)</code></li></ul></li>
 * <li>Add the style to the <code>HtmlDocument</code> in which your component's
 * HTML code will be rendered (and take note of its resulting assigned 
 * name):
 * <ul><li>
 * <code>String styleName = doc.addStyle(style);</code></li></ul></li>
 * <li>Reference the style from elements that use it by setting the 
 * &quot;class&quot; attribute:<br>
 * <ul><li>
 * <code>spanTag.addAttribute(ElementNames.Attributes.CLASS, 
 * styleName);</code></li></ul></li>
 * </ol>
 * Styles must be added using this sequence of operations.  Once a style has
 * been added to a document it may not be modified.  The reason for this is 
 * that the container will attempt to reuse styles for other components.
 */
public class ComponentStyle 
implements Serializable {

    /**
     * A prefix which will be appended to all style names.  Style names will 
     * consist of this prefix supplemented by an <code>Id</code>.
     */
    private static final String STYLE_PREFIX = "E-S-";

    /**
     * A unit suffix for percent units.
     */
    public static final String PERCENT_UNITS = Style.PERCENT_UNITS;

    /**
     * A unit suffix for pixel units.
     */
    public static final String PIXEL_UNITS = Style.PIXEL_UNITS;

    /**
     * A unit suffix for point units.
     */
    public static final String POINT_UNITS = Style.POINT_UNITS;
    
    /**
     * A constant used in the <code>options</code> attribute of constructors
     * to explicitly determine the background color (based on traversing
     * component hierarchy) and set it using CSS.
     */
    public static final int EXPLICIT_SET_BACKGROUND = 1;

    /**
     * A constant used in the <code>options</code> attribute of constructors
     * to explicitly determine the foreground color (based on traversing
     * component hierarchy) and set it using CSS.
     */
    public static final int EXPLICIT_SET_FOREGROUND = 2;

    /**
     * A constant used in the <code>options</code> attribute of constructors
     * to explicitly determine the font (based on traversing
     * component hierarchy) and set it using CSS.
     */
    public static final int EXPLICIT_SET_FONT = 4;

    /**
     * A constant used in the <code>options</code> attribute of constructors
     * to explicitly determine the font, foreground, and background (based on 
     * traversing component hierarchy) and set it using CSS.
     */
    public static final int EXPLICIT_SET_ALL = EXPLICIT_SET_BACKGROUND | EXPLICIT_SET_FOREGROUND | EXPLICIT_SET_FONT;

    /**
     * Creates and returns a <code>ComponentStyle</code> object for the 
     * provided <code>ComponentPeer</code>.
     *
     * @param componentPeer The peer for which the style is to be generated.
     * @return A <code>ComponentStyle</code> instance suitable for the given
     *         <code>ComponentPeer</code>.
     */
    public static ComponentStyle forComponent(ComponentPeer componentPeer) {
        return forComponent(componentPeer, EXPLICIT_SET_FONT | EXPLICIT_SET_FOREGROUND);
    }
    
    /**
     * Creates and returns a <code>ComponentStyle</code> object for the 
     * provided <code>ComponentPeer</code>.
     *
     * @param componentPeer The peer for which the style is to be generated.
     * @param findBackground If true, the component's ancestors will be searched
     *        upward from the specified component until a background color is
     *        found.  This background color will be explicitly specified in the
     *        style's CSS output.
     * @return A <code>ComponentStyle</code> instance suitable for the given
     *         <code>ComponentPeer</code>.
     * @deprecated Use the <code>forComponent(ComponentPeer, int)</code> 
     *             factory method.  Invoking it with an options value of
     *             <code>EXPLICIT_SET_BACKGROUND</code> will have the same
     *             effect as using this constructor with 
     *             <code>findBackground</code> true.
     */
    public static ComponentStyle forComponent(ComponentPeer componentPeer, boolean findBackground) {
        return forComponent(componentPeer, findBackground ? EXPLICIT_SET_ALL : EXPLICIT_SET_FONT | EXPLICIT_SET_FOREGROUND);
    }
    
    /**
     * Creates and returns a <code>ComponentStyle</code> object for the 
     * provided <code>ComponentPeer</code>.
     *
     * @param componentPeer The peer for which the style is to be generated.
     * @param options A bitmask indicating options to use when creating the
     *        style.
     * @return A <code>ComponentStyle</code> instance suitable for the given
     *         <code>ComponentPeer</code>.
     */
    public static ComponentStyle forComponent(ComponentPeer componentPeer, int options) {
        ComponentStyle style = new ComponentStyle();
        
        if ((options & EXPLICIT_SET_BACKGROUND) == 0) {
            style.setBackground(componentPeer.getComponent().getBackground()); // Avoid recursive background determination.
        } else {
            style.setBackground(componentPeer.getBackground());
        }
        if ((options & EXPLICIT_SET_FOREGROUND) == 0) {
            style.setForeground(componentPeer.getComponent().getForeground()); // Avoid recursive foreground determination.
        } else {
            style.setForeground(componentPeer.getForeground());
        }
        if ((options & EXPLICIT_SET_FONT) == 0) {
            style.setFont(componentPeer.getComponent().getFont()); // Avoid recursive foreground determination.
        } else {
            style.setFont(componentPeer.getFont());
        }
        return style;
    }

    /**
     * Converts a color to a seven character hexadecimal string of the form 
     * #RRGGBB
     *
     * @param color The color to be converted.
     * @return A presentation of the color in the format #RRGGBB.
     */
    public static String getColorHexValue(Color color) {
        StringBuffer sb = new StringBuffer("#");
        int r = color.getRed();
        int g = color.getGreen();
        int b = color.getBlue();
        
        if (r < 0x10) {
           sb.append("0");
        }
        sb.append(Integer.toString(r, 16));
        if (g < 0x10) {
           sb.append("0");
        }
        sb.append(Integer.toString(g, 16));
        if (b < 0x10) {
           sb.append("0");
        }
        sb.append(Integer.toString(b, 16));
        
        return sb.toString();
    }

    private Font font = null;
    private Color foreground = null;
    private Color background = null;
    private String backgroundImage = null;
    private Set elements = new HashSet();
    private Map attributes = null;
    private Map pseudonyms = null;
    private int id;
    private ComponentStyle forwardingStyle = null;
    
    /**
     * Adds a text attribute to the style.
     *
     * @param name The name of the attribute.
     * @param value The attribute's value.
     */
    public void addAttribute(String name, String value) {
        if (attributes == null) {
            attributes = new HashMap();
        }
        
        attributes.put(name, value);
    }
    
    /**
     * Adds an integer attribute to the style.
     *
     * @param name The name of the attribute.
     * @param value The attribute's value.
     */
    public void addAttribute(String name, int value) {
        addAttribute(name, Integer.toString(value));
    }
    
    /**
     * Adds an integer attribute and units to the style.
     *
     * @param name The name of the attribute.
     * @param value The attribute's value.
     * @param units The units of the value.
     */
    public void addAttribute(String name, int value, String units) {
        addAttribute(name, value + units);
    }
        
    /**
     * Adds an element type to the style.
     *
     * @param name The name of the element type to add.  Common element  
     *        type names are available from the 
     *        <code>nextapp.echoservlet.html.ElemenetNames</code> class.
     */
    public void addElementType(String name) {
        if (forwardingStyle == null) {
            elements.add(name);
        } else {
            // if this style was deemed to be equivalent to one already in the document, calls to addElementType effect that style
            // instead of this one.
            forwardingStyle.elements.add(name);
        }
    }
    
    /**
     * Adds all element types that are present in another style.
     *
     * @param style The style whose element types are to be retrieved and 
     *        added to this style's list of element types.
     */
    void addElementTypesOfStyle(ComponentStyle style) {
        elements.addAll(style.elements);
        style.forwardingStyle = this;
    }
    
    /**
     * Adds an attribute to a pseudoclass of the style.
     *
     * @param pseudonym The name of the pseudoclass (e.g. hover, visited, etc).
     * @param name The name of the attribute.
     * @param value The attribute's value.
     */
    public void addPseudoAttribute(String pseudonym, String name, String value) {
        if (pseudonyms == null) {
            pseudonyms = new HashMap();
        }
        
        Map pseudoAttributes = (Map) pseudonyms.get(pseudonym);
        if (pseudoAttributes == null) {
            pseudoAttributes = new HashMap();
            pseudonyms.put(pseudonym, pseudoAttributes);
        }
        
        pseudoAttributes.put(name, value);
    }
    
    /**
     * Adds an integer attribute to a pseudoclass of the style.
     *
     * @param pseudonym The name of the pseudoclass (e.g. hover, visited, etc).
     * @param name The name of the attribute.
     * @param value The attribute's value.
     */
    public void addPseudoAttribute(String pseudonym, String name, int value) {
        addPseudoAttribute(pseudonym, name, Integer.toString(value));
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object o) {
        boolean equal;
    
        if (this == o) {
            equal = true;
        } else if (o instanceof ComponentStyle) {
            ComponentStyle that = (ComponentStyle) o;
            equal = true;

            equal &= (this.font == that.font || (this.font != null && this.font.equals(that.font)));
            equal &= (this.foreground == that.foreground || (this.foreground != null && this.foreground.equals(that.foreground)));
            equal &= (this.background == that.background || (this.background != null && this.background.equals(that.background)));
            equal &= (this.backgroundImage == that.backgroundImage 
                    || (this.backgroundImage != null && this.backgroundImage.equals(that.backgroundImage)));
            equal &= (this.attributes == that.attributes || (this.attributes != null && this.attributes.equals(that.attributes)));
            equal &= (this.pseudonyms == that.pseudonyms || (this.pseudonyms != null && this.pseudonyms.equals(that.pseudonyms)));
        } else {
            equal = false;
        }
        
        return equal;
    }
    
    /**
     * Returns the value of the specified attribute.
     *
     * @param name The name of an attribute.
     * @return The named attribute's value, or null if the attribute is not 
     *         set.
     */
    public String getAttribute(String name) {
        return (String) attributes.get(name);
    }
    
    /**
     * Returns true if the style has any attributes set.
     * This method will return true if the foreground, background, font, 
     * backgroundImage, or custom attributes are set.
     * 
     * @return True if the style has any attributes set.
     */
    public boolean hasAttributes() {
        return attributes != null || foreground != null || background != null || font != null || backgroundImage != null;
    }
    
    /**
     * Returns the name of the style.
     */
    String getName() {
        return STYLE_PREFIX + id;
    }
        
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        int hashCode = 0;
        
        // merge together every other bit of background and foreground colors.
        if (foreground != null) {
            hashCode ^= (foreground.hashCode() & 0xa5a5a5);
        }
        if (background != null) {
            hashCode ^= (background.hashCode() & 0x5a5a5a);
        }
 
        if (font != null) {
            hashCode ^= font.hashCode();
        }
                
        return hashCode;
    }

    /**
     * Sets the background color of the style.
     *
     * @param background The new background color.
     */
    public void setBackground(Color background) {
        this.background = background;
    }
    
    /**
     * Sets the background image of the style.
     *
     * @param backgroundImage The new background image.
     */
    public void setBackgroundImage(String backgroundImage) {
        this.backgroundImage = backgroundImage;
    }

    /**
     * Sets the font of the style.
     *
     * @param font The new font.
     */
    public void setFont(Font font) {
        this.font = font;
    }
    
    /**
     * Sets the horizontal text alignment of the style based on a value
     * from the <code>nextapp.echo.EchoConstants</code> class.
     *
     * @param horizontalAlignment The horizontal text alignment, one of the 
     *        following values:
     *        <ul>
     *        <li><code>EchoConstants.LEFT</code></li>
     *        <li><code>EchoConstants.CENTER</code></li>
     *        <li><code>EchoConstants.RIGHT</code></li>
     *        </ul>
     */ 
    public void setHorizontalAlignment(int horizontalAlignment) {
        switch (horizontalAlignment) {
        case EchoConstants.LEFT:
            addAttribute(ElementNames.CssAttributes.TEXT_ALIGN, "left");
            break;
        case EchoConstants.CENTER:
            addAttribute(ElementNames.CssAttributes.TEXT_ALIGN, "center");
            break;
        case EchoConstants.RIGHT:
            addAttribute(ElementNames.CssAttributes.TEXT_ALIGN, "right");
            break;
        }
    }
    
    /**
     * Sets the vertical text alignment of the style based on a value
     * from the <code>nextapp.echo.EchoConstants</code> class.
     *
     * @param verticalAlignment The vertical text alignment, one of the 
     *        following values:
     *        <ul>
     *        <li><code>EchoConstants.TOP</code></li>
     *        <li><code>EchoConstants.CENTER</code></li>
     *        <li><code>EchoConstants.BOTTOM</code></li>
     *        </ul>
     */ 
    public void setVerticalAlignment(int verticalAlignment) {
        switch (verticalAlignment) {
        case EchoConstants.TOP:
            addAttribute(ElementNames.CssAttributes.VERTICAL_ALIGN, "top");
            break;
        case EchoConstants.CENTER:
            addAttribute(ElementNames.CssAttributes.VERTICAL_ALIGN, "middle");
            break;
        case EchoConstants.BOTTOM:
            addAttribute(ElementNames.CssAttributes.VERTICAL_ALIGN, "bottom");
            break;
        }
    }
    
    /**
     * Sets the foreground color of the style.
     *
     * @param foreground The new foreground color.
     */
    public void setForeground(Color foreground) {
        this.foreground = foreground;
    }
    
    /**
     * Sets the identifier of the style.
     */
    void setId(int id) {
        this.id = id;
    }
    
    /**
     * Prints state information (for debugging use).
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        StringBuffer out = new StringBuffer();
        out.append(ComponentStyle.class.getName());
        out.append(" id=");
        out.append(id);
        out.append("\n");
        out.append(" elements=");
        out.append(elements);
        out.append("\n");
        out.append(" attributes=");
        out.append(attributes);
        out.append("\n");
        return out.toString();
    }

    /**
     * Renders a CSS representation of the style.
     *
     * @return A CSS representation of the style.
     */
    Style toStyle() {
        Style style = new Style(getName());
        
        if (foreground != null) {
            style.addAttribute(ElementNames.CssAttributes.COLOR, getColorHexValue(foreground));
        }
        if (background != null) {
            style.addAttribute(ElementNames.CssAttributes.BACKGROUND_COLOR, getColorHexValue(background));
        }
        if (backgroundImage != null) {
            style.addAttribute(ElementNames.CssAttributes.BACKGROUND_IMAGE, "url(" + backgroundImage + ")");
        }
        if (font != null) {
            style.addAttribute(ElementNames.CssAttributes.FONT_FAMILY, Delimiter.toString(font.getNames(), ","));
            
            style.addAttribute(ElementNames.CssAttributes.FONT_SIZE, ExtentRenderer.render(font.getSizeExtent()));
            
            if (font.isBold()) {
                style.addAttribute(ElementNames.CssAttributes.FONT_WEIGHT, "bold");
            }
            if (font.isItalic()) {
                style.addAttribute(ElementNames.CssAttributes.FONT_STYLE, "italic");
            }

            StringBuffer decoration = null;
            if (font.isUnderline()) {
                decoration = new StringBuffer("underline");
            }
            if (font.isOverline()) {
                if (decoration == null) {
                    decoration = new StringBuffer("overline");
                } else {
                    decoration.append(" overline");
                }
            }
            if (font.isLineThrough()) {
                if (decoration == null) {
                    decoration = new StringBuffer("line-through");
                } else {
                    decoration.append(" line-through");
                }
            }
            if (decoration != null) {
                style.addAttribute(ElementNames.CssAttributes.TEXT_DECORATION, decoration.toString());
            }
        }
        if (attributes != null) {
            String name;
            for (Iterator it = attributes.keySet().iterator(); it.hasNext();) {
                name = (String) it.next();
                style.addAttribute(name, (String) attributes.get(name));
            }
        }
        if (pseudonyms != null) {
            for (Iterator pseudonymIt = pseudonyms.keySet().iterator(); pseudonymIt.hasNext();) {
                String pseudonym = (String) pseudonymIt.next();
                Map pseudoAttributes = (Map) pseudonyms.get(pseudonym);
                for (Iterator pseudoAttributeIt = pseudoAttributes.keySet().iterator(); pseudoAttributeIt.hasNext();) {
                    String name = (String) pseudoAttributeIt.next();
                    String value = (String) pseudoAttributes.get(name);
                    style.addPseudoAttribute(pseudonym, name, value);
                }
            }
        }
        
        for (Iterator it = elements.iterator(); it.hasNext();) {
            style.addElementType((String) it.next());
        }
        
        return style;
    }
}
