/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import nextapp.echo.EchoInstance;
import nextapp.echoservlet.util.ContentType;

/**
 * A wrapper that holds information about an HTTP request, provides 
 * facilities to generate a response, and provides access to the
 * application instance that is to process it.
 */
public final class Connection {

    /** 
     * The prefix that will be used to determine the key name under which
     * the application will be stored in <code>HttpSession</code>.  The
     * application's URI will be appended to the session prefix to form
     * a unique value.
     */
    private static final String INSTANCE_PEER_SESSION_KEY_PREFIX = "instancePeer.";
    
    /** A convenience for setting the cache header */
    private static final String CACHE_MAX_AGE = "max-age=" + EchoServer.CACHED_SERVICE_TIMEOUT;
    
    /**
     * The amount of free memory available.  Used for performance testing
     * purposes.
     */ 
    private static long lastFreeMemory = Runtime.getRuntime().freeMemory();
    
    /** 
     * A global counter of services invoked for the life of the VM. 
     * Useful only for debugging purposes. 
     */
    private static int serviceNumber = 0;

    /** Reference to the HTTP request object wrapped by this Connection. */
    private HttpServletRequest request;
    
    /** Reference to the HTTP response object wrapped by this Connection. */
    private HttpServletResponse response;
    
    /** 
     * Reference to the application's InstancePeer, may be null under 
     * initialization conditions. 
     */
    private InstancePeer instancePeer;
    
    /** Reference to the EchoServer. */
    private EchoServer server;
    
    /** Reference to servlet encoding adapter. */
    private ServletEncodingAdapter servletEncodingAdapter;
        
    /**
     * Holds the <code>VariableData</code> object associated with this
     * <code>Connection</code>.
     */
    private VariableData variableData = null;
    
    /** The session key with holds the instance peer as its value. */
    private String instancePeerSessionKey;
    
    /**
     * A flag that indicates whether any debug options may be enabled.
     * Debug options work only if this flag is set.
     */
    private boolean debugOptionsEnabled;

    /**
     * A flag that indicates whether the time interval during which each request
     * is processed will be recorded.  Note: the logged time is not the time 
     * required to service a particular request, as multiple requests may be 
     * processed simultaneously.
     */
    private boolean debugPerformanceLogging = false;

    /**
     * A flag that indicates whether the interactivecontroller frame should be 
     * made visible and provide debugging information.
     */
    private boolean debugInteractiveController = false;

    /**
     * A flag indicating verbose logging.  Enable to see a great deal of
     * debugging information.
     */
    private boolean debugVerboseLogging = false;
    
    /**
     * Creates a <code>connection</code> object that will handle the given 
     * request and response.  The InstancePeer will be pulled from the session 
     * if one exists.  A session will NOT be created if one does not exist.
     *
     * @param server The <code>EchoServer</code> generating the connection.
     * @param request The HTTP request object that was passed to the servlet.
     * @param response The HTTP response object that was passed to the servlet.
     */
    Connection(EchoServer server, HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException {
        super();
        
        this.request = request;
        this.response = response;
        this.server = server;
        this.servletEncodingAdapter = server.getServletEncodingAdapter();

        debugOptionsEnabled = server.getApplicationProperties().getBoolean(ApplicationProperties.DEBUG_OPTIONS_ENABLED);
        if (debugOptionsEnabled) {
            debugPerformanceLogging = server.getApplicationProperties().getBoolean(ApplicationProperties.DEBUG_PERFORMANCE_LOGGING);
            debugInteractiveController =
                    server.getApplicationProperties().getBoolean(ApplicationProperties.DEBUG_INTERACTIVE_CONTROLLER);
            debugVerboseLogging = server.getApplicationProperties().getBoolean(ApplicationProperties.DEBUG_VERBOSE_LOGGING);
        }
        
        instancePeerSessionKey = INSTANCE_PEER_SESSION_KEY_PREFIX + getApplicationUri();
        
        // Retrieve InstancePeer from session if available.
        HttpSession session = request.getSession(false);
        if (session == null) {
            instancePeer = null;
        } else {
            instancePeer = (InstancePeer) session.getAttribute(instancePeerSessionKey);
        }

        // Set character encoding for request.
        if (instancePeer != null) {
            try {
                servletEncodingAdapter.setRequestCharacterEncoding(request, instancePeer.getCharacterEncoding());
            } catch (UnsupportedEncodingException ex) {
                throw new EchoServletException("Unsupported encoding", ex);
            }
        }
        
        // Configure connection for Multipart Request if required.
        String contentType = request.getContentType();
        if (contentType != null && contentType.startsWith(ContentType.MULTIPART_FORM_DATA.getMimeType())) {
            if (EchoServer.getMultipartRequestWrapper() == null) {
                throw new EchoServletException(
                        "MultipartRequestWrapper was never set and client made an HTTP request "
                        + "encoded as multipart/form-data.");
            }
            this.request = EchoServer.getMultipartRequestWrapper().getWrappedRequest(request);
        }
    }
    
    /**
     * Returns the URI of the application.
     *
     * @return The URI of the application.
     */
    public String getApplicationUri() {
        return request.getRequestURI();
    }
    
    /** 
     * Returns a prefix to prepend to cache ids in order to ensure that 
     * incorrect cached resources are used by client in the event either
     * the server or the users session has restarted.
     * 
     * Returns an empty string if no <code>InstancePeer</code> has been 
     * created, otherwise defers responsibility to the
     * <code>InstancePeer</code>.
     * 
     * @return A prefix to prepend to cache ids to make them unique across
     *         server and session restarts.
     */
    public String getCacheIdPrefix() {
        if (instancePeer == null) {
            return "";
        } else {
            return instancePeer.getCacheIdPrefix();
        }
    }
    
    /**
     * Returns the specified request parameter as an Id value.  If the 
     * parameter is was not provided in the request, or if the parameter
     * is not a valid Id value, null is returned.
     *
     * @param name The parameter name.
     * @return The parameter as an Id.
     */
    public Id getIdParameter(String name) {
        Id id = null;
        String idString = request.getParameter(name);
        if (idString != null) {
            try {
                id = new Id(idString);
            } catch (NumberFormatException ex) { } // id = null
        }
        
        return id;
    }
    
    /**
     * Returns the <code>InstancePeer</code> associated with this connection.
     * If the session has not been initialized, null is returned.
     *
     * @return The <code>InstancePeer</code> associated with this connection.
     */
    public InstancePeer getInstancePeer() {
        return instancePeer;
    }
    
    /**
     * Returns the <code>OutputStream</code> object that may be used to 
     * generate a response.  This method may be called once.  If it is called, 
     * the getWriter() method may not be called.  This method wraps a call to 
     * <code>HttpServletResponse.getOutputStream()</code>.  The 
     * <code>OutputStream</code> will be closed by the servlet container.
     *
     * @return the <code>OutputStream</code> object that may be used to 
     *         generate a response to the client.
     */
    public OutputStream getOutputStream() {
        try {
            return response.getOutputStream();
        } catch (IOException ex) {
            throw new EchoServletException("Unable to get OutputStream.", ex);
        }
    }
    
    /**
     * Returns the value of the HTTP parameter associated with the given name.
     *
     * @param name The name of the parameter to return.
     * @return The value of the HTTP parameter associated with the given name.
     */
    public String getParameter(String name) {
        if (instancePeer == null) {
            return request.getParameter(name);
        } else {
            try {
                return servletEncodingAdapter.getParameter(request, name, instancePeer.getCharacterEncoding());
            } catch (UnsupportedEncodingException ex) {
                throw new EchoServletException("Unsupported encoding", ex);
            }
        }
    }
    
    /**
     * Returns an enumeration of all parameter names passed in the request.
     * 
     * @return An enumeration of all parameter names passed in the request.
     */
    public Enumeration getParameterNames() {
        return request.getParameterNames();
    }
    
    /**
     * Returns the <code>HttpServletRequest</code> wrapped by this 
     * <code>Connection</code>.
     *
     * @return The <code>HttpServletRequest</code> wrapped by this 
     *         <code>Connection</code>.
     */
    public HttpServletRequest getRequest() {
        return request;
    }
    
    /**
     * Returns the <code>HttpServletResponse</code> wrapped by this 
     * <code>Connection</code>.
     *
     * @return The <code>HttpServletResponse</code> wrapped by this 
     *         <code>Connection</code>.
     */
    public HttpServletResponse getResponse() {
        return response;
    }
    
    /**
     * Returns the <code>EchoServer</code> wrapped by this 
     * <code>Connection</code>.
     *
     * @return The <code>EchoServer</code> wrapped by this 
     *         <code>Connection</code>.
     */
    public EchoServer getServer() {
        return server;
    }
    
    /**
     * Returns the <code>VariableData</code> associated with this 
     * <code>Connection</code>.  If an <code>InstancePeer</code> exists, 
     * its <code>VariableData</code> is returned, otherwise, a temporary 
     * <code>VariableData</code> object is returned that contains 
     * defaults used for initialization.
     *
     * @return The <code>VariableData</code> object associated with this
     *         <code>Connection</code>.
     */
    public VariableData getVariableData() {
        return variableData;
    }
    
    /**
     * Returns the <code>PrintWriter</code> object that may be used to 
     * generate a response.  This method may be called once.  If it is called, 
     * the getOuputStream() method may not be called.  This method wraps a 
     * call to <code>HttpServletResponse.getWriter()</code>.  The 
     * <code>PrintWriter</code> will be closed by the servlet container.
     *
     * @return the <code>PrintWriter</code> object that may be used to 
     *         generate a response to the client.
     */
    public PrintWriter getWriter() {
        try {
            return response.getWriter();
        } catch (IOException ex) {
            throw new EchoServletException("Unable to get PrintWriter.", ex);
        }
    }

    /**
     * Corrects relative-path URIs to include the application name, such that 
     * they can properly retrieve the desired resource.
     *
     * @param uri The URI to correct.
     * @return The corrected URI.
     */
    public String correctUri(String uri) {
        if (uri != null && uri.indexOf("://") == -1 && uri.charAt(0) != '/') {
            // URI does not contain a protocol and is a relative path:
            // the application name will be added.
            uri = getApplicationUri().substring(0, getApplicationUri().lastIndexOf("/")) + "/" + uri;
        }
        return uri;
    }
    
    /**
     * Returns true if the interactive controller debug feature is enabled.
     *
     * @return True if the interactive controller debug feature is enabled.
     */
    boolean isDebugInteractiveControllerEnabled() {
        return debugInteractiveController;
    }
    
    /**
     * Returns true if the verbose logging debug feature is enabled.
     *
     * @return True if the verbose logging debug feature is enabled.
     */
    boolean isDebugVerboseLoggingEnabled() {
        return debugVerboseLogging;
    }

    /** 
     * Retrieves and executes the appropriate <code>Service</code> requested
     * by the client browser.  Sets cache control HTTP headers for the
     * response.  If the user does not yet have an <code>InstancePeer</code>,
     * then initialization services are executed.  Logs debugging information
     * if necessary.
     */
    void process() 
    throws IOException {
        boolean complete = false;
        long startTime = 0;

        if (debugPerformanceLogging || debugVerboseLogging) {
            startTime = System.currentTimeMillis();
            System.err.print(" -> ServiceReq ");
            System.err.print("#" + serviceNumber++ + ":");
            System.err.print("[ses=" + ((request.getSession(false) == null) 
                    ? "null" : request.getSession(false).getId()) + "] ");
        }

        try {
            Id id = getIdParameter(ClientObjects.PARAMETER_ID);
            if (debugVerboseLogging) {
                System.err.print(" id(" + id + ")");
            }
            
            Service service = retrieveService(id);
            if (service == null) {
                // No service could be found.  Log error and display render blank pixel image.
                // The blank pixel image is rendered in case we have the combination of an 
                // "click-happy" user and a lagging browser that is requesting the resulting 
                // recently-dead services.  This almost never occurs, and in such cases is harmless.
                service = UnknownIdService.SERVICE_UNKNOWN_ID;
                if (debugVerboseLogging) {
                    System.err.println("(Invalid service id: " + id + ")");
                }
            }
            
            if (instancePeer == null) {
                // No InstancePeer exists: Application has not started: create generic variable data. 
                variableData = Initializer.createInitialVariableData(this);
            } else {
                // InstancePeer exists: Application has started: use application's VariableData object.
                variableData = instancePeer.getVariableData();
                instancePeer.addConnection(this);
            }
            
            // Log request.
            if (debugVerboseLogging) {
                System.err.print(" class(" + service.getClass().getName() + ")");
            }

            // Set caching directives.
            if (service instanceof CacheableService) {
                response.setHeader("Cache-control", CACHE_MAX_AGE);
            } else {
                response.setHeader("Pragma", "no-cache");
                response.setHeader("Cache-Control", "no-cache");
                response.setHeader("Cache-Control", "no-store");
                response.setHeader("Expires", "0");
            }
            
            if (instancePeer != null) {
                // Mark the EchoInstance as active for this thread, such that invoking
                // EchoInstance.getEchoInstance() will return it.
                EchoInstance echoInstance = instancePeer.getInstance();
                echoInstance.markActive(true);
            }

            // Feed the request to the service.
            service.service(this);
            
            if (instancePeer != null) {
                // Mark the EchoInstance as inactive for this thread, such that invoking
                // EchoInstance.getEchoInstance() will return null
                EchoInstance echoInstance = instancePeer.getInstance();
                echoInstance.markActive(false);
            }
            
            complete = true; // For logging purposes.
        } finally {
            if (instancePeer != null) {
                instancePeer.removeConnection(this);
            }
            if (debugPerformanceLogging || debugVerboseLogging) {
                System.err.print(" time(" + (System.currentTimeMillis() - startTime) + "ms)");
                
                long freeMemory = Runtime.getRuntime().freeMemory();
                System.err.print(" dMem(" + ((freeMemory - lastFreeMemory) / 1024) + "kB)");   
                lastFreeMemory = Runtime.getRuntime().freeMemory();

                System.err.print(" fMem(" + (lastFreeMemory / 1024) + "kB)");   
                System.err.print(" tMem(" + (Runtime.getRuntime().totalMemory() / 1024) + "kB)");   

                if (complete) {
                    System.err.println(" - ok");
                } else {
                    System.err.println(" - FAIL");
                }
            }
        }
    }
    
    /**
     * Returns the service that corresponds to the specified Id.
     *
     * @param id The id of the service to return.
     * @return The service corresponding to the specified Id.
     */
    private Service retrieveService(Id id) {
        Service service;
        
        if (instancePeer == null) {
            if (id == null) {
                // No Id was provided: The Initializer service will be rendered.
                service = EchoServer.SERVICE_INITIALIZER;
            } else {
                // An Id was provided: Retrieve the appropriate global service.
                service = EchoServer.services.get(id);
                if (service == null) {
                    // The requested global service is not valid: this indicates that the session has expired, so the 
                    // session expiration handling service is rendered.
                    service = EchoServer.SERVICE_SESSION_EXPIRED;
                }
            }
        } else {
            if (id == null) {
                // No Id was provided: set the Id to the InstancePeer's default window.
                id = instancePeer.getDefaultWindowId();
                if (id == null) {
                    // No default window is available: unbind the instance peer and reinitialize.
                    unbindInstancePeer();
                    service = EchoServer.SERVICE_INITIALIZER;
                } else {
                    // Retrieve service associated with default window Id.
                    service = instancePeer.getServiceRegistry().get(id);
                }
            } else {
                // Attempt to retrieve service corresponding to specified Id.
                // Service might not exist, and if so null will be returned.  This will be dealt with by the caller.
                service = instancePeer.getServiceRegistry().get(id);
                if (service == null) {
                    service = EchoServer.services.get(id);
                }
            }
        }
        
        return service;
    }
    
    /**
     * Sets the content type of the response.
     * This method will automatically append a character encoding to
     * non-binary content types.
     * 
     * @param contentType The content type of the response.
     */
    public void setContentType(ContentType contentType) {
        if (contentType.isBinary() || instancePeer == null) {
            response.setContentType(contentType.getMimeType());
        } else {
            response.setContentType(contentType.getMimeType() + "; charset=" + instancePeer.getCharacterEncoding());
        }
    }
    
    /**
     * Sets the <code>InstancePeer</code> that has been created for the user
     * making this request.  This method is called once, by the 
     * <code>Initializer</code>.
     *
     * @param instancePeer The <code>InstancePeer</code> that will be used 
     *        by this user.
     */
    void setInstancePeer(InstancePeer instancePeer) {
        this.instancePeer = instancePeer;
        request.getSession(true).setAttribute(instancePeerSessionKey, instancePeer);
    }
    
    /**
     * Unbinds the application's <code>InstancePeer</code> from the session. 
     * This terminates the application.  If the user visits the application's
     * URI again, a new <code>InstancePeer</code> will be created.
     */
    void unbindInstancePeer() {
        HttpSession session = request.getSession(false);
        
        if (session != null) {
            session.removeAttribute(INSTANCE_PEER_SESSION_KEY_PREFIX + getApplicationUri());
        }
    }
}
