/*
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.PrintWriter;

import nextapp.echo.AbstractPane;
import nextapp.echo.ContainerPane;
import nextapp.echo.Extent;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.util.ContentType;

/**
 * A peer object for <code>ContainerPane</code>s.  This class is responsible
 * for rendering &lt;frameset&gt; documents to layout windows.
 */
public class ContainerPaneUI extends AbstractPaneUI 
implements PropertyChangeListener, Service {

    /**
     * Creates a &lt;frame&gt; element representation of a pane.
     *
     * @param conn The connection the document will be rendered to.
     * @param paneUI The paneUI to generate a frame element for.
     * @return A frame element for the specified pane.
     */
    public static Element createFrameElement(Connection conn, PaneUI paneUI) {
        Element paneElement = new Element(ElementNames.FRAME, false);

        if (paneUI == null) {
            paneElement.addAttribute("src", ClientObjects.getServiceUri(conn, EchoServer.SERVICE_BLANK_DOCUMENT));
            paneElement.addAttribute("scrolling", "no");
        } else {
            AbstractPane pane = (AbstractPane) ((ComponentPeer) paneUI).getComponent();
            paneElement.addAttribute("name", ClientObjects.getObjectName(paneUI.getId()));
            paneElement.addAttribute("src", paneUI.getUri(conn));
            if (! pane.isResizable()) {
                paneElement.addAttribute("frameborder", 0);
                paneElement.addAttribute("noresize");
            } 
            paneElement.addAttribute("scrolling", paneUI.getScrolling());
        }
        
        return paneElement;
    }
    
    /**
     * Creates a new <code>ContainerPaneUI</code>.
     */
    public ContainerPaneUI() {
        super();
    }
    
    /**
     * Returns the pane that this pane is &quot;forwarding&quot; requests to.
     * If the container pane has more than one child pane,
     * this method will return <code>this</code>.  In the case where there
     * is only one child pane, this method will return the child pane.
     *
     * @return The pane that this pane is &quot;forwarding&quot; requests to.
     */
    PaneUI getForwardingPane() {
        ComponentPeer[] children = getChildren();
        if (children.length == 1) {
            if (children[0] instanceof ContainerPaneUI) {
                return ((ContainerPaneUI) children[0]).getForwardingPane();
            } else {
                return (PaneUI) children[0];
            }
        } else {
            return this;
        }
    }
    
    /**
     * @see nextapp.echoservlet.PaneUI#getScrolling()
     */
    public String getScrolling() {
        if (isForwarding()) {
            return getForwardingPane().getScrolling();
        } else {
            return PaneUI.SCROLLING_AUTO;
        }
    }
    
    /**
     * @see nextapp.echoservlet.PaneUI#getUri(Connection)
     */
    public String getUri(Connection conn) {
        if (isForwarding()) {
            return getForwardingPane().getUri(conn);
        } else {
            return ClientObjects.getServiceUri(conn, this);
        }
    }
    
    /**
     * Returns true if the container pane is forwarding to its content.
     *
     * @return True if the container pane is forwarding to its content.
     * @see #getForwardingPane()
     */
    boolean isForwarding() {
        return getChildCount() == 1;
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        if (ContainerPane.PREFERRED_HEIGHT_CHANGED_PROPERTY.equals(e.getPropertyName())
                || ContainerPane.PREFERRED_WIDTH_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // In cases where scroll bar, width, or height settings have been changed it is necessary to redraw the parent.
            getParent().redraw();
        } else {
            redraw();
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }

    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn) {
        ContainerPane containerPane = (ContainerPane) getComponent();
        int childCount = containerPane.getComponentCount();
        HtmlDocument doc;
        
        if (getChildCount() > 0) {
            // FrameSet will contain child frames.
            doc = HtmlDocument.createFrameSet(conn);
            Extent sizeExtent = null;
            StringBuffer sizes = new StringBuffer();
            Element body = doc.getBodyElement();

            AbstractPane pane = null;
            PaneUI paneUI = null;
            boolean hasResizableFrames = false;
            int visiblePaneCount = 0;

            for (int index = 0; index < childCount; ++index) {
                pane = (AbstractPane) containerPane.getComponent(index);

                if (pane.isVisible()) {

                    if (containerPane.getOrientation() == ContainerPane.ORIENTATION_HORIZONTAL) {
                        sizeExtent = pane.getWidthExtent();
                    } else {
                        sizeExtent = pane.getHeightExtent();
                    }

                    if (sizeExtent == null) {
                        sizes.append("*");
                    } else {
                        sizes.append(ExtentRenderer.render(sizeExtent));
                    }
                    if (index < childCount - 1) {
                        sizes.append(",");
                    }
                    if (pane.isResizable()) {
                        hasResizableFrames = true;
                    }

                    paneUI = (PaneUI) getPeer(pane);
                    body.add(createFrameElement(conn, paneUI));

                    ++visiblePaneCount;
                }
            }

            if (containerPane.getOrientation() == ContainerPane.ORIENTATION_HORIZONTAL) {
                body.addAttribute("cols", sizes.toString());
            } else {
                body.addAttribute("rows", sizes.toString());
            }

            body.addAttribute("framespacing", 0);
            if (! hasResizableFrames) {
                body.addAttribute("border", 0);
                body.addAttribute("frameborder", 0);
            }
        } else {
            // FrameSet has no child frames: render blank dcoument.
            doc = HtmlDocument.create(conn);
        }

        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
