/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.PrintWriter;

import nextapp.echo.BackgroundImageProperties;
import nextapp.echo.Component;
import nextapp.echo.ContentPane;
import nextapp.echo.EchoConstants;
import nextapp.echo.Insets;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.image.ImageManager;
import nextapp.echoservlet.resource.ResourceNames;
import nextapp.echoservlet.util.ContentType;

/**
 * A peer object for <code>ContentPane</code>s.  This class is responsible
 * for rendering HTML documents that contain rendered components.
 */
public class ContentPaneUI extends AbstractPaneUI 
implements PaneUI, PropertyChangeListener, Service {

    private static final String BACKGROUND_IMAGE = "background";

    /**
     * A JavaScript include that contains functionality required for content 
     * panes, such as functions that input and action components will use to
     * notify the server of updated state.  This script will be referenced in
     * every content pane.
     */
    public static final Service SERVICE_CONTENT_PANE_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "cp", ResourceNames.CONTENT_PANE_SCRIPT);

    static {
        EchoServer.addGlobalService(SERVICE_CONTENT_PANE_SCRIPT);
    }    

    /**
     * An image manager that will handle the background image of the content
     * pane if one is used.
     */
    private ImageManager imageManager = new ImageManager(this);
    
    /**
     * Returns the JavaScript method call necessary to send an action command.
     * This method is called by components to generate script that will cause
     * server interaction.  For instance, a button that generates a link might
     * make its element's &quot;href&quot; attribute:
     * &quot;javascript:E_setAction('4f', 'press')&quot;,
     * such that the server would be notified that component '4f' triggered an
     * action with the command 'press'.  On the server, the ComponentPeer with 
     * Id '4f' would have its clientActionPerformed() method called with the 
     * parameter 'press'.
     *
     * @param id The Id of the component setting the action.
     * @param command The action command that is to be sent.
     */
    public static String getScriptSetAction(String id, String command) {
        return ClientObjects.STD_PREFIX + "setAction(" + id + ", " + command + ");";
    }
    
    /**
     * Returns the JavaScript method call necessary to store a parameter in the
     * Controller frame's hidden form.  During the next server interaction, 
     * the data held in this hidden parameter will be provided to the 
     * ComponentPeer with the corresponding Id.  The value will be sent as the
     * parameter of a clientInput() method call.
     *
     * @param id The Id of the component setting the parameter.
     * @param value The new value of the parameter.
     */
    public static String getScriptSetParameter(String id, String value) {
        return ClientObjects.STD_PREFIX + "setParameter(" + id + ", " + value + ");";
    }
    
    /** 
     * Creates a new <code>ContentPaneUI</code>.
     */
    public ContentPaneUI() {
        super();
    }
    
    /**
     * @see nextapp.echoservlet.PaneUI#getScrolling()
     */
    public String getScrolling() {
        String scrolling;
     
        switch (((ContentPane) getComponent()).getScrollBarPolicy()) {
        case ContentPane.SCROLL_BAR_AUTO:
            scrolling = PaneUI.SCROLLING_AUTO;
            break;
        case ContentPane.SCROLL_BAR_NEVER:
            scrolling = PaneUI.SCROLLING_NEVER;
            break;
        case ContentPane.SCROLL_BAR_ALWAYS:
            scrolling = PaneUI.SCROLLING_ALWAYS;
            break;
        default:
            throw new IllegalStateException("Unsupported scroll bar setting."); 
        }
            
        return scrolling;
    }
    
    /**
     * @see nextapp.echoservlet.PaneUI#getUri(Connection)
     */
    public String getUri(Connection conn) {
        return ClientObjects.getServiceUri(conn, this);
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        if (ContentPane.SCROLL_BAR_POLICY_CHANGED_PROPERTY.equals(e.getPropertyName())
                || ContentPane.HEIGHT_CHANGED_PROPERTY.equals(e.getPropertyName())
                || ContentPane.WIDTH_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // In cases where scroll bar, width, or height settings have been changed it is necessary to redraw the parent.
            getParent().redraw();
        } else {
            redraw();
        }

        if (ContentPane.BACKGROUND_IMAGE_PROPERTIES_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            ContentPane contentPane = (ContentPane) getComponent();
            imageManager.setImage(BACKGROUND_IMAGE, contentPane.getBackgroundImage());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        ContentPane contentPane = (ContentPane) getComponent();
        contentPane.addPropertyChangeListener(this);
        imageManager.setImage(BACKGROUND_IMAGE, contentPane.getBackgroundImage());
    }

    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn) {
        ContentPane contentPane = (ContentPane) getComponent();
        BackgroundImageProperties bgProperties = contentPane.getBackgroundImageProperties();

        HtmlDocument doc = HtmlDocument.create(conn);
        doc.addScriptInclude(SERVICE_CONTENT_PANE_SCRIPT);
        doc.addScriptInclude(conn.getInstancePeer().getClientErrorService());
        doc.setScrollBarPositions(contentPane.getScrollBarX(), contentPane.getScrollBarY());
        
        // Invoke JavaScript to focus server-side focused component on client.
        Component focusedComponent = contentPane.getEchoInstance().getFocusedComponent();
        if (focusedComponent != null && contentPane.isAncestorOf(focusedComponent) && focusedComponent.isShowing()) {
            ComponentPeer focusedComponentPeer = getInstancePeer().getPeer(focusedComponent);
            String focusId = focusedComponentPeer.getFocusedElementId();
            if (focusId != null) {
                doc.addScript("E_focusByElementId(\"" + focusId + "\");", HtmlDocument.EVENT_INITIALIZE);
            }
        }
        
        ComponentStyle style = ComponentStyle.forComponent(this);
        if (imageManager.getImage(BACKGROUND_IMAGE) != null) {
            String imageUri = imageManager.getImageUri(conn, BACKGROUND_IMAGE);
            BackgroundImagePropertiesRenderer.render(bgProperties, imageUri, style);
        }
        Insets insets = contentPane.getInsets();
        style.addAttribute("padding", 0, ComponentStyle.PIXEL_UNITS);
        if (insets.getTop() == insets.getBottom() && insets.getLeft() == insets.getRight() && insets.getTop() == insets.getLeft()) {
            style.addAttribute("margin", insets.getTop(), ComponentStyle.PIXEL_UNITS);
        } else {
            style.addAttribute("margin-left", insets.getLeft(), ComponentStyle.PIXEL_UNITS);
            style.addAttribute("margin-top", insets.getTop(), ComponentStyle.PIXEL_UNITS);
            style.addAttribute("margin-right", insets.getRight(), ComponentStyle.PIXEL_UNITS);
            style.addAttribute("margin-bottom", insets.getBottom(), ComponentStyle.PIXEL_UNITS);
        }
        style.addElementType(ElementNames.BODY);
        String styleName = doc.addStyle(style);
        doc.getBodyElement().addAttribute(ElementNames.Attributes.CLASS, styleName);
        
        RenderingContext rc = new RenderingContext(conn, doc);

        Element form = new Element(ElementNames.FORM);
        form.addAttribute("method", "GET");
        form.addAttribute("action", "javascript:;");
        form.addAttribute("style", "margin:0");
        
        doc.getBodyElement().setWhitespaceRelevant(true);
        if (contentPane.getHorizontalAlignment() != EchoConstants.LEFT 
                || contentPane.getVerticalAlignment() != EchoConstants.TOP) {
            // Create alignment table necesary to align content if left-top is not desired alignment.
            Element alignmentTable = new Element(ElementNames.TABLE);
            alignmentTable.addAttribute("cellmargin", 0);
            alignmentTable.addAttribute("cellpadding", 0);
            alignmentTable.addAttribute("border", 0);
            alignmentTable.addAttribute("width", "100%");
            alignmentTable.addAttribute("height", "100%");
            alignmentTable.setWhitespaceRelevant(true);
            Element alignmentRow = new Element(ElementNames.TR);
            alignmentRow.setWhitespaceRelevant(true);
            alignmentTable.add(alignmentRow);
            Element alignmentCell = new Element(ElementNames.TD);
            ComponentStyle alignmentCellStyle = new ComponentStyle();
            alignmentCellStyle.setFont(contentPane.getFont());
            alignmentCellStyle.addElementType(ElementNames.TD);
            String alignmentCellStyleName = doc.addStyle(alignmentCellStyle);
            alignmentCell.addAttribute(ElementNames.Attributes.CLASS, alignmentCellStyleName);
            alignmentCell.setWhitespaceRelevant(true);
            alignmentCell.addAttribute("width", "100%");
            alignmentCell.addAttribute("height", "100%");
            switch (contentPane.getHorizontalAlignment()) {
            case EchoConstants.CENTER:
                alignmentCell.addAttribute("align", "center");
                break;
            case EchoConstants.RIGHT:
                alignmentCell.addAttribute("align", "right");
                break;
            default:
                alignmentCell.addAttribute("align", "left");
            }
            switch (contentPane.getVerticalAlignment()) {
            case EchoConstants.CENTER:
                alignmentCell.addAttribute("valign", "middle");
                break;
            case EchoConstants.RIGHT:
                alignmentCell.addAttribute("valign", "bottom");
                break;
            default:
                alignmentCell.addAttribute("valign", "top");
            }
            alignmentRow.add(alignmentCell);
            doc.getBodyElement().add(alignmentTable);
            // Add form inside of alignment table.
            alignmentCell.add(form);
        } else {
            // Add form directly to document body.
            doc.getBodyElement().add(form);
        }
        
        Component[] children = contentPane.getComponents();
        for (int index = 0; index < children.length; ++index) {
            if (children[index].isVisible()) {
                getPeer(children[index]).render(rc, form);
            }
        }

        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        ContentPane contentPane = (ContentPane) getComponent();
        contentPane.removePropertyChangeListener(this);
        imageManager.setImage(BACKGROUND_IMAGE, null);
    }
}
