/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import nextapp.echo.Window;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Script;
import nextapp.echoservlet.resource.ResourceNames;
import nextapp.echoservlet.util.ContentType;
import nextapp.echoservlet.util.Delimiter;

/**
 * The Controller is a service that is used to receive input from the 
 * client and issue commands to it. <br>
 * <br>
 * Every window of an Echo application contains a frameset at its root.
 * This frameset contains two frames: The first, which fills 100% of the window,
 * is whatever content service (e.g. a ContainerPane or ContentPane) the 
 * application desires to display in the window.  The second frame is a
 * Controller frame, which occupies 0% of the window, and is thus entirely
 * hidden from view (although it can be made enabling certain debugging
 * options in your application's servlet properties file).<br>
 * <br>
 * The HTML document rendered by this service contains a form that provides
 * a hidden field for each form component currently visible in any window.
 * Each form component rendered on the client must copy its data to the 
 * controller's form when its data is changed by the user.
 * This enables ALL available client data to be submitted to the server with
 * every user gesture that may require server interaction.<br>
 * <br>
 * The controller form will be submitted when the setAction() script method
 * is called on the client.  When this form is submitted, its data is parsed
 * and each ComponentPeer is notified of the value of its respective hidden
 * field if said value has changed.  Then the component whose Id was provided
 * to the setAction() script is notified of its having had its action invoked.
 * This process will cause the ComponentPeers to notify their represented
 * components, resulting in the application receiving events and doing work.<br>
 * <br>
 * The application will (most likely) make changes as a result of receiving
 * input.  The ComponentPeers will be notified of the changes made to their
 * respective Components, and the ComponentPeers will add their containing
 * services to the OutgoingUpdateQueue such that they will be re-rendered.  When
 * the application has finished its work, the Controller will re-render
 * a new hidden form, in addition to JavaScript method calls that will
 * synchronize the client browser's state with the application's changes.<br>
 * <br>
 * NOTE: Controller implements CacheableService in order to avoid issues with
 * browsers (notably IE) whereby the user pressing the back button will cause
 * a client-side application failure due to the browser being unable to 
 * retrieve a copy of the previous Controller's document from cache, and 
 * being unable to request a new copy by virtue of the document having been 
 * generated in response to an HTTP POST request.
 */
final class Controller 
implements Serializable, CacheableService {

    /**
     * Flag indicating that all updates in the outgoing update queue should be
     * purged. Used when embedding an Echo application inside an
     * embedded browser for the following scenario:  
     * The state of the application is changed externally, and the embedding
     * browser is performing a reload to display the changed state.  In this 
     * case not purging the updates would result in a double-refresh when the
     * controller frame was rendered. 
     */
    private static final String PURGE_ALL_UPDATES_FLAG = "nextapp.echoservlet.Controller.purgeAllUpdates";
    
    static final String PARAMETER_SEQUENCE_NUMBER = ClientObjects.STD_PREFIX + "sequenceNumber";
    static final String PARAMETER_REQUEST_TYPE = ClientObjects.STD_PREFIX + "requestType";
    static final String PARAMETER_REQUEST_DATA = ClientObjects.STD_PREFIX + "requestData";
    
    static final String PARAMETER_INPUT_PREFIX = ClientObjects.STD_PREFIX + "in_";
    static final int PARAMETER_INPUT_PREFIX_LENGTH = PARAMETER_INPUT_PREFIX.length();
    static final String PARAMETER_PANE_SCROLL_POSITION_PREFIX = ClientObjects.STD_PREFIX + "psp_";
    static final int PARAMETER_PANE_SCROLL_POSITION_PREFIX_LENGTH = PARAMETER_PANE_SCROLL_POSITION_PREFIX.length();
    
    private static final Id ID = new Id(ClientObjects.STD_PREFIX + "cf");
    
    private static final String SET_READY_FLAG = ClientObjects.STD_PREFIX + "ready = true;";
    
    /** 
     * A parameter that is provided on the request URI when the controller's
     * form is being submitted in response to an action by the user.
     */
    static final String REQUEST_TYPE_ACTION = "a";
    
    /** 
     * A parameter that is provided on the request URI when the controller's
     * form is being submitted on a synchronize request.
     */
    static final String REQUEST_TYPE_SYNCHRONIZE = "s";

    /**
     * A parameter that is provided on the request URI when the controller's
     * form is being submitted in response to a window unloading.
     */
    static final String REQUEST_TYPE_WINDOW_UNLOAD = "u";
    
    /**
     * A parameter that is provided on the request URI when the controller's
     * form is being submitted in response to one or more windows being closed,
     * by way of their close buttons having been clicked.
     */
    static final String REQUEST_TYPE_WINDOWS_CLOSED = "c";
    
    /** 
     * A parameter that is provided on the request URI when the controller's
     * form is being submitted in response to the controller's script having
     * found an inconsistency between the windows that should be open on the 
     * client and the windows that actually are open on the client.
     */
    static final String REQUEST_TYPE_WINDOWS_MISSING = "m";

    /**
     * An acknowledgement from the client that the state of the client is 
     * valid.  This will cause any delayed updates to be processed.
     */
    static final String REQUEST_TYPE_STATE_VALID = "v";

    /** 
     * A JavaScript service containing methods used by the <code>Controller</code>. 
     */
    private static final Service SERVICE_CONTROLLER_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "ctrl", ResourceNames.CONTROLLER_SCRIPT);
    
    /**
     * A JavaScript service containing debugging methods.  
     */
    private static final Service SERVICE_CONTROLLER_DEBUG_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "ctrldebug", ResourceNames.CONTROLLER_DEBUG_SCRIPT);
            
    private static final Service SERVICE_COMPONENT_HIERARCHY_VIEWER = new ComponentHierarchyViewer();
            
    static {
        EchoServer.addGlobalService(SERVICE_CONTROLLER_SCRIPT);
        EchoServer.addGlobalService(SERVICE_CONTROLLER_DEBUG_SCRIPT);
        EchoServer.addGlobalService(SERVICE_COMPONENT_HIERARCHY_VIEWER);
    }
    
    private InstancePeer instancePeer;
    private IncomingUpdateQueue incomingUpdateQueue;
    private OutgoingUpdateQueue outgoingUpdateQueue;
    private int debugRenderCounter = 0;

    /**
     * Version number for <code>CacheableService</code>.
     */
    private int version = 0;

    /**
     * Create a new controller for the specified InstancePeer with the 
     * specified OutgoingUpdateQueue.
     *
     * @param instancePeer The instance peer for which this controller will be 
     *        used.
     */
    Controller(InstancePeer instancePeer) {
        super();
        
        this.instancePeer = instancePeer;
        this.incomingUpdateQueue = instancePeer.getIncomingUpdateQueue();
        this.outgoingUpdateQueue = instancePeer.getOutgoingUpdateQueue();
    }
    
    /**
     * Adds debugging options to (visible) controller frame.
     */
    private void addDebugOptions(Connection conn, HtmlDocument doc, Element parent) {
        doc.addScriptInclude(SERVICE_CONTROLLER_DEBUG_SCRIPT);
        Element debugLink = new Element(ElementNames.A);
        debugLink.setWhitespaceRelevant(true);
        debugLink.addAttribute("href", "javascript:E_getStatus();");
        debugLink.addText("[Status]");
        parent.add(debugLink);

        parent.addText("Render: " + debugRenderCounter++);

        parent.addText(" | Inspect:");

        Element inspect = new Element(ElementNames.INPUT, false);
        inspect.addAttribute("type", "text");
        inspect.addAttribute("size", 40);
        inspect.addAttribute("value", "");
        inspect.addAttribute("name", "E_inspect");
        parent.add(inspect);

        Element inspectFieldAction = new Element(ElementNames.A);
        inspectFieldAction.setWhitespaceRelevant(true);
        inspectFieldAction.addAttribute("href", "javascript:E_inspectField();");
        inspectFieldAction.addText("[Inspect Field]");
        parent.add(inspectFieldAction);

        Element inspectPropertyAction = new Element(ElementNames.A);
        inspectPropertyAction.setWhitespaceRelevant(true);
        inspectPropertyAction.addAttribute("href", "javascript:E_inspectProperty();");
        inspectPropertyAction.addText("[Inspect Property]");
        parent.add(inspectPropertyAction);

        Element frameHierarchyLink = new Element(ElementNames.A);
        frameHierarchyLink.setWhitespaceRelevant(true);
        frameHierarchyLink.addAttribute("href", ClientObjects.getServiceUri(conn, SERVICE_COMPONENT_HIERARCHY_VIEWER));
        frameHierarchyLink.addAttribute("target", "componentHierarchy");
        frameHierarchyLink.addText("[Component Hierarchy]");
        parent.add(frameHierarchyLink);
    }
    
    /**
     * @see nextapp.echoservlet.Service#getId()
     */
    public Id getId() {
        return ID;
    }
    
    /**
     * Returns a JavaScript method call which will adjust the Z-indices of 
     * raised and lowered windows.  The <code>OutgoingUpdateQueue</code>
     * will be queried to determine windows that need to be raised or lowered.
     * 
     * @param conn The connection wrapping the response.
     * @return A JavaScript method call to adjust the Z-indices of raised and
     *         lowered windows.
     */
    private String getScriptAdjustWindowZIndices(Connection conn) {
        StringBuffer sb = new StringBuffer();
        if (outgoingUpdateQueue.isWindowLowerRequired()) {
            Iterator it = outgoingUpdateQueue.dequeueWindowLower();
            while (it.hasNext()) {
                if (sb.length() > 0) {
                    sb.append(", ");
                }
                sb.append('\"');
                sb.append(((WindowUI) it.next()).getId());
                sb.append("\", -1");
                it.remove();
            }
        }
        if (outgoingUpdateQueue.isWindowRaiseRequired()) {
            Iterator it = outgoingUpdateQueue.dequeueWindowRaise();
            while (it.hasNext()) {
                if (sb.length() > 0) {
                    sb.append(", ");
                }
                sb.append('\"');
                sb.append(((WindowUI) it.next()).getId());
                sb.append("\", 1");
                it.remove();
            }
        }
        return ClientObjects.STD_PREFIX + "adjustWindowZIndices(" + sb + ");";
    }
    
    /**
     * Produces a JavaScript method invocation string which will cause
     * the client to update the content of one or more frames based on
     * the OutgoingUpdateQueue's collection of frames that require updates.
     * Removes all pane update tasks from the OutgoingUpdateQueue.
     *
     * @param conn The connection on which the JavaScript will be 
     *        rendered.
     * @return A JavaScript invocation string which will cause the update.
     */
    private String getScriptChangeFrameContent(Connection conn) {
        List args = new ArrayList();
        PaneUI paneUI, parentPaneUI;
        int startPoint;
        String uri;

        // Iterate through list of service updates
        for (Iterator paneUpdates = outgoingUpdateQueue.dequeuePaneUpdate(); paneUpdates.hasNext();) {
            paneUI = (PaneUI) paneUpdates.next();
            Id containingWindowId = instancePeer.getContainingWindow((ComponentPeer) paneUI).getId();
            
            if (paneUI instanceof ContainerPaneUI) {
                // For container panes, retrieve URI from pane that container is forwarding to.
                // The container pane will probably return itself, as most container panes
                // do have two or more child panes.  See ContainerPaneUI for more details.
                uri = ((ContainerPaneUI) paneUI).getForwardingPane().getUri(conn);
            } else {
                uri = paneUI.getUri(conn);
            }
            
            // record starting position
            startPoint = args.size();
            
            // find path to WindowUI
            parentPaneUI = instancePeer.getContainingPane(((ComponentPeer) paneUI).getParent());
            while (parentPaneUI != null) {
                if (!(parentPaneUI instanceof ContainerPaneUI && ((ContainerPaneUI) parentPaneUI).isForwarding())) {
                    // Parent pane is NOT a forwarding container pane: add id.
                    args.add(startPoint, paneUI.getId());
                }
                paneUI = parentPaneUI;
                parentPaneUI = instancePeer.getContainingPane(((ComponentPeer) paneUI).getParent());
            }
            args.add(startPoint, containingWindowId);
            args.add(startPoint, uri);
            args.add(startPoint, Integer.toString(args.size() - startPoint - 2));
            
            paneUpdates.remove();
        }

        return ClientObjects.STD_PREFIX + "changeFrameContent(\'" + Delimiter.toString(args, "\', \'") + "\');";
    }
    
    /**
     * Produces a JavaScript method invocation string which will cause
     * the client to update the root content of one or more windows
     * based on the OutgoingUpdateQueue's collection of windows that require updates.
     * Removes all window update tasks from the OutgoingUpdateQueue.
     *
     * @param conn The connection on which the JavaScript will be 
     *        rendered.
     * @return A JavaScript invocation string which will cause the update.
     */
    private String getScriptChangeWindowContent(Connection conn) {
        List args = new ArrayList();
        WindowUI windowUI;
        int startPoint;
        for (Iterator windowUpdates = outgoingUpdateQueue.dequeueWindowUpdate(); windowUpdates.hasNext();) {
            windowUI = (WindowUI) windowUpdates.next();
            startPoint = args.size();

            args.add(startPoint, windowUI.getId());
            args.add(startPoint, windowUI.getRootPaneUI().getUri(conn));
            args.add(startPoint, Integer.toString(0));

            windowUpdates.remove();
        }
        return ClientObjects.STD_PREFIX + "changeFrameContent(\'" + Delimiter.toString(args, "\', \'") + "\');";
    }
    
    /**
     * Returns a string containing a JavaScript method call that will cause
     * a window to close.
     *
     * @param windowUI The peer of the window to be closed.
     * @return A JavaScript method call to close the window.
     */
    private String getScriptCloseWindow(WindowUI windowUI) {
        return ClientObjects.STD_PREFIX + "closeWindow(\'" + windowUI.getId()  + "\');";
    }
    
    /**
     * Returns a string containing a JavaScript method call that will cause
     * a new window to open.
     *
     * @param conn The <code>Connection</code> wrapping the request.
     * @param windowUI The peer of the window to be opened.
     * @return A JavaScript method call that will open the window.
     */
    private String getScriptOpenWindow(Connection conn, WindowUI windowUI) {
        Window window = (Window) windowUI.getComponent();
    
        return ClientObjects.STD_PREFIX + "openWindow(\'" + windowUI.getId()  + "\', "
                + "\'" + ClientObjects.getServiceUri(conn, windowUI) + "\', " 
                + window.getWidth() + ", " + window.getHeight() + ", " +  windowUI.getFlags() + ");";
    }
        
    /**
     * Returns a string containing a JavaScript method call that will cause 
     * windows to have their titles changed.
     *
     * @return A JavaScript method call that will update window titles.
     */
    private String getScriptSetWindowTitles() {
        List args = new ArrayList(2);
        WindowUI windowUI;
        Window window;
        
        for (Iterator windowTitleUpdates = outgoingUpdateQueue.dequeueWindowTitleUpdate(); windowTitleUpdates.hasNext();) {
            windowUI = (WindowUI) windowTitleUpdates.next();
            args.add(windowUI.getId());
            window = (Window) windowUI.getComponent();
            String title = window.getTitle();
            args.add(Script.encodeQuotedString(title == null ? "" : title));
            
            windowTitleUpdates.remove();
        }
        
        return ClientObjects.STD_PREFIX + "setWindowTitles(\'" + Delimiter.toString(args, "\', \'") + "\');";
    }
    
    /**
     * Returns a string containing a JavaScript method call that will cause
     * the client to verify that the windows it has open are the same as the
     * windows the server wants open.
     *
     * @return A JavaScript method call that will verify the client state.
     */
    private String getScriptVerifyClientState(boolean acknowledgementRequired) {
        StringBuffer sb = new StringBuffer();
        Iterator it = instancePeer.getWindows();
        
        sb.append(ClientObjects.STD_PREFIX);
        if (acknowledgementRequired) {
            sb.append("verifyClientState(true, \'");
        } else {
            sb.append("verifyClientState(false, \'");
        }
        
        while (it.hasNext()) {
            sb.append(((WindowUI) it.next()).getId());
            if (it.hasNext()) {
                sb.append("\', \'");
            }
        }
        
        sb.append("\');");
        
        return sb.toString();    
    }
    
    /**
     * @see nextapp.echoservlet.CacheableService#getVersion()
     */
    public int getVersion() {
        return version++;
    }

    /**
     * Render a &quot;normal&quot; copy of the controller frame, with 
     * instuctions to synchronize the state of the client with that 
     * of the server.
     *
     * @param conn The connection wrapping the response.
     */
    private void render(Connection conn) {
        
        HtmlDocument doc = HtmlDocument.create(conn);
        WindowUI windowUI;
        
        doc.addScript("var E_pendingAction = null;"); 
        doc.addScript("var E_newWindows = null;");
        doc.addScript("var E_ready = false;");
        
        doc.addScriptInclude(instancePeer.getClientErrorService());
        doc.addScriptInclude(EchoServer.SERVICE_COLLECTIONS_SCRIPT);
        doc.addScriptInclude(SERVICE_CONTROLLER_SCRIPT);
        
        if (!incomingUpdateQueue.isDelayed()) {
            // Update code is only executed in the event updates are NOT being delayed.
            
            // If purge flag is set, clear all updates.
            if (instancePeer.getInstance().getAttribute(PURGE_ALL_UPDATES_FLAG) != null) {
                outgoingUpdateQueue.purgeAllUpdates();
                instancePeer.getInstance().setAttribute(PURGE_ALL_UPDATES_FLAG, null);
            }
        
            // add scripting to close browser windows.
            for (Iterator windows = outgoingUpdateQueue.dequeueWindowClose(); windows.hasNext();) {
                windowUI = (WindowUI) windows.next();
                doc.addScript(getScriptCloseWindow(windowUI), HtmlDocument.EVENT_INITIALIZE);
                windows.remove();
            }

            // add scripting to open browser windows.
            for (Iterator windows = outgoingUpdateQueue.dequeueWindowOpen(); windows.hasNext();) {
                windowUI = (WindowUI) windows.next();
                doc.addScript(getScriptOpenWindow(conn, windowUI), HtmlDocument.EVENT_INITIALIZE);
                windows.remove();
            }

            // add scripting to update changed browser frames
            if (outgoingUpdateQueue.isPaneUpdateRequired()) {
                doc.addScript(getScriptChangeFrameContent(conn), HtmlDocument.EVENT_INITIALIZE);
            }

            // add scripting to update window content if necessary
            if (outgoingUpdateQueue.isWindowUpdateRequired()) {
                doc.addScript(getScriptChangeWindowContent(conn), HtmlDocument.EVENT_INITIALIZE);
            }

            // add scripting to update window titles if necessary
            if (outgoingUpdateQueue.isWindowTitleUpdateRequired()) {
                doc.addScript(getScriptSetWindowTitles(), HtmlDocument.EVENT_INITIALIZE);
            }
            
            // add scripting to raise windows if necessary
            if (outgoingUpdateQueue.isWindowRaiseRequired() || outgoingUpdateQueue.isWindowLowerRequired()) {
                doc.addScript(getScriptAdjustWindowZIndices(conn), HtmlDocument.EVENT_INITIALIZE);
            }
        }
 
        // add scripting to verify client state
        doc.addScript(getScriptVerifyClientState(incomingUpdateQueue.isDelayed()), HtmlDocument.EVENT_INITIALIZE);
        
        // add scripting to set ready flag (THIS MUST BE DONE LAST)
        doc.addScript(SET_READY_FLAG, HtmlDocument.EVENT_INITIALIZE);
        
        // create form
        Element form = new Element(ElementNames.FORM);
        form.addAttribute("name", "E_controllerForm");
        form.addAttribute("method", "POST");
        form.addAttribute("action", conn.getApplicationUri());
        doc.getBodyElement().add(form);
        
        // create input field for request type and data
        form.add(HtmlElements.createHiddenField(ClientObjects.PARAMETER_ID, getId().toString()));
        form.add(HtmlElements.createHiddenField(PARAMETER_REQUEST_TYPE, null));
        form.add(HtmlElements.createHiddenField(PARAMETER_REQUEST_DATA, null));

        // add sequence number parameter
        form.add(HtmlElements.createHiddenField(PARAMETER_SEQUENCE_NUMBER, 
                Integer.toString(instancePeer.getSequenceNumber())));
        
        // create input fields in form for user-inputted data
        ClientInputProducer inputProducer;
        for (Iterator inputProducers = instancePeer.getClientInputProducers(); inputProducers.hasNext();) {
            inputProducer = (ClientInputProducer) inputProducers.next();
            // determine if the input producer also implements ControllerFieldInitialState, indicating that the hidden form
            // field should be rendered with a default value.
            if (inputProducer instanceof ControllerFieldInitialState) {
                String initialState = ((ControllerFieldInitialState) inputProducer).getControllerFieldValue();
                if (initialState == null) {
                    form.add(HtmlElements.createHiddenField(PARAMETER_INPUT_PREFIX + inputProducer.getId(), null));
                } else {
                    form.add(HtmlElements.createHiddenField(PARAMETER_INPUT_PREFIX + inputProducer.getId(), 
                            "-" + initialState));
                }
            } else {
                form.add(HtmlElements.createHiddenField(PARAMETER_INPUT_PREFIX + inputProducer.getId(), null));
            }
        }
        
        // create input fields in form for scrollbar positions of each frame
        ContentPaneUI contentPane;
        for (Iterator contentPanes = instancePeer.getContentPanes(); contentPanes.hasNext();) {
            contentPane = (ContentPaneUI) contentPanes.next();
            form.add(HtmlElements.createHiddenField(PARAMETER_PANE_SCROLL_POSITION_PREFIX + contentPane.getId(), null));
        }
        
        // add debugging code
        if (conn.isDebugInteractiveControllerEnabled()) {
            addDebugOptions(conn, doc, form);
        }
        
        // render
        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }

    /**
     * Renders output for the case where the application needs to shutdown and
     * redirect to a different URI.
     *
     * @param conn The connection wrapping the response.
     */
    private void renderShutdown(Connection conn) {
        String shutdownCommand = "parent." + ClientObjects.STD_PREFIX + "shutdown(\'" + 
                Script.encodeQuotedString(instancePeer.getTerminationUri()) + "\');";
        HtmlDocument doc = HtmlDocument.create(conn);
        doc.addScript(shutdownCommand, HtmlDocument.EVENT_INITIALIZE);
        
        if (instancePeer.isExiting()) {
            // Remove the InstancePeer from the HttpSession.
            conn.unbindInstancePeer();
        } else {
            // Clear the termination URI so that when the client re-requests the application it doesn't terminate again.
            instancePeer.clearTerminationUri();
        }

        // render
        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }
        
    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public synchronized void service(Connection conn) {
        ControllerInputProcessor.process(conn);
        instancePeer.getInstance().doValidation();
        if (instancePeer.getTerminationUri() == null) {
            render(conn);
        } else {
            renderShutdown(conn);
        }
    }
}
