/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import nextapp.echo.ContentPane;
import nextapp.echo.Window;

/**
 * Parses the HTTP request, storing component peers that will be used in its
 * processing.
 */
final class ControllerInputProcessor {

    /**
     * Processes the request to the controller.
     *
     * @param conn The connection provided to the Controller that contains the 
     *        HTTP request to be processed.
     */
    static void process(Connection conn) {
        new ControllerInputProcessor(conn);
    }
 
    private Connection conn;
    private InstancePeer instancePeer;
    private Map inputPeers = new HashMap();
    private Map scrollPositions = new HashMap();
    private String requestType;
    private ClientActionProducer actionProducer;
    private String actionCommand;
    private IncomingUpdateQueue incomingUpdateQueue;
    private OutgoingUpdateQueue outgoingUpdateQueue;

    /**
     * Processes a controller request.
     *
     * @param conn The connection provided to the Controller that contains the 
     *        HTTP request to be processed.
     */
    private ControllerInputProcessor(Connection conn) {
        super();
        this.conn = conn;
        instancePeer = conn.getInstancePeer();
        incomingUpdateQueue = instancePeer.getIncomingUpdateQueue();
        outgoingUpdateQueue = instancePeer.getOutgoingUpdateQueue();
        
        // Determine if request is an authentic new request.
        boolean validRequest = isNewRequest();

        try {
            storeRequestData();
        } catch (InvalidClientStateException ex) {
            validRequest = false;
        }

        if (validRequest) {
            // Clear focused component, as it is now in an indeterminate state (may have changed on client).
            instancePeer.getInstance().setFocusedComponent(null);

            if (Controller.REQUEST_TYPE_WINDOW_UNLOAD.equals(requestType)) {
                // Do not update server state to reflect changes made by user on client just yet.
                // The changes should be held in the delayed update map in the OutgoingUpdateQueue.
                incomingUpdateQueue.setDelayed(true);
                incomingUpdateQueue.setDelayedInputUpdateMap(inputPeers);
                incomingUpdateQueue.setDelayedScrollPositionUpdateMap(scrollPositions);
            } else if (Controller.REQUEST_TYPE_STATE_VALID.equals(requestType)
                    || Controller.REQUEST_TYPE_WINDOWS_CLOSED.equals(requestType)
                    || Controller.REQUEST_TYPE_WINDOWS_MISSING.equals(requestType)) {
                // Update server to reflect changes made by user on client before the window unload
                // event occurred.
                if (incomingUpdateQueue.isDelayed()) {
                    inputPeers = incomingUpdateQueue.getDelayedInputUpdateMap();
                    scrollPositions = incomingUpdateQueue.getDelayedScrollPositionUpdateMap();
                    incomingUpdateQueue.setDelayed(false);
                }
                if (scrollPositions != null) {
                    updateScrollbarPositions();
                }
                if (inputPeers != null) {
                    updateInputData();
                }
            } else {
                // Update server state to reflect changes made by user on client.
                updateScrollbarPositions();
                updateInputData();
            }
        }
        
        if (Controller.REQUEST_TYPE_ACTION.equals(requestType)) {
            // Action Request
            if (validRequest) {
                try {
                    doAction();
                } catch (InvalidClientStateException ex) {
                    throw new EchoServletException("Invalid Client State Exception Handler not yet implemented.", ex);
                }
            }
        } else if (Controller.REQUEST_TYPE_WINDOWS_CLOSED.equals(requestType)) {
            // Windows Closed On Client
            doWindowsClosed();
        } else if (Controller.REQUEST_TYPE_WINDOWS_MISSING.equals(requestType)) {
            // Windows Missing On Client
            doWindowsMissing();
        }

        instancePeer.getCookieManager().storeCookies(conn.getResponse());

        if (validRequest) {
            instancePeer.incrementSequenceNumber();
        }

        if (conn.isDebugVerboseLoggingEnabled()) {
            System.err.println("requestType=\"" + requestType + "\" data=\"" + conn.getParameter(Controller.PARAMETER_REQUEST_DATA)
                    + "\" valid=" + validRequest);
        }
    }
    
    /**
     * Processes a submission of the controller's form in response to an action
     * by the user.
     */
    private void doAction() 
    throws InvalidClientStateException {
        actionProducer.clientAction(actionCommand);
    }
    
    /**
     * Processes a submission of the controller's form in response to windows
     * having been closed by the user.  Notifies the application of the 
     * windows that were closed.
     */
    private void doWindowsClosed() {
        String closedWindows = conn.getParameter(Controller.PARAMETER_REQUEST_DATA);
        StringTokenizer st = new StringTokenizer(closedWindows, ",");
        Id windowId;
        WindowUI windowUI;
        Window window;
        
        while (st.hasMoreTokens()) {
            windowId = new Id(st.nextToken());
            windowUI = (WindowUI) instancePeer.getPeer(windowId);

            // If the report of the window being closed is false, i.e. it was 
            // already closed and destroyed and the controller was resubmitted
            // due to a reload, do nothing.
            
            if (windowUI != null) {
                // Remove all service updates under dead window.
                if (windowUI.getRootPaneUI() != null) {
                    // Window might have no content (unlikely but possible).
                    outgoingUpdateQueue.deleteComponentUpdate((ComponentPeer) windowUI.getRootPaneUI());
                }
                
                window = (Window) windowUI.getComponent();
                window.userClose();

                // Check if window is still registered.  If it is, reopen it.
                if (windowUI.getInstancePeer() != null) {
                    outgoingUpdateQueue.enqueueWindowOpen(windowUI);
                }
            }
        }
    }
    
    /**
     * Processes a submission of the controller's form in response to an
     * inconsistency being found between the windows that should be open
     * on the client and the windows that are actually open.
     * The windows will be reopened.
     */
    private void doWindowsMissing() {
        String missingWindows = conn.getParameter(Controller.PARAMETER_REQUEST_DATA);
        StringTokenizer st = new StringTokenizer(missingWindows, ",");
        Id windowId;
        WindowUI windowUI; 
        while (st.hasMoreTokens()) {
            windowId = new Id(st.nextToken());
            windowUI = (WindowUI) instancePeer.getPeer(windowId);
            outgoingUpdateQueue.enqueueWindowOpen(windowUI);
        }
    }

    /**
     * Determines whether the request to the controller is new by comparing
     * its sequence number to that of the server.
     *
     * @return True if the request is new.
     */
    private boolean isNewRequest() {
        int serverSequenceNumber = instancePeer.getSequenceNumber();
        int clientSequenceNumber;
        String clientSequenceNumberString = conn.getParameter(Controller.PARAMETER_SEQUENCE_NUMBER);
        if (clientSequenceNumberString == null) {
            clientSequenceNumber = 1; // initial request.
        } else {
            clientSequenceNumber = Integer.parseInt(clientSequenceNumberString);
        }
        
        return clientSequenceNumber > serverSequenceNumber;
    }
    
    /**
     * Stores the <code>ClientActionProducer</code> object whose action was
     * fired on the client.  The <code>ClientActionProducer</code> will be 
     * notified when the <code>doAction</code> method is later called.
     *
     * @throws InvalidClientStateException if this method determines that the
     *         client browser may be in an invalid state, i.e., the
     *         <code>ClientActionProducer</code> does not exist.
     */
    private void storeActionData() 
    throws InvalidClientStateException {
        String requestData = conn.getParameter(Controller.PARAMETER_REQUEST_DATA);
        int seperatorIndex = requestData.indexOf(",");
        Id actionId = new Id(requestData.substring(0, seperatorIndex));
        actionCommand = requestData.substring(seperatorIndex + 1);
        actionProducer = (ClientActionProducer) instancePeer.getPeer(actionId);
        if (actionProducer == null) {
            throw new InvalidClientStateException("Invalid action: no action producer was found with Id#"  + actionId);
        }
    }
     
    /**
     * Retrieves an input data parameter value from the HTTP request,
     * storing it in the <code>inputPeers</code> map under the key of
     * its <code>ClientInputProducer</code>.  The 
     * <code>ClientInputProducer</code> will be notified of the change in its 
     * value when the <code>updateInputData()</code> method is later called.
     *
     * @param parameterName The name of the parameter that contains the input.
     * @throws InvalidClientStateException if this method determines that the
     *         client browser may be in an invalid state, i.e., input data was
     *         sent for an unregistered <code>ClientInputProducer</code>.
     */
    private void storeInputData(String parameterName) 
    throws InvalidClientStateException {
        ComponentPeer peer = instancePeer.getPeer(new Id(parameterName.substring(Controller.PARAMETER_INPUT_PREFIX_LENGTH)));
        if (peer == null) {
            throw new InvalidClientStateException("Input Received for Unregistered Rendering Peer");
        } else {
            String parameterValue = conn.getParameter(parameterName);
            if (parameterValue != null && parameterValue.length() > 0 && parameterValue.charAt(0) == 'x') { 
                inputPeers.put(peer, parameterValue.substring(1));
            }
        }
    }
    
    /** 
     * Stores rendering peers that will be used in the request locally, such
     * that they can be accessed even if they are unregistered as a result of
     * an event fired to the application.
     */   
    private void storeRequestData() 
    throws InvalidClientStateException {
        Enumeration enum = conn.getParameterNames();
        String parameterName;
        
        // Store request type.
        requestType = conn.getParameter(Controller.PARAMETER_REQUEST_TYPE);

        while (enum.hasMoreElements()) {
            parameterName = (String) enum.nextElement();
            if (parameterName.startsWith(Controller.PARAMETER_INPUT_PREFIX)) {
                storeInputData(parameterName);
            } else if (parameterName.startsWith(Controller.PARAMETER_PANE_SCROLL_POSITION_PREFIX)) {
                storeScrollbarPosition(parameterName);
            }
        }
        
        if (Controller.REQUEST_TYPE_ACTION.equals(requestType)) {
            storeActionData();
        }
    }

    /**
     * Retrieves a scrollbar position parameter value from the HTTP request,
     * storing it in the <code>scrollPositions</code> map under the key of
     * its <code>ContentPaneUI</code>.  The <code>ContentPaneUI</code> will be
     * notified of the change in its scroll bar positions when the
     * <code>updateScrollbarPositions()</code> method is later called.
     *
     * @param parameterName The name of the parameter that contains scrollbar
     *        position information.
     * @throws InvalidClientStateException if this method determines that the
     *         client browser may be in an invalid state.
     */
    private void storeScrollbarPosition(String parameterName) 
    throws InvalidClientStateException {
        // Parameter is a ContentPane scroll position.  Store the content pane peer.
        ContentPaneUI contentPaneUI;
        contentPaneUI = (ContentPaneUI) instancePeer.getPeer(
                new Id(parameterName.substring(Controller.PARAMETER_PANE_SCROLL_POSITION_PREFIX_LENGTH)));
        String parameterValue = conn.getParameter(parameterName);
        int commaIndex = parameterValue.indexOf(",");
        if (commaIndex != -1) {
            try {
                scrollPositions.put(contentPaneUI, new int[]{
                        Integer.parseInt(parameterValue.substring(0, commaIndex)),
                        Integer.parseInt(parameterValue.substring(commaIndex + 1))});
            } catch (NumberFormatException ex) { } 
        }
    }
    
    /**
     * Updates the state of all <code>ClientInputProducer</code>s, notifying
     * them of data received from the client.
     */
    private void updateInputData() {
        for (Iterator it = inputPeers.keySet().iterator(); it.hasNext();) {
            ClientInputProducer inputProducer = (ClientInputProducer) it.next();
            inputProducer.clientInput((String) inputPeers.get(inputProducer));
        }
    }
    
    /**
     * Updates the scroll bar positions of any content panes whose positions 
     * were changed of their new state.
     */
    private void updateScrollbarPositions() {
        for (Iterator it = scrollPositions.keySet().iterator(); it.hasNext();) {
            ContentPaneUI contentPaneUI = (ContentPaneUI) it.next();
            int[] coord = (int[]) scrollPositions.get(contentPaneUI);
            ContentPane contentPane = (ContentPane) contentPaneUI.getComponent();
            contentPane.setScrollBarX(coord[0]);
            contentPane.setScrollBarY(coord[1]);
        }
    }
}
