/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
 
package nextapp.echoservlet;

import java.io.Serializable;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A utility class used to cookies set by an application on the client browser.
 * Applications may obtain access to this object by retrieving their
 * <code>ServerContext</code> from their <code>EchoInstance</code> and calling
 * <code>ServerContext.getCookieManager()</code>.
 */
public class CookieManager 
implements Serializable {
    
    /**
     * Contains the cookies that were most recently sent in by the client's
     * most recent HTTP request.  If no cookies are present on the client,
     * clientCookies may be null.
     */
    private transient Cookie[] clientCookies = null;
    
    /**
     * Contains the Echo applications "working copy" of the cookies.  This
     * map references cookies by their names, and may contain cookies that the
     * Echo application has requested to be set but that have not yet been set
     * on the client.
     */
    private Map localCookieMap = null;

    /**
     * A set of cookies that have changed.  Cookies are stored in this set when
     * addCookie() is called, such that they will be updated in the HttpResponse
     * generated by the Connection.  Deleted cookies are also stored in this 
     * set, with there expiration set to zero, such that the client will delete
     * them.  The contents of this set are sent to the client at the next 
     * possible opportunity.
     */    
    private Set updatedCookies = new HashSet();

    /**
     * Adds a new cookie or updates an existing one.  In the case of updating an
     * existing cookie, be certain to reset its maximum age with setMaxAge() if
     * necessary.  (Cookies on the client browser do not report their maximum 
     * age to the server, only their value.)
     *
     * @param cookie The cookie to add or update.
     */
    public void add(Cookie cookie) {
        getLocalCookieMap().put(cookie.getName(), cookie);
        
        updatedCookies.add(cookie);
    }
    
    /**
     * Clears the locally held information about cookies.
     */
    private void clearLocalCookieMap() {
        localCookieMap = null;
    }
    
    /**
     * Returns a cookie with the given name if it is exists or null if it
     * does not.
     *
     * @param name The name of the cookie to return.
     * @return The cookie identified by <code>name</code>.
     */
    public Cookie get(String name) {
        return (Cookie) getLocalCookieMap().get(name);
    }

    /**
     * Creates a new local cookie set, copying the client cookies array into it.
     * This method is lazily called by other methods that need to work with the
     * local cookie map.
     *
     * @return A map containing the client's cookies.
     */
    private Map getLocalCookieMap() {
        if (localCookieMap == null) {
            localCookieMap = new TreeMap();

            if (clientCookies != null) {
                for (int index = 0; index < clientCookies.length; ++index) {
                    localCookieMap.put(clientCookies[index].getName(), clientCookies[index]);
                }
            }
        }
        
        return localCookieMap;
    }
    
    /**
     * Returns an iterator over all present cookies.
     *
     * @return An iterator over all present cookies.
     */
    public Iterator iterator() {
        return getLocalCookieMap().values().iterator();
    }
    
    /**
     * Removes the specified cookie.  This is accomplished by setting
     * the cookie on the client with a maximum age of 0.
     *
     * @param name The name of the cookie to remove.
     */
    public void remove(String name) {
        Cookie cookie = get(name);
        
        cookie.setMaxAge(0);
        
        if (localCookieMap != null) {
            localCookieMap.remove(name);
        }

        updatedCookies.add(cookie);
    }

    /**
     * Retrieves cookies from a client HTTP request and stores them in the 
     * CookieManager.  
     * When this method is called, all "working information" about cookies 
     * is destroyed.
     *
     * @param request An <code>HttpServletRequest</code> from which to 
     *        extract cookies.
     */
    void retrieveCookies(HttpServletRequest request) {
        clientCookies = request.getCookies();
        clearLocalCookieMap();
    }
    
    /** 
     * Returns the number of cookies present.  This is the number of cookies
     * that the Echo application is tracking at the current time.  Calls to the
     * add and remove methods will increase or decrease this number even though
     * the cookie has not yet been added or removed on the client.
     *
     * @return The number of cookies present.
     */
    public int size() {
        return getLocalCookieMap().size();
    }
    
    /**
     * Stores cookies on the client by adding them to the HTTP response.
     * When this method is called, all "working information" about cookies 
     * is destroyed.
     *
     * @param response An HttpServletResponse to feed the cookies to.
     */
    void storeCookies(HttpServletResponse response) {
        if (updatedCookies != null) {
            Iterator it = updatedCookies.iterator();

            while (it.hasNext()) {
                response.addCookie((Cookie) it.next());
                it.remove();
            }
        }
    }
}
