/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import nextapp.echo.EchoInstance;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * Holds global information about an application and handles HTTP requests for 
 * an application.  This class is extended by an application.
 */
public abstract class EchoServer extends HttpServlet {

    public static final String CHARACTER_ENCODING_ISO_8859_1 = "iso-8859-1";
    public static final String CHARACTER_ENCODING_ISO_8859_2 = "iso-8859-2";
    public static final String CHARACTER_ENCODING_ISO_8859_3 = "iso-8859-3";
    public static final String CHARACTER_ENCODING_ISO_8859_4 = "iso-8859-4";
    public static final String CHARACTER_ENCODING_ISO_8859_5 = "iso-8859-5";
    public static final String CHARACTER_ENCODING_US_ASCII = "us-ascii";
    public static final String CHARACTER_ENCODING_UTF_8 = "utf-8";

    /**
     * Default 'Client Not Ready' error message.
     */
    static final String DEFAULT_CLIENT_ERROR_NOT_READY_MESSAGE 
            = "The server is currently processing your request.\\n\\nIf it fails to respond within a reasonable amount of time, "
            + "try pressing your browser's \\\"refresh\\\" or \\\"reload\\\" button.";

    /**
     * The app container name and version, as well as the framework name and 
     * version.  This information will be printed in META tags in all generated
     * documents.
     */
    public static final String ID_STRING = "NextApp EchoServer v1.1.4 (" + EchoInstance.ID_STRING + ")";
    
    /**
     * The time delta (in seconds) when cached services will expire.
     */
    static final int CACHED_SERVICE_TIMEOUT = 3600;
    
    /** Holds global content for all users */
    static final ServiceRegistry services = new ServiceRegistry();
    
    /**
     * The handler of multipart/form-data encoded HTTP requests.
     */
    private static MultipartRequestWrapper multipartRequestWrapper = null; 
       
    /** The service responsible for initializing applications */
    public static final Service SERVICE_INITIALIZER = new Initializer();
    
    /** A blank HTML page, suitable for filling empty ContainerPanes, etc. */
    public static final Service SERVICE_BLANK_DOCUMENT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "blank", ResourceNames.BLANK_DOCUMENT);
    
    /** 
     * The JavaScript include which provides collections-framework-like 
     * functionality to JavaScript objects. 
     */
    public static final Service SERVICE_COLLECTIONS_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "col", ResourceNames.COLLECTIONS_SCRIPT);
    
    /**
     * The service a client browser will be directed to in the event that
     * his/her session expires.
     */
    public static final Service SERVICE_SESSION_EXPIRED 
            = Template.createFromResource(ClientObjects.STD_PREFIX + "sex", ResourceNames.SESSION_EXPIRED);

    /** The default safe method invocation service */
    public static final Service SERVICE_SAFE_INVOKE_DEFAULT
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "si_def", ResourceNames.SAFE_INVOKE_DEFAULT_SCRIPT);

    /** 
     * The safe method invocation service  for browsers that support 
     * exception handling using try/catch/finally blocks. 
     */
    public static final Service SERVICE_SAFE_INVOKE_EXCEPTION_SUPPORT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "si_exs", 
            ResourceNames.SAFE_INVOKE_EXCEPTION_SUPPORT_SCRIPT);
            
    public static final Service SERVICE_CLIENT_ERROR_SCRIPT
            = Template.createFromResource(ClientObjects.STD_PREFIX + "error",
            ResourceNames.CLIENT_ERROR_SCRIPT);
            
    private final ApplicationProperties appProperties = new ApplicationProperties(getClass().getName());
    
    private ServletEncodingAdapter servletEncodingAdapter;
            
    static {
        PeerDiscovery.loadPeers();
        
        services.add(SERVICE_INITIALIZER);
        services.add(SERVICE_COLLECTIONS_SCRIPT);
        services.add(SERVICE_SESSION_EXPIRED);
        services.add(SERVICE_SAFE_INVOKE_DEFAULT);
        services.add(SERVICE_SAFE_INVOKE_EXCEPTION_SUPPORT);
        services.add(SERVICE_CLIENT_ERROR_SCRIPT);
        services.add(SERVICE_BLANK_DOCUMENT);
    }
    
    /**
     * An interface implemented by a supporting object that will handle 
     * multipart/form-data encoded HTTP requests.  This type of request is
     * required for file uploads.  Echo does not provide internal support
     * for file uploads, but instead provides hooks for file-upload handling
     * components.  
     */
    public interface MultipartRequestWrapper {
    
        /**
         * Returns a replacement <code>HttpServletRequest</code> object that
         * may be used to handle a multipart/form-data encoded HTTP request.
         *
         * @param request The HTTP request provided from the servlet container
         *        that has multipart/form-data encoding.
         * @return An HTTP request that is capable of handling 
         *         multipart/form-data encoding.
         */
        public HttpServletRequest getWrappedRequest(HttpServletRequest request)
        throws IOException, ServletException;
    }
    
    /**
     * Adds a globally available service that will be available to all
     * applications deployed with this EchoServer object.  Global services
     * may not be removed.
     *
     * @param service The global service to be added.
     */
    public static void addGlobalService(Service service) {
        services.add(service);
    }
    
    /**
     * Returns the multipart/form-data encoded HTTP request handler.
     * 
     * @return The multipart/form-data encoded HTTP request handler.
     * @see #setMultipartRequestWrapper
     */
    public static final MultipartRequestWrapper getMultipartRequestWrapper() {
        return multipartRequestWrapper;
    }
    
    /**
     * Loads peer bindings for additional components.  This method is used to
     * associate custom component peers with custom components.
     *
     * @param propertiesFile The name of the properties file containing the 
     *        peer binding data.
     */
    public static final void loadPeerBindings(String propertiesFile) {
        ComponentPeerFactory.addAssociations(propertiesFile);
    }
    
    /**
     * Sets the multipart/form-data encoded HTTP request handler.
     * The multipart request wrapper can only be set one time.  It should be set
     * in a static block of your Echo application.  This method will disregard
     * additional attempts to set the wrapper if the provided wrapper's class
     * is identical to the existing one.  If the wrapper is already set and the
     * new wrapper object's class is different or the wrapper is null, an
     * exception is thrown.
     *
     * @param multipartRequestWrapper The handler for multipart/form-data 
     *        encoded HTTP requests.
     * @throws IllegalStateException if the application attempts to change
     *        a previously set multipart request handler.
     */
    public static final void setMultipartRequestWrapper(MultipartRequestWrapper multipartRequestWrapper) {
        if (EchoServer.multipartRequestWrapper == null) {
            EchoServer.multipartRequestWrapper = multipartRequestWrapper;
        } else {
            if (multipartRequestWrapper == null || 
                    !EchoServer.multipartRequestWrapper.getClass().getName().equals(
                    multipartRequestWrapper.getClass().getName()));        
        }
    }
    
    /**
     * Handles a GET request.
     *
     * @see #process(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public final void doGet(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException {
        process(request, response);
    }
    
    /**
     * Handles a POST request.
     *
     * @see #process(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public final void doPost(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException {
        process(request, response);
    }
    
    /**
     * Returns the Application Properties for this Echo Servlet.
     *
     * @return The Application Properties for this Echo Servlet.
     */
    public final ApplicationProperties getApplicationProperties() {
        return appProperties;
    }

    /**
     * Returns the default locale based on the request.
     */
    public Locale getDefaultLocale(HttpServletRequest request) {
        return request.getLocale();
    }
    
    /**
     * Returns the <code>ServletEncodingAdapter</code> object that serves
     * this <code>EchoServer</code>.
     * 
     * @return The <code>ServletEncodingAdapter</code>.
     */
    ServletEncodingAdapter getServletEncodingAdapter() {
        return servletEncodingAdapter;
    }
        
    /**
     * Returns the URI that should be redirected to in the event a user's 
     * session within an Echo application expires.  The application's URI
     * is provided.  The default implementation simply returns the 
     * application's URI, restarting the app.  This method should be
     * overridden by applications that desire to direct users to an alternate
     * URI.
     *
     * @param applicationUri The URI of the Echo application.
     * @return The URI to redirect to in the event the user's session
     *         expires.
     */
    public String getSessionExpirationUri(String applicationUri) {
        return applicationUri;
    }

    /**
     * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
     */
    public void init(ServletConfig config)
    throws ServletException {
        super.init(config);
        servletEncodingAdapter = new ServletEncodingAdapter(this);
    }
    
    /**
     * A method to be provided by the derived class that will return a new 
     * EchoInstance (an instance of an Echo application).
     *
     * @return A new EchoInstance for the new user.
     */
    public abstract EchoInstance newInstance();
    
    /**
     * Handles an HTTP request to the EchoServer.  This method will create
     * a <code>Connection</code> to wrap the request and response objects
     * provided by the servlet container and invoke <code>Connection</code>'s
     * <code>process()</code> method to handle it.  If the request is
     * encoded as multipart/form-data, the MultipartRequestWrapper will be 
     * used.
     *
     * @param request The <code>HTTPServletRequest</code> object provided by 
     *        the servlet container.
     * @param response The <code>HTTPServletResponse</code> object provided by 
     *        the servlet container.
     */
    protected void process(HttpServletRequest request, HttpServletResponse response) 
    throws IOException, ServletException {
        Connection conn = null;
        try {
            conn = new Connection(this, request, response);
            conn.process();
        } catch (Exception ex) {
            if (conn == null) {
                processException(null, request, response, ex);
            } else {
                processException(conn.getInstancePeer(), request, response, ex);
            }

            // Attempt to invalidate the session.
            try {
                HttpSession session = request.getSession(false);
                if (session != null) {
                    session.invalidate();
                }
            } catch (Exception ex2) { 
                log("Error while attempting to invalidate session as a result of previous error.", ex2);
            }
        }
    }
    
    /**
     * Processes an exception which occurred during an invocation of the 
     * <code>process()</code> method.  This method may be overriden to take
     * additional actions when errors occur.
     * 
     * @param instancePeer The <code>InstancePeer</code> object of the 
     *        failed application.
     * @param request The <code>HttpServletRequest</code> in use during 
     *        failure.
     * @param response The <code>HttpServletResponse</code> in use during 
     *        failure.
     * @param ex The <code>Exception</code> that occurred.
     */
    protected void processException(InstancePeer instancePeer, HttpServletRequest request, HttpServletResponse response,
            Exception ex) 
    throws IOException, ServletException {
        ErrorHandler.process(instancePeer, request, response, this, ex);
    }
}
