/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * An application error handler/logger that will notify the
 * user with both a simple HTML page and JavaScript alert.  
 */
class ErrorHandler {

    /** Text that preceeds the error message */
    private static final String PRE_ERROR_TEXT = "An application error occurred: ";

    /** Text that follows the error message */
    private static final String POST_ERROR_TEXT = "The error has been logged.";
    
    /** Error id counter */
    private static int errorIdCount = 1000;

    /** 
     * The first segment of the &quot;template&quot; used to display the HTML
     * error page. 
     */
    private static final String HTML_1 
            = "<html>\n"
            + "<head>\n"
            + "<title>Error</title>\n"
            + "<script language=\"JavaScript\">\n"
            + "alert(\"";

    /** 
     * The second segment of the &quot;template&quot; used to display the HTML
     * error page. 
     */
    private static final String HTML_2 
            = "\");\n"
            + "if (top.E_shutdown) {\n"
            + "    top.E_shutdown(\"";

    /** 
     * The third segment of the &quot;template&quot; used to display the HTML
     * error page. 
     */
    private static final String HTML_3 
            = "\");\n"
            + "}\n"
            + "</script>\n"
            + "</head>\n"
            + "<body>\n";

    /** 
     * The fourth segment of the &quot;template&quot; used to display the HTML
     * error page. 
     */
    private static final String HTML_4 
            =  "\n" 
            + "</body>\n" 
            + "</html>";

    /**
     * Renders an error page and shuts down the application.
     * The application's session will be invalidated, and JavaScript
     * will be executed to close extra browser windows if they have
     * been opened.  The error will be logged to the Servlet log file.
     *
     * @param request The HTTP request object provided by the servlet 
     *        container.
     * @param response The HTTP response object provided by the servlet
     *        container.
     * @param servlet A reference to the application's underlying 
     *        <code>HttpServlet</code>.  The servlet is required to
     *        access the logging facility.
     * @param ex The fatal exception.
     */
    static void process(InstancePeer instancePeer, HttpServletRequest request, HttpServletResponse response, 
            HttpServlet servlet, Exception ex) 
    throws ServletException, IOException {
        String errorId = ++errorIdCount + "_" + Long.toString(System.currentTimeMillis());
        servlet.log("EchoServer Error Log Reference #" + errorId, ex);
        
        // Attempt to obtain the error redirection URI.
        String errorUri = "about:blank";
        try {
            if (instancePeer != null) {
                errorUri = instancePeer.getErrorUri();
            }
        } catch (Exception ex2) { 
            servlet.log("Error while attempting to obtain error URI as a result of previous error: " + errorId, ex2);
        }

        // Generate error page.
        PrintWriter pw = response.getWriter();
        pw.print(HTML_1);
        pw.print(PRE_ERROR_TEXT);
        pw.print("\\n");
        pw.print(errorId);
        pw.print("\\n");
        pw.print(POST_ERROR_TEXT);
        pw.print(HTML_2);
        pw.print(errorUri);
        pw.print(HTML_3);
        pw.print(PRE_ERROR_TEXT);
        pw.print("<br>\n");
        pw.print(errorId);
        pw.print("<br>\n");
        pw.print(POST_ERROR_TEXT);
        pw.print(HTML_4);
        pw.close();
    }
    
    /** Non-instantiable class. */
    private ErrorHandler() { }
}
