/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Script;
import nextapp.echoservlet.html.ScriptElement;
import nextapp.echoservlet.html.StyleElement;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A representation of an HTML document.  This class functions as the root 
 * node for the tree of HTMLElements that make up a renderable document.
 */
public class HtmlDocument 
implements Serializable {

    /**
     * A document header for an HTML 4.01 Transitional document.
     */
    private static final String DOCTYPE_HTML401_TRANS = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">";
    
    /**
     * The generator meta tag.
     */
    private static final Element META_GENERATOR;
    
    /**
     * A script event type for &quot;initialization&quot; tasks.  This 
     * constant is a suitable value for the <code>eventName</code> parameter
     * of the <code>addScript</code> method.  A script added with
     * this event type will be executed once its document's frame's
     * parents have executed their initialization scripts.
     */
    public static final int EVENT_INITIALIZE = 1;
    
    /**
     * A script event type for &quot;onload&quot; tasks.   This constant
     * is a suitable value for the <code>eventName</code> parameter
     * of the <code>addScript</code> method.  A script added with
     * this event type will be executed once its document's frame's
     * child frames have executed their onload scripts.
     */
    public static final int EVENT_ONLOAD = 2;
    
    /**
     * A script event type for &quot;onunload&quot; tasks.  This constant
     * is a suitable value for the <code>evetName</code> parameter
     * of the <code>addScript</code> method.  A script added with
     * this event type will be executed when a document is removed
     * from service, such as in the event that its content is changed
     * or its window is closed or reloaded.
     */
    public static final int EVENT_ONUNLOAD = 3;
    
    private static final Service SERVICE_INIT_FRAME_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "init", ResourceNames.INIT_FRAME_SCRIPT);

    static {
        META_GENERATOR = new Element(ElementNames.META, false);
        META_GENERATOR.addAttribute("name", "generator");
        META_GENERATOR.addAttribute("content", EchoServer.ID_STRING);
        EchoServer.addGlobalService(SERVICE_INIT_FRAME_SCRIPT);
    }
    
    /**
     * Creates a normal document (not a frameset).
     *
     * @param conn The connection to which the document will be 
     *             rendered.
     * @return An HTML document that whose body is a 
     *         &lt;body&gt; tag.
     */
    public static HtmlDocument create(Connection conn) {
        Element body = new Element(ElementNames.BODY, true);
        return new HtmlDocument(conn, body);
    }
    
    /**
     * Creates a frameset document.
     *
     * @param conn The connection to which the document will be 
     *             rendered.
     * @return An HTML document that whose body is a 
     *         &lt;frameset&gt; tag.
     */
    public static HtmlDocument createFrameSet(Connection conn) {
        Element body = new Element(ElementNames.FRAMESET, true);
        return new HtmlDocument(conn, body);
    }

    private Element body;
    private Element head;
    private Set scriptIncludes = null;
    private List initializationEvents = null;
    private List onloadEvents = null;
    private List onunloadEvents = null;
    private Map styles = null;
    private Connection conn;
    private boolean cursorOnNewLine = true;
    private List rootScripts = null;
    private int xScrollBarPosition = 0;
    private int yScrollBarPosition = 0;
    private int nextId = 0;
    
    /** 
     * Creates a new instance rendered through the specified connection
     * and containing <code>body</code> as its root content element.
     *
     * @param conn The connection for which this document is being put into
     *        service.
     * @param body The element that will be used as the root content
     *        element of this document (a &lt;body&gt; or &lt;frame&gt;
     *        element).
     */
    protected HtmlDocument(Connection conn, Element body) {
        super();

        this.conn = conn;
        this.body = body;

        head = new Element(ElementNames.HEAD, true);
        head.add(META_GENERATOR);
    }
    
    /** 
     * Adds a line of JavaScript that will be invoked as the document is
     * rendered.  Such scripts may invoked before all resources are available,
     * and thus most script calls should be added using the 
     * addScript(script, eventName) method.  This method is only useful for
     * adding global script variables to the document body.
     *
     * @param script A line of JavaScript code.
     */
    public void addScript(String script) {
        if (rootScripts == null) {
            rootScripts = new ArrayList(3);
        }
        
        rootScripts.add(script);
    }
    
    /**
     * Adds a line of JavaScript that will be invoked when an event occurs.
     * These lines of JavaScript will be executed in the order that they were
     * added.
     *
     * @param script A line of JavaScript code.
     * @param eventType The name of the event, one of the following values:
     *        <ul>
     *        <li><code>EVENT_INITIALIZE</code>
     *        execute the script when all frames have loaded.</li> 
     *        <li><code>EVENT_ONLOAD</code>
     *        execute the script when all child frames have loaded.</li>
     *        <li><code>EVENT_ONUNLOAD</code>
     *        execute the script when the frame unloads.</li>
     *        </ul>
     */
    public void addScript(String script, int eventType) {
        List eventActions;
        switch (eventType) {
            case EVENT_INITIALIZE:
                if (initializationEvents == null) {
                    initializationEvents = new ArrayList(5);
                }
                eventActions = initializationEvents;
                break;
            case EVENT_ONLOAD:
                if (onloadEvents == null) {
                    onloadEvents = new ArrayList(5);
                }
                eventActions = onloadEvents;
                break;
            case EVENT_ONUNLOAD:
                if (onunloadEvents == null) {
                    onunloadEvents = new ArrayList(5);
                }
                eventActions = onunloadEvents;
                break;
            default:
                throw new IllegalArgumentException("Invalid event type.");
        }
        eventActions.add(script);
    }
    
    /**
     * Adds a referenced JavaScript include.
     *
     * @param service The service that renders the JavaScript include.
     */
    public void addScriptInclude(Service service) {
        if (scriptIncludes == null) {
            scriptIncludes = new HashSet();
        }
        
        scriptIncludes.add(service);
    }

    /**
     * Adds a component style to the document.  Returns the name of the style
     * in the document.
     *
     * @param componentStyle The component style to add to the document.
     * @return The document-specific name of the style.  The document-specific
     *         name of the style is not necessarily identical to the style's 
     *         name.  The name of the style provided here must be used in all
     *         element &quot;class&quot; attributes that use the style to 
     *         guarantee proper rendering.
     */
    public String addStyle(ComponentStyle componentStyle) {
        if (styles == null) {
            styles = new HashMap();
        }
        
        ComponentStyle existingStyle = (ComponentStyle) styles.get(componentStyle);
        
        if (existingStyle == null) {
            componentStyle.setId(nextId++);
            styles.put(componentStyle, componentStyle);
        } else {
            existingStyle.addElementTypesOfStyle(componentStyle);
            componentStyle = existingStyle;
        }
        
        return componentStyle.getName();
    }
    
    /**
     * Returns the &quot;body&quot; element of the document.  This will either
     * be a &lt;body&gt; element in the case of a content-holding document
     * or a &lt;frameset&gt; element in the case of a frameset document.
     *
     * @return The &quot;body&quot; tag of the document.
     */
    public Element getBodyElement() {
        return body;
    }
    
    /**
     * Returns the &lt;head&gt; element of the document.
     *
     * @return The &lt;head&gt; element of the document.
     */
    public Element getHeadElement() {
        return head;
    }
    
    /**
     * Returns the if the browser's &quot;rendering cursor&quot; is at the 
     * beginning of a new line.  This is used to inform a component that is
     * being rendered whether the last component will automatically move the
     * browser's rendering cursor to the start of a new line, as is the case
     * after a &lt;br&gt; or closing &lt;/table&gt; tag is encountered.
     *
     * @return True if the cursor is on a new line.
     */
    public boolean isCursorOnNewLine() {
        return cursorOnNewLine;
    }

    /**
     * Renders the entire document to the specified PrintWriter.
     *
     * @param pw A PrintWriter to render the document to.
     */
    public void render(PrintWriter pw) {
        Element html = new Element(ElementNames.HTML, true);
        
        html.add(head);

        head.add(HtmlElements.createScriptInclude(conn, SERVICE_INIT_FRAME_SCRIPT));
        head.add(HtmlElements.createScriptInclude(conn, conn.getInstancePeer().getSafeInvokeService()));
        if (scriptIncludes != null) {
            for (Iterator it = scriptIncludes.iterator(); it.hasNext();) {
                head.add(HtmlElements.createScriptInclude(conn, (Service) it.next()));
            }
        }
        
        // add style information
        if (styles != null) {
            ComponentStyle componentStyle;
            StyleElement styleElement = new StyleElement();
            for (Iterator it = styles.keySet().iterator(); it.hasNext();) {
                componentStyle = (ComponentStyle) it.next();
                styleElement.add(componentStyle.toStyle());
            }
            head.add(styleElement);
        }

        ScriptElement scriptElement = new ScriptElement();
        
        // Add root script calls
        if (rootScripts != null) {
            Script script = new Script();
            for (Iterator it = rootScripts.iterator(); it.hasNext();) {
               script.add(0, (String) it.next());
            }
            scriptElement.add(script);
        }

        // Add event-triggered script calls
        Script script = null;

        // Add Onunload Events (if necessary).
        if (onunloadEvents != null) {
            body.addAttribute("onunload", ClientObjects.STD_PREFIX + "onunload()");
            script = new Script();
            script.add(0, "function " + ClientObjects.STD_PREFIX + "onunload() {");
            for (Iterator eventIterator = onunloadEvents.iterator(); eventIterator.hasNext();) {
                script.add(1, (String) eventIterator.next());
            }
            script.add(0, "}");
            scriptElement.add(script);
        }
        
        // Add Onload Events (if necessary).
        if (onloadEvents != null) {
            body.addAttribute("onload", ClientObjects.STD_PREFIX + "onload()");
            script = new Script();
            script.add(0, "function " + ClientObjects.STD_PREFIX + "onload() {");
            for (Iterator eventIterator = onloadEvents.iterator(); eventIterator.hasNext();) {
                script.add(1, (String) eventIterator.next());
            }
            script.add(1, ClientObjects.STD_PREFIX + "initialize();");
            script.add(0, "}");
            scriptElement.add(script);
        } else {
            // If no Onload Events were added, directly add a call to initialize() as the body onload attribute.
            body.addAttribute("onload", ClientObjects.STD_PREFIX + "initialize()");
        }

        // Add Initialization Events (always).
        script = new Script();
        script.add(0, "function " + ClientObjects.STD_PREFIX + "initializeTasks() {");
        // add scripting to ensure browser at end of history (prevent the back
        // button from having an effect).
        if (!conn.getServer().getApplicationProperties().getBoolean(ApplicationProperties.DISABLE_BACK_BUTTON_DISABLE_SUPPORT) &&
                conn.getInstancePeer().getClientProperties().getBoolean(ClientProperties.BACK_BUTTON_DISABLE_SUPPORT)) {
            script.add(1, "history.forward();");
        }
        // Add scroll bar position adjustments if necessary.
        if (xScrollBarPosition != 0 || yScrollBarPosition != 0) {
            script.add(1, "scrollTo(" + xScrollBarPosition +  ", " + yScrollBarPosition + ");");
        }
        // Add "custom" initialization events.
        if (initializationEvents != null) {
            for (Iterator eventIterator = initializationEvents.iterator(); eventIterator.hasNext();) {
                script.add(1, (String) eventIterator.next());
            }
        }
        script.add(0, "}");
        scriptElement.add(script);

        head.add(scriptElement);

        html.add(body);
        
        pw.println(DOCTYPE_HTML401_TRANS);
        pw.println();
        html.render(pw);
        pw.println();
    }
    
    /**
     * Sets whether the browser's rendering cursor is on a new line.
     *
     * @param cursorOnNewLine True if the browser's rendering cursor
     *        is on a new line.
     * @see #isCursorOnNewLine()
     */
    public void setCursorOnNewLine(boolean cursorOnNewLine) {
        this.cursorOnNewLine = cursorOnNewLine;
    }
    
    /**
     * Sets the intial scroll bar positions of the document.
     * 
     * @param x The initial horizontal scroll bar position (0 is the default).
     * @param y The initial vertical scroll bar position (0 is the default).
     */
    public void setScrollBarPositions(int x, int y) {
        this.xScrollBarPosition = x;
        this.yScrollBarPosition = y;
    }
}
