/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.SecureRandom;

/**
 * A unique identifier.  Each subsequent construction of an identifier will 
 * have a value greater than the previous.
 */
public final class Id 
implements Comparable, Serializable {
    
    private static SecureRandom seeder;
    private static String middle;
    
    /**
     * Converts the sepecified byte array into a hexdecimal String.
     */
    private static String hexValue(byte[] bytes) {
        StringBuffer sb = new StringBuffer();
        for (int index = 0; index < bytes.length; ++index) {
            int value = bytes[index];
            if (value < 0) {
                value += 256;
            }
            String s = Integer.toString(value, 16);
            if (s.length() == 1) {
                s = "0" + s;
            }
            sb.append(s);
        }
        return sb.toString();
    }
    
    /**
     * Converts an integer to an eight-byte hexadecimal String.
     */
    private static String hexValue(int i) {
        String s = Integer.toHexString(i);
        if (s.length() != 8) {
            s = "00000000".substring(s.length()) + s;
        }
        return s;
    }
    
    static {
        try {
            middle = hexValue(InetAddress.getLocalHost().getAddress()) 
                   + hexValue(System.identityHashCode("x"));
            seeder = new SecureRandom();
        } catch (UnknownHostException ex) {
            // Should not ever occur.
            throw new EchoServletException(ex.toString());
        }
    }
    
    /** Holds the identifier value */
    private String value;
    
    /**
     * Creates a new identifier.
     */
    public Id() {
        super();
        int time = (int) (System.currentTimeMillis() & 0xffffffff);
        int random = seeder.nextInt();
        String value = hexValue(time) + middle + hexValue(random);
        this.value = value;
    }
    
    /**
     * Creates a new identifier from a string.
     *
     * @param value A string representation of the desired identifier.
     */
    public Id(String value) {
        super();
        this.value = value;
    }
    
    /**
     * @see java.lang.Comparable#compareTo(Object)
     */
    public int compareTo(Object o) {
        int comparisonValue;

        if (o instanceof Id) {
            Id that = (Id) o;
            comparisonValue = this.value.compareTo(that.value);
        } else {
            comparisonValue = this.toString().compareTo(o);
        }
        
        return comparisonValue;
    }
    
    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object o) {
        boolean equal;
        
        if (this == o) {
            equal = true;
        } else if (o instanceof Id) {
            Id that = (Id) o;
            equal = this.value.equals(that.value);
        } else {
            equal = false;
        }
        
        return equal;
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return value.hashCode();
    }
    
    /**
     * Returns the identifier as a string.  The contents of the string 
     * will be a hexadecimal value.
     *
     * @return A string representation of the identifier.
     */
    public String toString() {
        return value;
    }
}
