/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.Map;

/**
 * Manages incoming updates from the client.
 */
final class IncomingUpdateQueue 
implements Serializable {

    /**
     * A flag indicating whether the updates are being delayed due to a
     * client-server synchronization.
     */
    private boolean delayed = false;

    /** 
     * A map containing input data submitted through the controller that is 
     * being held temporarily during a second controller round-trip to 
     * synchronize the client.
     */
    private Map delayedInputUpdateMap = null;

    /** 
     * A map containing scrollbar position data submitted through the 
     * controller that is being held temporarily during a second controller
     * round-trip to synchronize the client.
     */
    private Map delayedScrollPositionUpdateMap = null;

    /**
     * Constructor for IncomingUpdateQueue.
     */
    public IncomingUpdateQueue() {
        super();
    }

    /**
     * Returns the delayed input update map, which holds input data
     * from a controller request that requires a second round-trip
     * to the browser.
     *
     * @return The delayed input update map.
     */
    Map getDelayedInputUpdateMap() {
        if (!delayed) {
            throw new IllegalStateException("Cannot retrieve delayed data when update queue is not delayed.");
        }
        return delayedInputUpdateMap;
    }
    
    /**
     * Returns the delayed scrollbar position update map, which holds 
     * scrollbar position data from a controller request that requires 
     * a second round-trip to the browser.
     *
     * @return The delayed scrollbar position update map.
     */
    Map getDelayedScrollPositionUpdateMap() {
        if (!delayed) {
            throw new IllegalStateException("Cannot retrieve delayed data when update queue is not delayed.");
        }
        return delayedScrollPositionUpdateMap;
    }

    /**
     * Returns true if the UpdateQueue is currently delayed due to a 
     * client-server resynchronization.
     *
     * @return True if the UpdateQueue is delayed.
     */
    boolean isDelayed() {
        return delayed;
    }
    
    /**
     * Sets whether the update queue is being delayed by a client-server
     * synchronization.  If set to false, clears all delayed input data.
     *
     * @param delayed The new delayed state.
     */
    void setDelayed(boolean delayed) {
        this.delayed = delayed;
        if (!delayed) {
            delayedInputUpdateMap = null; 
            delayedScrollPositionUpdateMap = null;
        }
    }

    /**
     * Sets the delayed input update map, which holds input data
     * from a controller request that requires a second round-trip
     * to the browser.
     *
     * @param delayedInputUpdateMap A map containing the input parameters 
     *        received by the controller.
     */
    void setDelayedInputUpdateMap(Map delayedInputUpdateMap) {
        if (!delayed) {
            throw new IllegalStateException("Cannot store delayed data when update queue is not delayed.");
        }
        this.delayedInputUpdateMap = delayedInputUpdateMap; 
    }
    
    /**
     * Sets the delayed scrollbar position update map, which holds 
     * scrollbar position data from a controller request that requires 
     * a second round-trip to the browser.
     *
     * @param delayedScrollPositionUpdateMap A map containing the scrollbar 
     *        position parameters received by the controller.
     */
    void setDelayedScrollPositionUpdateMap(Map delayedScrollPositionUpdateMap) {
        if (!delayed) {
            throw new IllegalStateException("Cannot store delayed data when update queue is not delayed.");
        }
        this.delayedScrollPositionUpdateMap = delayedScrollPositionUpdateMap; 
    }
}
