/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.util.Iterator;
import java.util.Map;

/**
 * An interface provided to an EchoInstance such that it may access the 
 * parameters that were initially passed to the application container.  
 * An instance of an implementation of this interface is set as 
 * an attribute with <code>ATTRIBUTE_NAME</code>
 */
public interface InitialParameters {
    
    public static final String ATTRIBUTE_NAME = "nextapp.echoservlet.InitialParameters";
    
    /**
     * Returns the value of an initial request parameter as a String, or null 
     * if the parameter does not exist. 
     * <p>
     * You should only use this method when you are sure the parameter has only 
     * one value.  If the parameter might have more than one value, use 
     * getParameterValues(java.lang.String).
     * <P>
     * If you use this method with a multivalued parameter, the value returned 
     * is equal to the first value in the array returned by getParameterValues. 
     *  
     * @param name - a String specifying the name of the parameter
     * @return a String representing the single value of the parameter
     */
    public String getParameter(String name);
    
    /**
     * Returns a java.util.Map of the parameters of the initial request. 
     *   
     * @return an immutable java.util.Map containing parameter names as keys 
     * and parameter values as map values. The keys in the parameter map are
     * of type String.  The values in the parameter map are of type String[].
     */
    public Map getParameterMap();
    
    /**
     * Returns an Iterator of String objects containing the names of 
     * the parameters contained in the initial request. If the initial 
     * request has no parameters, the method returns an empty Enumeration.
     * 
     * @return an Iterator of String objects, each String containing 
     * the name of an initial request parameter; or an empty Enumeration if the 
     * request has no parameters
     */
    public Iterator getParameterNames();
    
    /**
     * Returns an array of all the initial request parameters names.  
     * If the initial request has no parameters, the method returns an zero 
     * length String array.
     *  
     * @return an array of String objects, each String containing 
     * the name of a request parameter; or a zero length array if the 
     * request has no parameters
     */
    public String[] getAllParameterNames();

    /**
     * Returns an array of String objects containing all of the values 
     * the given initial request parameter has, or null if the 
     * parameter does not exist.
     * <p>
     * If the parameter has a single value, the array has a length of 1.
     * @param name - a String containing the name of the parameter 
     * whose value is requested
     * @return an array of String objects containing the parameter's values
     */
    public String[] getParameterValues(String name);
}
