/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpSession;

import nextapp.echo.EchoInstance;

import nextapp.echoservlet.html.Html;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * The service is responsible for initializing applications.  This is 
 * the first service that will be invoked when an application starts up.  It 
 * provides a client detection page that will use JavaScript to gather 
 * information about public properties of the client browser, storing them in
 * a form.  The form is submitted immediately thereafter, invoking this
 * service a second time.  The form data will be parsed by a new 
 * <code>ClientProperties</code> object that is created
 * from this service.  An InstancePeer and related objects are then created 
 * for the application, stored in the user's session, and initialized.
 * The service() method of the application's default window is then invoked,
 * allowing the application to take control.
 */
final class Initializer 
implements Serializable, Service {

    /** 
     * A prefix for the session key used to temporarily store initial parameters.
     */
    public static final String PARAMETERS_KEY_PREFIX = "InitialParameters.";

    /** The Id of this service */
    private static final Id ID = new Id(ClientObjects.STD_PREFIX + "start");

    /**
     * The HTTP form parameter name used by the client detection script 
     * to indicate client detection information has been retrieved.
     */ 
    public static final String PARAMETER_DETECT_COMPLETE = ClientObjects.STD_PREFIX + "detectComplete";

    /**
     * The client detection service.  It is not necessary that this service 
     * be registered with the EchoServer.
     */
    private static final Service SERVICE_CLIENT_DETECT 
            = Template.createFromResource(ClientObjects.STD_PREFIX + "clidet", ResourceNames.CLIENT_DETECT);

    /**
     * The client detection service.  It is not necessary that this service 
     * be registered with the EchoServer.
     */
    private static final Service SERVICE_STARTUP_CONTAINER 
            = Template.createFromResource(ClientObjects.STD_PREFIX + "startcon", ResourceNames.STARTUP_CONTAINER);
    
    static {
        EchoServer.addGlobalService(SERVICE_CLIENT_DETECT);
    }
    
    /**
     * Creates one-time-use variable data used to start the application.
     *
     * @param conn The connection for which the variable data will be used.
     * @return The generated <code>VariableData</code> object.
     */
    public static VariableData createInitialVariableData(Connection conn) {
        VariableData initialVariableData = new VariableData();
        EchoServer server = conn.getServer();
        
        initialVariableData.set(VariableData.APPLICATION_URI, conn.getApplicationUri());
        initialVariableData.set(VariableData.STARTUP_WINDOW_TITLE,
                Html.encode(server.getApplicationProperties().get(ApplicationProperties.STARTUP_WINDOW_TITLE)));
        initialVariableData.set(VariableData.STARTUP_WINDOW_TEXT,
                Html.encode(server.getApplicationProperties().get(ApplicationProperties.STARTUP_WINDOW_TEXT)));
        initialVariableData.set(VariableData.STARTUP_NO_SCRIPT_ERROR_TEXT,
                Html.encode(server.getApplicationProperties().get(ApplicationProperties.STARTUP_NO_SCRIPT_ERROR_TEXT)));
        initialVariableData.set(VariableData.STARTUP_DELAY,
                server.getApplicationProperties().get(ApplicationProperties.STARTUP_DELAY));
        initialVariableData.set(VariableData.SPLASH_SCREEN_URI,
                conn.correctUri(server.getApplicationProperties().get(ApplicationProperties.SPLASH_SCREEN_URI)));
        initialVariableData.set(VariableData.SESSION_EXPIRATION_URI, server.getSessionExpirationUri(conn.getApplicationUri()));
        if (conn.getServer().getApplicationProperties().get(ApplicationProperties.SPLASH_SCREEN_URI) == null) {
            initialVariableData.set(VariableData.CLIENT_DETECT_TARGET_FRAME, "_self");
        } else {
            initialVariableData.set(VariableData.CLIENT_DETECT_URI, ClientObjects.getServiceUri(conn, SERVICE_CLIENT_DETECT));
            initialVariableData.set(VariableData.CLIENT_DETECT_TARGET_FRAME, "_parent");
        }
        
        return initialVariableData;
    }
        
    /**
     * Creates an Initializer service.  Only initializer service is used by the
     * entire application.
     */
    Initializer() {
        super();
    }
    
    /**
     * @see nextapp.echoservlet.Service#getId()
     */
    public Id getId() {
        return ID;
    }
    
    /**
     * Saves the initial servlet parameters so that they
     * can later be available to the application
     * instance.
     *  
     * @param conn The Connection with the parameters
     * @param parameterMapSessionKey The session key in which to store the parameter map.
     */
    private void saveInitialParameters(Connection conn, String parameterMapSessionKey) {
        Map parameterMap = new HashMap();
        for (Enumeration e = conn.getParameterNames(); e.hasMoreElements();) {
            String paramName = (String) e.nextElement();
            parameterMap.put(paramName,conn.getRequest().getParameterValues(paramName));
        }
        if (parameterMap.size() > 0) {
            // We only save the parameter map into the session if there are any extra parameters
            // on startup.  Otherwise the session will not be created here.
            conn.getRequest().getSession(true).setAttribute(parameterMapSessionKey,parameterMap);   
        }
    }

    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn)
    throws IOException {

        // Our initial parameter session key
        String parameterMapSessionKey = PARAMETERS_KEY_PREFIX + conn.getApplicationUri();
        
        if (conn.getParameter(PARAMETER_DETECT_COMPLETE) == null) {
            // First request (client detection has not taken place): Do client detection.
            if (conn.getServer().getApplicationProperties().get(ApplicationProperties.SPLASH_SCREEN_URI) == null) {
                // save any intial URL parameters for later
                saveInitialParameters(conn,parameterMapSessionKey);

                // Display visible client detection page with "application starting" message.
                SERVICE_CLIENT_DETECT.service(conn);
            } else {
                // Display frameset containing hidden client detection page and visible splash screen.
                SERVICE_STARTUP_CONTAINER.service(conn);
            }
        } else {
            // Second request (client detection has taken place): Start application, display initial window content.
            
            // Create client properties object from connection.
            ClientProperties clientProperties = new ClientProperties(conn);
            
            // Create a new instance of the application.  (The instance is not yet initialized, as its init() method has not yet 
            // been called.)
            EchoInstance instance = conn.getServer().newInstance();
            
            // Mark the EchoInstance as active for this thread, such that invoking
            // EchoInstance.getEchoInstance() will return it.
            instance.markActive(true);
            
            if (instance.getLocale() == null) {
                // Only set locale if application has not set it.
                instance.setLocale(conn.getServer().getDefaultLocale(conn.getRequest()));
            }
            
            // Create an InstancePeer for the new instance.  
            InstancePeer instancePeer = new InstancePeer(instance, conn.getApplicationUri());
            
            // Add the connection to the InstancePeer.
            instancePeer.addConnection(conn);
            
            // Store the client properties.
            instancePeer.setClientProperties(clientProperties);
            
            // Store the principal.
            instancePeer.setUserPrincipal(conn.getRequest().getUserPrincipal());
            
            // Assign the connection to the InstancePeer.
            conn.setInstancePeer(instancePeer);
            
            // Store initial parameters if required.
            HttpSession session = conn.getRequest().getSession(true);
            Map parameterMap = (Map) session.getAttribute(parameterMapSessionKey);
            session.removeAttribute(parameterMapSessionKey);
            if (parameterMap != null) {
                instance.setAttribute(InitialParameters.ATTRIBUTE_NAME, new InitialParametersImpl(parameterMap));
            }
            
            // Retrieve any persistent cookies.
            instancePeer.getCookieManager().retrieveCookies(conn.getRequest());
            
            // Initialize the InstancePeer.  The InstancePeer initialization code will initialize the application
            // instance by calling EchoInstance.init(),
            instancePeer.init();
            
            // Set any cookies stored in the initialization request.
            instancePeer.getCookieManager().storeCookies(conn.getResponse());
            
            // Render the main window.
            ((Service) instancePeer.getPeer(instancePeer.getDefaultWindowId())).service(conn);

            // Remove the connection from the InstancePeer.
            instancePeer.removeConnection(conn);
            
            // Mark the EchoInstance as inactive for this thread, such that invoking
            // EchoInstance.getEchoInstance() will return null
            instance.markActive(false);
        }
    }
}
