/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.security.Principal;
import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.servlet.ServletConfig;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;

import nextapp.echo.Component;
import nextapp.echo.EchoConstants;
import nextapp.echo.EchoInstance;
import nextapp.echo.Window;
import nextapp.echoservlet.image.ImageRepository;

/**
 * A peer object for an user instance of an Echo application.  Like 
 * EchoInstance, one instance of this class is created for each user.
 */
public final class InstancePeer
implements HttpSessionBindingListener, PropertyChangeListener, Serializable {
    
    /**
     * A reference to the HTTP Session object in which this instance peer is being held.
     * 
     * Marked as transient to avoid serializing the session when individual applications
     * are being serialized.
     */
    private transient HttpSession session = null;

    /**
     * Generated three-digit hexadecimal prefix for all cache Ids.
     * This is included to eliminate programmer headaches where the server
     * has been restarted and is reassigning ids that may be cached by their
     * test browser.  This is also required to prevent similar problems when
     * a user's session has expired and a new one has been created, but 
     * resources remain cached on the server.
     * This String will be of the form "f15_".
     */
    private final String cacheIdPrefix = Integer.toString((int) (System.currentTimeMillis() / 1000 % 0x1000), 16) + "_";

    /**
     * A mapping between components and component peers.
     */
    private final Map componentMap = new HashMap(50);
    
    /**
     * A mapping between ids and component peers.
     */
    private final Map idMap = new HashMap(50);
    
    /**
     * A container of all visible windows.
     */
    private final Set windows = new HashSet();
    
    /** 
     * A container of all visible action-producing component peers. 
     */
    private final Set clientActionProducers = new HashSet();
    
    /**
     * A container of all visible input-producing component peers. 
     */
    private final Set clientInputProducers = new HashSet();
    
    /** 
     * A container of all visible content panes. 
     */
    private final Set contentPanes = new HashSet();
    
    /**
     * A container of all variables used for templated services in this
     * application instance.
     */
    private final VariableData variableData = new VariableData();
    
    /**
     * A per-user cacheable wrapper for the client error service.  This enables
     * the client error service to be cached unless locale is changed.
     */
    private final CacheableServiceWrapper clientErrorService;

    /**
     * A container of all services available to this application instance.
     */
    private final ServiceRegistry services;
    
    /**
     * Changes incoming from the client.
     */
    private final IncomingUpdateQueue incomingUpdateQueue;

    /**
     * Content that needs to be re-rendered. 
     */
    private final OutgoingUpdateQueue outgoingUpdateQueue;
    
    /** 
     * A service that is used to coordinate re-rendering of content and other
     * updates of the client's browser.
     **/
    private final Controller controller;

    /**
     * The application instance.
     */
    private final EchoInstance echoInstance;
    
    /**
     * The URI required to make HTTP requests to this application.
     */
    private final String applicationUri;
    
    /**
     * Information about the client browser.
     */
    private ClientProperties clientProperties = null;
    
    /**
     * The Id of the window that will be rendered if no Id is provided.
     */
    private Id defaultWindowId = null;
    
    /**
     * The URI to redirect the browser to in the event of an unrecoverable
     * error.
     */
    private String errorUri = "about:blank";
    
    /**
     * A flag indicating whether initialization has taken place 
     * (whether the init() method has been invoked).
     */
    private boolean initialized = false;
    
    /**
     * The character encoding that will be used for all text-based 
     * renderings in the application.
     */
    private String characterEncoding = EchoServer.CHARACTER_ENCODING_UTF_8;
    
    /**
     * True when the application is exiting.
     */
    private boolean exiting = false;

    /**
     * Null if the application does not desire to exit or a URI
     * to direct the browser to if the application does desire to exit.
     */
    private String terminationUri = null;
    
    /**
     * The CookieManager object responsible for handling cookies for this 
     * application instance.
     */
    private CookieManager cookieManager = new CookieManager();
    
    /**
     * The server context that will be provided to this application's
     * <code>EchoInstance</code>.
     */
    private ServerContext serverContext;
    
    /**
     * The principal using the application.
     */
    private Principal userPrincipal;
    
    /**
     * The next id value assigned to a component.
     */
    private long nextComponentId = 0;
    
    /**
     * The request sequence number.  Ensures that operations are not performed
     * based on input received from previously processed controller frames that
     * may get submitted as a result of the user hitting the reload button on
     * the browser.
     */
    private int sequenceNumber = 0;
    
    /**
     * A set of active <code>Connection</code> objects using this 
     * <code>InstancePeer</code>.  An active <code>Connection</code> may be 
     * needed for use by the <code>InstancePeer</code> by certain operations, 
     * such as <code>isUserInRole()</code> which requires the use of a 
     * <code>Connection</code>'s <code>HttpServletRequest</code>.
     */
    private transient Set connections = null;
    
    /**
     * A repository of the <code>ImagePeer</code>s used by this 
     * <code>InstancePeer</code>.
     */
    private ImageRepository imageRepository = new ImageRepository(this);
    
    /**
     * Refrence to relevant <code>ApplicationProperties</code> for this 
     * application container instance.
     */
    private ApplicationProperties applicationProperties;

    /**
     * Creates a new instance peer for the provided instance.
     * Stores the application's URI for future use.
     *
     * @param echoInstance The <code>EchoInstance</code> object for which
     *        this peer is being created.
     * @param applicationUri The URI of the application.
     */
    InstancePeer(EchoInstance echoInstance, String applicationUri) {
        super();
        
        this.echoInstance = echoInstance;
        this.applicationUri = applicationUri;

        // Create and store ServerContext.
        serverContext = new ServerContextImpl(this);
        echoInstance.setAttribute(ServerContext.ATTRIBUTE_NAME, serverContext);
        
        // Create ServiceRegistry.
        services = new ServiceRegistry();
       
        // Create update queues.
        incomingUpdateQueue = new IncomingUpdateQueue();
        outgoingUpdateQueue = new OutgoingUpdateQueue(this);
        
        // Create and register Controller.
        controller = new Controller(this);
        services.add(controller);
        
        // Create and register client error service.
        clientErrorService = new CacheableServiceWrapper(generateId(), EchoServer.SERVICE_CLIENT_ERROR_SCRIPT);
        services.add(clientErrorService);
        
        // Store application URI in variable data.
        variableData.set(VariableData.APPLICATION_URI, applicationUri);
        
        echoInstance.addPropertyChangeListener(this);
    }
    
    /**
     * Registers the given component, and all visible components with visible 
     * parents beneath it.  Adds a property change listener to any non-visible
     * components that are the immediate children of any visible components 
     * that are being registered, so that if they become visible, this object
     * will be notified.
     *
     * @param c The component to be added to the hierarchy.
     * @param visibilityChange A flag indicating whether the component is being
     *        added because it changed visibility or not.  If this flag is 
     *        set, a property change listener won't be added to the component,
     *        because it already has one.
     */
    private void add(Component c, boolean visibilityChange) {
        if (c.isVisible()) {
            // The Component is visible.
        
            if (! visibilityChange) {
                // If not a visibility change, add a listener.  (if the component is 
                // only changing visibility, it already has the listener)
                c.addPropertyChangeListener(this);
            }

            // Register the component before its children are registered.
            register(c);

            Component[] children = c.getComponents();
            for (int index = 0; index < children.length; ++index) {
                add(children[index], false);
            }
        } else {
            // The component is currently not visible:
            // Add only a property change listener such that the peer will be
            // notified when the component does become visible.
            c.addPropertyChangeListener(this);
        }

        if (c instanceof Window && c.isVisible()) {
            // If the component is a visible window, add it to the updateQueue
            // manually to be opened.
            outgoingUpdateQueue.enqueueWindowOpen((WindowUI) getPeer(c));
        }
    }
    
    /**
     * Adds  an active connection to the internal list of connections.
     *
     * @param conn The <code>Connection to be added.
     */
    void addConnection(Connection conn) {
        if (connections == null) {
            connections = new HashSet();
        }
        synchronized(connections) {
            connections.add(conn);
        }
    }
    
    /**
     * Processes a property change event where a registered component has had
     * children added or removed.
     *
     * @param e A property change event indicating that children have been
     *        added to or removed from a registered component.
     */
    private void childrenChange(PropertyChangeEvent e) {
        Component parent = ((Component) e.getSource());
        
        // PropertyChangeListeners remained attached to invisible components which are the immediate children
        // of visible registered components.  These listeners will inform the InstancePeer if it needs to take
        // action due to such an invisible component being made visible.  This method must therefore ensure that
        // the component in question is visible before adding/removing child peers.
        if (!parent.isVisible()) {
            // If parent is not visible, take no action.
            return;
        }

        Component child = (Component) e.getNewValue();
        if (child == null) {
            child = (Component) e.getOldValue();
        }
        
        boolean componentRegistered = child.isRegistered();
        boolean peerRegistered = hasPeer(child);

        if (!componentRegistered && peerRegistered) {
            // child being removed
            remove(child, false);
        } else if (componentRegistered && !peerRegistered) {
            // child being added
            add(child, false);
        }
    }
    
    /**
     * Clears the termination URI.  A termination URI is set if the application
     * is exiting.  The termination URI is the URI that the remaining open 
     * window will be directed to once the application has shut down and all 
     * of its other windows have been closed.  A non-null termination URI 
     * indicates that the application SHOULD IMMEDIATELY EXIT to that URI. 
     * When the exit operation has been completed, the termination URI must be
     * cleared such that if the application is again visited it will 
     * reinitialize instead of again redirecting to the termination URI.
     */
    void clearTerminationUri() {
        terminationUri = null;
    }
    
    /**
     * Exits the application, redirecting an available browser window to the 
     * specified URI.
     *
     * @param uri The URI to redirect the browser window to.
     */
    void exit(String uri) {
        terminationUri = uri;
        exiting = true;
    }
    
    /**
     * Processes a property change event where the focused component has been
     * changed.
     *
     * @param e A property change event indicating that the focus has changed.
     */
    private void focusChange(PropertyChangeEvent e) {
        Component component = ((Component) e.getNewValue());
        if (component == null || !hasPeer(component)) {
            return;
        }
        redraw(getPeer(component));
    }
    
    /**
     * Generates a unique (to the application) identifier.
     * 
     * @return An application-unique identifier.
     */
    public Id generateId() {
        return new Id("a_" + nextComponentId++);
    }
    
    /**
     * Returns the URI from which this application is available.
     *
     * @return The URI from which this application is available.
     */
    String getApplicationUri() {
        return applicationUri;
    }
    
    /**
     * Returns the character encoding that will be used when communicating 
     * with the client browser.
     *
     * @return The character encoding that will be used when communicating 
     * with the client browser.
     */
    String getCharacterEncoding() {
        return characterEncoding;
    }
    
    /** 
     * Returns a prefix to prepend to cache ids in order to ensure that 
     * incorrect cached resources are used by client in the event either
     * the server or the users session has restarted.
     * 
     * @return A prefix to prepend to cache ids to make them unique across
     *         server and session restarts.
     */
    String getCacheIdPrefix() {
        int hour = (int) ((System.currentTimeMillis() / 3600000) % 24);
        return Integer.toString(hour + 10, 30) + cacheIdPrefix;
    }
    
    /**
     * Returns the client error service for this user.
     * 
     * @return The client error service for this user.
     */
    Service getClientErrorService() {
        return clientErrorService;
    }
    
    /**
     * Returns an iterator over all registered 
     * <code>ClientInputProducers</code>s.
     *
     * @return An iterator over all registered 
     *         <code>ClientInputProducers</code>s.
     */
    Iterator getClientInputProducers() {
        return clientInputProducers.iterator();
    }
    
    /**
     * Returns the &quot;client properties&quot; data collected about the
     * client of this <code>InstancePeer</code>.  This data contains 
     * information such as the browser name, version, and the clients
     * screen resolution.
     *
     * @return The &quot;client properties&quot; data collected about the
     *         client of this <code>InstancePeer</code>.
     */
    public ClientProperties getClientProperties() {
        return clientProperties;
    }
    
    /**
     * Returns the <code>PaneUI</code> in which the given
     * <code>ComponentPeer</code> is contained.
     *
     * @param componentPeer The <code>ComponentPeer</code> whose containing
     *        <code>PaneUI</code> is to be returned.
     * @return The <code>PaneUI</code> containing the given
     *         <code>ComponentPeer</code>.
     */ 
    public PaneUI getContainingPane(ComponentPeer componentPeer) {
        while (componentPeer != null && !(componentPeer instanceof PaneUI)) {
            componentPeer = componentPeer.getParent();
        }
        
        return (PaneUI) componentPeer;
    }
    
    /**
     * Returns the <code>WindowUI</code> in which the given
     * <code>ComponentPeer</code> is contained.
     *
     * @param componentPeer The <code>ComponentPeer</code> whose containing
     *        <code>WindowUI</code> is to be returned.
     * @return The <code>WindowUI</code> containing the given
     *         <code>ComponentPeer</code>.
     */ 
    public WindowUI getContainingWindow(ComponentPeer componentPeer) {

        if (! (componentPeer instanceof WindowUI)) {
            do {
                componentPeer = componentPeer.getParent();
            } while (componentPeer != null && (! (componentPeer instanceof WindowUI)));
        }
    
        return (WindowUI) componentPeer;
    }
    
    /**
     * Returns an iterator over all registered content panes.
     *
     * @return An iterator over all registered content panes.
     */
    Iterator getContentPanes() {
        return contentPanes.iterator();
    }
    
    /**
     * Returns this instance's <code>Controller<code> service.
     *
     * @return This instance's <code>Controller<code> service.
     */
    public Service getController() {
        return controller;
    }
    
    /**
     * Returns the <code>CookieManager</code> used by this instance.
     *
     * @return The <code>CookieManager</code> used by this instance.
     */
    public CookieManager getCookieManager() {
        return cookieManager;
    }
    
    /**
     * Returns the Id of the application's default window.  This is the 
     * window that will occupy the original browser window the user typed the
     * application's URI into.  If the original default window has been
     * closed, an arbitrary window id will be returned.
     *
     * @return The application's default window.
     */
    Id getDefaultWindowId() {
        if (defaultWindowId == null || getPeer(defaultWindowId) == null) {
            // Find new default window Id.
            if (windows.size() > 0) {
                defaultWindowId = ((WindowUI) windows.iterator().next()).getId();
            } else {
                defaultWindowId = null;
            }
        }
        
        return defaultWindowId;
    }
    
    /**
     * Returns the URI the client is directed to in the event of an 
     * unrecoverable error.
     *
     * @return The URI the client is directed to in the event of an
     *         unrecoverable error.
     */
    String getErrorUri() {
        return errorUri;
    }
    
    /**
     * Returns the image repository.
     * 
     * @return The image repository.
     */
    public ImageRepository getImageRepository() {
        return imageRepository;
    }
    
    /**
     * Returns the application's incoming update queue.
     *
     * @return The application's incoming update queue.
     */
    IncomingUpdateQueue getIncomingUpdateQueue() {
        return incomingUpdateQueue;
    }
    
    /**
     * Returns the EchoInstance object this peer supports.
     *
     * @return The EchoInstance object this peer supports.
     */
    public EchoInstance getInstance() {
        return echoInstance;
    }
    
    /**
     * Returns the application's outgoing update queue.
     *
     * @return The application's outgoing update queue.
     */
    OutgoingUpdateQueue getOutgoingUpdateQueue() {
        return outgoingUpdateQueue;
    }
    
    /**
     * Returns the ComponentPeer that corresponds to the given Component.
     * A component will have a peer only if it has been registered.  A component
     * will be registered if all of its ancestors are visible, and the window
     * that contains it is registered with this InstancePeer.  If a
     * peer cannot be found for the component, an EchoServletException
     * (A RuntimeException) will be thrown.
     *
     * @param c The component whose peer is to be retrieved.
     * @return The peer for the specified component.
     */
    public ComponentPeer getPeer(Component c) {
        ComponentPeer peer = (ComponentPeer) componentMap.get(c);
    
        if (peer == null) {
            throw new EchoServletException("Peer requested for unregistered component: " + c);
        }
    
        return peer;
    }

    /**
     * Returns the ComponentPeer that corresponds to the given Id.
     * If a peer cannot be found which corresponds to the specified Id, a
     * null value is returned.
     *
     * @param id The id of the peer to be retrieved.
     * @return The peer identified by id.
     */
    public ComponentPeer getPeer(Id id) {
        return (ComponentPeer) idMap.get(id);
    }
    
    /**
     * Returns the service that renders the script containing the safeInvoke()
     * method.  The safeInvoke() method is used to execute script code which 
     * may generate errors.  Different safeInvoke() implementations are used 
     * on different browsers based on the browser's support of JavaScript 1.4's
     * try/catch/finally blocks.
     *
     * @return The service that renders the script containing the safeInvoke()
     *         method.
     */
    public Service getSafeInvokeService() {
        if (ClientProperties.TRUE.equals(clientProperties.get(ClientProperties.EXCEPTION_SUPPORT))) {
            return EchoServer.SERVICE_SAFE_INVOKE_EXCEPTION_SUPPORT;
        } else {
            return EchoServer.SERVICE_SAFE_INVOKE_DEFAULT;
        }
    }
    
    /**
     * Returns the sequence number of the last successfully processed request
     * from the client controller frame.  The sequence number prevents the same
     * request from being made multiple times, i.e., as the result of the user
     * clicking the reload button in the browser.
     *
     * @return The sequence number of the last successfully processed request
     * from the client controller frame.
     */
    int getSequenceNumber() {
        return sequenceNumber;
    }
    
    /**
     * Returns the HttpSession object to which this instance peer is bound,
     * or null if is not bound to a session.
     *
     * @return the HttpSession object to which this instance peer is bound.
     */
    HttpSession getSession() {
        return session;
    }
    
    /**
     * Returns the <code>ServiceRegistry</code> that is used by this instance
     * to hold all <code>Service</code> objects available to it.
     *
     * @return This instance's <code>ServiceRegistry</code>.
     */
    ServiceRegistry getServiceRegistry() {
        return services;
    }
    
    /**
     * Returns the <code>ServletConfig</code> object containing that the 
     * servlet container sent to the EchoServer's init() method (EchoServer
     * extends HttpServlet).
     * 
     * return The <code>ServletConfig</code> object for this Echo servlet.
     */
    ServletConfig getServletConfig() {
        synchronized(connections) {
            Iterator connIt = connections.iterator();
            if (!connIt.hasNext()) {
                throw new EchoServletException("No available Connection.");
            }
            Connection conn = (Connection) connIt.next();
            return conn.getServer().getServletConfig();
        }
    }
    
    /**
     * Returns null if the application does not desire to exit or a URI
     * to direct the browser to if the application does desire to exit.  This
     * method is called by the Controller every time it receives a service
     * request.  If a URI is provided, the Controller will terminate the 
     * <b>client</b> session and redirect the browser.
     *
     * @return The termination URI.
     */
    String getTerminationUri() {
        return terminationUri;
    }
    
    /**
     * Returns the principal (user) of this application.
     * This method provides the identical capability of 
     * <code>HttpServletRequest.getUserPrincipal()</code>.
     * 
     * @return The principal (user) of this application.
     */
    public Principal getUserPrincipal() {
        return userPrincipal;
    }
    
    /**
     * Returns the <code>VariableData</code> object used by this instance
     * peer when rendering templated services.
     *
     * @return The <code>VariableData</code> object used by this instance peer.
     */ 
    VariableData getVariableData() {
        return variableData;
    }
    
    /**
     * Returns an iterator of all registered windows.
     *
     * @return An iterator of all registered windows.
     */
    Iterator getWindows() {
        return windows.iterator();
    }
    
    /**
     * Determines whether the specified component has a peer.
     * 
     * @param c The component to analyze.
     * @return True if the component has a peer.
     */
    private boolean hasPeer(Component c) {
        return componentMap.containsKey(c);
    }
    
    /**
     * Increments the value of the controller frame sequence number by one.
     * This method is called by the controller after a successful request.
     *
     * @see #getSequenceNumber()
     */
    void incrementSequenceNumber() {
        sequenceNumber++;
    }
    
    /**
     * Initializes this <code>InstancePeer</code>, causing the represented
     * <code>EchoInstance</code> to also initialize.  The default window
     * is obtained from the <code>EchoInstance.init()</code> method and 
     * added to the application.
     */
    void init() {
        // Obtain application properties.
        synchronized(connections) {
            Iterator connIt = connections.iterator();
            if (!connIt.hasNext()) {
                // Should not occur.
                throw new EchoServletException("No available Connection.");
            }
            Connection conn = (Connection) connIt.next();
            applicationProperties =  conn.getServer().getApplicationProperties();
        }
        
        Window initialWindow = echoInstance.doInit();
        updateLocalizedResources();
        add(initialWindow, false);
        WindowUI initialWindowUI = (WindowUI) getPeer(initialWindow);
        defaultWindowId = initialWindowUI.getId();
        
        if (clientProperties != null) {
            // Set initial window data based on ClientProperties.
            try {
                if (initialWindow.getWidth() == EchoConstants.UNDEFINED_SIZE) {
                    initialWindow.setWidth(Integer.parseInt(clientProperties.get(ClientProperties.INITIAL_WINDOW_WIDTH)));
                }
                if (initialWindow.getHeight() == EchoConstants.UNDEFINED_SIZE) {
                    initialWindow.setHeight(Integer.parseInt(clientProperties.get(ClientProperties.INITIAL_WINDOW_HEIGHT)));
                }
                int flags = 0;
                flags |= ClientProperties.TRUE.equals(clientProperties.get(ClientProperties.INITIAL_WINDOW_HAS_MENU_BAR)) 
                        ? WindowUI.MENU_BAR : 0;
                flags |= ClientProperties.TRUE.equals(clientProperties.get(ClientProperties.INITIAL_WINDOW_HAS_STATUS_BAR)) 
                        ? WindowUI.STATUS_BAR : 0;
                flags |= ClientProperties.TRUE.equals(clientProperties.get(ClientProperties.INITIAL_WINDOW_HAS_TOOL_BAR)) 
                        ? WindowUI.TOOL_BAR : 0;
                initialWindowUI.setFlags(flags);
            } catch (NumberFormatException ex) { }
        }

        // Prevent default window from being opened by updateQueue.
        outgoingUpdateQueue.enqueueWindowClose(initialWindowUI);
        
        initialized = true;
    }
    
    /**
     * Returns true if the application is exiting and should have therefore 
     * have its session invalidated.  This method will be called by the 
     * controller which will remove the InstancePeer from the session.
     *
     * @return True if the application is exiting.
     */
    boolean isExiting() {
        return exiting;
    }
    
    /**
     * Returns true if the user of the application is in the specified role.
     * This method provides the identical capability of 
     * <code>HttpServletRequest.isUserInRole()</code>.
     *
     * @param role The role for which to test membership.
     * @return True if the user is in the specified role.
     */
    boolean isUserInRole(String role) {
        synchronized(connections) {
            Iterator connIt = connections.iterator();
            if (!connIt.hasNext()) {
                throw new EchoServletException("No available Connection.");
            }
            Connection conn = (Connection) connIt.next();
            return conn.getRequest().isUserInRole(role);
        }
    }

    /**
     * Lowers the specified window to the background.
     * 
     * @param windowUI The window to lower.
     */
    void lowerWindow(WindowUI windowUI) {
        outgoingUpdateQueue.enqueueWindowLower(windowUI);
    }

    /**
     * Handler for PropertyChangeEvents.  This class registers with all 
     * registered components and their immediately non-visible siblings 
     * to receive property changes.  The InstancePeer needs to monitor
     * all changes in its represented <code>EchoInstance</code> and its
     * components such that it can update the client browser to reflect
     * the application's state.
     *
     * @param e A property change event describing a property change to
     *          the application represented by this peer or one of its
     *          components.
     */
    public void propertyChange(PropertyChangeEvent e) {
        if (Component.CHILDREN_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // child being added or removed
            childrenChange(e);
        } else if (EchoInstance.FOCUSED_COMPONENT_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // focus changed
            focusChange(e);
        } else if (Component.VISIBLE_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // child being set visible or nonvisible
            visibilityChange(e);
        } else if (EchoInstance.WINDOWS_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            // Handle windows being added/removed.
            if (e.getNewValue() == null) {
                // Window being removed.
                remove((Window) e.getOldValue(), false);
            } else {
                // Window being added.
                add((Window) e.getNewValue(), false);
            }
        } else if (e.getSource() == echoInstance && EchoInstance.LOCALE_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            updateLocalizedResources();
        }
    }

    /**
     * Raises the specified window to the foreground.
     * 
     * @param windowUI The window to raise.
     */
    void raiseWindow(WindowUI windowUI) {
        outgoingUpdateQueue.enqueueWindowRaise(windowUI);
    }
    
    /**
     * Causes the service containing the given component to be re-rendered.
     *
     * @param componentPeer A <code>ComponentPeer</code> whose represented
     *        <code>Component</code> has changed and therefore needs to be
     *        updated on the client browser.
     */
    public void redraw(ComponentPeer componentPeer) {
        outgoingUpdateQueue.enqueueComponentUpdate(componentPeer);
    }
    
    /**
     * Registers a component.  Components are registered using the following 
     * procedure:  
     * <ol>
     * <li>A peer is created for the component.</li>
     * <li>The peer is sored in the InstancePeer's Id-to-ComponentPeer and
     *     Component-to-ComponentPeer mappings.</li>
     * <li>If the peer is a service, it is added to the service registry.</li>
     * <li>If the peer has any ancillary services, they are added to the
     *     service registry.</li>
     * <li>If the peer produces client actions (by implementing
     *     <code>ClientActionProducer</code>, then it is added to this 
     *     instance's inventory of action producing components.</li>
     * <li>If the peer produces client input (by implementing
     *     <code>ClientInputProducer</code>, then it is added to this
     *     instance's inventory of input producing components.</li>
     * <li>The component is notified of its registration by having
     *     its <code>registered()</code>method invoked.</li>
     * </ol>
     *
     * @param c The component to be registered.
     */
    private synchronized void register(Component c) {
        if (! c.isVisible()) {
            throw new IllegalStateException("Illegal attempt to register non-visible component.");
        }
    
        ComponentPeer componentPeer = ComponentPeerFactory.createComponentPeer(c);
        componentPeer.setId(generateId());
        componentPeer.setInstancePeer(this);
        
        // Add the component peer to the id and component based maps.
        idMap.put(componentPeer.getId(), componentPeer);
        componentMap.put(componentPeer.getComponent(), componentPeer);

        // File the component in the appropriate containers according
        // to its type.

        if (componentPeer instanceof Service) {
            services.add((Service) componentPeer);
            
            if (componentPeer instanceof ContentPaneUI) {
                contentPanes.add(componentPeer);
            } else if (componentPeer instanceof WindowUI) {
                windows.add(componentPeer);
            }
        }
        
        componentPeer.registerAncillaryServices();

        if (componentPeer instanceof ClientActionProducer) {
            clientActionProducers.add(componentPeer);
        }
        
        if (componentPeer instanceof ClientInputProducer) {
            clientInputProducers.add(componentPeer);
        }

        // notify the component peer that it was registered.
        componentPeer.registered();
    }
    
    /**
     * Recursively removes a component and its descendants from the 
     * InstancePeer by unregistering their peer objects and removing
     * the InstancePeer from their lists of property change listeners.
     * If the reason for removal is that the component has become
     * invisible, the property change listener on the component
     * being removed will remain such that the InstancePeer will be 
     * notified if/when it again becomes visible.
     *
     * @param c The component to be removed from the hierarchy.
     * @param visibilityChange A flag indicating whether the component is being
     *        removed because it changed visibility or not.  If this flag is 
     *        set, a property change listener will be kept on the component such
     *        that it can be re-added if it becomes visible again.
     */
    private void remove(Component c, boolean visibilityChange) {
        if (c instanceof Window && (c.isVisible() || visibilityChange)) {
            outgoingUpdateQueue.enqueueWindowClose((WindowUI) getPeer(c));
        }
    
        if (!visibilityChange && !c.isVisible() && !hasPeer(c)) {
            // Removing an invisible Component:
            // Just remove PropertyChangeListener.
            c.removePropertyChangeListener(this);
        } else {
            // Recursively call remove() on all children of the component.
            Component[] children = c.getComponents();
            for (int index = 0; index < children.length; ++index) {
                remove(children[index], false);
            }

            // If the component is not just changing visibility, but is actually 
            // being removed, remove its property change listener.  If it is only
            // changing visibility, the listener is kept so this object will be 
            // notified if it becomes visible again.
            if (! visibilityChange) {
                c.removePropertyChangeListener(this);
            }

            // Unregister the component after its children have been unregistered.
            unregister(c);
        }
    }
    
    /**
     * Removes an active connection from the internal list of connections.
     *
     * @param conn The <code>Connection to be removed.
     */
    void removeConnection(Connection conn) {
        synchronized(connections) {
            connections.remove(conn);
        }
    }
    
    /**
     * Restarts application by suspending to applciation's own URI.
     */
    void restart() {
        suspend(applicationUri);
    }

    /**
     * Returns the character encoding that will be used when communicating 
     * with the client browser.
     *
     * @param newValue The character encoding that will be used when 
     *        communicating with the client browser.
     */
    void setCharacterEncoding(String newValue) {
        String oldValue = characterEncoding;
        characterEncoding = newValue;
        if (initialized && !oldValue.equals(newValue)) {
            restart();
        }
    }
    
    /**
     * Sets the client properties for the user.
     *
     * @param clientProperties The client properties data to associate with this instance.
     */
    void setClientProperties(ClientProperties clientProperties) {
        this.clientProperties = clientProperties;
    }
    
    /**
     * Sets the URI the client is directed to in the event of an unrecoverable
     * error.
     *
     * @param errorUri The new error URI.
     */
    void setErrorUri(String errorUri) {
        this.errorUri = errorUri;
    }
    
    /**
     * Causes the specified window's title to be updated on the client.
     *
     * @param windowUI The peer of the window whose title should be redrawn.
     */
    public void setTitle(WindowUI windowUI) {
        outgoingUpdateQueue.enqueueWindowTitleUpdate(windowUI);
    }

    /**
     * Sets the principal (user) that is using the application.
     *
     * @param userPrincipal The principal (user) that is using the application.
     */
    void setUserPrincipal(Principal userPrincipal) {
        this.userPrincipal = userPrincipal;
    }
    
    /**
     * Suspends the application, redirecting an available browser window to the
     * specified URI.
     *
     * @param uri The URI to redirect the browser.
     */
    void suspend(String uri) {
        terminationUri = uri;
    }
    
    /**
     * Unregisters a component.
     *
     * @param c The component to be unregistered.
     */
    private synchronized void unregister(Component c) {
        ComponentPeer componentPeer = getPeer(c);
    
        componentPeer.unregisterAncillaryServices();

        idMap.remove(componentPeer.getId());
        componentMap.remove(componentPeer.getComponent());
        
        // Remove the component from all filing containers.
        // It is assumed that it is faster to try and fail to remove the
        // component than it is to first do an instanceof check.

        clientActionProducers.remove(componentPeer);
        clientInputProducers.remove(componentPeer);
        if (componentPeer instanceof Service) {
            services.remove((Service) componentPeer);
            contentPanes.remove(componentPeer);
            windows.remove(componentPeer);
        }
        
        // Notify the component peer that it was unregistered.
        componentPeer.unregistered();
        
        // Disconnect the component peer from the instance.
        componentPeer.setInstancePeer(null);
        
        // Remove any queued updates for the component or its children.
        outgoingUpdateQueue.deleteComponentUpdate(componentPeer);
    }
    
    /**
     * Updates localized resources.
     */
    private void updateLocalizedResources() {
        // Load localized client-side error messages.
        if (applicationProperties.getBoolean(ApplicationProperties.DISPLAY_CLIENT_ERROR_FEEDBACK)) {
            String clientErrorMessageResourceUri = applicationProperties.get(ApplicationProperties.CLIENT_ERROR_MESSAGE_RESOURCE);
            if (clientErrorMessageResourceUri != null) {
                Locale locale = echoInstance.getLocale();
                ResourceBundle resource = ResourceBundle.getBundle(clientErrorMessageResourceUri, locale);
                String notReadyMessage = resource.getString(VariableData.CLIENT_ERROR_NOT_READY);
                if (notReadyMessage != null) {
                    variableData.set(VariableData.CLIENT_ERROR_NOT_READY, notReadyMessage);
                } else {
                    variableData.set(VariableData.CLIENT_ERROR_NOT_READY, EchoServer.DEFAULT_CLIENT_ERROR_NOT_READY_MESSAGE);
                }
            } else {
                variableData.set(VariableData.CLIENT_ERROR_NOT_READY, EchoServer.DEFAULT_CLIENT_ERROR_NOT_READY_MESSAGE);
            }
        } else {
            variableData.set(VariableData.CLIENT_ERROR_NOT_READY, null);
        }
        clientErrorService.update();
    }
    
    /**
     * Listener implementation of <code>HttpSessionBindingListener</code>.
     * Stores reference to session when invoked.
     *
     * @see javax.servlet.http.HttpSessionBindingListener#valueBound(HttpSessionBindingEvent)
     */
    public void valueBound(HttpSessionBindingEvent e) {
        session = (HttpSession) e.getSource();
    }
    
    /**
     * Listener implementation of Mcode>HttpSessionBindingListener</code>.
     * Removes reference to session when invoked.
     *
     * @see javax.servlet.http.HttpSessionBindingListener#valueUnbound(HttpSessionBindingEvent)
     */
    public void valueUnbound(HttpSessionBindingEvent e) {
        echoInstance.markActive(true);
        echoInstance.doExit();
        echoInstance.markActive(false);
        session = null;
    }
    
    /**
     * Handles the case where a registered component or immediate invisible
     * descendent of a registered component has changed visibility.
     *
     * A @param e property change event from a component that whose visibility 
     * has changed.
     */
    private void visibilityChange(PropertyChangeEvent e) {
        Component c = (Component) e.getSource();

        if (c.isRegistered()) {
            // If a component is not registered with an EchoInstance, do nothing.
            
            boolean hasPeer = hasPeer(c);
            if (c.isVisible()) {
                // Register component hierarchy that has become visible
                // and re-render.
                if (!hasPeer) {
                    add(c, true);
                    if (c instanceof Window) {
                        redraw(getPeer(c));
                    } else {
                        redraw(getPeer(c).getParent());
                    }
                }
            } else {
                if (hasPeer) {
                    // Unregister component hierarchy that has become invisible
                    // and re-render.
                    if (c instanceof Window) {
                        redraw(getPeer(c));
                    } else {
                        redraw(getPeer(c).getParent());
                    }
                    remove(c, true);
                }
            }
        }
    }
}
