/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nextapp.echo.Component;

/**
 * Manages outgoing updates to the client.
 * 
 * Managed updates include:
 * <ul>
 * <li>Panes requiring refresh</li>
 * <li>Windows requiring refresh</li>
 * <li>Windows requiring title update</li>
 * <li>Windows requiring z-index adjustment (raising or lowering)</li>
 * <li>Windows to be opened or closed</li>
 * </ul>
 */
final class OutgoingUpdateQueue 
implements Serializable {
        
    /**
     * A set containing WindowUIs that need to be closed on the client.
     */
    private Set closedWindows = new HashSet();
    
    /**
     * A set containing WindowUIs that need to be raised on the client.
     */
    private Set raisedWindows = new HashSet();
    
    /**
     * A set containing WindowUIs that need to be lowered on the client.
     */
    private Set loweredWindows = new HashSet();
    
    /**
     * The InstancePeer with which this OutgoingUpdateQueue is associated.
     */
    private InstancePeer instancePeer;
    
    /** 
     * A set containing WindowUI's that need to be opened on the client.
     */
    private Set openedWindows = new HashSet();
    
    /**
     * A set containing all services that need to be refreshed on the client.
     */
    private Set paneUpdates = new HashSet();
    
    /**
     * A set containing all WindowUI's whose titles need to be updated on the 
     * client.
     */
    private Set windowTitleUpdates = new HashSet();

    /**
     * A set containing all WindowUI's whose content needs to be updated on 
     * the client.
     */
    private Set windowUpdates = new HashSet();

    /**
     * Creates a new OutgoingUpdateQueue for the specified InstancePeer.
     * There should be one OutgoingUpdateQueue per InstancePeer.
     *
     * @param instancePeer The InstancePeer that this OutgoingUpdateQueue will handle 
     *        updates for.
     */
    OutgoingUpdateQueue(InstancePeer instancePeer) {
        super();
        
        this.instancePeer = instancePeer;
    }
    
    /**
     * Removes all previously scheduled service updates for services that
     * are descendants of the specified <code>ComponentPeer</code>.  It is
     * necessary to remove descendant services if the componentPeer has been
     * removed or has become invisible.
     *
     * @param componentPeer The component peer whose descendant services
     *        are to be removed.
     */
    void deleteComponentUpdate(ComponentPeer componentPeer) {
        Component component = componentPeer.getComponent();
        Component testComponent;
        ComponentPeer testComponentPeer;
        
        for (Iterator it = paneUpdates.iterator(); it.hasNext();) {
            testComponentPeer = (ComponentPeer) it.next();
            testComponent = testComponentPeer.getComponent();
            if (component.isAncestorOf(testComponent)) {
                it.remove();
            }
        }
    }

    /**
     * Returns all services (panes and windows) that are present on the client
     * and need to be refreshed such that they are in sync with their 
     * respective server-side representations.
     *
     * @return An iterator of services needing to be refreshed.
     */
    Iterator dequeuePaneUpdate() {
        // Handle forwarding container panes: Any forwarding container pane found will have its first parent component that is not
        // a forwarding container pnae redrawn instead.
        Set forwardedContainerUpdates = null;
        Iterator it = paneUpdates.iterator();
        while (it.hasNext()) {
            PaneUI paneUI = (PaneUI) it.next();
            if (paneUI instanceof ContainerPaneUI && 
                    !(((ContainerPaneUI) paneUI).getForwardingPane() instanceof ContainerPaneUI)) {
                // Pane update is a forwarding container pane, that is forwarding to a non-container pane: 
                // Update its parent instead.  This is done as otherwise the scrollbar policy desired by the pane being forwarded
                // to will not be observed.
                ComponentPeer componentPeer = (ComponentPeer) paneUI;
                
                // Find first parent component that is not a forwarding container pane.
                while (componentPeer instanceof ContainerPaneUI 
                        && ((ContainerPaneUI) componentPeer).getForwardingPane() != componentPeer) {
                    componentPeer = componentPeer.getParent();
                }
    
                // Add peer to update list.
                if (forwardedContainerUpdates == null) {
                    // lazy-create forwarded container update set
                    forwardedContainerUpdates = new HashSet();
                }
                forwardedContainerUpdates.add(componentPeer);
            }
        }
        if (forwardedContainerUpdates != null) {
            it = forwardedContainerUpdates.iterator();
            while (it.hasNext()) {
                enqueueComponentUpdate((ComponentPeer) it.next());
            }
        }
    
        return paneUpdates.iterator();
    }
    
    /**
     * Returns an iterator over all windows that are presently open on the 
     * client and need to be closed.  The controller will use the 
     * Iterator.remove() method to remove each window from this queue after
     * it has generated script to close the window.
     *
     * @return An iterator of windows needing to be closed.
     */    
    Iterator dequeueWindowClose() {
        return closedWindows.iterator();
    }
    
    /**
     * Returns an iterator over all windows that need to be lowered
     * on the client.  The controller will use the
     * Iterator.remove() method to remove each window from this queue after
     * it has generated script to lower the window.
     * 
     * @return An iterator of windows needing to be lowered.
     */
    Iterator dequeueWindowLower() {
        return loweredWindows.iterator();
    }
    
    /**
     * Returns an iterator over all windows that are presently closed on the 
     * client and need to be opened.  The controller will use the 
     * Iterator.remove() method to remove each window from this queue after
     * it has generated script to open the window.
     *
     * @return An iterator of windows needing to be opened.
     */    
    Iterator dequeueWindowOpen() {
        return openedWindows.iterator();
    }
    
    /**
     * Returns an iterator over all windows that need to be raised
     * on the client.  The controller will use the 
     * Iterator.remove() method to remove each window from this queue after
     * it has generated script to raise the window.
     *
     * @return An iterator of windows needing to be raised.
     */    
    Iterator dequeueWindowRaise() {
        return raisedWindows.iterator();
    }
    
    /**
     * Returns an iterator over all open windows that need to have their titles 
     * updated on the client.
     *
     * @return An iterator of windows needing their titles updated.
     */
    Iterator dequeueWindowTitleUpdate() {
        return windowTitleUpdates.iterator();
    }
    
    /**
     * Returns an iterator over all open windows that need to have their 
     * content updated on the client.
     *
     * @return An iterator of windows needing their content updated.
     */
    Iterator dequeueWindowUpdate() {
        return windowUpdates.iterator();
    }

    /**
     * Causes the service containing the given component to be re-rendered.
     *
     * @param componentPeer A <code>ComponentPeer</code> whose represented
     *        <code>Component</code> has changed and therefore needs to be
     *        updated on the client browser.
     */
    void enqueueComponentUpdate(ComponentPeer componentPeer) {
        if (componentPeer instanceof WindowUI) {
            enqueueWindowUpdate((WindowUI) componentPeer);
        } else {
            enqueuePaneUpdate(componentPeer);
        }
    }

    /**
     * Adds the <code>Service</code> that is responsible for rendering the 
     * specified <code>ComponentPeer</code> to the list of services needing
     * to be refreshed.
     *
     * @param componentPeer The component that needs to be re-rendered.
     */
    private void enqueuePaneUpdate(ComponentPeer componentPeer) {
        if (componentPeer instanceof WindowUI) {
            throw new IllegalArgumentException("Cannot redraw window.");
        }
        
        ComponentPeer parent = componentPeer.getParent();
        if (parent instanceof WindowUI) {
            // If the componentPeer is the root pane of the window, then forward the redraw request to
            // redrawWindow().
            enqueueWindowUpdate((WindowUI) parent);
        } else {
            PaneUI containingPane = instancePeer.getContainingPane(componentPeer);
    
            // Quick check to ensure that the service containing this component is not already queued for update.
            // This check is done first because often many components under the same service will be updated at once.
            if (paneUpdates.contains(containingPane)) {
                return;
            }
    
            Iterator it;
            Component testComponent;
            Component component = componentPeer.getComponent();      
    
            // Determine if any services that are ancestors of the component are already scheduled for updating.
            // At the same time, remove any descendant services of the component from being scheduled for updates. 
            it = paneUpdates.iterator();
            while (it.hasNext()) {
                testComponent = ((ComponentPeer) it.next()).getComponent();
    
                // Check for ancestors of the test component.
                if (testComponent.isAncestorOf(component)) {
                    return;
                }
    
                if (component.isAncestorOf(testComponent)) {
                    // A child of the component was found in the queue, therefore, the component should be added.  There is
                    // no possiblility that the components parent could be in the queue.  Therefore, the iterator is now
                    // used only to look for more child components.
                    it.remove();
                    while (it.hasNext()) {
                        testComponent = ((ComponentPeer) it.next()).getComponent();
                        if (component.isAncestorOf(testComponent)) {
                            it.remove();
                        }
                    }
                    paneUpdates.add(containingPane);
                    return;
                }
            }
    
            if (openedWindows.size() > 0) {
                // Determine if the component is a descendant of a window that is scheduled to be opened.
                // If this is the case, the component's containing service will not be scheduled for update.
                it = openedWindows.iterator();
                while (it.hasNext()) {
                    testComponent = ((ComponentPeer) it.next()).getComponent();
                    if (testComponent.isAncestorOf(component)) {
                        // The window that contains the component is scheduled to be opened.  There is no reason
                        // to schedule the component's containing service for an update.
                        return;
                    }
                }
            }
            
            if (windowUpdates.size() > 0) {
                // Determine if the component is a descendant of a window that is scheduled to be refreshed.
                // If this is the case, the component's containing service will not be scheduled for update.
                it = windowUpdates.iterator();
                while (it.hasNext()) {
                    testComponent = ((ComponentPeer) it.next()).getComponent();
                    if (testComponent.isAncestorOf(component)) {
                        // The window that contains the component is scheduled to be opened.  There is no reason
                        // to schedule the component's containing service for an update.
                        return;
                    }
                }
            }
            
            if (closedWindows.size() > 0) {
                // Determine if the component is a descendant of a window that is scheduled to be closed.
                // If this is the case, the component's containg service will not be scheduled for update.
                it = closedWindows.iterator();
                while (it.hasNext()) {
                    testComponent = ((ComponentPeer) it.next()).getComponent();
                    if (testComponent.isAncestorOf(component)) {
                        // The window that contains the component is scheduled to be closed.  There is no reason
                        // to schedule the component's containing service for an update.
                        return;
                    }
                }
            }
    
            // The method has not prematurely returned, therefore no services that contain the component were found.
            paneUpdates.add(containingPane);
        }
    }

    /**
     * Removes a window from the application.  The window will be closed
     * on the client browser if necessary (the only case where this is not
     * necessary is if the window is not yet open but was scheduled to be 
     * opened by a call to <code>addWindow()</code>).
     *
     * @param windowUI The window to be removed.
     */
    void enqueueWindowClose(WindowUI windowUI) {
        if (openedWindows.contains(windowUI)) {
            openedWindows.remove(windowUI);
        } else {
            closedWindows.add(windowUI);
            windowUpdates.remove(windowUI);
            windowTitleUpdates.remove(windowUI);
            deleteComponentUpdate(windowUI);
            loweredWindows.remove(windowUI);
            raisedWindows.remove(windowUI);
        }
    }

    /** 
     * Schedules a window to be lowered to the background of the client 
     * application.
     * 
     * @param windowUI The window to lower.
     */
    void enqueueWindowLower(WindowUI windowUI) {
        if (!openedWindows.contains(windowUI)) {
            if (loweredWindows.contains(windowUI)) {
                loweredWindows.remove(windowUI);
            }
            if (raisedWindows.contains(windowUI)) {
                raisedWindows.remove(windowUI);
            }
            loweredWindows.add(windowUI);
        }
    }
    
    /**
     * Adds a window to the application.  The window will be opened on the 
     * client if necessary (the only case where this is not necessary is if
     * the window is already opened and was scheduled to be closed by a call
     * to <code>removeWindow()</code>).
     *
     * @param windowUI The window to be added.
     */
    void enqueueWindowOpen(WindowUI windowUI) {
        if (closedWindows.contains(windowUI)) {
            // The window was closed, and is being opened again, in the same step.  Therefore, the window on the user's screen, 
            // The window is thus removed from the closedWindows collections. 
            closedWindows.remove(windowUI);

            // The window may also need to be refreshed, and  because its state was not being monitored between the time it was
            // closed and reopened, there is no way to tell  if an update is necessary.  Thus the window's content is marked to be
            // updated.
            windowUpdates.add(windowUI);
            windowTitleUpdates.add(windowUI);
        } else {
            // The window needs to be opened on the client.  
            openedWindows.add(windowUI);
            
            // Remove any service updates that are scheduled underneath the window.
            deleteComponentUpdate(windowUI);
            
            // Remove the window from being updated as a service.
            windowUpdates.remove(windowUI);
            
            // Remove any window title updates.
            windowTitleUpdates.remove(windowUI);
            
            // Remove any window raising/lowering tasks
            loweredWindows.remove(windowUI);
            raisedWindows.remove(windowUI);
        }
    }
    
    /**
     * Adds the specified <code>WindowUI</code> to the list of windows that
     * need to have be raised on the client.
     * 
     * @param windowUI the peer of a <code>Window</code> which should be 
     *        raised on the client.
     */
    void enqueueWindowRaise(WindowUI windowUI) {
        if (!openedWindows.contains(windowUI)) {
            if (loweredWindows.contains(windowUI)) {
                loweredWindows.remove(windowUI);
            }
            if (raisedWindows.contains(windowUI)) {
                raisedWindows.remove(windowUI);
            }
            raisedWindows.add(windowUI);
        }
    }

    /**
     * Adds the specified <code>WindowUI</code> to the list of windows that
     * need to have their titles updated.
     *
     * @param windowUI The peer of a <code>Window</code> whose title has been
     *        changed.
     */
    void enqueueWindowTitleUpdate(WindowUI windowUI) {
        if (!openedWindows.contains(windowUI) && !closedWindows.contains(windowUI)) {
            windowTitleUpdates.add(windowUI);
        }
    }

    /**
     * Adds the specified <code>WindowUI</code> to the list of windows that
     * need to have their content updated.
     *
     * @param windowUI The peer of a <code>Window</code> whose content has been
     *        changed.
     */
    private void enqueueWindowUpdate(WindowUI windowUI) {
        // If the window is scheduled to be opened (and is therefore not yet open), return immediately.
        if (openedWindows.contains(windowUI)) {
            return;
        }
        
        windowUpdates.add(windowUI);
    }

    /**
     * Returns true if there are services that need to be refreshed on the
     * client.
     *
     * @return True if there are services that need to be refreshed on the
     *         client.
     */
    boolean isPaneUpdateRequired() {
        return paneUpdates.size() > 0;
    }

    /**
     * Returns true if there are windows on the client that need to be closed.
     *
     * @return True if there are windows on the client that need to be closed.
     */
    boolean isWindowCloseRequired() {
        return closedWindows.size() > 0;
    }

    /**
     * Returns true if there are windows that need to be lowered to the 
     * background of the screen.
     * 
     * @return True if there are windows that need to be lowered to the
     *         background of the screen.
     */
    boolean isWindowLowerRequired() {
        return loweredWindows.size() > 0;
    }
    
    /**
     * Returns true if there are windows that need to be opened on the client.
     *
     * @return True if there are windows that need to be opened on the client.
     */
    boolean isWindowOpenRequired() {
        return openedWindows.size() > 0;
    }
    
    /**
     * Returns true if there are windows that need to be raised to the 
     *         foreground of the screen.
     * 
     * @return True if there are windows that need to be raised to the 
     *         foreground of the screen.
     */
    boolean isWindowRaiseRequired() {
        return raisedWindows.size() > 0;
    }
    
    /**
     * Returns true if there are windows that need their titles updated on the 
     * client.
     *
     * @return True if there are windows that need their titles updated on the 
     *         client.
     */
    boolean isWindowTitleUpdateRequired() {
        return windowTitleUpdates.size() > 0;
    }
    
    /**
     * Returns true if there are windows that need their content updated on 
     * the client.
     *
     * @return True if there are windows that need their content updated on 
     *         the client.
     */
    boolean isWindowUpdateRequired() {
        return windowUpdates.size() > 0;
    }
    
    /**
     * Purges all updates.
     */
    void purgeAllUpdates() {
        windowUpdates.clear();
        windowTitleUpdates.clear();
        paneUpdates.clear();
        openedWindows.clear();
        closedWindows.clear();
        raisedWindows.clear();
        loweredWindows.clear();
    }
}
