/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.security.Principal;
import javax.servlet.ServletConfig;
import javax.servlet.http.HttpSession;

/**
 * An interface provided to an EchoInstance such that it may access certain 
 * facilities available in the application container.  An instance of an
 * implementation of this interface is set as an attribute with 
 * <code>ATTRIBUTE_NAME</code>
 */ 
public interface ServerContext {

    public static final String ATTRIBUTE_NAME = "nextapp.echoservlet.ServerContext";

    /**
     * Exits the application.  The application will be removed from the HTTP
     * session, and if its URI is requested again, a new instance of the 
     * application will be provided.  When an application exits, all browser 
     * windows used by the application will be closed except one.  
     * The remaining window will be redirected to the default exit URI.
     */
    public void exit();
    
    /**
     * Exits the application.  The application will be removed from the HTTP
     * session, and if its URI is requested again, a new instance of the 
     * application will be provided.  When an application exits, all browser 
     * windows used by the application will be closed except one.  
     * The remaining window will be redirected to the specified URI.
     *
     * @param uri The URI to redirect the browser to.
     */
    public void exit(String uri);
    
    /**
     * Returns the abnormal exit URI.  
     * The browser is redirected to the abnormal exit URI automatically if the
     * application encounters an unrecoverable error.
     *
     * @return The abnormal exit URI.
     */
    public String getAbnormalExitUri();

    /**
     * Returns the character encoding that will be used when communicating 
     * with the client.
     *
     * @return The character encoding that will be used when
     *        communicating with the client.
     */    
    public String getCharacterEncoding();
    
    /**
     * Returns the client properties data that contains data about the 
     * client browser.
     *
     * @return The instance's client properties data.
     */
    public ClientProperties getClientProperties();

    /**
     * Returns the CookieManager instance for the application.
     *
     * @return The CookieManager instance for the application.
     */
    public CookieManager getCookieManager();
    
    /**
     * Returns the default exit URI.
     * The browser is redirected to the default exit URI automatically if the
     * exit() or suspend() are called without a specific URI.
     *
     * @return The default exit URI.
     */
    public String getDefaultExitUri();
    
    /**
     * Returns the <code>ServletConfig</code> object provided to the 
     * <code>EchoServer</code> when it was initialized.  The 
     * <code>ServletConfig</code> object provides access to servlet 
     * initialization parameters.  Use
     * <code>ServletConfig.getServletContext()</code> to obtain access
     * to the <code>ServletContext</code>.
     * 
     * @return The <code>ServletConfig</code> object provided to the
     *         <code>EchoServer</code> when it was initialized.
     */
    public ServletConfig getServletConfig();

    /**
     * Returns the HTTP session that in which the Echo application is held.
     *
     * @return the HTTP session that in which the Echo application is held.
     */
    public HttpSession getSession();
    
    /**
     * Returns the principal (user) of the application.
     *
     * @return The principal (user) of the application.
     */
    public Principal getUserPrincipal();
    
    /**
     * Returns true if the application's user is in the specified role.
     *
     * @return True if the application's user is in the specified role.
     */
    public boolean isUserInRole(String role);

    /**
     * Sets the abnormal exit URI.
     * The browser is redirected to the abnormal exit URI automatically if the
     * application encounters an unrecoverable error.
     *
     * @param abnormalExitUri The abnormal exit URI.
     */
    public void setAbnormalExitUri(String abnormalExitUri);
    
    /**
     * Sets the character encoding that will be used when communicating with 
     * the client.
     *
     * @param characterEncoding The character encoding that will be used when
     *        communicating with the client.
     */    
    public void setCharacterEncoding(String characterEncoding);

    /**
     * Sets the default exit URI
     * The browser is redirected to the default exit URI automatically if the
     * exit() or suspend() are called without a specific URI.
     *
     * @param uri The default exit URI.
     */
    public void setDefaultExitUri(String uri);
    
    /**
     * Suspends the application.  The application will still be held in the HTTP
     * session, and if its URI is requested again, the application will 
     * reinitialize to its pre-suspended state.  When an application is suspended,
     * all browser windows used by the application will be closed except one.  
     * The remaining window will be redirected to the default exit URI.
     */
    public void suspend();
    
    /**
     * Suspends the application.  The application will still be held in the HTTP
     * session, and if its URI is requested again, the application will 
     * reinitialize to its pre-suspended state.  When an application is suspended,
     * all browser windows used by the application will be closed except one.  
     * The remaining window will be redirected to the specified URI.
     *
     * @param uri The URI to redirect the browser to.
     */
    public void suspend(String uri);
}
