/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;

/**
 * A utility class which fixes character encoding issues which occur during
 * HTTP transactions as a result of browsers not properly setting the 
 * character encoding of their requests.
 */
class ServletEncodingAdapter {
    
    private static final boolean SERVLET_2_2_TEST_MODE = false;
    
    /**
     * Indicates whether the servlet container supports the
     * <code>ServletRequest.setCharacterEncoding()</code> method.
     */
    private boolean mutableRequestEncoding;
    
    /**
     * Indicates whether character encoding must be corrected on request
     * parameters.
     */
    private boolean encodingCorrectionRequired;
    
    /**
     * The servlet container's default character encoding.
     * This is the encoding with which the container will encode strings by
     * default if the browser has not set the encoding and the encoding
     * has not been set via 
     * <code>ServletRequest.setCharacterEncoding()</code>.
     */
    private String servletContainerDefaultEncoding;
    
    /**
     * A reflection reference to the 
     * <code>ServletRequest.setCharacterEncoding()</code> method.
     */
    private Method setCharacterEncodingMethod;

    /**
     * Creates a new <code>ServletEncodingAdapter</code> for an
     * <code>EchoServer</code>.
     * 
     * @param server The <code>EchoServer</code>.
     */
    ServletEncodingAdapter(EchoServer server) {
        super();

        ServletContext context = server.getServletContext();
        if (SERVLET_2_2_TEST_MODE || (context.getMajorVersion() == 2 && context.getMinorVersion() == 2)) {
            // Servlet 2.2 Containers do not support the ServletRequest.setCharacterEncoding() method.
            // Earlier versions are not relevant because they are not supported by Echo.
            mutableRequestEncoding = false;
            
            // Determine if encoding corrections are required based on data in ApplicationProperites object.
            encodingCorrectionRequired 
                    = server.getApplicationProperties().getBoolean(ApplicationProperties.FIX_PARAMETER_ENCODING);
            
             // Determine the encoding FROM which corrections will be made based on data in ApplicationProperties object.            
            servletContainerDefaultEncoding 
                    = server.getApplicationProperties().get(ApplicationProperties.DEFAULT_PARAMETER_ENCODING);
        } else {
            // Servlet 2.3 and higher Containers support the ServletRequest.setCharacterEncoding() method.
            mutableRequestEncoding = true;
            
            // Encoding corrections are not required.
            encodingCorrectionRequired = false;
            
            // Servlet Container's Default Encoding is not relevant.
            servletContainerDefaultEncoding = null;

            // Store a Method object for using reflection to call ServletRequest.setCharacterEncoding().
            try {            
                setCharacterEncodingMethod = ServletRequest.class.getMethod("setCharacterEncoding", new Class[]{String.class});
            } catch (NoSuchMethodException ex) {
                // Should never occur.
                throw new EchoServletException(ex.toString());
            }
        }
    }
    
    /**
     * Returns a parameter value from a <code>ServletRequest</code>,
     * performing any necessary character encoding corrections.
     * 
     * @param request The <code>ServletRequest</code> containing the 
     *        parameter.
     * @param parameterName The name of the parameter to return.
     * @param encoding The proper encoding of the parameter.
     * @return The correctly encoded value of the parameter.
     */
    String getParameter(ServletRequest request, String parameterName, String encoding) 
    throws UnsupportedEncodingException {
        String value = request.getParameter(parameterName);
        if (encodingCorrectionRequired) {
            if (value != null) {
                value = new String(value.getBytes(servletContainerDefaultEncoding), encoding);
            }
        }
        return value;
    }
    
    /**
     * Sets the character encoding of a <code>ServletRequest</code>,
     * if the operation is supported.
     * 
     * @param request The <code>ServletRequest</code> to be configured.
     * @param encoding The character encoding. 
     */
    void setRequestCharacterEncoding(ServletRequest request, String encoding) 
    throws UnsupportedEncodingException {
        if (!mutableRequestEncoding) {
            // Do nothing for servlet 2.2 containers.
            return;
        }
        
        try {
            setCharacterEncodingMethod.invoke(request, new Object[]{encoding});
        } catch (IllegalAccessException ex) {
            // Should never occur.
            throw new EchoServletException(ex.toString());
        } catch (InvocationTargetException ex) {
            Throwable cause = ex.getTargetException();
            if (cause instanceof UnsupportedEncodingException) {
                throw (UnsupportedEncodingException) cause;
            } else {
                // Should never occur.
                throw new EchoServletException(ex.toString());
            }
        }
    }
}
