/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;

import nextapp.echoservlet.util.ContentType;
import nextapp.echoservlet.util.Extensions;
import nextapp.echoservlet.util.JavaScriptCompressor;
import nextapp.echoservlet.util.Resource;

/**
 * A service that renders static text data.
 */
public class StaticText
implements CacheableService, Serializable {

    /**
     * Creates a <code>StaticText</code> object by way of a resource name.  
     * The content type will be determined from the resource's extension.
     * JavaScript resources will be compressed.
     *
     * @param resourceName The name of the resource from which to create the
     *        <code>StaticText</code> object.
     * @return A <code>StaticText</code> object containing the content of the
     *         specified resource.
     * @deprecated The createFromResource(String, String) method
     *             should be used instead.  Use of this version may result
     *             in errors if applications are migrated between VMs,
     *             due to the Service having a different identifier on each VM.
     */
    public static StaticText createFromResource(String resourceName) {
        return createFromResource(null, resourceName);
    }

    /**
     * Creates a <code>StaticText</code> object by way of a resource name.  
     * The content type will be determined from the resource's extension.
     * JavaScript resources will be compressed.
     *
     * @param identifier A unique identifier for this service.
     * @param resourceName The name of the resource from which to create the
     *        <code>StaticText</code> object.
     * @return A <code>StaticText</code> object containing the content of the
     *         specified resource.
     */
    public static StaticText createFromResource(String identifier, String resourceName) {
        ContentType contentType = Extensions.getFileNameContentType(resourceName);
        String resource = Resource.getResourceAsString(resourceName);
        if (contentType.equals(ContentType.TEXT_JAVASCRIPT)) {
            resource = JavaScriptCompressor.compress(resource);
        }
        StaticText staticText = new StaticText(identifier, contentType, resource);
        return staticText;
    }

    private ContentType contentType = null;
    private String data = null;
    private Id id = null;

    /**
     * Creates a new text content object.
     *
     * @param contentType The content type of the object.
     * @param data A string referencing the content.
     * @deprecated The StaticText(String, ContentType, String) constructor
     *             should be used instead.  Use of this version may result
     *             in errors if applications are migrated between VMs,
     *             due to the Service having a different identifier on each VM.
     */
    public StaticText(ContentType contentType, String data) {
        this(null, contentType, data);
    }
    
    /**
     * Creates a new text content object.
     *
     * @param identifier A unique identifier for this service.
     * @param contentType The content type of the object.
     * @param data A string referencing the content.
     */
    public StaticText(String identifier, ContentType contentType, String data) {
        super();

        this.contentType = contentType;
        this.data = data;
        if (identifier == null) {
            id = new Id();
        } else {
            id = new Id(identifier);
        }
    }
    
    /**
     * Returns the content type.
     *
     * @return The content type.
     */
    public ContentType getContentType() {
        return contentType;
    }
    
    /**
     * @see nextapp.echoservlet.Service#getId()
     */
    public Id getId() {
        return id;
    }

    /**
     * Returns zero to indicate that the content will never change.
     *
     * @return Zero to indicate that the content will never change.
     */ 
    public int getVersion() {
        return 0;
    }
    
    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn) 
    throws IOException {
        conn.setContentType(contentType);
        PrintWriter pw = conn.getWriter();
        
        try {
            pw.write(data);
        } finally {
            pw.close(); 
        }
    }
}
