/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import nextapp.echoservlet.util.ContentType;
import nextapp.echoservlet.util.Extensions;
import nextapp.echoservlet.util.JavaScriptCompressor;
import nextapp.echoservlet.util.Resource;

/**
 * A service which produces templated text output.  Templates are used in 
 * cases where the server must render a mostly static document that contains
 * text that may vary.  This service is similar to <code>StaticText</code>,
 * but offers the ability to have dynamically inserted text at specified
 * points.  A dynamic area is specified by adding a &quot;variable name&quot;
 * to the template.  Variables names are enclosed in dollar signs.<br>
 * <br>
 * The following example document shows how a Template can be used:<br>
 * <br>
 * <code>In the year $Year$, annual sales were $$$AnnualSales$.</code><br>
 * <br>
 * This sample has two variables, &quot;Year&quot; and &quot;Annual 
 * Sales&quot;.  The double dollar sign will be rendered as a single dollar 
 * sign.  The variable data will be extracted from the renderer's 
 * <code>VariableData</code> object.
 */
public class Template 
implements Service, Serializable {

    private static final String MARKERSTRING = "$";
    
    /**
     * Creates a <code>Template</code> object by way of a resource name.  
     * The content type will be determined from the resource's extension.
     * JavaScript resources will be compressed.
     *
     * @param resourceName The name of the resource from which to create the
     *        <code>StaticText</code> object.
     * @return A <code>Template</code> object containing the content of the
     *         specified resource.
     * @deprecated The createFromResource(String, String) method
     *             should be used instead.  Use of this version may result
     *             in errors if applications are migrated between VMs,
     *             due to the Service having a different identifier on each VM.
     */
    public static Template createFromResource(String resourceName) {
        return createFromResource(null, resourceName);
    }

    /**
     * Creates a <code>Template</code> object by way of a resource name.  
     * The content type will be determined from the resource's extension.
     * JavaScript resources will be compressed.
     *
     * @param identifier A unique identifier for this service.
     * @param resourceName The name of the resource from which to create the
     *        <code>StaticText</code> object.
     * @return A <code>Template</code> object containing the content of the
     *         specified resource.
     */
    public static Template createFromResource(String identifier, String resourceName) {
        ContentType contentType = Extensions.getFileNameContentType(resourceName);
        String resource = Resource.getResourceAsString(resourceName);
        if (contentType.equals(ContentType.TEXT_JAVASCRIPT)) {
            resource = JavaScriptCompressor.compress(resource);
        }
        Template template =  new Template(identifier, contentType, resource);
        return template;
    }

    private String[] staticSections = null;
    private String[] fieldNames = null;
    
    private ContentType contentType = null;
    private Id id = null;

    /**
     * Creates a content template from the given String.
     * 
     * @param contentType The content type of the output.
     * @param templateData A String containing the template data.
     * @deprecated The Template(String, ContentType, String) constructor
     *             should be used instead.  Use of this version may result
     *             in errors if applications are migrated between VMs,
     *             due to the Service having a different identifier on each VM.
     */
    public Template(ContentType contentType, String templateData) {
        this(null, contentType, templateData);
    }
    
    /**
     * Creates a content template from the given String.
     * 
     * @param contentType The content type of the output.
     * @param templateData A String containing the template data.
     * @param identifier A unique identifier for this service.
     */
    public Template(String identifier, ContentType contentType, String templateData) {
        super();
        
        if (identifier == null) {
            id = new Id();
        } else {
            id = new Id(identifier);
        }
        this.contentType = contentType;
        
        List fieldNameList = new ArrayList();
        List staticSectionList = new ArrayList();
        
        int startPosition = 0;
        int endPosition = 0;
        do {
            // Search for next field name
            endPosition = templateData.indexOf(MARKERSTRING, startPosition);
            
            // Add static text up to variable to list of static content.
            if (endPosition == -1) {
                staticSectionList.add(templateData.substring(startPosition));
            } else {
                staticSectionList.add(templateData.substring(startPosition, endPosition));
            }
            
            if (endPosition != -1) {
                // determine variable name.
                startPosition = endPosition + 1;
                endPosition = templateData.indexOf(MARKERSTRING, startPosition);
                
                if (endPosition == startPosition) {
                    // MARKERSTRING detected twice, therefore MARKERSTRING is added to last static
                    // content section.
                    staticSectionList.set(staticSectionList.size() - 1, 
                            staticSectionList.get(staticSectionList.size() - 1) + MARKERSTRING);
                } else if (endPosition == -1) {
                    // MARKERSTRING was not found again, meaning document is not valid.
                    throw new IllegalArgumentException("Data contains unterminated variable string.");
                } else {
                    // Add field name to list
                    fieldNameList.add(templateData.substring(startPosition, endPosition));
                }
                
                startPosition = endPosition + 1;
            }
        } while (endPosition != -1);
        
        staticSections = (String[]) staticSectionList.toArray(new String[staticSectionList.size()]);
        fieldNames = (String[]) fieldNameList.toArray(new String[fieldNameList.size()]);
    }
    
    /**
     * Returns the content type.
     *
     * @return The content type.
     */
    public ContentType getContentType() {
        return contentType;
    }
    
    /**
     * @see nextapp.echoservlet.Service#getId()
     */
    public Id getId() {
        return id;
    }
    
    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn) 
    throws IOException {
        conn.setContentType(contentType);
        PrintWriter pw = conn.getWriter();
        
        VariableData variableData = conn.getVariableData();
        String customField = null;
        for (int index = 0; index < staticSections.length; ++index) {
            if (staticSections[index] != null) {
                pw.write(staticSections[index]);
            }
            if (index < fieldNames.length && fieldNames[index] != null) {
                customField = variableData.get(fieldNames[index]);
                if (customField != null) {
                    pw.write(customField);
                }
            }
        }
        
        pw.close();
    }
    
    /**
     * Renders the content template as a String.  Should be used only for 
     * debugging purposes.
     *
     * @return A String representation of the template.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
    
        for (int index = 0; index < staticSections.length; ++index) {
            if (staticSections[index] != null) {
                sb.append("STATIC: " + staticSections[index]);
            }
            sb.append("\n");
            if (fieldNames.length - 1 >= index && fieldNames[index] != null) {
                sb.append("DYNAMIC: \"" + fieldNames[index] + "\"");
            }
            sb.append("\n");
        }
        
        return sb.toString();
    }
}
