/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.PrintWriter;

import nextapp.echo.Window;

import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.resource.ResourceNames;
import nextapp.echoservlet.util.ContentType;

/**
 * A peer object for <code>Window</code>s.  This class is responsible for
 * rendering the root document of each browser window, which contains a
 * &lt;frameset&gt; that references the window's content and the
 * <code>Controller</code> document.  Windows also contain the 
 * client model object script and additional JavaScript functions for
 * maintaining application state.
 */
public class WindowUI extends ComponentPeer 
implements PropertyChangeListener, Service {

    public static final int MENU_BAR = 0x4;
    public static final int STATUS_BAR = 0x8;
    public static final int TOOL_BAR = 0x2;

    /**
     * The service containing the ClientModel script.
     */
    private static final Service SERVICE_CLIENT_MODEL_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "cmodel", ResourceNames.CLIENT_MODEL_SCRIPT);
    
    /**
     * The service containing the window's built-in JavaScript functions.
     */
    private static final Service SERVICE_WINDOW_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "win", ResourceNames.WINDOW_SCRIPT);
    
    static {
       EchoServer.addGlobalService(SERVICE_CLIENT_MODEL_SCRIPT);
       EchoServer.addGlobalService(SERVICE_WINDOW_SCRIPT);
    }

    /**
     * Returns a line of JavaScript code that will invoke the window unload 
     * handler.
     *
     * @return A line of JavaScript code that will invoke the window unload 
     *         handler.
     */
    public static String getScriptHandleUnload() {
        return ClientObjects.STD_PREFIX + "handleUnload();";
    }
    
    private int flags = 0;
    
    /**
     * Returns the PaneUI of the WindowUI's content.
     *
     * @return The PaneUI of the WindowUI's content.
     */
    public PaneUI getRootPaneUI() {
        Window window = (Window) getComponent();
        PaneUI paneUI;
        if (window.getContent() == null || !window.getContent().isVisible()) {
            paneUI = null;
        } else {
            paneUI = (PaneUI) getPeer(window.getContent());
            if (paneUI instanceof ContainerPaneUI) {
                // Display forwarded container pane if container pane is empty.
                paneUI = ((ContainerPaneUI) paneUI).getForwardingPane();
            }
        } 
        return paneUI;
    }
    
    /**
     * Returns the script necessary to initialize the "ClientModel" 
     * JavaScript object.
     *
     * @return The script necessary to initialize the "ClientModel" 
     * JavaScript object.
     */
    private String getScriptInitClientModel() {
        return "E_initClientModel(\'" + getId() + "\');";
    }
    
    /**
     * Returns the flags enabled on this window.
     *
     * @return The flags enabled on this window, which can consist of zero or
     *         more of the following values ORed together:
     *         <ul>
     *         <li>MENU_BAR</li>
     *         <li>STATUS_BAR</li>
     *         <li>TOOL_BAR</li>
     *         </ul>
     */
    public int getFlags() {
        return flags;
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        if (Window.TITLE_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            getInstancePeer().setTitle(this);
        } else if (Window.Z_INDEX_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (((Integer)e.getNewValue()).intValue() > 0) {
                getInstancePeer().raiseWindow(this);
            } else {
                getInstancePeer().lowerWindow(this);
            }
        } else {
            redraw();
        }
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }

    /**
     * @see nextapp.echoservlet.Service#service(Connection)
     */
    public void service(Connection conn) 
    throws IOException {
        Window window = (Window) getComponent();
        
        HtmlDocument doc = HtmlDocument.createFrameSet(conn);
        doc.addScriptInclude(EchoServer.SERVICE_COLLECTIONS_SCRIPT);
        doc.addScriptInclude(SERVICE_CLIENT_MODEL_SCRIPT);
        doc.addScriptInclude(SERVICE_WINDOW_SCRIPT);
        doc.addScript(getScriptInitClientModel(), HtmlDocument.EVENT_INITIALIZE);
        if (!conn.getServer().getApplicationProperties().getBoolean(ApplicationProperties.DISABLE_ONUNLOAD)) {
            doc.addScript(getScriptHandleUnload(), HtmlDocument.EVENT_ONUNLOAD);
        }
        
        // Add Title.
        Element title = new Element(ElementNames.TITLE, true);
        title.setWhitespaceRelevant(true);
        if (window.getTitle() != null) {
            title.addText(window.getTitle());
        }
        doc.getHeadElement().add(title);
        
        Element frameSet = doc.getBodyElement();
        if (conn.isDebugInteractiveControllerEnabled()) {
            // The interactive controller is visible.
            frameSet.addAttribute("rows", "*,40");
        } else {
            frameSet.addAttribute("rows", "100%,*");
        }
        frameSet.addAttribute("id", ClientObjects.getObjectName(getId()));
        frameSet.addAttribute("frameborder", 0);
        frameSet.addAttribute("framespacing", 0);
        frameSet.addAttribute("border", 0);
        
        // Add the window's content.
        PaneUI paneUI;
        if (window.getContent() == null || !window.getContent().isVisible()) {
            paneUI = null;
        } else {
            paneUI = (PaneUI) getPeer(window.getContent());
        } 

        frameSet.add(ContainerPaneUI.createFrameElement(conn, paneUI));
        
        // Add Controller Frame.
        Element controlFrame = new Element(ElementNames.FRAME, false);
        controlFrame.addAttribute("src", ClientObjects.getServiceUri(conn, getInstancePeer().getController()));
        controlFrame.addAttribute("name", ClientObjects.CONTROLLER_FRAME_NAME);
        controlFrame.addAttribute("scrolling", "no");
        controlFrame.addAttribute("frameborder", 0);
        controlFrame.addAttribute("marginwidth", 0);
        controlFrame.addAttribute("marginheight", 0);
        controlFrame.addAttribute("noresize");
        frameSet.add(controlFrame);
        
        conn.setContentType(ContentType.TEXT_HTML);
        PrintWriter pw = conn.getWriter();
        doc.render(pw);
        pw.close();
    }
    
    /**
     * Sets the flags enabled on this window.
     *
     * @param flags The flags enabled on this window, which can consist of zero or
     *        more of the following values ORed together:
     *        <ul>
     *        <li>MENU_BAR</li>
     *        <li>STATUS_BAR</li>
     *        <li>TOOL_BAR</li>
     *        </ul>
     */
    public void setFlags(int flags) {
        this.flags = flags;
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
