/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.html;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * A representation of an HTML element designed to reside in a hierarchy
 * of elements (an HTML document) and be rendered with good performance.
 */
public class Element 
implements Renderable, Serializable {

    private String name = null;
    private Map attributes = null;
    private ElementContent contents = null;
    private boolean whitespaceRelevant = false;
    private boolean sticky = false;
    private boolean closingTagRequired = true;
    
    /**
     * Creates a new Element of the specified name with a required closing tag.
     *
     * @param name The name of the element.
     */
    public Element(String name) {
        this(name, true);
    }
    
    /**
     * Creates a new Element of the specified name and closing tag requirement.
     *
     * @param name The name of the element.
     * @param closingTagRequired Specifies whether a closing tag should be 
     *        applied if no child elements or content is added to the element.
     */
    public Element(String name, boolean closingTagRequired) {
        super();
        
        this.name = name;
        this.closingTagRequired = closingTagRequired;
    }
    
    /**
     * Adds a renderable object to the element.
     *
     * @param child The renderable object to add to the element.
     */
    public void add(Renderable child) {
        if (contents == null) {
            contents = new ElementContent();
        }
        
        contents.add(child);
    }
    
    /**
     * Adds a renderable object to the element at a specified index.
     * 
     * @param index The index at which to add the renderable.
     * @param child The renderable object to add to the element.
     */
    public void add(int index, Renderable child) {
        if (contents == null) {
            contents = new ElementContent();
        }
        
        contents.add(index, child);
    }
    
    /**
     * Adds a null attribute to the element.
     *
     * @param attr The name of the attribute.
     */
    public void addAttribute(String attr) {
        if (attributes == null) {
            attributes = new HashMap();
        }
        
        attributes.put(attr, null);
    }
    
    /**
     * Adds an integer attribute to the element.
     *
     * @param attr The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addAttribute(String attr, int value) {
        if (attributes == null) {
            attributes = new HashMap();
        }
        
        attributes.put(attr, Integer.toString(value));
    }
    
    /**
     * Adds the a text attribute to the element.
     *
     * @param attr The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addAttribute(String attr, String value) {
        if (attributes == null) {
            attributes = new HashMap();
        }
        
        attributes.put(attr, value);
    }
    
    /**
     * Adds a child element to the element.
     * 
     * @deprecated The add() method with the equivalent signature should 
     *             now be used.
     *
     * @param child The child element to add to the element.
     */
    public void addElement(Element child) {
        add(child);
    }
    
    /**
     * Adds a child element to the element at a specified index.
     * 
     * @deprecated The add() method with the equivalent signature should 
     *             now be used.
     *
     * @param index The index at which to add the child element.
     * @param child The child element to add to the element.
     */
    public void addElement(int index, Element child) {
        add(index, child);
    }
    
    /**
     * Adds content from an <code>ElementContent</code> object to this element.
     *
     * @deprecated The add() method with the equivalent signature should 
     *             now be used.
     * 
     * @param content The content to add.
     */
    public void addElementContent(ElementContent content) {
        add(content);
    }
    
    /**
     * Adds content from an <code>ElementContent</code> object to this element
     * at a specified index.
     * 
     * @deprecated The add() method with the equivalent signature should 
     *             now be used.
     *
     * @param index The index at which to add the element content.
     * @param content The content to add.
     */
    public void addElementContent(int index, ElementContent content) {
        add(index, content);
    }
    
    /**
     * Adds raw HTML to the body of the element.  The given HTML string will 
     * not be encoded.
     *
     * @param html Encoded HTML content to be added to this element.
     */
    public void addHtml(String html) {
        if (contents == null) {
            contents = new ElementContent();
        }
        
        contents.addHtml(html);
    }
    
    /**
     * Adds raw HTML to the body of the element at a specified index.  
     * The given HTML string will not be encoded.
     *
     * @param index The index at which to add the HTML.
     * @param html Encoded HTML content to be added to this element.
     */
    public void addHtml(int index, String html) {
        if (contents == null) {
            contents = new ElementContent();
        }
        
        contents.addHtml(index, html);
    }
    
    /**
     * Adds plain text to the body of the element.  This text will 
     * automatically be encoded into valid HTML.
     */
    public void addText(String text) {
        addText(text, true);
    }
    
    /**
     * Adds plain text to the body of the element.  This text will 
     * automatically be encoded into valid HTML.
     *
     * @param text Unencoded text to be added as HTML to the body of the 
     *        element.
     * @param encodeNewLines A flag indicating if new line characters should
     *        be translated to &lt;BR&gt;s.
     */
    public void addText(String text, boolean encodeNewLines) {
        if (contents == null) {
            contents = new ElementContent();
        }

        contents.addText(text, encodeNewLines);
    }
    
    /**
     * Adds plain text to the body of the element at a specified index.  
     * This text will automatically be encoded into valid HTML.
     *
     * @param index The index at which to add the text.
     * @param text Unencoded text to be added as HTML to the body of the 
     *        element.
     * @param encodeNewLines A flag indicating if new line characters should
     *        be translated to &lt;BR&gt;s.
     */
    public void addText(int index, String text, boolean encodeNewLines) {
        if (contents == null) {
            contents = new ElementContent();
        }

        contents.addText(index, text, encodeNewLines);
    }
    
    /**
     * Returns the value of the given attribute.
     *
     * @param name The name of the attribute.
     * @return The attribute value.
     */
    public String getAttribute(String name) {
        if (attributes == null) {
            return null;
        } else {
            return (String) attributes.get(name);
        }
    }
    
    /**
     * Returns the name of the element.
     *
     * @return The name of the element.
     */
    public String getName() {
        return name;
    }
    
    /**
     * Returns the index of the specified object if it is a child of this
     * Element.  If the object is not found, -1 is returned.
     *
     * @param child The child object whose index is desired.
     * @return The index of the specified object.
     */
    public int indexOf(Object child) {
        if (contents == null) {
            return -1;
        }
        return contents.indexOf(child);
    }
    
    /**
     * Determines whether the elment is &quot;sticky&quot;  Sticky elements 
     * will be rendered immediately after and adjacent to the previous 
     * element (without whitespace).
     * 
     * @return True if the element is sticky.
     */
    public boolean isSticky() {
        return sticky;
    }
    
    /**
     * Renders this element and its children to a PrintWriter.
     *
     * @param pw The print writer to render the element to.
     */
    public void render(PrintWriter pw) {
        render(pw, 0, false);
    }
    
    /**
     * Renders this element and its children to a PrintWriter.
     * The element will be indented to the specified depth, and its
     * children will be indented to the specified depth + 1.
     * If white space is relevant (whitespaceRelevant==true), then no white 
     * space will be left between this element's opening and closing tags.
     *
     * @param pw The print writer to render the element to.
     * @param depth The indentation depth to indent the element to.
     * @param parentWhitespaceRelevant A flag indicating whether the parent
     *        element had the WhitespaceRelevant property set.
     */
    public void render(PrintWriter pw, int depth, boolean parentWhitespaceRelevant) {
        if (!parentWhitespaceRelevant && !sticky) {
            for (int index = 0; index < depth; ++index) {
                pw.print(INDENT_STRING);
            }
        }

        pw.print("<");
        pw.print(name);
        
        // render attributes
        if (attributes != null) {
            String attributeName = null;
            String attributeValue = null;
        
            for (Iterator it = attributes.keySet().iterator(); it.hasNext();) {
                attributeName = (String) it.next();
                attributeValue = (String) attributes.get(attributeName);

                pw.print(" ");
                pw.print(attributeName);

                if (attributeValue != null) {
                    pw.print("=\"");
                    pw.print(attributeValue);
                    pw.print("\"");
                }
            }
        }
        
        pw.print(">");

        if (! whitespaceRelevant && (closingTagRequired || contents != null)) {
            pw.println();
        }
        
        // Render child objects
        if (contents != null ) {
        
            contents.render(pw, depth, whitespaceRelevant);
            
            // Always print closing tag if children exist.
            if (! whitespaceRelevant) {
                for (int index = 0; index < depth; ++index) {
                    pw.print(INDENT_STRING);
                }
            }

            pw.print("</");
            pw.print(name);
            pw.print(">");
        } else if (closingTagRequired) {
            if (! whitespaceRelevant) {
                for (int index = 0; index < depth; ++index) {
                    pw.print(INDENT_STRING);
                }
            }

            pw.print("</");
            pw.print(name);
            pw.print(">");
        }
    }
    
    /**
     * Sets whether the element requires an explicit closing tag.  An element 
     * will always be explicitly closed if it has child elements.
     *
     * @param closingTagRequired True if the tag should be explicitly closed 
     *        event if it has no child elements.  False if the tag should be 
     *        explicitly closed only when it has child elements.
     */
    public void setClosingTagRequired(boolean closingTagRequired) {
        this.closingTagRequired = closingTagRequired;
    }
    
    /**
     * Sets whether the element is &quot;sticky&quot;  Sticky elements will be
     * rendered immediately after and adjacent to the previous element (without
     * whitespace).
     * 
     * @param sticky True if the element should be sticky.
     */
    public void setSticky(boolean sticky) {
        this.sticky = sticky;
    }
    
    /**
     * Sets whether the rendering of this element is adversely effected by 
     * white space.  If this flag is set, child elements will be rendered 
     * immediately adjacent to the opening and closing tags of this element, 
     * with no whitespace in between.  If white space is not relevant, this
     * flag should be left in its default false state as the document will
     * be more human-readable.
     *
     * @param whitespaceRelevant True if this tag is sensitive to whitespace
     *        inside it.
     */
    public void setWhitespaceRelevant(boolean whitespaceRelevant) {
        this.whitespaceRelevant = whitespaceRelevant;
    }
}
