/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.html;

import java.io.PrintWriter;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A representation of an <code>Element</code>s content.
 */
public class ElementContent 
implements Renderable, Serializable {

    private List contents = null;

    /**
     * Creates a new Element Content object.
     */    
    public ElementContent() {
        super();
        
        contents = new ArrayList(5);
    }
    
    /**
     * Adds a child element to this element.
     *
     * @param child The child element to be added.
     */
    public void add(Renderable child) {
        contents.add(child);
    }
    
    /**
     * Adds a child element to this element at the specified index.
     * 
     * @param index The index at which to  add the child.
     * @param child The child element to be added.
     */
    public void add(int index, Renderable child) {
        contents.add(index, child);
    }
    
    /**
     * Adds raw HTML to the body of the tag.  The given HTML string will not
     * be encoded.
     *
     * @param html A string of encoded HTML.
     */
    public void addHtml(String html) {
        contents.add(html);
    }
    
    /**
     * Adds raw HTML to the body of the tag at the specified index.  The given
     * HTML string will not be encoded.
     *
     * @param index The index at which to  add the HTML.
     * @param html A string of encoded HTML.
     */
    public void addHtml(int index, String html) {
        contents.add(index, html);
    }
    
    /**
     * Adds plain text to the tag.  This text will automatically be encoded
     * into valid HTML.
     *
     * @param text A string of unencoded text that will be encoded to HTML.
     * @param encodeNewLines A flag indicating whether the encoding should
     *        translate newline characters to &lt;br&gt; elements.
     */
    public void addText(String text, boolean encodeNewLines) {
        contents.add(Html.encode(text, encodeNewLines ? Html.NEWLINE_TO_BR : Html.NEWLINE_TRANSPARENT));
    }
    
    /**
     * Adds plain text to the tag at the specified index.  This text will 
     * automatically be encoded into valid HTML.
     *
     * @param index The index at which to  add the text.
     * @param text A string of unencoded text that will be encoded to HTML.
     * @param encodeNewLines A flag indicating whether the encoding should
     *        translate newline characters to &lt;br&gt; elements.
     */
    public void addText(int index, String text, boolean encodeNewLines) {
        contents.add(index, Html.encode(text, encodeNewLines ? Html.NEWLINE_TO_BR : Html.NEWLINE_TRANSPARENT));
    }
    
    /**
     * Returns the index of the specified object if it is a child of this
     * ElementContent object.  If the object is not found, -1 is returned.
     *
     * @param child The child object whose index is desired.
     * @return The index of the specified object.
     */
    public int indexOf(Object child) {
        return contents.indexOf(child);
    }
    
	/**
	 * @see nextapp.echoservlet.html.Renderable#render(PrintWriter, int, boolean)
	 */
    public void render(PrintWriter pw, int depth, boolean whitespaceRelevant) {
        Object child = null;
        Object nextChild = null;
        
        Iterator it = contents.iterator();
        
        if (it.hasNext()) {
            nextChild = it.next();
        }
        
        while (nextChild != null) {
            child = nextChild;
            if (it.hasNext()) {
                nextChild = it.next();
            } else {
                nextChild = null;
            }
            
            if (child instanceof String) {
                pw.print((String) child);
            } else if (child instanceof Renderable) {
                ((Renderable) child).render(pw, depth + (whitespaceRelevant ? 0 : 1), whitespaceRelevant);
            }

            if (! whitespaceRelevant) {
                if (nextChild == null || !(nextChild instanceof Element) || !((Element) nextChild).isSticky()) {
                    pw.println();
                }
            }
        }
    }
}
