/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.html;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * A container of JavaScript code.  Provides the ability to add lines of 
 * JavaScript code that will be automatically indented correctly for the
 * document in which they are rendered.
 */
public class Script 
implements Renderable, Serializable {

    private List code = null;
    private static final String INDENT = "    ";
    private static final String TAB = "    ";
    
    /**
     * Encodes single and double quotation marks found in a String by 
     * backslash escaping them. 
     *
     * @param s The string to be encoded.
     * @return The encoded string.
     */
    public static String encodeQuotedString(String s) {
        StringBuffer sb = new StringBuffer();
        int length = s.length();
        int startIndex = 0;
        int index = 0;
        
        while (index < length) {
            switch (s.charAt(index)) {
            case '\'':
                if (startIndex < index) {
                    sb.append(s.substring(startIndex, index));
                }
                sb.append("\\\'");
                startIndex = index + 1;
                break;
            case '\"':
                if (startIndex < index) {
                    sb.append(s.substring(startIndex, index));
                }
                sb.append("\\\"");
                startIndex = index + 1;
                break;
            }
            ++index;
        }

        if (startIndex < index) {
            sb.append(s.substring(startIndex));
        }

        return sb.toString();
    }
    
    /** 
     * Creates a new script object.
     */
    public Script() {
        super();
        
        code = new ArrayList(5);
    }

    /**
     * Adds a line of script code.  The line should not contain any newline
     * characters.
     *
     * @param tab The tab index to which the line should be indented.
     * @param line The line of code (with no leading spaces and no newline
     *        characters.
     */
    public void add(int tab, String line) {
        code.add(INDENT);
        for (int index = 0; index < tab; ++index) {
            code.add(TAB);
        }
        code.add(line);
        code.add("\n");
    }
    
    /**
     * @see nextapp.echoservlet.html.Renderable#render(PrintWriter, int, boolean)
     */
    public void render(PrintWriter pw, int depth, boolean parentWhitespaceRelevant) {
        for (Iterator it = code.iterator(); it.hasNext();) {
            pw.print((String) it.next());
        }
    }
} 
