/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.html;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * A representation of a CSS style that may be associated with multiple
 * types of elements.
 */
public class Style 
implements Renderable, Serializable {

    private static final String DOUBLE_INDENT = INDENT_STRING + INDENT_STRING;
    private static final String TRIPLE_INDENT = INDENT_STRING + INDENT_STRING + INDENT_STRING;

    private Map attributes = new HashMap();
    private Map pseudonyms = null;
    private Set elementTypes = new HashSet();
    private String id;
    
    /**
     * A unit suffix for percent units.
     */
    public static final String PERCENT_UNITS = "%";

    /**
     * A unit suffix for pixel units.
     */
    public static final String PIXEL_UNITS = "px";

    /**
     * A unit suffix for point units.
     */
    public static final String POINT_UNITS = "pt";
    
    /**
     * Creates a new style with the given identifier.
     *
     * @param id The identifier of this style.
     */
    public Style(String id) {
        super();
        
        this.id = id;
    }
    
    /**
     * Adds an element type to the style.  The style
     * will be rendered identically for each element type that is specified.
     * For example, a single style might be used for the elements
     * &lt;A&gt;, &lt;INPUT&gt; and &lt;TD&gt;.
     *
     * @param elementType The element to be added, e.g.
     *        &quot;a&quot;, &quot;input&quot;, or &quot;td&quot;.
     */
    public void addElementType(String elementType) {
        elementTypes.add(elementType);
    }
    
    /**
     * Adds a text attribute to the style.
     *
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addAttribute(String name, String value) {
        attributes.put(name, value);
    }

    /**
     * Adds an integer attribute (with no units) to this style.
     *
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addAttribute(String name, int value) {
        attributes.put(name, Integer.toString(value));
    }
    
    /**
     * Adds an integer attribute with the specified units to this style.
     *
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     * @param units The units of the value.
     */
    public void addAttribute(String name, int value, String units) {
        attributes.put(name, value + units);
    }
    
    /**
     * Adds a pseudoclass attribute to the style.
     *
     * @param pseudonym The name of the pseudoclass.
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addPseudoAttribute(String pseudonym, String name, String value) {
        if (pseudonyms == null) {
            pseudonyms = new HashMap();
        }
        
        Map pseudoAttributes = (Map) pseudonyms.get(pseudonym);
        if (pseudoAttributes == null) {
            pseudoAttributes = new HashMap();
            pseudonyms.put(pseudonym, pseudoAttributes);
        }
        
        pseudoAttributes.put(name, value);
    }
    
    /**
     * Adds a pseudoclass integer attribute to the style.
     *
     * @param pseudonym The name of the pseudoclass.
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addPseudoAttribute(String pseudonym, String name, int value) {
        addPseudoAttribute(pseudonym, name, Integer.toString(value));
    }
    
    /**
     * Renders this style to the given <code>PrintWriter</code>.
     * 
     * @see nextapp.echoservlet.html.Renderable#render(PrintWriter, int, boolean)
     */
    public void render(PrintWriter pw, int depth, boolean parentWhitespaceRelevant) {
        for (Iterator elementTypeIt = elementTypes.iterator(); elementTypeIt.hasNext();) {
            String elementType = (String) elementTypeIt.next();
            renderClass(pw, elementType, null, attributes);
            
            if (pseudonyms != null) {
                pw.println();
                
                for (Iterator pseudonymIt = pseudonyms.keySet().iterator(); pseudonymIt.hasNext();) {
                    String pseudonym = (String) pseudonymIt.next();
                    Map pseudoAttributes = (Map) pseudonyms.get(pseudonym);
                    renderClass(pw, elementType, pseudonym, pseudoAttributes);

                    if (pseudonymIt.hasNext()) {
                        pw.println();
                    }
                }
            }
            
            if (elementTypeIt.hasNext()) {
                pw.println();
            }
        }
    }
    
    /**
     * Renders a class element of the style to the given
     * <code>PrintWriter</code>.
     *
     * @param pw The <code>PrintWriter</code> to print the output to.
     * @param elementType The element type which the style is describing,
     *        e.g. &quot;a&quot;, &quot;span&quot;, etc.
     * @param pseudonym The pseudoclass being rendered (null if the main
     *        class is being rendered).
     * @param attributeData A map containing attributes (name/value pairs)
     *        as Strings.
     */
    private void renderClass(PrintWriter pw, String elementType, String pseudonym, Map attributeData) {
        String attributeName;

        pw.print(DOUBLE_INDENT);
        pw.print(elementType);
        pw.print(".");
        pw.print(id);
        if (pseudonym != null) {
            pw.print(":");
            pw.print(pseudonym);
        }
        pw.println(" {");

        for (Iterator attributeIt = attributeData.keySet().iterator(); attributeIt.hasNext();) {
            attributeName = (String) attributeIt.next();
            pw.print(TRIPLE_INDENT);
            pw.print(attributeName);
            pw.print(": ");
            pw.print((String) attributeData.get(attributeName));
            pw.println(";");
        }

        pw.print(DOUBLE_INDENT);
        pw.print("}");
    }
}
