/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.image;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import nextapp.echo.ImageReference;
import nextapp.echo.event.ImageUpdateListener;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.Connection;
import nextapp.echoservlet.Service;

/**
 * Manages the images referenced by a <code>ComponentPeer</code>.
 */
public class ImageManager 
implements Serializable {

    private static final int INITIAL_CAPACITY = 3;

    /**
     * A map that maps image names to image peers.
     */
    private Map images = null;
    
    /**
     * The ComponentPeer for which this ImageManager is managing images.
     */
    private ComponentPeer componentPeer;

    /** 
     * Creates an image manager for the specified component.
     *
     * @param componentPeer The <code>ComponentPeer</code> whose images will 
     *        be managed by this instance.
     */
    public ImageManager(ComponentPeer componentPeer) {
        super();
        
        this.componentPeer = componentPeer;
    }
    
    /** 
     * Returns an image managed by this <code>ImageManager</code>, 
     * identified by <code>imageName</code>.
     *
     * @param imageName The name of the image to return.
     * @return The <code>ImageReference</code> that corresponds to the 
     *         specified name.
     */ 
    public ImageReference getImage(String imageName) {
        ImageReference ref;
        
        if (images == null) {
            ref = null;
        } else {
            ImagePeer peer = (ImagePeer) images.get(imageName);

            if (peer == null) {
                ref = null;
            } else {
                ref = peer.getImageReference();
            }
        }
        
        return ref;
    }
    
    /**
     * Returns the number of images managed by this <code>ImageManager</code>.
     * 
     * @return The number of images managed by this <code>ImageManager</code>.
     */
    public int getImageCount() {
        if (images == null) {
            return 0;
        } else {
            return images.size();
        }
    }
    
    /**
     * Returns an <code>Iterator</code> over the names of images managed by
     * this <code>ImageManager</code>.
     * 
     * @return An <code>Iterator</code> over the names of images managed by
     *         this <code>ImageManager</code>.
     */
    public Iterator getImageNames() {
        if (images == null) {
            return Collections.EMPTY_SET.iterator();
        } else {
            return Collections.unmodifiableSet(images.keySet()).iterator();
        }
    }

    /**
     * Returns the URI of the image with provided name.
     *
     * @param conn The connection rendering the image URI.
     * @param imageName The name of the image whose URI is to be returned.
     * @return The URI of the image that corresponds to the specified name.
     */
    public String getImageUri(Connection conn, String imageName) {
        String uri;

        if (images == null) {
            uri = null;
        } else {
            ImagePeer peer = (ImagePeer) images.get(imageName);

            if (peer == null) {
                uri = null;
            } else {
                uri = peer.getUri(conn);
            }
        }
        
        return uri;
    }
    
    /**
     * Removes all images.
     */
    public void removeAllImages() {
        String[] imageNames = (String[]) images.keySet().toArray(new String[images.size()]);
        for (int index = 0; index < imageNames.length; ++index) {
            setImage(imageNames[index], null);
        }
    }
    
    /**
     * Adds an image to the image manager, identified by imageName.
     *
     * @param imageName A identifier that will be used for the added image.
     * @param newImage The new image to be added.
     */
    public void setImage(String imageName, ImageReference newImage) {
        if (images == null) {
            if (newImage == null) {
                return;
            } else {
                images = new HashMap(INITIAL_CAPACITY);
            }
        }
            
        ImagePeer oldImagePeer = (ImagePeer) images.get(imageName);
        Service service;

        if (oldImagePeer != null) {
            // An image peer already exists under this name, remove it.
            
            if (componentPeer instanceof ImageUpdateListener) {
                // The componentPeer implements ImageUpdateListener, unregister 
                // it from being informed about updates to the image.
                oldImagePeer.getImageReference().removeImageUpdateListener((ImageUpdateListener) componentPeer);
            }

            // Clear the image from being associated with this ImageManager.
            images.remove(imageName);

            // Retrieve the ancillary service (if any) used by the image peer.
            service = oldImagePeer.getAncillaryService();

            if (service != null) {
                // The image has an ancillary service.
                if (! images.values().contains(service)) {
                    // The image is not used under another name in 
                    // the ImageManager: remove its ancillary service from
                    // the ComponentPeer.
                    componentPeer.removeAncillaryService(service);
                }
            }
        }

        if (newImage != null) {
            // The image is being set, not removed.
        
            // Create an ImagePeer through the Image Peer Factory.
            ImagePeer newImagePeer = componentPeer.getInstancePeer().getImageRepository().getImagePeer(newImage);

            if (componentPeer instanceof ImageUpdateListener) {
                // The componentPeer implements ImageUpdateListener, register 
                // it to be informed about updates to the image.
                newImagePeer.getImageReference().addImageUpdateListener((ImageUpdateListener) componentPeer);
            }
            
            // Retrieve the ancillary service (if any) used by the image peer.
            service = newImagePeer.getAncillaryService();
            
            if (service != null) {
                // The image has an ancillary service.
                if (! images.values().contains(service)) {
                    // The image is NOT already being used under another name
                    // in the ImageManager: its ancillary service needs to be
                    // registered with the ComponentPeer.
                    componentPeer.addAncillaryService(service);
                }
            }

            // Add the image to be associated with this ImageManager.
            // This must be done after the ancillary service check.
            images.put(imageName, newImagePeer);
        }
    }
}
