/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.image;

import java.io.Serializable;

import nextapp.echo.ImageReference;
import nextapp.echoservlet.Connection;
import nextapp.echoservlet.Id;
import nextapp.echoservlet.Service;

/**
 * A base class from which all image peers are derived.
 */ 
public abstract class ImagePeer
implements Serializable {
    
    /**
     * A value assigned by the <code>ImageRepository</code> to indicate how
     * recently the image peer has been used.
     */
    private int recency;

    /**
     * A reference to the represented <code>ImageReference</code>
     */
    private ImageReference imageRef;

    /**
     * Default constructor.
     */
    public ImagePeer() {
        super();
    }
    
    /**
     * Returns the ancillary service used for rendering this image on the 
     * client.  Returns null if no ancillary service is required.
     *
     * @return The ancillary service used for rendering this image on the 
     * client.
     */
    public abstract Service getAncillaryService();

    /**
     * Returns the underlying image reference.
     * 
     * @return The underlying image reference.
     */
    public ImageReference getImageReference() {
        return imageRef;
    }
    
    /**
     * Returns the recency value of this peer.
     * 
     * @return The recency value of this peer.
     */
    int getRecency() {
        return recency;
    }
    
    /**
     * Returns the URI required to reference this image from a document
     * rendered by the given <code>Connection</code>.
     *
     * @param conn The connection that is rendering the document that 
     * references the image.
     * @return The URI of the image.
     */
    public abstract String getUri(Connection conn);
    
    /**
     * Called to notify the image peer that it has been registered to an image.
     * This method should be overridden when notification of registration
     * is required.
     */
    public void registered() { }
    
    /**
     * Sets the identifier for the ancillary service if one is required.
     * 
     * @param id The identifier for the ancillary service.
     */
    public void setId(Id id) { }
    
    /**
     * Initializes the image peer: this method must be called before any other 
     * methods.
     *
     * @param imageRef The image that is represented by this
     *        <code>ImagePeer</code>.
     */
    void setImageReference(ImageReference imageRef) {
        this.imageRef = imageRef;
    }
    
    /**
     * Sets the recency value of this peer.
     * 
     * @param recency The recency value.
     */
    void setRecency(int recency) {
        this.recency = recency;
    }
    
}
