/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.image;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;
import java.util.WeakHashMap;
import java.lang.ref.WeakReference;

import nextapp.echo.ImageReference;
import nextapp.echoservlet.InstancePeer;

/**
 * A caching repository of image peers.
 */
public class ImageRepository 
implements Serializable {
    
    private static final int CACHE_SIZE = 100;
    private static final int REDUCTION_SIZE = 10;
    private transient Map ref2peer = null;
    private transient int currentRecencyValue = 0;
    private InstancePeer instancePeer;
    
    /**
     * Creates a new <code>ImageRepository</code>.
     */
    public ImageRepository(InstancePeer instancePeer) {
        super();
        this.instancePeer = instancePeer;
    }
    
    /**
     * Returns an image peer for the specified image.  A peer will be 
     * retrieved from the cache if possible, otherwise one will be created.
     * 
     * @param imageRef The image for which to return a peer.
     * @return A peer suitable for the specified image reference.
     */
    public ImagePeer getImagePeer(ImageReference imageRef) {
        ImagePeer imagePeer = loadPeer(imageRef);
        if (imagePeer == null) {
            imagePeer = ImagePeerFactory.createImagePeer(imageRef);
            storePeer(imageRef, imagePeer);
            imagePeer.setId(instancePeer.generateId());
        }
        imagePeer.setRecency(++currentRecencyValue);
        return imagePeer;
    }
    
    /**
     * Retreives an image peer from the cache for the specified image.
     * 
     * @param imageRef The image for which to return a peer.
     * @return An image peer for the specified image, or null if one 
     *         cannot be found.
     */
    private ImagePeer loadPeer(ImageReference imageRef) {
        if (ref2peer == null) {
            return null;
        } else {
            WeakReference peerWeakRef = (WeakReference) ref2peer.get(imageRef);
            if (peerWeakRef == null) {
                return null;
            } else {
                return (ImagePeer) peerWeakRef.get();
            }
        }
    }
    
    /**
     * Reduces the number of cached image peers by removing those that have
     * not been used recently.
     */
    private void reduce() {
        Iterator it = ref2peer.keySet().iterator();
        int minimumReceny = currentRecencyValue - (CACHE_SIZE - REDUCTION_SIZE);
        while (it.hasNext()) {
            ImageReference imageRef = (ImageReference) it.next();
            ImagePeer peer = (ImagePeer) ((WeakReference) ref2peer.get(imageRef)).get();
            if (peer == null || peer.getRecency() < minimumReceny) {
                it.remove();
            }
        }
    }
    
    /**
     * Stores an image peer in the cache for the specifeid image.
     * 
     * @param imageRef The image reference whose peer is to be stored.
     * @param peer The image reference's peer.
     */
    private void storePeer(ImageReference imageRef, ImagePeer peer) {
        if (ref2peer == null) {
            ref2peer = new WeakHashMap();
        }
        WeakReference peerWeakRef = new WeakReference(peer);
        ref2peer.put(imageRef, peerWeakRef);
        if (ref2peer.size() > CACHE_SIZE) {
            reduce();
        }
    }
}
