/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.AbstractButton;
import nextapp.echo.event.ImageUpdateEvent;
import nextapp.echo.event.ImageUpdateListener;
import nextapp.echoservlet.Alignment;
import nextapp.echoservlet.ClientActionProducer;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.image.ImageManager;

/**
 * A base peer class for components derived from <code>AbstractButton</code>.
 */
public abstract class AbstractButtonUI extends ComponentPeer 
implements Alignment, ClientActionProducer, ImageUpdateListener, PropertyChangeListener {
    
    /**
     * Indicates whether the button was most recently rendered enabled.
     */
    protected boolean renderedActive = true;
    
    /**
     * Default constructor.
     */
    public AbstractButtonUI() {
        super();
    }
    
    /**
     * The image manager that will handle images for the button,
     * including custom state images for toggle buttons.
     */
    protected ImageManager imageManager = null;
    
    /** 
     * A constant used to identify the default icon for the 
     * in the <code>ImageManager</code>.
     */
    protected static final String ICON_IMAGE = "icon";

    /** 
     * A constant used to identify the rollover icon for the 
     * in the <code>ImageManager</code>.
     */
    protected static final String ROLLOVER_ICON_IMAGE = "rolloverIcon";

    /**
     * @see nextapp.echoservlet.ClientActionProducer#clientAction(java.lang.String)
     */
    public void clientAction(String action) {
        if (renderedActive) {
            ((AbstractButton) getComponent()).doAction();
        }
    }
    
    /** 
     * Returns JavaScript code that will be executed when the button is
     * clicked.  On plain <code>Button</code>s, this will normally
     * send input to the server.  On <code>ToggleButton</code>s, the
     * selection state will change and input may also be sent to the
     * server if desired by the application.
     *
     * @param rc The <code>RenderingContext</code> being used for the
     *        rendering of the button.
     * @return A JavaScript method call that will be executed when the
     *         button is clicked.  The returned string must be valid
     *         HTML.
     */
    public abstract String getScriptAction(RenderingContext rc);

    /**
     * @see nextapp.echoservlet.Alignment#getHorizontalAlignment()
     */
    public int getHorizontalAlignment() {
        return ((AbstractButton) getComponent()).getHorizontalAlignment();
    }
    
    /**
     * @see nextapp.echoservlet.Alignment#getVerticalAlignment()
     */
    public int getVerticalAlignment() {
        return ((AbstractButton) getComponent()).getVerticalAlignment();
    }
    
    /**
     * @see nextapp.echo.event.ImageUpdateListener#imageUpdate(ImageUpdateEvent)
     */
    public void imageUpdate(ImageUpdateEvent e) {
        redraw();
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
        
        if (AbstractButton.ICON_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            AbstractButton button = (AbstractButton) getComponent();
            imageManager.setImage(ICON_IMAGE, button.getIcon());
        } else if (AbstractButton.ROLLOVER_ICON_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            AbstractButton button = (AbstractButton) getComponent();
            imageManager.setImage(ROLLOVER_ICON_IMAGE, button.getRolloverIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        AbstractButton button = (AbstractButton) getComponent();
        getComponent().addPropertyChangeListener(this);
        if (button.getIcon() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(ICON_IMAGE, button.getIcon());
        }
        if (button.getRolloverIcon() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(ROLLOVER_ICON_IMAGE, button.getRolloverIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        renderedActive = getComponent().isEnabled();
        Element buttonElement = ButtonRenderer.getElement(rc, this);
        if (buttonElement != null) {
            parent.add(buttonElement);
        }
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
        if (imageManager != null) {
            imageManager.setImage(ICON_IMAGE, null);
            imageManager.setImage(ROLLOVER_ICON_IMAGE, null);
        }
    }
}
