/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.io.Serializable;

import nextapp.echo.AbstractButton;
import nextapp.echo.CButton;
import nextapp.echo.EchoConstants;
import nextapp.echo.ImageReference;
import nextapp.echo.Insets;
import nextapp.echo.ToggleButton;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.ClientProperties;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.ExtentRenderer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementContent;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Renderable;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A utility class for rendering <code>Button</code>, <code>RadioButton</code>,
 * <code>CheckBox</code>, and <code>CButton</code> components.
 */ 
public class ButtonRenderer 
implements Serializable {
    
    public static final Service SERVICE_BUTTON_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "btn", ResourceNames.BUTTON_SCRIPT);
    
    static {
        EchoServer.addGlobalService(SERVICE_BUTTON_SCRIPT);
    }

    /**
     * Adds alignment (text-align/vertical-align) properties to a 
     * TD element.
     *
     * @param tdElement The TD element to which the properties are to be added.
     * @param horizontalAlignment The horizontal alignment setting desired
     *        for the style, one of the following value:
     *        <ul>
     *        <li>EchoConstants.LEFT</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     * @param verticalAlignment The vertical alignment setting desired
     *        for the style, one of the following value:
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     */
    private static void addAlignmentSettings(Element tdElement, int horizontalAlignment, int verticalAlignment) {
        switch (horizontalAlignment) {
        case EchoConstants.LEFT:    tdElement.addAttribute(ElementNames.Attributes.ALIGN, "left");       break;
        case EchoConstants.CENTER:  tdElement.addAttribute(ElementNames.Attributes.ALIGN, "center");     break;
        case EchoConstants.RIGHT:   tdElement.addAttribute(ElementNames.Attributes.ALIGN, "right");      break;
        }
        switch (verticalAlignment) {
        case EchoConstants.TOP:     tdElement.addAttribute(ElementNames.Attributes.VALIGN, "top");    break;
        case EchoConstants.CENTER:  tdElement.addAttribute(ElementNames.Attributes.VALIGN, "middle"); break;
        case EchoConstants.BOTTOM:  tdElement.addAttribute(ElementNames.Attributes.VALIGN, "bottom"); break;
        }
    }
    
    /**
     * Adds <code>border-style</code> property to a 
     * <code>ComponentSyle</code>.
     * 
     * @param style The <code>ComponentStyle</code> to modify.
     * @param borderStyle The border style constant value, one of the 
     *        following values:
     *        <ul>
     *         <code>CButton.BORDER_SOLID</code>
     *         <code>CButton.BORDER_INSET</code>
     *         <code>CButton.BORDER_OUTSET</code>
     *         <code>CButton.BORDER_NONE</code>
     *        </ul>
     */
    private static void addBorderStyleSetting(ComponentStyle style, int borderStyle) {
        switch (borderStyle) {
        case CButton.BORDER_SOLID:
            style.addAttribute(ElementNames.CssAttributes.BORDER_STYLE, "solid");
            break;
        case CButton.BORDER_INSET:
            style.addAttribute(ElementNames.CssAttributes.BORDER_STYLE, "inset");
            break;
        case CButton.BORDER_OUTSET:
            style.addAttribute(ElementNames.CssAttributes.BORDER_STYLE, "outset");
            break;
        case CButton.BORDER_NONE:
            style.addAttribute(ElementNames.CssAttributes.BORDER_STYLE, "none");
            break;
        }
    }

    /**
     * Returns an Element containing an HTML representation of the provided 
     * button.
     *
     * @param rc The <code>RenderingContext</code> that is rendering the 
     *        button.
     * @param buttonUI The peer of the button to be rendered.
     * @return A hierarchy of HTML elements that visually represent the button.
     */
    public static Element getElement(RenderingContext rc, AbstractButtonUI buttonUI) {
        return new ButtonRenderer(rc, buttonUI).getRoot();
    }
    
    /**
     * The root of the rendered HTML <code>Element</coe> hierarchy.
     */
    private Element rootElement;
    
    /**
     * The <code>RenderingContext</code> being used for this rendering.
     */
    private RenderingContext rc;

    /**
     * The <code>AbstractButtonUI</code> being rendered.
     */
    private AbstractButtonUI buttonUI;
    
    /**
     * The underlying <code>AbstractButton</code> component being rendered.
     */
    private AbstractButton button;
    
    /**
     * Text HTML code.
     */
    private Renderable textRenderable;
    
    /**
     * Icon HTML code.
     */
    private Renderable iconRenderable;
    
    /**
     * State icon HTML code.
     */
    private Renderable stateRenderable;
    
    /**
     * Counter indicating the next 'sub id' available to identify an HTML
     * element that will take part in the rollover effect.
     */
    private int nextRolloverSubIdValue = 0;
    
    /**
     * Name of style of root element.
     */
    private String rootStyleName;
    
    /**
     * Name of style applied to border TD element
     * (used for CButton renderings).
     */
    private String borderTdStyleName;
    
    /**
     * Creates a ButtonRenderer for the given Button.
     *
     * @param rc The <code>RenderingContext</code> that is rendering the 
     *        button.
     * @param buttonUI The peer of the button to be rendered.
     */
    private ButtonRenderer(RenderingContext rc, AbstractButtonUI buttonUI) {
        super();
        
        // Store initial properties.
        this.rc = rc;
        this.buttonUI = buttonUI;
        button = (AbstractButton) buttonUI.getComponent();
        
        // Create the main Renderable objects of the button: the text, icon, and state icon.
        createMainRenderables();
        
        // Create renderable containing main renderables.
        Renderable containerRenderable = createContainerRenderable();

        // Create border renderable (for CButton) if required.
        // Store previous top-level renderable in 'innerRenderable',
        // Store border renderable in 'containerRenderable'.
        Renderable innerRenderable;
        if (buttonUI instanceof CButtonUI) {
            innerRenderable = containerRenderable;
            containerRenderable = createCButtonBorderElement(containerRenderable);
        } else {
            innerRenderable = null;
        }
        
        // Create root element.
        if (containerRenderable instanceof Element) {
            // 'containerRenderable' is an element: cast and assign.
            rootElement = (Element) containerRenderable;
        } else if (containerRenderable != null) {
            // 'containerRenderable' is not an element, i.e., it is text:
            // Create a 'SPAN' element in which to house it.
            Element spanElement = new Element(ElementNames.SPAN);
            spanElement.addAttribute(ElementNames.Attributes.ID, getNextRolloverElementId());
            spanElement.setWhitespaceRelevant(true);
            spanElement.add(containerRenderable);
            rootElement = spanElement;
        }
        
        if (rootElement == null) {
            // Root element is null, no reason to do any more work: 
            // nothing will be rendered.
            return;
        }
        
        String scriptAction = buttonUI.getScriptAction(rc);
        
        // Add onclick event to root element to invoke desired script action.
        rootElement.addAttribute("onclick", scriptAction);
        
        // Add tab index so button is focusable.
        rootElement.addAttribute("tabindex", buttonUI.getTabIndex());
        
        rootElement.addAttribute("onkeypress", "E_processButtonKeyPress(event, &quot;" + scriptAction + "&quot;)");
        rc.getDocument().addScriptInclude(SERVICE_BUTTON_SCRIPT);
        
        // Create root style for component.
        ComponentStyle rootStyle = ComponentStyle.forComponent(buttonUI);
        rootStyle.addAttribute("cursor", "pointer");
        
        // Modify root style to account for properties (width/height) of
        // CButton.
        if (button instanceof CButton) {
            modifyCButtonBorderTableStyle(rootStyle);
        }
        
        // Add root style.
        rootStyle.addElementType(rootElement.getName());
        rootStyleName = rc.getDocument().addStyle(rootStyle);
        rootElement.addAttribute("class", rootStyleName);
        
        // Add font style for inner element if required.
        if (innerRenderable instanceof Element) {
            ComponentStyle innerStyle = new ComponentStyle();
            innerStyle.setFont(buttonUI.getFont());
            Element innerElement = (Element) innerRenderable;
            innerStyle.addElementType(innerElement.getName());
            String innerStyleName = rc.getDocument().addStyle(innerStyle);
            innerElement.addAttribute("class", innerStyleName);
        }
        
        // Add events for effects.
        if (button.isEnabled()) {
            if (button.isRolloverEnabled()) {
                // Create events for when button is rolled over.
                createRolloverEvents();
            }
            if (button instanceof CButton) {
                // Create events for when button is pressed.
                createPressEvents();
            }
        }
        
        // Add tool tip text.
        if (button.getToolTipText() != null) {
            rootElement.addAttribute(ElementNames.Attributes.TITLE, button.getToolTipText());
        }
    }
    
    /**
     * Creates and returns a <code>Renderable</code> that contains the icon, 
     * state, and text renderables of the button.
     * 
     * @return A <code>Renderable</code> that contains the icon, 
     * state, and text renderables of the button.
     */
    private Renderable createContainerRenderable() {
        // Determine number of cells in table.
        int cellCount = (textRenderable == null ? 0 : 1) + (iconRenderable == null ? 0 : 1) + (stateRenderable == null ? 0 : 1);
        
        Renderable containerRenderable;
        
        if (cellCount == 1) {
            // Render single entity button.
            if (textRenderable != null) {
                // Text only
                containerRenderable = textRenderable;
            } else if (iconRenderable != null) {
                // Icon only
                containerRenderable = iconRenderable;
            } else  {
                // State only
                containerRenderable = stateRenderable;
            }
        } else if (cellCount == 2) {
            // Render two entity button.
            TriCellTable tct;
            if (stateRenderable == null) {
                // Not a ToggleButton.
                int iconTextOrientation = TriCellTable.getOrientation(button.getHorizontalTextPosition(), 
                        button.getVerticalTextPosition());
                tct = new TriCellTable(rc.getConnection(), iconTextOrientation, button.getIconTextMargin());
                tct.getTdElement(0).add(textRenderable);
                tct.getTdElement(1).add(iconRenderable);
                addAlignmentSettings(tct.getTdElement(0), button.getHorizontalAlignment(),
                        button.getVerticalAlignment());
                addAlignmentSettings(tct.getTdElement(1), button.getHorizontalAlignment(),
                        button.getVerticalAlignment());
            } else {
                // Is a ToggleButton.
                ToggleButton toggleButton = (ToggleButton) button;
                int stateOrientation = TriCellTable.getOrientation(toggleButton.getHorizontalStatePosition(),
                        toggleButton.getVerticalStatePosition());
                tct = new TriCellTable(rc.getConnection(), stateOrientation ^ TriCellTable.INVERTED, 
                        toggleButton.getStateMargin());
                if (textRenderable == null) {
                    tct.getTdElement(0).add(iconRenderable);
                } else {
                    tct.getTdElement(0).add(textRenderable);
                }
                tct.getTdElement(1).add(stateRenderable);
                addAlignmentSettings(tct.getTdElement(0), toggleButton.getHorizontalAlignment(),
                        toggleButton.getVerticalAlignment());
                addAlignmentSettings(tct.getTdElement(1), toggleButton.getHorizontalStateAlignment(),
                        toggleButton.getVerticalStateAlignment());
            }
            tct.addAttribute(ElementNames.Attributes.ID, getNextRolloverElementId());
            containerRenderable = tct;
        } else if (cellCount == 3) {
            // Render triple entity button (definitely a ToggleButton).
            ToggleButton toggleButton = (ToggleButton) button;
            int iconTextOrientation = TriCellTable.getOrientation(toggleButton.getHorizontalTextPosition(), 
                        toggleButton.getVerticalTextPosition());
            int stateOrientation = TriCellTable.getOrientation(toggleButton.getHorizontalStatePosition(),
                        toggleButton.getVerticalStatePosition());
            
            TriCellTable tct = new TriCellTable(rc.getConnection(), iconTextOrientation, toggleButton.getIconTextMargin(), 
                        stateOrientation ^ TriCellTable.INVERTED, toggleButton.getStateMargin());
            tct.getTdElement(0).add(textRenderable);
            tct.getTdElement(1).add(iconRenderable);
            tct.getTdElement(2).add(stateRenderable);
            addAlignmentSettings(tct.getTdElement(0), toggleButton.getHorizontalAlignment(),
                    toggleButton.getVerticalAlignment());
            addAlignmentSettings(tct.getTdElement(1), toggleButton.getHorizontalAlignment(),
                    toggleButton.getVerticalAlignment());
            addAlignmentSettings(tct.getTdElement(2), toggleButton.getHorizontalStateAlignment(),
                        toggleButton.getVerticalStateAlignment());
            tct.addAttribute("id", getNextRolloverElementId());
            containerRenderable = tct;
        } else {
            containerRenderable = null;
        }
        
        return containerRenderable;
    }
    
    /**
     * Modifies style of border table to take into account properties
     * of <code>CButton</code>.
     * 
     * @param borderTableStyle The <code>ComponentStyle</code> to modify.
     */
    private void modifyCButtonBorderTableStyle(ComponentStyle borderTableStyle) {
        CButton cButton = (CButton) button;
        if (cButton.getHeightExtent() != null) {
            borderTableStyle.addAttribute("height", ExtentRenderer.render(cButton.getHeightExtent()));
        }
        if (cButton.getWidthExtent() != null) {
            borderTableStyle.addAttribute("width", ExtentRenderer.render(cButton.getWidthExtent()));
        }
    }
    
    /**
     * Creates and returns border element for CButton.
     * 
     * @param innerRenderable button contents.
     * @return Border <code>Element</code> containing contents of button. 
     */
    private Element createCButtonBorderElement(Renderable innerRenderable) {
        CButton cButton = (CButton) button;
        
        ComponentStyle borderTdStyle = new ComponentStyle();
        if (cButton.getBorderSize() != 0) {
            borderTdStyle.addAttribute("border-width", cButton.getBorderSize(), "px");
            if (cButton.getBorderColor() != null) {
                borderTdStyle.addAttribute("border-color", ComponentStyle.getColorHexValue(cButton.getBorderColor()));
            }
            addBorderStyleSetting(borderTdStyle, cButton.getBorderStyle());
        }
        Insets insets = cButton.getInsets();
        if (insets != null) {
            if (insets.getTop() == insets.getBottom() && insets.getLeft() == insets.getRight() 
                    && insets.getTop() == insets.getLeft()) {
                borderTdStyle.addAttribute("padding", insets.getTop(), ComponentStyle.PIXEL_UNITS);
            } else {
                borderTdStyle.addAttribute("padding-left", insets.getLeft(), ComponentStyle.PIXEL_UNITS);
                borderTdStyle.addAttribute("padding-top", insets.getTop(), ComponentStyle.PIXEL_UNITS);
                borderTdStyle.addAttribute("padding-right", insets.getRight(), ComponentStyle.PIXEL_UNITS);
                borderTdStyle.addAttribute("padding-bottom", insets.getBottom(), ComponentStyle.PIXEL_UNITS);
            }
        }
        
        borderTdStyle.addElementType(ElementNames.TD);
        borderTdStyleName = rc.getDocument().addStyle(borderTdStyle);
        
        Element tableElement = new Element(ElementNames.TABLE);
        tableElement.addAttribute("id", getNextRolloverElementId());
        tableElement.addAttribute("cellspacing", 0);
        Element trElement = new Element(ElementNames.TR);
        tableElement.add(trElement);
        Element tdElement = new Element(ElementNames.TD);
        tdElement.addAttribute(ElementNames.Attributes.ID, getBorderTableCellId());
        trElement.add(tdElement);
        trElement.setWhitespaceRelevant(true);
        tdElement.addAttribute("class", borderTdStyleName);
        addAlignmentSettings(tdElement, cButton.getHorizontalAlignment(), cButton.getVerticalAlignment());
        
        tdElement.setWhitespaceRelevant(true);
        tdElement.add(innerRenderable);
        return tableElement;
    }

    /**
     * Creates and returns an <code>Renderable</code> representing the
     * icon part of the button.
     * 
     * @return The created <code>Renderable</code>.
     */
    private Renderable createIconRenderable() {
        // Determine URIs of default and rollover images.
        String iconUri; 
        String rolloverIconUri;
        if (buttonUI.imageManager == null) {
            iconUri = null;
            rolloverIconUri = null;
        } else {
            iconUri = buttonUI.imageManager.getImage(AbstractButtonUI.ICON_IMAGE) == null 
                    ? null : buttonUI.imageManager.getImageUri(rc.getConnection(), AbstractButtonUI.ICON_IMAGE);
            rolloverIconUri = buttonUI.imageManager.getImage(AbstractButtonUI.ROLLOVER_ICON_IMAGE) == null 
                    ? null : buttonUI.imageManager.getImageUri(rc.getConnection(), AbstractButtonUI.ROLLOVER_ICON_IMAGE);
        }

        // Image element
        if (iconUri != null) {
            Element iconElement = new Element(ElementNames.IMG, false);
            iconElement.addAttribute(ElementNames.Attributes.SRC, iconUri);
            iconElement.addAttribute(ElementNames.Attributes.ALT, "");
            iconElement.addAttribute(ElementNames.Attributes.BORDER, 0);
            if (rolloverIconUri != null) {
                iconElement.addAttribute(ElementNames.Attributes.NAME, ClientObjects.getImageName(buttonUI.getId()));
            }

            AbstractButton button = (AbstractButton) buttonUI.getComponent();
            ImageReference icon = button.getIcon();
            if (icon.getWidth() > 0) {
                iconElement.addAttribute(ElementNames.Attributes.WIDTH, icon.getWidth());
            } 
            if (icon.getHeight() > 0) {
                iconElement.addAttribute(ElementNames.Attributes.HEIGHT, icon.getHeight());
            }
            
            return iconElement;
        } else {
            return null;
        }
    }
    
    /**
     * Creates the main (text/icon/state) renderable components that will
     * make up the button.  Created elements are store in instance variables
     * <code>textRenderable</code>, <code>iconRenderable</code>, and
     * <code>stateRenderable</code>.
     */
    private void createMainRenderables() {
        // Create the text, icon, and state Renderable objects of the button.
        textRenderable = button.getText() == null ? null : createTextRenderable();
        iconRenderable = button.getIcon() == null ? null : createIconRenderable();

        boolean isToggle = buttonUI instanceof ToggleButtonUI;
        stateRenderable = isToggle ? createStateRenderable((Element) iconRenderable) : null;
        
        if (iconRenderable == stateRenderable) {
            // Icon is being used to indicate state, only render state icon.
            iconRenderable = null;
        }
    }
    
    /**
     * Creates and returns an <code>Renderable</code> representing the
     * state part of the button.
     * 
     * @return The created <code>Renderable</code>.
     */
    private Renderable createStateRenderable(Element iconImgElement) {
        // Determine if button will use a custom state icon.
        boolean hasCustomStateIcon = buttonUI.imageManager != null
                && buttonUI.imageManager.getImage(AbstractButtonUI.ICON_IMAGE) != null  
                && buttonUI.imageManager.getImage(ToggleButtonUI.SELECTED_ICON_IMAGE) != null;

        // Create state element if necessary
        if (hasCustomStateIcon) {
            // Will modify button's icon for state.
            iconImgElement.addAttribute(ElementNames.Attributes.SRC, 
                    ((ToggleButtonUI) buttonUI).getStateImageUri(rc, button.isSelected()));
            iconImgElement.addAttribute(ElementNames.Attributes.NAME, 
                    ToggleButtonUI.TOGGLE_IMAGE_PREFIX + buttonUI.getId());
            return iconImgElement;
        } else {
            // Will not use button's icon for state.
            Element img = new Element(ElementNames.IMG, false);
            img.addAttribute(ElementNames.Attributes.SRC, ((ToggleButtonUI) buttonUI).getStateImageUri(rc, 
                    button.isSelected()));
            img.addAttribute(ElementNames.Attributes.NAME, ToggleButtonUI.TOGGLE_IMAGE_PREFIX + buttonUI.getId());
            img.addAttribute(ElementNames.Attributes.BORDER, 0);
            return img;
        }
    }

    /**
     * Creates and returns an <code>Renderable</code> representing the
     * text part of the button.
     * 
     * @return The created <code>Renderable</code>.
     */
    private Renderable createTextRenderable() {
        // Text element
        ElementContent textContent = new ElementContent();
        textContent.addText(button.getText(), true);
        return textContent;
    }
    
    /**
     * Creates CSS style for pressed state of <code>CButton</code>
     * which should be applied to all rendered <code>TABLE</code>
     * elements via a rollover effect when button is pressed on client.
     * 
     * @return The name of the created style name, or null if a
     *         style is not necessary.
     */
    private String createTablePressedCssStyle() {
        CButton button = (CButton) this.button;
        if (button.getPressedForeground() != null || button.getPressedBackground() != null
                || button.getPressedBorderColor() != null || button.getPressedBorderStyle() != 0) {
            ComponentStyle pressedStyle = new ComponentStyle();
            pressedStyle.setForeground(button.getPressedForeground());
            pressedStyle.setBackground(button.getPressedBackground());
            pressedStyle.addElementType("");
            rc.getDocument().addScriptInclude(CssSwapScript.SERVICE_CSS_SWAP_SCRIPT);
            return rc.getDocument().addStyle(pressedStyle);
        }
        return null;
    }
    
    /**
     * Creates CSS style for pressed state of <code>CButton</code>
     * which should be applied to the border <code>TD</code>
     * element via a rollover effect when button is pressed on client.
     * 
     * @return The name of the created style name, or null if a
     *         style is not necessary.
     */
    private String createTdPressedCssStyle() {
        CButton button = (CButton) this.button;
        if (button.getPressedBorderColor() != null || button.getPressedBorderStyle() != 0) {
            ComponentStyle pressedStyle = new ComponentStyle();
            if (button.getPressedBorderColor() != null) {
                pressedStyle.addAttribute(ElementNames.CssAttributes.BORDER_COLOR, 
                        ComponentStyle.getColorHexValue(button.getPressedBorderColor()));
            }
            addBorderStyleSetting(pressedStyle, button.getPressedBorderStyle());
            pressedStyle.addElementType("");
            rc.getDocument().addScriptInclude(CssSwapScript.SERVICE_CSS_SWAP_SCRIPT);
            return rc.getDocument().addStyle(pressedStyle);
        }
        return null;
    }
    
    /**
     * Adds mouse up/down event code to <code>rootElement</code> to create 
     * button-press effects when mouse is depressed on button.
     */
    private void createPressEvents() {
        if (!rc.getConnection().getInstancePeer().getClientProperties()
                .getBoolean(ClientProperties.CSS_MODIFICATION_SUPPORT)) {
            // Only render CSS-style modifying rollovers if supported by browser.
            return;
        }

        String pressedTableStyleName = createTablePressedCssStyle();
        String pressedTdStyleName = createTdPressedCssStyle();
        if (pressedTableStyleName == null && pressedTdStyleName == null) {
            // Do nothing.
            return;
        }
        
        rc.getDocument().addScriptInclude(RolloverScript.SERVICE_ROLLOVER_SCRIPT);
        
        String id = buttonUI.getId().toString();
        
        RolloverScript.MultipleRolloverScriptElement tableDown = new RolloverScript.MultipleRolloverScriptElement(
                id, null, pressedTableStyleName);
        RolloverScript.MultipleRolloverScriptElement tdDown = new RolloverScript.MultipleRolloverScriptElement(
                id + "_border", null, pressedTdStyleName);
        RolloverScript.MultipleRolloverScriptElement tableUp = new RolloverScript.MultipleRolloverScriptElement(
                id, null, rootStyleName);
        RolloverScript.MultipleRolloverScriptElement tdUp = new RolloverScript.MultipleRolloverScriptElement(
                id + "_border", null, borderTdStyleName);
        
        String scriptDown = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                new RolloverScript.MultipleRolloverScriptElement[]{tableDown, tdDown});
        String scriptUp = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                new RolloverScript.MultipleRolloverScriptElement[]{tableUp, tdUp});
        
        rootElement.addAttribute(ElementNames.Attributes.ONMOUSEDOWN, scriptDown);
        rootElement.addAttribute(ElementNames.Attributes.ONMOUSEUP, scriptUp);        
    }
    
    /**
     * Creates CSS style for rollover state of button
     * which should be applied to all rendered <code>TABLE</code>
     * elements via a rollover effect when button is rolled over on client.
     * 
     * @return The name of the created style name, or null if a
     *         style is not necessary.
     */
    private String createTableRolloverCssStyle() {
        if (button.getRolloverForeground() != null || button.getRolloverBackground() != null
                || button.getRolloverFont() != null) {
            ComponentStyle rolloverStyle = new ComponentStyle();
            rolloverStyle.setForeground(button.getRolloverForeground());
            rolloverStyle.setBackground(button.getRolloverBackground());
            rolloverStyle.setFont(button.getRolloverFont());
            rolloverStyle.addElementType("");
            rc.getDocument().addScriptInclude(CssSwapScript.SERVICE_CSS_SWAP_SCRIPT);
            return rc.getDocument().addStyle(rolloverStyle);
        }
        return null;
    }
    
    /**
     * Creates CSS style for rolled over state of <code>CButton</code>
     * which should be applied to the border <code>TD</code>
     * element via a rollover effect when button is rolled over on client.
     * 
     * @return The name of the created style name, or null if a
     *         style is not necessary.
     */
    private String createTdRolloverCssStyle() {
        
        if (button instanceof CButton && ((CButton) button).getRolloverBorderColor() != null) {
            ComponentStyle tdRolloverStyle = new ComponentStyle();
            tdRolloverStyle.addAttribute(ElementNames.CssAttributes.BORDER_COLOR,
                    ComponentStyle.getColorHexValue(((CButton) button).getRolloverBorderColor()));
            tdRolloverStyle.addElementType("");
            rc.getDocument().addScriptInclude(CssSwapScript.SERVICE_CSS_SWAP_SCRIPT);
            return rc.getDocument().addStyle(tdRolloverStyle);
        }
        return null;
    }
    
    /**
     * Adds mouse over/out event code to <code>rootElement</code> to create 
     * button-press effects when mouse is depressed on button.
     */
    private void createRolloverEvents() {
        String tableRolloverStyleName;
        String tdRolloverStyleName;
        
        if (rc.getConnection().getInstancePeer().getClientProperties()
                    .getBoolean(ClientProperties.CSS_MODIFICATION_SUPPORT)) {
            tableRolloverStyleName = createTableRolloverCssStyle();
            tdRolloverStyleName = createTdRolloverCssStyle();
        } else {
            tableRolloverStyleName = null;
            tdRolloverStyleName = null;
        }
        
        boolean hasIconRollover = button.getIcon() != null && button.getRolloverIcon() != null;
        
        if (tableRolloverStyleName == null && tdRolloverStyleName == null && !hasIconRollover) {
            // Rollover effect not required.
            return;
        }

        if (hasIconRollover) {
            rc.getDocument().addScriptInclude(ImageSwapScript.SERVICE_IMAGE_SWAP_SCRIPT);
        }
        rc.getDocument().addScriptInclude(RolloverScript.SERVICE_ROLLOVER_SCRIPT);
        
        String iconUri; 
        String rolloverIconUri;
        if (!hasIconRollover || buttonUI.imageManager == null) {
            iconUri = null;
            rolloverIconUri = null;
        } else {
            iconUri = buttonUI.imageManager.getImage(AbstractButtonUI.ICON_IMAGE) == null 
                    ? null : buttonUI.imageManager.getImageUri(rc.getConnection(), AbstractButtonUI.ICON_IMAGE);
            rolloverIconUri = buttonUI.imageManager.getImage(AbstractButtonUI.ROLLOVER_ICON_IMAGE) == null 
                    ? null : buttonUI.imageManager.getImageUri(rc.getConnection(), AbstractButtonUI.ROLLOVER_ICON_IMAGE);
        }
        
        String scriptRolloverEnter;
        String scriptRolloverExit;
        String id = buttonUI.getId().toString();
        
        if (stateRenderable != null && buttonUI.imageManager != null 
                && buttonUI.imageManager.getImage(ToggleButtonUI.SELECTED_ICON_IMAGE) != null) {
            String selectedIconUri = buttonUI.imageManager.getImage(ToggleButtonUI.SELECTED_ICON_IMAGE) == null 
                        ? null : buttonUI.imageManager.getImageUri(rc.getConnection(), ToggleButtonUI.SELECTED_ICON_IMAGE);
            String rolloverSelectedIconUri 
                        = buttonUI.imageManager.getImage(ToggleButtonUI.ROLLOVER_SELECTED_ICON_IMAGE) == null 
                        ? null 
                        : buttonUI.imageManager.getImageUri(rc.getConnection(), ToggleButtonUI.ROLLOVER_SELECTED_ICON_IMAGE);

            String stateId = ((ToggleButtonUI) buttonUI).getStateId().toString();
            
            scriptRolloverEnter = ToggleButtonUI.getScriptCustomStateIconToggleRollover(rc.getConnection(), id, stateId,
                    iconUri == null ? null : rolloverIconUri, rolloverSelectedIconUri, tableRolloverStyleName);
            scriptRolloverExit = ToggleButtonUI.getScriptCustomStateIconToggleRollover(rc.getConnection(), id, stateId,
                    rolloverIconUri == null ? null : iconUri, selectedIconUri, tableRolloverStyleName == null ? null : rootStyleName);
        } else {
            RolloverScript.MultipleRolloverScriptElement enterTableRolloverScriptElement
                    = new RolloverScript.MultipleRolloverScriptElement(id, 
                    iconUri == null ? null : rolloverIconUri, tableRolloverStyleName); 
            RolloverScript.MultipleRolloverScriptElement exitTableRolloverScriptElement
                    = new RolloverScript.MultipleRolloverScriptElement(id, 
                    rolloverIconUri == null ? null : iconUri, tableRolloverStyleName == null ? null : rootStyleName);
            if (tdRolloverStyleName != null) {
                RolloverScript.MultipleRolloverScriptElement enterTdRolloverScriptElement
                        = new RolloverScript.MultipleRolloverScriptElement(id + "_border", 
                        iconUri == null ? null : rolloverIconUri, tdRolloverStyleName);
                RolloverScript.MultipleRolloverScriptElement exitTdRolloverScriptElement
                        = new RolloverScript.MultipleRolloverScriptElement(id + "_border", 
                        rolloverIconUri == null ? null : iconUri, borderTdStyleName);
                scriptRolloverEnter = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                        new RolloverScript.MultipleRolloverScriptElement[]{
                        enterTableRolloverScriptElement, enterTdRolloverScriptElement});
                scriptRolloverExit = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                        new RolloverScript.MultipleRolloverScriptElement[]{
                        exitTableRolloverScriptElement, exitTdRolloverScriptElement});
            } else {
                scriptRolloverEnter = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                        new RolloverScript.MultipleRolloverScriptElement[]{enterTableRolloverScriptElement});
                scriptRolloverExit = RolloverScript.getScriptMultiRollover(rc.getConnection(), 
                        new RolloverScript.MultipleRolloverScriptElement[]{exitTableRolloverScriptElement});
            }
        }

        rootElement.addAttribute(ElementNames.Attributes.ONMOUSEOVER, scriptRolloverEnter);
        rootElement.addAttribute(ElementNames.Attributes.ONMOUSEOUT, scriptRolloverExit);        
    }
    
    /**
     * Returns the element id of the border table cell's TD.
     * 
     * @return The element id.
     */
    private String getBorderTableCellId() {
        return ClientObjects.COMPONENT_ID_PREFIX + buttonUI.getId().toString() + "_border_0";
    }

    /**
     * Returns the next sequential id available to assign to an elemnet
     * participating in the rollover effect.
     * 
     * @return The next available id.
     */
    private String getNextRolloverElementId() {
        return ClientObjects.COMPONENT_ID_PREFIX + buttonUI.getId().toString() + "_" + nextRolloverSubIdValue++;
    }

    /**
     * Returns the rendered root element.
     * 
     * @return The rendered root element.
     */
    private Element getRoot() {
        return rootElement;
    }
}
