/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.EchoConstants;
import nextapp.echo.Column;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.ExtentRenderer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/**
 * A peer for <code>Column</code> components.
 */
public class ColumnUI extends ComponentPeer 
implements PropertyChangeListener {
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        Column column = (Column) getComponent();
        
        // Create style for table.
        String tableStyleName;
        if (column.getWidthExtent() != null || column.getHeightExtent() != null) {
            ComponentStyle tableStyle = new ComponentStyle();
            if (column.getWidthExtent() != null) {
                tableStyle.addAttribute(ElementNames.CssAttributes.WIDTH, ExtentRenderer.render(column.getWidthExtent()));
            }
            if (column.getHeightExtent() != null) {
                tableStyle.addAttribute(ElementNames.CssAttributes.HEIGHT, ExtentRenderer.render(column.getHeightExtent()));
            }
            tableStyle.addElementType(ElementNames.TABLE);
            tableStyleName = rc.getDocument().addStyle(tableStyle);
        } else {
            tableStyleName = null;
        }
        
        // Create style for content cells.
        ComponentStyle cellStyle = ComponentStyle.forComponent(this, 
                ComponentStyle.EXPLICIT_SET_FOREGROUND | ComponentStyle.EXPLICIT_SET_FONT);
        if (column.getCellHeight() != null) {
            cellStyle.addAttribute(ElementNames.CssAttributes.HEIGHT, ExtentRenderer.render(column.getCellHeight()));
        }
        cellStyle.addElementType(ElementNames.TD);
        String cellStyleName = rc.getDocument().addStyle(cellStyle);
        
        // Create style for spacing cells.
        String spacingCellStyleName;
        if (column.getCellSpacing() != null && column.getCellSpacing().getValue() > 0) {
            ComponentStyle spacingCellStyle = new ComponentStyle();
            spacingCellStyle.addElementType(ElementNames.TD);
            spacingCellStyle.addAttribute(ElementNames.CssAttributes.HEIGHT, ExtentRenderer.render(column.getCellSpacing()));
            spacingCellStyleName = rc.getDocument().addStyle(spacingCellStyle);
        } else {
            spacingCellStyleName = null;
        }
        
        
        // Determine cell alignment (cannot be done via css due to browser misrendering).
        String horizontalAlignmentValue;
        switch (column.getHorizontalAlignment()) {
        case EchoConstants.LEFT:    horizontalAlignmentValue = "left";    break;
        case EchoConstants.CENTER:  horizontalAlignmentValue = "center";  break;
        case EchoConstants.RIGHT:   horizontalAlignmentValue = "right";   break;
        default:                    horizontalAlignmentValue = null;
        }
        String verticalAlignmentValue;
        switch (column.getVerticalAlignment()) {
        case EchoConstants.TOP:     verticalAlignmentValue = "top";    break;
        case EchoConstants.CENTER:  verticalAlignmentValue = "middle";  break;
        case EchoConstants.BOTTOM:  verticalAlignmentValue = "bottom";   break;
        default:                    verticalAlignmentValue = null;
        }
        
        // Render.
        Element table = new Element(ElementNames.TABLE);
        if (tableStyleName != null) {
            table.addAttribute(ElementNames.Attributes.CLASS, tableStyleName);
        }
        table.addAttribute(ElementNames.Attributes.BORDER, 0);
        table.addAttribute(ElementNames.Attributes.CELLPADDING, 0);
        table.addAttribute(ElementNames.Attributes.CELLSPACING, 0);
        
        ComponentPeer[] children = getChildren();
        for (int index = 0; index < children.length; ++index) {
            Element tr = new Element(ElementNames.TR);
            table.add(tr);

            Element td = new Element(ElementNames.TD);
            td.setWhitespaceRelevant(true);
            td.addAttribute(ElementNames.Attributes.CLASS, cellStyleName);

            if (horizontalAlignmentValue != null) {
                td.addAttribute("align", horizontalAlignmentValue);
            }
            if (verticalAlignmentValue != null) {
                td.addAttribute("valign", verticalAlignmentValue);
            }
            
            rc.getDocument().setCursorOnNewLine(true);
            
            children[index].render(rc, td);
            tr.add(td);
            
            if (spacingCellStyleName != null && index < children.length - 1) {
                tr = new Element(ElementNames.TR);
                table.add(tr);
                
                Element spacingTd = new Element(ElementNames.TD);
                spacingTd.addAttribute(ElementNames.Attributes.CLASS, spacingCellStyleName);
                tr.add(spacingTd);
            }
        }
        
        parent.add(table);
        rc.getDocument().setCursorOnNewLine(true);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
