/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.Grid;
import nextapp.echo.event.ImageUpdateEvent;
import nextapp.echo.event.ImageUpdateListener;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.Connection;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.image.ImageManager;

/**
 * A peer for <code>Grid.Cell</code> components.
 */
public class GridCellUI extends ComponentPeer 
implements ImageUpdateListener, PropertyChangeListener {

    private static final String BACKGROUND_IMAGE = "background";
    private ImageManager imageManager = null;

    /**
     * Returns the URI of the background image of this grid cell or null if it
     * has no background image.
     *
     * @param conn The rendering connection.
     * @return The URI of the background image.
     */
    String getBackgroundImageUri(Connection conn) {
        if (imageManager == null) {
            return null;
        } else {
            return imageManager.getImageUri(conn, BACKGROUND_IMAGE);
        }
    }

    /**
     * @see nextapp.echo.event.ImageUpdateListener#imageUpdate(ImageUpdateEvent)
     */
    public void imageUpdate(ImageUpdateEvent e) {
        redraw();
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();

        if (Grid.Cell.BACKGROUND_IMAGE_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            Grid.Cell cell = (Grid.Cell) getComponent();
            imageManager.setImage(BACKGROUND_IMAGE, cell.getBackgroundImage());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        Grid.Cell cell = (Grid.Cell) getComponent();
        cell.addPropertyChangeListener(this);
        if (cell.getBackgroundImage() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(BACKGROUND_IMAGE, cell.getBackgroundImage());
        }
    }   
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        ComponentPeer[] children = getChildren();
        rc.getDocument().setCursorOnNewLine(true);
        
        for (int index = 0; index < children.length; ++index) {
            children[index].render(rc, parent);
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
        if (imageManager != null) {
            imageManager.setImage(BACKGROUND_IMAGE, null);
        }
    }
}
