/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.io.Serializable;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import nextapp.echo.Coordinate;
import nextapp.echo.Dimension;
import nextapp.echo.EchoConstants;
import nextapp.echo.Grid;

/**
 * A utility class used when rendering <code>Grid</code> components.
 */
final class GridRenderingModel 
implements Serializable {

    private static final FlowOrderComparator FLOW_ORDER_COMPARATOR = new FlowOrderComparator();

    public static final int CELL_TYPE_EMPTY          = 0;
    public static final int CELL_TYPE_ORIGIN         = 1;
    public static final int CELL_TYPE_POINTER        = 2;
    
    /**
     * A comparator used to determine the order in which cells will 
     * &quot;flow&quot; within a grid to consume empty space.
     */
    private static final class FlowOrderComparator
    implements Comparator, Serializable {

        /**
         * @see java.util.Comparator#compare(Object, Object)
         */
        public int compare(Object o1, Object o2) {
            int comparisonValue;
            
            if (o1.equals(o2)) {
                comparisonValue = 0;
            } else {
                comparisonValue =  (((Grid.Cell) o2).getFlowPriority() - ((Grid.Cell) o1).getFlowPriority()) * 0x10;
                if (comparisonValue == 0) {
                    comparisonValue = 1;
                }
            } 
            
            return comparisonValue;
        }
    }
    
    /**
     * An object used to describe the origin of (top-left coordinate) of a
     * grid cell.
     */
    private static final class OriginCell 
    implements Serializable {

        private Grid.Cell cell;
        
        private int columnSpan = -1;
        private int rowSpan = -1;
        private PointerCell pointerCell = null;
        
        /**
         * Creates a new <code>OriginCell</code> to represent a 
         * <code>Grid.Cell</code> during rendering.
         */
        public OriginCell(Grid.Cell cell) {
            super();
            
            this.cell = cell;
            this.columnSpan = cell.getColumnSpan();
            this.rowSpan = cell.getRowSpan();
        }
        
        /**
         * Returns the <code>Grid.Cell</code> object that this 
         * <code>OriginCell</code> is representing during rendering.
         * 
         * @return The represented <code>Grid.Cell</code>.
         */
        public Grid.Cell getCell() {
            return cell;
        }
        
        /**
         * Returns the number of columns spanned by the cell.
         * 
         * @return The number of columns spanned by the cell.
         */
        public int getColumnSpan() {
            return columnSpan;
        }
        
        /**
         * Returns the <code>PointerCell</code> that will be used in the
         * <code>cellArray</code> grid to indicate which coordinates are
         * occupied by this occupied by this cells row- and column-spanning.
         * 
         * @return The <code>PointerCell</code> for this cell.
         */
        public PointerCell getPointerCell() {
            if (pointerCell == null) {
                pointerCell = new PointerCell(this);
            }
            
            return pointerCell;
        }
        
        /**
         * Returns the number of rows spanned by the cell.
         * 
         * @return The number of rows spanned by the cell.
         */
        public int getRowSpan() {
            return rowSpan;
        }
        
        /**
         * Sets the number of columns spanned by the cell.
         * 
         * @param columnSpan The number of columns spanned by the cell.
         */
        public void setColumnSpan(int columnSpan) {
            this.columnSpan = columnSpan;
        }
        
        /**
         * Sets the number of rows spanned by the cell.
         * 
         * @param rowSpan The number of rows spanned by the cell.
         */
        public void setRowSpan(int rowSpan) {
            this.rowSpan = rowSpan;
        }
    }
    
    /**
     * An object used to describe a grid coordinate that is occupied
     * by a multi-row and/or multi-column cell, but is not the origin
     * (top-left cell) of that cell.  A single instance of this object
     * will be used for each origin cell.  This single instance will
     * appear in every location the origin cell occupies except its 
     * origin.
     */
    private static final class PointerCell 
    implements Serializable {

        private OriginCell originCell;
     
        /**
         * Creates a new <code>PointerCell</code> to represent grid
         * coordinates occupied by a given <code>OriginCell</code>.
         */
        public PointerCell(OriginCell originCell) {
            super();
            
            this.originCell = originCell;
        }
        
        /**
         * Returns the represented <code>OriginCell</code>.
         * 
         * @return The represented <code>OriginCell</code>.
         */
        public OriginCell getOriginCell() {
            return originCell;
        }
    }

    private Object[][] cellArray;
    private boolean[] visibleRows;
    private boolean[] visibleColumns;
    private int[] rowMapping;
    private int[] rowHeights;
    private int[] columnMapping;
    private int[] columnWidths;
    private int columnCount;
    private int rowCount;
    private Grid grid;
    private boolean columnWidthsUsePercentUnits;
    private boolean rowHeightsUsePercentUnits;
  
    /**
     * Creates a <code>GridRenderingModel</code> for the specified <code>Grid</code>.
     * 
     * @param grid The <code>Grid</code> to be rendered.
     */
    public GridRenderingModel(Grid grid) {
        super();
        
        this.grid = grid;
        
        columnWidthsUsePercentUnits = grid.getWidthUnits() == Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS ||  
                grid.getWidthUnits() == Grid.PERCENT_UNITS;
        rowHeightsUsePercentUnits = grid.getHeightUnits() == Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS ||  
                grid.getHeightUnits() == Grid.PERCENT_UNITS;

        Grid.Cell cell;
        Iterator cellIterator;
        Dimension size = grid.getSize();
        cellArray = new Object[size.getHeight()][size.getWidth()];

        if (size.getHeight() > 0) {
            cellIterator = grid.iterator();
            while(cellIterator.hasNext()) {
                cell = (Grid.Cell) cellIterator.next();
                if (cell.isVisible()) {
                    OriginCell originCell = new OriginCell(cell);
                    Coordinate position = grid.getCellPosition(cell);
                    int left = position.getX();
                    int top = position.getY();
                    if (cell.getColumnSpan() > 1 || cell.getRowSpan() > 1) {
                        int right = cell.getColumnSpan() + left - 1;
                        int bottom = cell.getRowSpan() + top - 1;
                        PointerCell pointerCell = originCell.getPointerCell();
                        for (int row = top; row <= bottom; ++row) {
                            for (int column = left; column <= right; ++column) {
                                cellArray[row][column] = pointerCell;
                            }
                        }
                    }
                    cellArray[top][left] = originCell;
                }
            }

            // Flowfill cells
            SortedSet flowOrderSet = new TreeSet(FLOW_ORDER_COMPARATOR);
            cellIterator = grid.iterator();
            while(cellIterator.hasNext()) {
                cell = (Grid.Cell) cellIterator.next();
                if (cell.isVisible() && cell.getFlow() != 0) {
                    flowOrderSet.add(cell);
                }
            }
            Iterator it = flowOrderSet.iterator();
            while (it.hasNext()) {
                cell = (Grid.Cell) it.next();
                flowCell(cell);
            }

            // Row/Column reduction
            reduceRows();
            reduceColumns();
        }
    }

    /**
     * Grows cells that have a flow property set.
     *
     * @param cell The cell to flow.  This cell must be visible.
     */
    private void flowCell(Grid.Cell cell) {
        Coordinate position = grid.getCellPosition(cell);
        int flow = cell.getFlow();
        int row, column;
        int left = position.getX();
        int top = position.getY();
        int right = cell.getColumnSpan() + left - 1;
        int bottom = cell.getRowSpan() + top - 1;
        OriginCell originCell = (OriginCell) cellArray[top][left];

        boolean obstructionFound;
        boolean originChanged;

        // flow right
        if ((flow & Grid.Cell.FLOW_RIGHT) != 0) {
            obstructionFound = false;
            column = right + 1;
            while (column < cellArray[0].length && !obstructionFound) {
                row = top;
                while (row <= bottom && !obstructionFound) {
                    obstructionFound = cellArray[row][column] != null;
                    ++row;
                }
                if (!obstructionFound) {
                    for (row = top; row <= bottom; ++row) {
                        cellArray[row][column] = originCell.getPointerCell();
                    }
                    originCell.setColumnSpan(originCell.getColumnSpan() + 1);
                }
                ++column;
            }
            right = left + originCell.getColumnSpan() - 1;
        }

        // flow down
        if ((flow & Grid.Cell.FLOW_DOWN) != 0) {
            obstructionFound = false;
            row = bottom + 1;
            while (row < cellArray.length && !obstructionFound) {
                column = left;
                while (column <= right && !obstructionFound) {
                    obstructionFound = cellArray[row][column] != null;
                    ++column;
                }
                if (!obstructionFound) {
                    for (column = left; column <= right; ++column) {
                        cellArray[row][column] = originCell.getPointerCell();

                    }
                    originCell.setRowSpan(originCell.getRowSpan() + 1);
                }
                ++row;
            }
            bottom = top + originCell.getRowSpan() - 1;
        }

        // flow left
        if ((flow & Grid.Cell.FLOW_LEFT) != 0) {
            obstructionFound = false;
            originChanged = false;
            column = left - 1;
            while (column >= 0 && !obstructionFound) {
                row = top;
                while (row <= bottom && !obstructionFound) {
                    obstructionFound = cellArray[row][column] != null;
                    ++row;
                }
                if (!obstructionFound) {
                    for (row = top; row <= bottom; ++row) {
                        cellArray[row][column] = originCell.getPointerCell();
                    }
                    originCell.setColumnSpan(originCell.getColumnSpan() + 1);
                    originChanged = true;
                }
                --column;
            }
            if (originChanged) {
                cellArray[top][left] = originCell.getPointerCell();
                left = right - originCell.getColumnSpan() + 1;
                cellArray[top][left] = originCell;
            }
        }

        // flow up
        if ((flow & Grid.Cell.FLOW_UP) != 0) {
            obstructionFound = false;
            originChanged = false;
            row = top - 1;
            while (row >= 0 && !obstructionFound) {
                column = left;
                while (column <= right && !obstructionFound) {
                    obstructionFound = cellArray[row][column] != null;
                    ++column;
                }
                if (!obstructionFound) {
                    for (column = left; column <= right; ++column) {
                        cellArray[row][column] = originCell.getPointerCell();
                    }
                    originCell.setRowSpan(originCell.getRowSpan() + 1);
                    originChanged = true;
                }
                --row;
            }
            if (originChanged) {
                cellArray[top][left] = originCell.getPointerCell();
                top = bottom - originCell.getRowSpan() + 1;
                cellArray[top][left] = originCell;
            }
        }
    }

    /**
     * Returns the <code>Grid.Cell</code> at the specified coordinate.
     * The cells position and row and column spans may have changed if
     * the cell flow-filled up or to the left and/or rows and columns of
     * the grid were reduced.
     */
    public Grid.Cell getCell(int column, int row) {
        Grid.Cell cell;

        if (cellArray[rowMapping[row]][columnMapping[column]] instanceof OriginCell) {
            cell = ((OriginCell) cellArray[rowMapping[row]][columnMapping[column]]).getCell();
        } else {
            cell = ((PointerCell) cellArray[rowMapping[row]][columnMapping[column]]).getOriginCell().getCell();
        }

        return cell;
    }

    /**
     * Returns the cell type at the specified coordinate.
     * The cell type may be one of the following values:
     * <ul>
     * <li><code>CELL_TYPE_EMPTY</code> No cell exists at the coordinate, a 
     * filler cell should be rendered.</li>
     * <li><code>CELL_TYPE_ORIGIN</code> A cell's origin is at the specified
     * coordinate.  The cell should be rendered here.</li>
     * <li><code>CELL_TYPE_POINTER</code>A cell occupies this space, but this
     * is not the coordinate of its origin.  NOTHING should be rendered.</li>
     * </ul>
     */
    public int getCellType(int column, int row) { 
        int cellType;

        if (cellArray[rowMapping[row]][columnMapping[column]] == null) {
            cellType = CELL_TYPE_EMPTY;
        } else if (cellArray[rowMapping[row]][columnMapping[column]] instanceof OriginCell) {
            cellType = CELL_TYPE_ORIGIN;
        } else {
            cellType = CELL_TYPE_POINTER;
        }

        return cellType;
    }
        
    /**
     * Returns the height of the specified cell, or 
     * <code>EchoConstants.UNDEFINED_SIZE</code> if it cannot be determined.
     *
     * Will throw IllegalArgumentException if OriginCell does not
     * exist at specified coordinate.
     */
    public int getCellHeight(int column, int row) {
        OriginCell cell;
        try {
            cell = (OriginCell) cellArray[rowMapping[row]][columnMapping[column]];
        } catch (ClassCastException ex) {
            throw new IllegalArgumentException("Cell at specified coordinates is not an origin cell.");
        }
        
        int rowSpan = cell.getRowSpan();
        int height = 0;
        for (int index = row; index < rowSpan + row; ++index) {
            if (rowHeights[index] == EchoConstants.UNDEFINED_SIZE) {
                height = EchoConstants.UNDEFINED_SIZE;
                break;
            } else {
                height += rowHeights[index];
            }
        }
        
        if (height != EchoConstants.UNDEFINED_SIZE && !rowHeightsUsePercentUnits) {
            height += (rowSpan - 1) * (grid.getBorderSize() + grid.getCellMargin() * 2);
        }
            
        return height;
    }

    /**
     * Returns the width of the specified cell, or 
     * <code>EchoConstants.UNDEFINED_SIZE</code> if it cannot be determined.
     *
     * Will throw IllegalArgumentException if OriginCell does not
     * exist at specified coordinate.
     */
    public int getCellWidth(int column, int row) {
        OriginCell cell;
        try {
            cell = (OriginCell) cellArray[rowMapping[row]][columnMapping[column]];
        } catch (ClassCastException ex) {
            throw new IllegalArgumentException("Cell at specified coordinates is not an origin cell.");
        }
        
        int columnSpan = cell.getColumnSpan();
        int width = 0;
        for (int index = column; index < columnSpan + column; ++index) {
            if (columnWidths[index] == EchoConstants.UNDEFINED_SIZE) {
                width = EchoConstants.UNDEFINED_SIZE;
                break;
            } else {
                width += columnWidths[index];
            }
        }
        
        if (width != EchoConstants.UNDEFINED_SIZE && !columnWidthsUsePercentUnits) {
            width += (columnSpan - 1) * (grid.getBorderSize() + grid.getCellMargin() * 2);
        }
            
        return width;
    }

    /**
     * Returns the number of columns in the rendered version of the grid.
     */
    public int getColumnCount() {
        return columnCount;
    }
    
    /**
     * Returns the column span of the cell at the specified coordinates
     * in the rendered version of the grid.
     */
    public int getColumnSpan(int column, int row) {
        return ((OriginCell) cellArray[rowMapping[row]][columnMapping[column]]).getColumnSpan();
    }

    /**
     * Returns the rendered width of the given column.
     */
    public int getColumnWidth(int column) {
        return columnWidths[column]; 
    }

    /**
     * Returns the number of rows in the rendered version of the grid.
     */
    public int getRowCount() {
        return rowCount;
    }
    
    /**
     * Returns the rendered height of the given row.
     */
    public int getRowHeight(int row) {
        return rowHeights[row];
    }

    /**
     * Returns the row span of the cell at the specified coordinates
     * in the rendered version of the grid.
     */
    public int getRowSpan(int column, int row) {
        return ((OriginCell) cellArray[rowMapping[row]][columnMapping[column]]).getRowSpan();
    }

    /**
     * Removes empty columns that are spanned across in every row of the grid.
     * Having these columns present may confuse browsers.  The appearence of the
     * table is unchanged.
     */
    private void reduceColumns() {
        visibleColumns = new boolean[cellArray[0].length];
        columnWidths = new int[cellArray[0].length];
        columnMapping = new int[cellArray[0].length];
        int visibleColumn = 0;
        for (int column = 0; column < cellArray[0].length; ++column) {
            columnWidths[column] = 0; // guarantee by the time a column is reached its width will be initialized to 0.
            visibleColumns[column] = false;
            // Search column for any non-PointerCells and if found add it to the column mapping.
            for (int row = 0; row < cellArray.length && !visibleColumns[column]; ++row) {
                if (!(cellArray[row][column] instanceof PointerCell)) {
                    visibleColumns[column] = true;
                    columnMapping[visibleColumn] = column;
                }
            }
            if (!visibleColumns[column]) {
                // No non-PointerCells were found.  Reduce columnspans of all origin cells crossing
                // this column.
                OriginCell cell = null;
                for (int row = 0; row < cellArray.length; ++row) {
                    if (((PointerCell) cellArray[row][column]).getOriginCell() != cell) {
                        cell = ((PointerCell) cellArray[row][column]).getOriginCell();
                        cell.setColumnSpan(cell.getColumnSpan() - 1);
                    }
                }
            }
            
            // Calculate column width (Column widths may have increased due to reduced columns).
            if (columnWidths[visibleColumn] != EchoConstants.UNDEFINED_SIZE) {
                if (grid.getColumnWidth(column) == EchoConstants.UNDEFINED_SIZE) {
                    columnWidths[visibleColumn] = EchoConstants.UNDEFINED_SIZE;
                } else {
                    if (columnWidths[visibleColumn] == 0) {
                        columnWidths[visibleColumn] = grid.getColumnWidth(column);
                    } else {
                        columnWidths[visibleColumn] += grid.getColumnWidth(column);
                        if (!columnWidthsUsePercentUnits) {
                            columnWidths[visibleColumn] += grid.getBorderSize() + grid.getCellMargin() * 2;
                        }
                    }
                }
            }

            if (visibleColumns[column]) {
                ++visibleColumn;
            }
        }
        
        columnCount = visibleColumn;
    }

    /**
     * Removes empty rows that are spanned across in every column of the grid.
     * Having these rows present may confuse browsers.  The appearence of the
     * table is unchanged.
     */
    private void reduceRows() {
        visibleRows = new boolean[cellArray.length];
        rowHeights = new int[cellArray.length];
        rowMapping = new int[cellArray.length];
        int visibleRow = 0;
        for (int row = 0; row < cellArray.length; ++row) {
            rowHeights[row] = 0; // guarantee by the time a row is reached its height will be initialized to 0.
            visibleRows[row] = false;
            // Search row for any non-PointerCells and if found add it to the row mapping.
            for (int column = 0; column < cellArray[0].length && !visibleRows[row]; ++column) {
                if (!(cellArray[row][column] instanceof PointerCell)) {
                    visibleRows[row] = true;
                    rowMapping[visibleRow] = row;
                }
            }
            if (!visibleRows[row]) {
                // No non-PointerCells were found.  Reduce rowspans of all origin cells crossing
                // this row.
                OriginCell cell = null;
                for (int column = 0; column < cellArray[0].length; ++column) {
                    if (((PointerCell) cellArray[row][column]).getOriginCell() != cell) {
                        cell = ((PointerCell) cellArray[row][column]).getOriginCell();
                        cell.setRowSpan(cell.getRowSpan() - 1);
                    }
                }
            }

            // Calculate row height (Row heights may have increased due to reduced rows).
            if (rowHeights[visibleRow] != EchoConstants.UNDEFINED_SIZE) {
                if (grid.getRowHeight(row) == EchoConstants.UNDEFINED_SIZE) {
                    rowHeights[visibleRow] = EchoConstants.UNDEFINED_SIZE;
                } else {
                    if (rowHeights[visibleRow] == 0) {
                        rowHeights[visibleRow] = grid.getRowHeight(row);
                    } else {
                        rowHeights[visibleRow] += grid.getRowHeight(row);
                        if (!rowHeightsUsePercentUnits) {
                           rowHeights[visibleRow] += grid.getBorderSize() + grid.getCellMargin() * 2;
                        }
                    }
                }
            }

            if (visibleRows[row]) {
                ++visibleRow;
            }
        }
        
        rowCount = visibleRow;
    }
}
