/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.Color;
import nextapp.echo.Component;
import nextapp.echo.EchoConstants;
import nextapp.echo.EchoInstance;
import nextapp.echo.Grid;
import nextapp.echo.Insets;
import nextapp.echoservlet.ClientProperties;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/**
 * A peer for <code>Grid</code> components.
 */
public class GridUI extends ComponentPeer 
implements PropertyChangeListener {

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
        
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }   
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        Grid grid = (Grid) getComponent();
        Grid.Cell cell;
        GridCellUI cellUI;
        GridRenderingModel renderModel = new GridRenderingModel(grid);
        boolean borderTableRequired = grid.getBorderColor() != null && grid.getBorderSize() > 0;
        boolean columnWidthsUsePercentUnits = grid.getWidthUnits() == Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        grid.getWidthUnits() == Grid.PERCENT_UNITS;
        boolean rowHeightsUsePercentUnits = grid.getHeightUnits() == Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        grid.getHeightUnits() == Grid.PERCENT_UNITS;
        boolean someColumnsHaveWidths = false;
        boolean allColumnsHaveWidths = true;
        boolean allRowsHaveHeights = true;
        boolean cssAlignment 
               = getInstancePeer().getClientProperties().getBoolean(ClientProperties.CSS_TABLE_CELL_ALIGNMENT_SUPPORT);
        String emptyCellStyleName = null;
        
        // Render HTML
        Element table = new Element(ElementNames.TABLE);
        Element td;
        table.addAttribute("border", 0);
        table.addAttribute("cellpadding", grid.getCellMargin());
        table.addAttribute("cellspacing", grid.getBorderSize());
        
        int columnCount = renderModel.getColumnCount();
        int rowCount = renderModel.getRowCount();

        Element col;
        Element colGroup = new Element(ElementNames.COLGROUP);
        for (int column = 0; column < columnCount; ++column) {
            col = new Element(ElementNames.COL, false);
            if (renderModel.getColumnWidth(column) == EchoConstants.UNDEFINED_SIZE) {
                allColumnsHaveWidths = false;
            } else {
                if (columnWidthsUsePercentUnits) {
                    col.addAttribute(ElementNames.Attributes.WIDTH, renderModel.getColumnWidth(column) + "%");
                } else {
                    col.addAttribute(ElementNames.Attributes.WIDTH, renderModel.getColumnWidth(column));
                }
                someColumnsHaveWidths = true;
            }
            colGroup.add(col);
        }
        if (someColumnsHaveWidths) {
            table.add(colGroup);
        }
        
        // Determine default background color for cell that do not have a 
        // background, but due to the use of a bordering table, will be 
        // required to have a background color set in the CSS style sheet
        // in order to render properly.
        Color defaultCellBackground = grid.getBackground();
        if (defaultCellBackground == null && grid.getBorderColor() != null) {
            Component c = grid;
            while (defaultCellBackground == null && c != null) {
                defaultCellBackground = c.getBackground();
                c = c.getParent();
            }
            if (defaultCellBackground == null) {
                defaultCellBackground = EchoInstance.DEFAULT_BACKGROUND;
            }
        }
            
        String rowHeight;
        for (int row = 0; row < rowCount; ++row) {
            Element tr = new Element(ElementNames.TR);
            
            // Height calculation
            if (renderModel.getRowHeight(row) == EchoConstants.UNDEFINED_SIZE) {
                allRowsHaveHeights = false;
                rowHeight = null;
            } else {
                if (rowHeightsUsePercentUnits) {
                    rowHeight = renderModel.getRowHeight(row) + "%";
                } else {
                    rowHeight = Integer.toString(renderModel.getRowHeight(row));
                }
            }
            
            table.add(tr);
            
            for (int column = 0; column < columnCount; ++column) {
                ComponentStyle cellStyle;
                String cellStyleName;
                
                switch(renderModel.getCellType(column, row)) {
                case GridRenderingModel.CELL_TYPE_ORIGIN:
                    cell = renderModel.getCell(column, row);
                    cellUI = (GridCellUI) getInstancePeer().getPeer(cell);
                    
                    // Create Cell Element
                    td = new Element(ElementNames.TD);
                    td.setWhitespaceRelevant(true);
                    
                    // Create Style for Cell
                    cellStyle = ComponentStyle.forComponent(cellUI, 
                            ComponentStyle.EXPLICIT_SET_FOREGROUND | ComponentStyle.EXPLICIT_SET_FONT);
                    if (cell.getBackgroundImage() != null) {
                        // Cell has background image:
                        cellStyle.setBackgroundImage(cellUI.getBackgroundImageUri(rc.getConnection()));
                    } else if (cell.getBackground() == null && defaultCellBackground != null) {
                        // Cell has neither background image nor background color, but default cell background does exist:
                        // Set background color to default cell background to avoid border "showing through":
                        cellStyle.setBackground(defaultCellBackground);
                    }
                    
                    // Set cell line wrap policy. 
                    if (!cell.getLineWrap()) {
                        cellStyle.addAttribute("white-space", "nowrap");
                    }
                    
                    cellStyle.addElementType(ElementNames.TD);
                    
                    // Set cell horizontal alignment.
                    switch (cell.getHorizontalAlignment()) {
                    case EchoConstants.LEFT:
                        if (cssAlignment) {
                            cellStyle.addAttribute("text-align", "left");
                        } else {
                            td.addAttribute("align", "left");
                        }
                        break;
                    case EchoConstants.CENTER:
                        if (cssAlignment) {
                            cellStyle.addAttribute("text-align", "center");
                        } else {
                            td.addAttribute("align", "center");
                        }
                        break;
                    case EchoConstants.RIGHT:
                        if (cssAlignment) {
                            cellStyle.addAttribute("text-align", "right");
                        } else {
                            td.addAttribute("align", "right");
                        }
                        break;
                    }
                    
                    // Set cell vertical alignment.
                    switch (cell.getVerticalAlignment()) {
                    case EchoConstants.TOP:
                        if (cssAlignment) {
                            cellStyle.addAttribute("vertical-align", "top");
                        } else {
                            td.addAttribute("valign", "top");
                        }
                        break;
                    case EchoConstants.CENTER:
                        if (cssAlignment) {
                            cellStyle.addAttribute("vertical-align", "middle");
                        } else {
                            td.addAttribute("valign", "middle");
                        }
                        break;
                    case EchoConstants.BOTTOM:
                        if (cssAlignment) {
                            cellStyle.addAttribute("vertical-align", "bottom");
                        } else {
                            td.addAttribute("valign", "bottom");
                        }
                        break;
                    }
                    
                    // Set cell insets.
                    Insets insets = cell.getInsets();
                    if (insets != null) {
                        if (insets.getTop() == insets.getBottom() && insets.getLeft() == insets.getRight() 
                                && insets.getTop() == insets.getLeft()) {
                            cellStyle.addAttribute("padding", insets.getTop());
                        } else {
                            cellStyle.addAttribute("padding-left", insets.getLeft());
                            cellStyle.addAttribute("padding-top", insets.getTop());
                            cellStyle.addAttribute("padding-right", insets.getRight());
                            cellStyle.addAttribute("padding-bottom", insets.getBottom());
                        }
                    }
                    cellStyleName = rc.getDocument().addStyle(cellStyle);
                    
                    // Render Cell Tag Attributes
                    td.addAttribute(ElementNames.Attributes.CLASS, cellStyleName);
                    int rowSpan = renderModel.getRowSpan(column, row);
                    if (rowSpan > 1) {
                        td.addAttribute("rowspan", rowSpan);
                    }
                    int columnSpan = renderModel.getColumnSpan(column, row);
                    if (columnSpan > 1) {
                        td.addAttribute("colspan", columnSpan);
                    }
                    
                    int width = renderModel.getCellWidth(column, row);
                    if (width != EchoConstants.UNDEFINED_SIZE) {
                        if (columnWidthsUsePercentUnits) {
                            td.addAttribute(ElementNames.Attributes.WIDTH, width + "%");
                        } else {
                            td.addAttribute(ElementNames.Attributes.WIDTH, width);
                        }
                    }
                    if (rowHeight != null) {
                        td.addAttribute(ElementNames.Attributes.HEIGHT, rowHeight);
                    }
                    
                    // Render Cell Content
                    cellUI.render(rc, td);

                    tr.add(td);
                    break;
                    
                case GridRenderingModel.CELL_TYPE_EMPTY:
                    if (emptyCellStyleName == null) {
                        // Create Style for empty cell, if necessary.
                        ComponentStyle emptyCellStyle = new ComponentStyle();
                        emptyCellStyle.setBackground(defaultCellBackground);
                        emptyCellStyle.addElementType(ElementNames.TD);
                        emptyCellStyleName = rc.getDocument().addStyle(emptyCellStyle);
                    }

                    td = new Element(ElementNames.TD);
                    td.setWhitespaceRelevant(true);
                    td.addAttribute(ElementNames.Attributes.CLASS, emptyCellStyleName);
                    if (rowHeight != null) {
                        td.addAttribute(ElementNames.Attributes.HEIGHT, rowHeight);
                    }
                    
                    tr.add(td);
                    break;
                }
            }
        }

        if (grid.getWidth() > 0) {
            switch (grid.getWidthUnits()) {
            case Grid.PIXEL_UNITS:
                // If all columns have widths, they override total width.
                if (! allColumnsHaveWidths) {
                    table.addAttribute(ElementNames.Attributes.WIDTH, grid.getWidth());
                }
                break;
            case Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS:
                table.addAttribute(ElementNames.Attributes.WIDTH, grid.getWidth());
                break;
            case Grid.PERCENT_UNITS:
                if (borderTableRequired) {
                    table.addAttribute(ElementNames.Attributes.WIDTH, "100%");
                } else {
                    table.addAttribute(ElementNames.Attributes.WIDTH, grid.getWidth() + "%");
                }
                break;
            }
        }
        if (grid.getHeight() > 0) {
            switch (grid.getHeightUnits()) {
            case Grid.PIXEL_UNITS:
                // If all columns have widths, they override total width.
                if (! allRowsHaveHeights) {
                    table.addAttribute(ElementNames.Attributes.HEIGHT, grid.getHeight());
                }
                break;
            case Grid.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS:
                table.addAttribute(ElementNames.Attributes.HEIGHT, grid.getHeight());
                break;
            case Grid.PERCENT_UNITS:
                if (borderTableRequired) {
                    table.addAttribute(ElementNames.Attributes.HEIGHT, "100%");
                } else {
                    table.addAttribute(ElementNames.Attributes.HEIGHT, grid.getHeight() + "%");
                }
                break;
            }
        }
        
        if (borderTableRequired) {
            // Embed table inside another table cell with a background color of the border color.

            ComponentStyle borderStyle = new ComponentStyle();
            borderStyle.setBackground(grid.getBorderColor());
            borderStyle.addElementType(ElementNames.TD);
            String borderStyleName = rc.getDocument().addStyle(borderStyle);

            Element borderTable = new Element(ElementNames.TABLE);
            borderTable.setWhitespaceRelevant(true);
            borderTable.addAttribute("border", 0);
            borderTable.addAttribute("cellpadding", 0);
            borderTable.addAttribute("cellspacing", 0);
            if (grid.getWidthUnits() == Grid.PERCENT_UNITS && grid.getWidth() > 0) {
                borderTable.addAttribute(ElementNames.Attributes.WIDTH, grid.getWidth() + "%");
            }
            if (grid.getHeightUnits() == Grid.PERCENT_UNITS && grid.getHeight() > 0) {
                borderTable.addAttribute(ElementNames.Attributes.HEIGHT, grid.getHeight() + "%");
            }
            
            Element borderTr = new Element(ElementNames.TR);
            borderTr.setWhitespaceRelevant(true);
            
            Element borderTd = new Element(ElementNames.TD);
            borderTd.setWhitespaceRelevant(true);
            borderTd.addAttribute(ElementNames.Attributes.CLASS, borderStyleName);
            
            borderTable.add(borderTr);
            borderTr.add(borderTd);
            borderTd.add(table);
            parent.add(borderTable);
        } else {
            parent.add(table);
        }
        
        rc.getDocument().setCursorOnNewLine(true);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
