/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.ImageReference;
import nextapp.echo.Label;
import nextapp.echo.event.ImageUpdateEvent;
import nextapp.echo.event.ImageUpdateListener;
import nextapp.echoservlet.Alignment;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.image.ImageManager;

/**
 * A peer for <code>Label</code> components.
 */
public class LabelUI extends ComponentPeer
implements Alignment, ImageUpdateListener, PropertyChangeListener {

    /** 
     * A string that will indentify the label's icon in the 
     * <code>ImageManager</code>.
     */
    private static final String LABEL_IMAGE = "label";

    /**
     * An image manager that will handle the label's icon.
     * The image manager is lazy-created.
     */
    private ImageManager imageManager = null;

    /**
     * Creates the image <code>Element</code> that will reference the
     * label's image.
     *
     * @param rc The <code>RenderingContext</code> on which this label is
     *           being rendered.
     * @return An &lt;img&gt; element reference the label's image.
     */
    private Element createIconElement(RenderingContext rc) {
        ImageReference icon = ((Label) getComponent()).getIcon();
        Element img = new Element(ElementNames.IMG, false);
        
        img.addAttribute("src", imageManager.getImageUri(rc.getConnection(), LABEL_IMAGE));
        img.addAttribute("alt", "");
        img.addAttribute("border", 0);
        
        if (icon.getWidth() > 0) {
           img.addAttribute("width", icon.getWidth());
        }
        if (icon.getHeight() > 0) {
           img.addAttribute("height", icon.getHeight());
        }
        
        return img;
    }
    
    /**
     * @see nextapp.echoservlet.Alignment#getHorizontalAlignment()
     */
    public int getHorizontalAlignment() {
        return ((Label) getComponent()).getHorizontalAlignment();
    }
    
    /**
     * @see nextapp.echoservlet.Alignment#getVerticalAlignment()
     */
    public int getVerticalAlignment() {
        return ((Label) getComponent()).getVerticalAlignment();
    }
    
    /**
     * @see nextapp.echo.event.ImageUpdateListener#imageUpdate(ImageUpdateEvent)
     */
    public void imageUpdate(ImageUpdateEvent e) {
        redraw();
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
        
        if (Label.ICON_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            Label label = (Label) getComponent();
            imageManager.setImage(LABEL_IMAGE, label.getIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        Label label = (Label) getComponent();
        label.addPropertyChangeListener(this);
        if (label.getIcon() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(LABEL_IMAGE, label.getIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        Label label = (Label) getComponent();
    
        if (label.getText() != null) {
            // has icon
            if (label.getIcon() != null) {
                // has icon and text
                
                // Will be contained in table cell, style should explicitly set foreground, font, to avoid inheriting
                // defaults from table cell.
                ComponentStyle style = ComponentStyle.forComponent(this, 
                        ComponentStyle.EXPLICIT_SET_FOREGROUND | ComponentStyle.EXPLICIT_SET_FONT);
                
                style.addElementType(ElementNames.TD);
                String styleName = rc.getDocument().addStyle(style);
                TriCellTable tct = new TriCellTable(rc.getConnection(), 
                        TriCellTable.getOrientation(label.getHorizontalTextPosition(), label.getVerticalTextPosition()),
                        label.getIconTextMargin());
                tct.getTdElement(0).addText(label.getText());
                tct.getTdElement(0).addAttribute(ElementNames.Attributes.CLASS, styleName);
                tct.getTdElement(1).add(createIconElement(rc));
                tct.getTdElement(1).addAttribute(ElementNames.Attributes.CLASS, styleName);
                if (tct.getMarginTdElement(0) != null) {
                    tct.getMarginTdElement(0).addAttribute(ElementNames.Attributes.CLASS, styleName);
                }
                parent.add(tct);
                rc.getDocument().setCursorOnNewLine(true);
            } else {
                // has text only

                // Don't explicitly set foreground/font.
                ComponentStyle style = ComponentStyle.forComponent(this,0); 
                
                if (!style.hasAttributes()) {
                    // Style is empty or Parent element style is identical to necessary style:
                    // DO NOT enclose element in SPAN tag.
                    parent.addText(label.getText());
                    rc.getDocument().setCursorOnNewLine(false);
                } else {
                    // Parent element style is not identical to necessary style:
                    // Enclose Element in SPAN tag.
                    String styleName = rc.getDocument().addStyle(style);
                    style.addElementType(ElementNames.SPAN);
                    Element span = new Element(ElementNames.SPAN);
                    span.setWhitespaceRelevant(true);
                    span.addAttribute(ElementNames.Attributes.CLASS, styleName);
                    span.addText(label.getText());
                    parent.add(span);
                    rc.getDocument().setCursorOnNewLine(false);
                }
            }
        } else if (label.getIcon() != null) {
            // has icon only
            parent.add(createIconElement(rc));
            rc.getDocument().setCursorOnNewLine(false);
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
        if (imageManager != null) {
            imageManager.setImage(LABEL_IMAGE, null);
        }
    }
}
