/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.StringTokenizer;

import nextapp.echo.ListBox;
import nextapp.echo.ListCellRenderer;
import nextapp.echo.ListModel;
import nextapp.echo.ListSelectionModel;

import nextapp.echoservlet.ClientInputProducer;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Html;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A peer for <code>ListBox</code> components.
 */
public class ListBoxUI extends ComponentPeer 
implements ClientInputProducer, PropertyChangeListener {

    /**
     * A service that will provide the JavaScript used to manage a list box's
     * state.
     */
    private static final Service SERVICE_LIST_BOX_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "listbox", ResourceNames.LIST_BOX_SCRIPT);
    
    static {
        EchoServer.addGlobalService(SERVICE_LIST_BOX_SCRIPT);
    }

    /**
     * Indicates whether the component was most recently rendered editable 
     * and enabled.
     */
    protected boolean renderedActive = true;

    /**
     * @see nextapp.echoservlet.ClientInputProducer#clientInput(String)
     */
    public void clientInput(String input) {
        if (renderedActive) {
            ListBox listBox = (ListBox) getComponent();
            StringTokenizer st = new StringTokenizer(input, ",");
            int size = listBox.getModel().size();
            int nextSelectedIndex = -1;
            for (int index = 0; index < size; ++index) {
                if (nextSelectedIndex == -1 && st.hasMoreTokens()) {
                    nextSelectedIndex = Integer.parseInt(st.nextToken());
                }
                
                if (index == nextSelectedIndex) {
                    nextSelectedIndex = -1;
                    listBox.setSelectedIndex(index, true);
                } else {
                    listBox.setSelectedIndex(index, false);
                }
            }
        }
    }

    /**
     * Returns the JavaScript method call that will be invoked when the state
     * of the list box is altered by the user.
     *
     * @return The JavaScript method call that will be invoked when the state
     *         of the list box is altered by the user.
     */
    private String getOnChangeScript() {
        StringBuffer sb = new StringBuffer(ClientObjects.STD_PREFIX);
        
        sb.append("listSelectionChanged(\'");
        sb.append(getId().toString());
        sb.append("\', this);");
        
        return sb.toString();
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        rc.getDocument().addScriptInclude(SERVICE_LIST_BOX_SCRIPT);

        ListBox listBox = (ListBox) getComponent();
        renderedActive = listBox.isEnabled();
        ListCellRenderer renderer = listBox.getCellRenderer();
        ListModel model = listBox.getModel();
        ListSelectionModel selectionModel = listBox.getSelectionModel();
        int size = model.size();
        Object item;

        ComponentStyle style = ComponentStyle.forComponent(this, ComponentStyle.EXPLICIT_SET_ALL);
        if (listBox.getWidth() > 0) {
            switch (listBox.getWidthUnits()) {
            case ListBox.PIXEL_UNITS:
                style.addAttribute(ElementNames.Attributes.WIDTH, listBox.getWidth(), ComponentStyle.PIXEL_UNITS);
                break;
            case ListBox.PERCENT_UNITS:
                style.addAttribute(ElementNames.Attributes.WIDTH, listBox.getWidth(), ComponentStyle.PERCENT_UNITS);
                break;
            }
        }
        style.addElementType(ElementNames.SELECT);
        String styleName = rc.getDocument().addStyle(style);

        Element select = new Element(ElementNames.SELECT);
        select.addAttribute(ElementNames.Attributes.ID, getId().toString());
        Element option;
        select.addAttribute(ElementNames.Attributes.CLASS, styleName);
        select.addAttribute("tabindex", getTabIndex());
        select.addAttribute("name", ClientObjects.getObjectName(getId()));
        select.addAttribute("size", listBox.getVisibleRowCount());
        select.addAttribute("onchange", getOnChangeScript());
        if (listBox.getSelectionMode() == ListSelectionModel.MULTIPLE_SELECTION) {
            select.addAttribute("multiple");
        }
        if (!listBox.isEnabled()) {
            select.addAttribute("disabled");
        }
        if (listBox.getToolTipText() != null) {
            select.addAttribute("title", Html.encode(listBox.getToolTipText(), Html.NEWLINE_TO_SPACE));
        }
        
        for (int index = 0; index < size; ++index) {
            option = new Element(ElementNames.OPTION);
            item = model.get(index);
            Object renderedValue = renderer.getListCellRendererComponent(listBox, item, index);
            option.addText(renderedValue.toString());
            if (renderedValue instanceof ListCellRenderer.StyledListCell) {
                ComponentStyle cellStyle = new ComponentStyle();
                cellStyle.setBackground(((ListCellRenderer.StyledListCell) renderedValue).getBackground());
                cellStyle.setFont(((ListCellRenderer.StyledListCell) renderedValue).getFont());
                cellStyle.setForeground(((ListCellRenderer.StyledListCell) renderedValue).getForeground());
                cellStyle.addElementType(ElementNames.OPTION);
                String cellStyleName = rc.getDocument().addStyle(cellStyle);
                option.addAttribute("class", cellStyleName);
            }
            option.setWhitespaceRelevant(true);
            option.addAttribute("value", index);
            if (selectionModel.isSelectedIndex(index)) {
                option.addAttribute("selected");
            }
            select.add(option);
        }
        
        parent.add(select);
        rc.getDocument().setCursorOnNewLine(false);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
