/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.util.Iterator;

import nextapp.echo.AbstractButton;
import nextapp.echo.ButtonGroup;
import nextapp.echo.RadioButton;
import nextapp.echo.ToggleButton;

import nextapp.echoservlet.ClientInputProducer;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.ControllerFieldInitialState;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.Id;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticBinary;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A peer for <code>RadioButton</code> components.
 */
public class RadioButtonUI extends ToggleButtonUI 
implements ClientInputProducer, ControllerFieldInitialState {

    /** A service that renders the default deselected radio button image. */
    public static final Service SERVICE_RADIO_BUTTON_OFF 
            = StaticBinary.createFromResource(ClientObjects.STD_PREFIX + "rb0", ResourceNames.RADIO_BUTTON_OFF);
    
    /** A service that renders the default selected radio button  image. */
    public static final Service SERVICE_RADIO_BUTTON_ON 
            = StaticBinary.createFromResource(ClientObjects.STD_PREFIX + "rb1", ResourceNames.RADIO_BUTTON_ON);
    
    static {
        EchoServer.addGlobalService(SERVICE_RADIO_BUTTON_OFF);
        EchoServer.addGlobalService(SERVICE_RADIO_BUTTON_ON);
    }

    /**
     * @see nextapp.echoservlet.ClientInputProducer#clientInput(String)
     */
    public void clientInput(String input) {
        // client input from a radio button does not necessarily indicate that THIS radio button was selected,
        // only that this button's Id was used as the groupId.
        Id id = new Id(input);
        RadioButtonUI radioButtonUI = (RadioButtonUI) getInstancePeer().getPeer(id);
        
        // Note: It is theoretically possible, but EXTREMELY unlikely, that the radio button that was selected was 
        // unregistered due to another event, so it must be ensured that the peer still exists.
        if (radioButtonUI != null && radioButtonUI.renderedActive) {
            RadioButton radioButton = (RadioButton) radioButtonUI.getComponent();
            radioButton.setSelected(true);
        }
    }
    
    /**
     * @see nextapp.echoservlet.ControllerFieldInitialState#getControllerFieldValue()
     */
    public String getControllerFieldValue() {
        if (getId().equals(getStateId())) {
            Id selectedId = getSelectedId();
            if (selectedId == null) {
                return null;
            } else {
                return selectedId.toString();
            }
        } else {
            return null;
        }
    }
    
    /**
     * @see nextapp.echoservlet.ui.ToggleButtonUI#getDefaultStateImageUri(RenderingContext, boolean)
     */
    public String getDefaultStateImageUri(RenderingContext rc, boolean selected) {
        return ClientObjects.getServiceUri(rc.getConnection(), 
                selected ? SERVICE_RADIO_BUTTON_ON : SERVICE_RADIO_BUTTON_OFF);
    }
    
    /**
     * Returns the <code>Id</code> of the <code>ButtonGroup</code>.  The 
     * <code>Id</code> of the <code>ButtonGroup</code> is the
     * <code>Id</code> of the first visible <code>RadioButtonUI</code> 
     * in the group.
     *
     * @return The <code>Id</code> of the <code>ButtonGroup</code>. 
     */
    private Id getGroupId() {
        RadioButton radioButton = (RadioButton) getComponent();
        ToggleButton.ToggleButtonModel model = (ToggleButton.ToggleButtonModel) radioButton.getModel();
        ButtonGroup group = model.getGroup();
        Id id = null;

        if (group != null) {
            Iterator it = group.getElements();
            while (id == null && it.hasNext()) {
                AbstractButton groupButton = (AbstractButton) it.next();
                if (groupButton.isShowing()) {
                    id = getInstancePeer().getPeer(groupButton).getId();
                }
            }
        }
        
        return id;
    }
    
    /**
     * @see nextapp.echoservlet.ui.AbstractButtonUI#getScriptAction(RenderingContext)
     */
    public String getScriptAction(RenderingContext rc) {
        RadioButton radioButton = (RadioButton) getComponent();

        if (radioButton.isEnabled()) {
            rc.getDocument().addScriptInclude(SERVICE_TOGGLE_BUTTON_SCRIPT);

            StringBuffer sb = new StringBuffer(ClientObjects.STD_PREFIX + "radioButtonAction(\'");

            Id selectedId = getSelectedId();   
            Id groupId;

            if (selectedId == null) {
                groupId = getGroupId();
            } else {
                groupId = selectedId;
            }

            sb.append(getId().toString());
            sb.append("\', ");
            if (groupId == null) {
                sb.append("null, ");
            } else {
                sb.append("\'");
                sb.append(groupId.toString());
                sb.append("\', ");
            }
            if (selectedId == null) {
                sb.append("null, ");
            } else {
                sb.append("\'");
                sb.append(selectedId.toString());
                sb.append("\', ");
            }
            sb.append(radioButton.hasActionListeners() ? "true, \'" : "false, \'");
            sb.append(getStateImageUri(rc, false));
            sb.append("\', \'");
            sb.append(getStateImageUri(rc, true));
            sb.append("\');");

            return sb.toString();
        } else {
            return ";";
        }
    }
    
    /**
     * Returns the <code>Id</code> of the selected button in the 
     * <code>ButtonGroup</code>.
     *
     * @return The <code>Id</code> of the selected button in the 
     *         <code>ButtonGroup</code>.
     */
    private Id getSelectedId() {
        RadioButton radioButton = (RadioButton) getComponent();
        ToggleButton.ToggleButtonModel model = (ToggleButton.ToggleButtonModel) radioButton.getModel();
        ButtonGroup group = model.getGroup();
        Id id = null;

        if (group == null) {
            id = getId();
        } else {
            AbstractButton groupButton;
            AbstractButtonUI ui;
            
            Iterator it = group.getElements();
            while (id == null && it.hasNext()) {
                groupButton = (AbstractButton) it.next();
                if (groupButton.isShowing()) {
                    ui = (AbstractButtonUI) getInstancePeer().getPeer(groupButton);
                    if (groupButton.isSelected()) {
                        id = ui.getId();
                    }
                }
            }
        }
        
        return id;
    }

    /**
     * @see nextapp.echoservlet.ui.ToggleButtonUI#getStateId()
     */
    public Id getStateId() {
        Id stateId = getSelectedId();   
        if (stateId == null) {
            stateId = getGroupId();
        } 
        return stateId;
    }
}
