/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.Connection;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A utility class that wraps JavaScript code required to create mouse rollover
 * effects in HTML pages on the client without server interaction.
 */
public class RolloverScript {
    
    /**
     * A data object used to describe a rollover action when multiple actions
     * are required for a rollover effect.
     */
    public static class MultipleRolloverScriptElement {
        
        private String id, imageUri, styleName; 

        /**
         * Creates a new <code>MultipleRolloverScriptElement</code>.
         * 
         * @param id The Id of the button that will be given the rollover
         *        effect.
         * @param imageUri The URI of the replacement image.
         * @param styleName The URI of the replacement style.
         */
        public MultipleRolloverScriptElement(String id, String imageUri, String styleName) {
            super();
            this.id = id; 
            this.imageUri = imageUri; 
            this.styleName = styleName;
        }
    }

    /**
     * A service containing JavaScript code used to create rollover effects.
     */
    public static final Service SERVICE_ROLLOVER_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "rlov", ResourceNames.ROLLOVER_SCRIPT);
    
    static {
        EchoServer.addGlobalService(SERVICE_ROLLOVER_SCRIPT);
    }

    /**
     * Returns the necessary script to produce a button rollover effect.
     *
     * @param conn The connection whose response the generated script will be
     *             written to.
     * @param id The Id of the button that will be given the rollover 
     *        effect.
     * @param imageUri The URI of the replacement image.
     * @param styleName The URI of the replacement style.
     * @return A JavaScript method call that should be invoked in the rollover
     *         event listener.
     */
    public static String getScriptRollover(Connection conn, String id, String imageUri, String styleName) {
        StringBuffer sb = new StringBuffer(ClientObjects.STD_PREFIX);
        
        sb.append("rollover(\'");
        sb.append(id);
        sb.append("\', ");
        if (imageUri == null) {
            sb.append("null, ");
        } else {
            sb.append("\'");
            sb.append(imageUri);
            sb.append("\', ");
        }
        if (styleName == null) {
            sb.append("null);");
        } else {
            sb.append("\'");
            sb.append(styleName);
            sb.append("\');");
        }
        
        return sb.toString();
    }
    
    /**
     * Returns the necessary script to produce a button rollover effect where
     * multiple elements will be modified.
     *
     * @param conn The connection whose response the generated script will be
     *        written to.
     * @param elements An array of <code>MultipleRolloverScriptElement</code>s
     *        describing the rollover action.
     */
    public static String getScriptMultiRollover(Connection conn, MultipleRolloverScriptElement[] elements) {
        int count = elements.length;
        StringBuffer sb = new StringBuffer(ClientObjects.STD_PREFIX);
        sb.append("rollover(");
        
        for (int index = 0; index < count; ++index) {
            sb.append("\'");
            sb.append(elements[index].id);
            sb.append("\', ");
            if (elements[index].imageUri == null) {
                sb.append("null, ");
            } else {
                sb.append("\'");
                sb.append(elements[index].imageUri);
                sb.append("\', ");
            }
            if (elements[index].styleName == null) {
                sb.append("null");
            } else {
                sb.append("\'");
                sb.append(elements[index].styleName);
                sb.append("\'");
            }
            if (index < count - 1) {
                sb.append(", ");
            }
        }
        sb.append(");");
        
        return sb.toString();
    }
    
    /** Non-instantiable class */
    private RolloverScript() { }
}
