/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.ListCellRenderer;
import nextapp.echo.SelectField;
import nextapp.echo.SelectFieldModel;

import nextapp.echoservlet.ClientActionProducer;
import nextapp.echoservlet.ClientInputProducer;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Html;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A peer for <code>SelectField</code> components.
 */
public class SelectFieldUI extends ComponentPeer 
implements ClientActionProducer, ClientInputProducer, PropertyChangeListener {

    private static final Service SERVICE_SELECT_FIELD_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "selfd", ResourceNames.SELECT_FIELD_SCRIPT);
    
    static {
        EchoServer.addGlobalService(SERVICE_SELECT_FIELD_SCRIPT);
    }
    
    /**
     * Indicates whether the component was most recently rendered editable 
     * and enabled.
     */
    protected boolean renderedActive = true;

    /**
     * @see nextapp.echoservlet.ClientActionProducer#clientAction(String)
     */
    public void clientAction(String command) {
        if (renderedActive) {
            SelectField selectField = (SelectField) getComponent();
            selectField.fireActionEvent();
        }
    }

    /**
     * @see nextapp.echoservlet.ClientInputProducer#clientInput(String)
     */
    public void clientInput(String input) {
        if (renderedActive) {
            try {
                int index = Integer.parseInt(input);
                SelectField selectField = (SelectField) getComponent();
                if (index >=0 && index < selectField.getModel().size()) {
                    selectField.setSelectedIndex(index);
                }
            } catch (NumberFormatException ex) {
                // Do nothing.
            }
        }
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        rc.getDocument().addScriptInclude(SERVICE_SELECT_FIELD_SCRIPT);

        SelectField selectField = (SelectField) getComponent();
        renderedActive = selectField.isEnabled();
        ListCellRenderer renderer = selectField.getCellRenderer();
        SelectFieldModel model = selectField.getModel();
        int size = model.size();
        Object item;
        Object selectedItem = model.getSelectedItem();

        ComponentStyle style = ComponentStyle.forComponent(this, ComponentStyle.EXPLICIT_SET_ALL);
        if (selectField.getWidth() > 0) {
            switch (selectField.getWidthUnits()) {
            case SelectField.PIXEL_UNITS:
                style.addAttribute(ElementNames.Attributes.WIDTH, selectField.getWidth(), ComponentStyle.PIXEL_UNITS);
                break;
            case SelectField.PERCENT_UNITS:
                style.addAttribute(ElementNames.Attributes.WIDTH, selectField.getWidth(), ComponentStyle.PERCENT_UNITS);
                break;
            }
        }
        style.addElementType(ElementNames.SELECT);
        String styleName = rc.getDocument().addStyle(style);

        Element select = new Element(ElementNames.SELECT);
        select.addAttribute(ElementNames.Attributes.ID, getId().toString());
        
        Element option;
        select.addAttribute("class", styleName);
        select.addAttribute("tabindex", getTabIndex());
        select.addAttribute("name", ClientObjects.getObjectName(getId()));
        select.addAttribute("onchange", ClientObjects.STD_PREFIX + "selectFieldAction(\'" + getId() + "\', this.selectedIndex, " 
                + (selectField.hasActionListeners() ? "true);" : "false);"));
        if (selectField.getToolTipText() != null) {
            select.addAttribute("title", Html.encode(selectField.getToolTipText(), Html.NEWLINE_TO_SPACE));
        }
        
        for (int index = 0; index < size; ++index) {
            option = new Element(ElementNames.OPTION);
            item = model.get(index);
            Object renderedValue = renderer.getListCellRendererComponent(selectField, item, index);
            option.addText(renderedValue.toString());
            if (renderedValue instanceof ListCellRenderer.StyledListCell) {
                ComponentStyle cellStyle = new ComponentStyle();
                cellStyle.setBackground(((ListCellRenderer.StyledListCell) renderedValue).getBackground());
                cellStyle.setFont(((ListCellRenderer.StyledListCell) renderedValue).getFont());
                cellStyle.setForeground(((ListCellRenderer.StyledListCell) renderedValue).getForeground());
                cellStyle.addElementType(ElementNames.OPTION);
                String cellStyleName = rc.getDocument().addStyle(cellStyle);
                option.addAttribute("class", cellStyleName);
            }
            option.setWhitespaceRelevant(true);
            option.addAttribute("value", index);
            if (item.equals(selectedItem)) {
                option.addAttribute("selected");
            }
            select.add(option);
        }
        
        if (!selectField.isEnabled()) {
            select.addAttribute("disabled");
        }

        parent.add(select);
        rc.getDocument().setCursorOnNewLine(false);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
