/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.Component;
import nextapp.echo.EchoConstants;
import nextapp.echo.Table;
import nextapp.echo.table.TableColumnModel;
import nextapp.echo.table.TableHeader;
import nextapp.echoservlet.Alignment;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/** 
 * A peer for <code>TableHeader</code> components.
 * 
 * Rendering responsibilities for <code>TableHeader</code>s are handled by 
 * the <code>Table</code> component.
 */
public class TableHeaderUI extends ComponentPeer
implements PropertyChangeListener {

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#render(nextapp.echoservlet.RenderingContext, nextapp.echoservlet.html.Element)
     */
    public void render(RenderingContext rc, Element parent) {
        Element trElement = new Element(ElementNames.TR);
        TableHeader tableHeader = (TableHeader) getComponent();
        Table table = tableHeader.getTable();
        TableColumnModel columnModel = table.getColumnModel();
        int columnCount = table.getColumnCount();

        boolean columnWidthsUsePercentUnits = table.getWidthUnits() == Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        table.getWidthUnits() == Table.PERCENT_UNITS;
        boolean rowHeightsUsePercentUnits = table.getHeightUnits() == Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        table.getHeightUnits() == Table.PERCENT_UNITS;

        boolean renderHeader = false;
        for (int column = 0; column < columnCount; ++column) {
            Component cell = tableHeader.getRenderedComponent(column);
            if (cell != null) {
                renderHeader = true;
                break;
            }
        }
        if (!renderHeader) {
            return;
        }
        
        // Create a style for empty header cells.  A style must be set on empty cells to avoid the border color
        // "showing through".
        ComponentStyle emptyHeaderCellStyle = new ComponentStyle();
        emptyHeaderCellStyle.addElementType(ElementNames.TD);
        emptyHeaderCellStyle.setBackground(
                tableHeader.getBackground() == null ? table.getBackground() : tableHeader.getBackground());
        String emptyHeaderCellStyleName = rc.getDocument().addStyle(emptyHeaderCellStyle);
        
        // Height calculation
        String rowHeight;
        if (rowHeightsUsePercentUnits) {
            rowHeight = tableHeader.getHeight() + "%";
        } else {
            rowHeight = Integer.toString(tableHeader.getHeight());
        }

        for (int column = 0; column < columnCount; ++column) {
            Element tdElement = new Element(ElementNames.TD);
            tdElement.setWhitespaceRelevant(true);
            Component cell = tableHeader.getRenderedComponent(column);
            if (cell == null || !cell.isVisible()) {
                tdElement.addAttribute(ElementNames.Attributes.CLASS, emptyHeaderCellStyleName);
            } else {
                // Cell does have a component.
                ComponentPeer peer = getPeer(cell);
                // Create style for cell.
                ComponentStyle cellStyle = ComponentStyle.forComponent(peer, ComponentStyle.EXPLICIT_SET_ALL);
                if (peer instanceof Alignment) {
                    cellStyle.setHorizontalAlignment(((Alignment) peer).getHorizontalAlignment());
                    cellStyle.setVerticalAlignment(((Alignment) peer).getVerticalAlignment());
                }
                cellStyle.addElementType(ElementNames.TD);
                String cellStyleName = rc.getDocument().addStyle(cellStyle);
                tdElement.addAttribute(ElementNames.Attributes.CLASS, cellStyleName);
                peer.render(rc, tdElement);
            }

            int width = columnModel.getColumn(column).getWidth();
            if (width != EchoConstants.UNDEFINED_SIZE) {
                if (columnWidthsUsePercentUnits) {
                    tdElement.addAttribute(ElementNames.Attributes.WIDTH, width + "%");
                } else {
                    tdElement.addAttribute(ElementNames.Attributes.WIDTH, width);
                }
            }
            if (rowHeight != null) {
                tdElement.addAttribute(ElementNames.Attributes.HEIGHT, rowHeight);
            }

            trElement.add(tdElement);
        }
        parent.add(trElement);
    }

}
