/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.Color;
import nextapp.echo.Component;
import nextapp.echo.Coordinate;
import nextapp.echo.EchoConstants;
import nextapp.echo.EchoInstance;
import nextapp.echo.Table;
import nextapp.echo.table.TableColumn;
import nextapp.echo.table.TableColumnModel;
import nextapp.echo.table.TableHeader;
import nextapp.echoservlet.Alignment;
import nextapp.echoservlet.ComponentPeer;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/**
 * A peer for <code>Table</code> components.
 */
public class TableUI extends ComponentPeer
implements PropertyChangeListener {

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        redraw();
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        getComponent().addPropertyChangeListener(this);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        Table table = (Table) getComponent();
        TableHeader header = table.getTableHeader();
        TableColumnModel columnModel = table.getColumnModel();
        boolean borderTableRequired = table.getBorderColor() != null && table.getBorderSize() > 0;
        boolean columnWidthsUsePercentUnits = table.getWidthUnits() == Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        table.getWidthUnits() == Table.PERCENT_UNITS;
        boolean rowHeightsUsePercentUnits = table.getHeightUnits() == Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS || 
                        table.getHeightUnits() == Table.PERCENT_UNITS;
        boolean someColumnsHaveWidths = false;
        boolean allColumnsHaveWidths = true;
        boolean allRowsHaveHeights = true;
        int rowCount = table.getRowCount();
        int columnCount = table.getColumnCount();
        
        Element tableElement = new Element(ElementNames.TABLE);
        tableElement.addAttribute(ElementNames.Attributes.BORDER, 0);
        tableElement.addAttribute(ElementNames.Attributes.CELLPADDING, table.getCellMargin());
        tableElement.addAttribute(ElementNames.Attributes.CELLSPACING, table.getBorderSize());
        
        // Generate <colgroup> element (only added if necessary).
        // Determine if all columns have widths (set the allColumnsHaveWidths property,
        // which will be used later.
        Element colGroupElement = new Element(ElementNames.COLGROUP);
        for (int column = 0; column < columnCount; column++) {
            TableColumn tableColumn = columnModel.getColumn(column);
            int width = tableColumn.getWidth();
            Element colElement = new Element(ElementNames.COL, false);
            if (width == EchoConstants.UNDEFINED_SIZE) {
                colElement.addAttribute(ElementNames.Attributes.WIDTH, "1*");                    
                allColumnsHaveWidths = false;
            } else {
                if (columnWidthsUsePercentUnits) {
                    colElement.addAttribute(ElementNames.Attributes.WIDTH, width + "%");
                } else {
                    colElement.addAttribute(ElementNames.Attributes.WIDTH, width);
                }
                someColumnsHaveWidths = true;
            }
            colGroupElement.add(colElement);
        }
        if (someColumnsHaveWidths) {
            tableElement.add(colGroupElement);
        }

        // Determine background color for table cells that do not have the background property set.
        // This is required to prevent the table's border color from "showing through."
        Color defaultBackground = table.getBackground();
        if (defaultBackground == null && table.getBorderColor() != null) {
            Component c = table;
            while (defaultBackground == null && c != null) {
                defaultBackground = c.getBackground();
                c = c.getParent();
            }
            if (defaultBackground == null) {
                defaultBackground = EchoInstance.DEFAULT_BACKGROUND;
            }
        }

        // Create a style for empty cells.
        ComponentStyle emptyCellStyle = new ComponentStyle();
        emptyCellStyle.addElementType(ElementNames.TD);
        emptyCellStyle.setBackground(defaultBackground);
        String emptyCellStyleName = rc.getDocument().addStyle(emptyCellStyle);

        Element trElement, tdElement;

        if (header != null && header.isVisible()) {
            getPeer(header).render(rc, tableElement);
        }
        
        for (int row = 0; row < rowCount; ++row) {
            trElement = new Element(ElementNames.TR);

            // Height calculation
            String rowHeight;
            if (table.getRowHeight(row) == EchoConstants.UNDEFINED_SIZE) {
                allRowsHaveHeights = false;
                rowHeight = null;
            } else {
                if (rowHeightsUsePercentUnits) {
                    rowHeight = table.getRowHeight(row) + "%";
                } else {
                    rowHeight = Integer.toString(table.getRowHeight(row));
                }
            }            

            for (int column = 0; column < columnCount; ++column) {
                tdElement = new Element(ElementNames.TD);
                tdElement.setWhitespaceRelevant(true);
                Coordinate coordinate = new Coordinate(column, row);
                Component cell = table.getRenderedComponent(coordinate);
                if (cell == null || !cell.isVisible()) {
                    // Cell does not have a component to render, set style to the default.
                    tdElement.addAttribute(ElementNames.Attributes.CLASS, emptyCellStyleName);
                } else {
                    // Cell does have a component.
                    ComponentPeer peer = getPeer(cell);
                    // Create style for cell.
                    ComponentStyle cellStyle = ComponentStyle.forComponent(peer, ComponentStyle.EXPLICIT_SET_ALL);
                    
                    if (cell.getBackground() == null) {
                        // Apply default background to empty cells or cells without backgrounds.
                        cellStyle.setBackground(defaultBackground);
                    }
                    if (peer instanceof Alignment) {
                        cellStyle.setHorizontalAlignment(((Alignment) peer).getHorizontalAlignment());
                        cellStyle.setVerticalAlignment(((Alignment) peer).getVerticalAlignment());
                    }
                    cellStyle.addElementType(ElementNames.TD);
                    String cellStyleName = rc.getDocument().addStyle(cellStyle);
                    tdElement.addAttribute(ElementNames.Attributes.CLASS, cellStyleName);
                    peer.render(rc, tdElement);
                }

                int width = columnModel.getColumn(column).getWidth();
                if (width != EchoConstants.UNDEFINED_SIZE) {
                    if (columnWidthsUsePercentUnits) {
                        tdElement.addAttribute(ElementNames.Attributes.WIDTH, width + "%");
                    } else {
                        tdElement.addAttribute(ElementNames.Attributes.WIDTH, width);
                    }
                }
                if (rowHeight != null) {
                    tdElement.addAttribute(ElementNames.Attributes.HEIGHT, rowHeight);
                }

                trElement.add(tdElement);
            }
            tableElement.add(trElement);
        }
        
        if (table.getWidth() > 0) {
            switch (table.getWidthUnits()) {
            case Table.PIXEL_UNITS:
                // If all columns have widths, they override the total width.
                if (! allColumnsHaveWidths) {
                    tableElement.addAttribute(ElementNames.Attributes.WIDTH, table.getWidth());
                }
                break;
            case Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS:
                tableElement.addAttribute(ElementNames.Attributes.WIDTH, table.getWidth());
                break;
            case Table.PERCENT_UNITS:
                if (borderTableRequired) {
                    tableElement.addAttribute(ElementNames.Attributes.WIDTH, "100%");
                } else {
                    tableElement.addAttribute(ElementNames.Attributes.WIDTH, table.getWidth() + "%");
                }
                break;
            }
        }
        if (table.getHeight() > 0) {
            switch (table.getHeightUnits()) {
            case Table.PIXEL_UNITS:
                // If all columns have heights, they override the total height.
                if (! allRowsHaveHeights) {
                    tableElement.addAttribute(ElementNames.Attributes.HEIGHT, table.getHeight());
                }
                break;
            case Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS:
                tableElement.addAttribute(ElementNames.Attributes.HEIGHT, table.getHeight());
                break;
            case Table.PERCENT_UNITS:
                if (borderTableRequired) {
                    tableElement.addAttribute(ElementNames.Attributes.HEIGHT, "100%");
                } else {
                    tableElement.addAttribute(ElementNames.Attributes.HEIGHT, table.getHeight() + "%");
                }
                break;
            }
        }

        if (borderTableRequired) {
            // Embed table inside another table cell with a background color of the border color.

            ComponentStyle borderStyle = new ComponentStyle();
            borderStyle.setBackground(table.getBorderColor());
            borderStyle.addElementType(ElementNames.TD);
            String borderStyleName = rc.getDocument().addStyle(borderStyle);

            Element borderTableElement = new Element(ElementNames.TABLE);
            borderTableElement.setWhitespaceRelevant(true);
            borderTableElement.addAttribute(ElementNames.Attributes.BORDER, 0);
            borderTableElement.addAttribute(ElementNames.Attributes.CELLPADDING, 0);
            borderTableElement.addAttribute(ElementNames.Attributes.CELLSPACING, 0);
            if (table.getWidthUnits() == Table.PERCENT_UNITS && table.getWidth() > 0) {
                borderTableElement.addAttribute(ElementNames.Attributes.WIDTH, table.getWidth() + "%");
            }
            if (table.getHeightUnits() == Table.PERCENT_UNITS && table.getHeight() > 0) {
                borderTableElement.addAttribute(ElementNames.Attributes.HEIGHT, table.getHeight() + "%");
            }
            
            Element borderTrElement = new Element(ElementNames.TR);
            borderTrElement.setWhitespaceRelevant(true);
            
            Element borderTdElement = new Element(ElementNames.TD);
            borderTdElement.setWhitespaceRelevant(true);
            borderTdElement.addAttribute(ElementNames.Attributes.CLASS, borderStyleName);

            borderTableElement.add(borderTrElement);
            borderTrElement.add(borderTdElement);
            borderTdElement.add(tableElement);
            parent.add(borderTableElement);
        } else {
            parent.add(tableElement);
        }

        rc.getDocument().setCursorOnNewLine(true);
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        getComponent().removePropertyChangeListener(this);
    }
}
