/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import nextapp.echo.PasswordField;
import nextapp.echo.TextField;
import nextapp.echoservlet.ClientActionProducer;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.ComponentStyle;
import nextapp.echoservlet.ContentPaneUI;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;
import nextapp.echoservlet.html.Html;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A peer for <code>TextField</code> and <code>PasswordField</code> components.
 */
public class TextFieldUI extends TextComponentUI 
implements ClientActionProducer {

    public static final Service SERVICE_TEXT_FIELD_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "txtfd", ResourceNames.TEXT_FIELD_SCRIPT);
    
    static {
        EchoServer.addGlobalService(SERVICE_TEXT_FIELD_SCRIPT);
    }
    
    /**
     * @see nextapp.echoservlet.ClientActionProducer#clientAction(String)
     */
    public void clientAction(String command) {
        if (renderedActive) {
            TextField textField = (TextField) getComponent();
            textField.fireActionEvent();
        }
    }

    /**
     * @see nextapp.echoservlet.ComponentPeer#render(RenderingContext, Element)
     */
    public void render(RenderingContext rc, Element parent) {
        TextField textField = (TextField) getComponent();
        Element element = new Element(ElementNames.INPUT, false);
        element.addAttribute(ElementNames.Attributes.ID, getId().toString());
        
        renderedActive = textField.isEditable() && textField.isEnabled();
        
        ComponentStyle style = getStyle();
        style.addElementType(ElementNames.INPUT);
        
        if (!textField.isEditable()) {
            element.addAttribute("readonly");
        }
        if (!textField.isEnabled()) {
            element.addAttribute("disabled");
        }
        switch (textField.getColumnUnits()) {
        case TextField.CHARACTER_UNITS:
            element.addAttribute("size", textField.getColumns());
            break;
        case TextField.PIXEL_UNITS:
            style.addAttribute("width", textField.getColumns(), ComponentStyle.PIXEL_UNITS);
            break;
        case TextField.PERCENT_UNITS:
            style.addAttribute("width", textField.getColumns(), ComponentStyle.PERCENT_UNITS);
            break;
        default:
            throw new IllegalStateException("Unsupported column units.");
        }
        if (textField instanceof PasswordField) {
            element.addAttribute("type", "password");
            if (textField.getText() != null) {
                int length = textField.getText().length();
                StringBuffer sb = new StringBuffer();
                for (int index = 0; index < length; ++index) {
                    sb.append("*");
                }
                element.addAttribute("value", sb.toString());
            }
            element.addAttribute("onfocus", "this.value=\'\'");
        } else {
            element.addAttribute("type", "text");
            element.addAttribute("value", Html.encode(textField.getText(), Html.NEWLINE_TO_SPACE));
        }
        
        if (textField.getToolTipText() != null) {
            element.addAttribute("title", Html.encode(textField.getToolTipText(), Html.NEWLINE_TO_SPACE));
        }
        
        if (textField.hasActionListeners()) {
            // Add text field script (only necessary for retrieving action events)
            rc.getDocument().addScriptInclude(SERVICE_TEXT_FIELD_SCRIPT);
            element.addAttribute("onkeypress", "E_textFieldKeyPress(event, '" + getId() + "', this.value);");
        }
        
        if (textField.getMaximumLength() >= 0) {
            element.addAttribute("maxlength", textField.getMaximumLength());
        }
        element.addAttribute("tabindex", getTabIndex());
        element.addAttribute("name", ClientObjects.getObjectName(getId()));
        element.addAttribute("onblur", ContentPaneUI.getScriptSetParameter("\'" + getId() + "\'", "this.value"));
        
        String styleName = rc.getDocument().addStyle(style);
        element.addAttribute(ElementNames.Attributes.CLASS, styleName);
        
        parent.add(element);
        rc.getDocument().setCursorOnNewLine(false);
    }
}
