/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import nextapp.echo.ToggleButton;
import nextapp.echoservlet.Connection;
import nextapp.echoservlet.ClientObjects;
import nextapp.echoservlet.EchoServer;
import nextapp.echoservlet.Id;
import nextapp.echoservlet.RenderingContext;
import nextapp.echoservlet.Service;
import nextapp.echoservlet.StaticText;
import nextapp.echoservlet.image.ImageManager;
import nextapp.echoservlet.resource.ResourceNames;

/**
 * A base peer class for components derived from <code>ToggleButton</code>.
 */
public abstract class ToggleButtonUI extends AbstractButtonUI 
implements PropertyChangeListener {

    /**
     * A prefix used in the value of the image &quot;name&quot; attribute of a
     * an &lt;img&gt; element rendered to represent a toggle button's state 
     * icon.
     */
    public static final String TOGGLE_IMAGE_PREFIX = ClientObjects.STD_PREFIX + "TBIMG_";
    
    /**
     * A service providing JavaScript to handle client-side management of 
     * toggle buttons.
     */
    protected static final Service SERVICE_TOGGLE_BUTTON_SCRIPT 
            = StaticText.createFromResource(ClientObjects.STD_PREFIX + "togbu", ResourceNames.TOGGLE_BUTTON_SCRIPT);
    
    /** 
     * A constant used to identify the selected icon for the 
     * in the <code>ImageManager</code>.
     */
    protected static final String SELECTED_ICON_IMAGE = "selectedIcon";
    
    /** 
     * A constant used to identify the rollover selected icon for the 
     * in the <code>ImageManager</code>.
     */
    protected static final String ROLLOVER_SELECTED_ICON_IMAGE = "rolloverSelectedIcon";

    static {
        EchoServer.addGlobalService(SERVICE_TOGGLE_BUTTON_SCRIPT);
    }
    
    /**
     * Returns the necessary script to produce a button rollover effect.
     *
     * @param conn The <code>connection</code> on which the button is being 
     *             rendered.
     * @param id The <code>Id</code> of the toggle button.
     * @param stateId The <code>Id</code> of the toggle button that holds the 
     *        selected state of the toggle button.  This property is different
     *        from the <code>id</code> on RadioButtons.
     * @param imageUri The URI of the icon that should be displayed when the
     *                 button is not selected.
     * @param selectedImageUri The URI of the icon that should be displayed when the
     *                 button is selected.
     * @param styleName The toggle button's CSS style name.
     * @return A JavaScript method call that may be invoked to produce
     *         mouse rollover effects.
     */
    public static String getScriptCustomStateIconToggleRollover(Connection conn, String id, String stateId, 
            String imageUri, String selectedImageUri, String styleName) {
        StringBuffer sb = new StringBuffer(ClientObjects.STD_PREFIX);
        
        sb.append("toggleRollover(\'");
        sb.append(id);
        sb.append("\', \'");
        sb.append(stateId);
        sb.append("\', ");
        if (imageUri == null) {
            sb.append("null, ");
        } else {
            sb.append("\'");
            sb.append(imageUri);
            sb.append("\', ");
        }
        if (selectedImageUri == null) {
            sb.append("null, ");
        } else {
            sb.append("\'");
            sb.append(selectedImageUri);
            sb.append("\', ");
        }
        if (styleName == null) {
            sb.append("null);");
        } else {
            sb.append("\'");
            sb.append(styleName);
            sb.append("\');");
        }
        
        return sb.toString();
    }
    
    /**
     * Returns the URI of the default image used to represent the toggle
     * button's state.
     *
     * @param rc The <code>RenderingContext</code> to which the button is being
     *           rendered.
     * @param selected True if the selected state image is being requested.
     * @return The URI of the default state image.
     */
    public abstract String getDefaultStateImageUri(RenderingContext rc, boolean selected);
    
    /**
     * Returns the <code>Id</code> of the controller input field in which the 
     * <code>ToggleButton</code>'s state is to be stored.  In the case where 
     * multiple <code>RadioButton</code>s are in a <code>ButtonGroup</code>,
     * the state will be stored in only one field, specified by this
     * method.<br>
     * <br>
     * The default implementation returns the <code>Id</code> of this peer.
     * 
     * @return The <code>Id</code> of the controller input field used to 
     *         store the <code>ToggleButton</code>'s state.
     */
    public Id getStateId() {
        return getId();
    }
    
    /**
     * Returns the URI of the image that will be used to represent state
     * for this particular toggle button.
     *
     * @param rc The <code>RenderingContext</code> to which the button is being
     *           rendered.
     * @param selected True if the selected state image is being requested.
     * @return The URI of the state image.
     */
    public String getStateImageUri(RenderingContext rc, boolean selected) {
        String imageUri;
        Connection conn = rc.getConnection();
        
        if (imageManager != null && imageManager.getImage(ICON_IMAGE) != null 
                && imageManager.getImage(SELECTED_ICON_IMAGE) != null) {
            if (selected) {
                imageUri = imageManager.getImageUri(conn, SELECTED_ICON_IMAGE);
            } else {
                imageUri = imageManager.getImageUri(conn, ICON_IMAGE);
            }
        } else {
            imageUri = getDefaultStateImageUri(rc, selected);
        }
        
        return imageUri;
    }
    
    /**
     * @see java.beans.PropertyChangeListener#propertyChange(PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
        super.propertyChange(e);
        
        if (ToggleButton.ROLLOVER_SELECTED_ICON_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            ToggleButton button = (ToggleButton) getComponent();
            imageManager.setImage(ROLLOVER_SELECTED_ICON_IMAGE, button.getRolloverSelectedIcon());
        } else if (ToggleButton.SELECTED_ICON_CHANGED_PROPERTY.equals(e.getPropertyName())) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            ToggleButton button = (ToggleButton) getComponent();
            imageManager.setImage(SELECTED_ICON_IMAGE, button.getSelectedIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#registered()
     */
    public void registered() {
        super.registered();
        ToggleButton button = (ToggleButton) getComponent();
        if (button.getSelectedIcon() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(SELECTED_ICON_IMAGE, button.getSelectedIcon());
        }
        if (button.getRolloverSelectedIcon() != null) {
            if (imageManager == null) {
                // Lazy-create image manager.
                imageManager = new ImageManager(this);
            }
            imageManager.setImage(ROLLOVER_SELECTED_ICON_IMAGE, button.getRolloverSelectedIcon());
        }
    }
    
    /**
     * @see nextapp.echoservlet.ComponentPeer#unregistered()
     */
    public void unregistered() {
        super.unregistered();
        if (imageManager != null) {
            imageManager.setImage(SELECTED_ICON_IMAGE, null);
            imageManager.setImage(ROLLOVER_SELECTED_ICON_IMAGE, null);
        }
    }
}
