/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.ui;

import nextapp.echo.EchoConstants;

import nextapp.echoservlet.Connection;
import nextapp.echoservlet.LayoutStrut;
import nextapp.echoservlet.html.Element;
import nextapp.echoservlet.html.ElementNames;

/**
 * Renders an HTML table that has two or three &quot;container&quot; cells and
 * independently settable margins between them.  These tables are useful for
 * rendering buttons that have two or three elements (images, text labels, and 
 * state indicators).  This class supports all possible permutations for
 * placement of each of the two or three contained components.
 */
class TriCellTable extends Element {

    static final int INVERTED = 1;
    static final int VERTICAL = 2;
    
    static final int LEFT_RIGHT = 0;
    static final int RIGHT_LEFT = INVERTED;
    static final int TOP_BOTTOM = VERTICAL;
    static final int BOTTOM_TOP = INVERTED | VERTICAL;
    
    /**
     * Translates horizontal and vertical orientation constants from the
     * <code>nextapp.echo.EchoContstants</code> class into the constants
     * used by <code>TriCellTable</code>:
     * <ul>
     * <li>LEFT_RIGHT</li>
     * <li>RIGHT_LEFT</li>
     * <li>TOP_BOTTOM</li>
     * <li>BOTTOM_TOP</li>
     * </ul>
     * @param horizontalOrientation The horizontal orientation desired, one of
     *        the following values:
     *        <ul>
     *        <li>EchoConstants.LEFT</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.RIGHT</li>
     *        </ul>
     * @param verticalOrientation The vertical orientation desired, one of the
     *        following values:
     *        <ul>
     *        <li>EchoConstants.TOP</li>
     *        <li>EchoConstants.CENTER</li>
     *        <li>EchoConstants.BOTTOM</li>
     *        </ul>
     * @return Am orientation constant from <code>TriCellTable</code> that 
     *         equates to the orientation parameters provided.
     */
    static int getOrientation(int horizontalOrientation, int verticalOrientation) {
        int tctOrientation;
    
        switch (verticalOrientation) {
        case EchoConstants.TOP:
            tctOrientation = TriCellTable.TOP_BOTTOM;
            break;
        case EchoConstants.BOTTOM:
            tctOrientation = TriCellTable.BOTTOM_TOP;
            break;
        default:
            if (horizontalOrientation == EchoConstants.LEFT) {
                tctOrientation = TriCellTable.LEFT_RIGHT;
            } else {
                tctOrientation = TriCellTable.RIGHT_LEFT;
            }
            break;
        }
        
        return tctOrientation;
    }

    private Element[] tds;
    private Element[] marginTds = null;
    
    /**
     * This constructor is called by the non-private constructors to set up 
     * common properties.
     */
    private TriCellTable() {
        super(ElementNames.TABLE);

        addAttribute("border", 0);
        addAttribute("cellpadding", 0);
        addAttribute("cellspacing", 0);
    }

    /**
     * Creates a two-celled <code>TriCellTable</code>.
     * 
     * @param conn The connection which is to render the underlying button 
     *        object.
     * @param orientation0_1 The orientation of Element 0 with respect to 
     *        Element 1, one of the following values:
     *        <ul>
     *        <li>LEFT_RIGHT (element 0 is to the left of element 1)</li>
     *        <li>RIGHT_LEFT (element 1 is to the left of element 0)</li>
     *        <li>TOP_BOTTOM (element 0 is above element 1)</li>
     *        <li>BOTTOM_TOP (element 1 is above element 0)</li>
     *        </ul>
     * @param margin0_1 The margin size (in pixels) between element 0 and
     *        element 1.
     */
    TriCellTable(Connection conn, int orientation0_1, int margin0_1) {
        this();
        
        marginTds = new Element[1];
        tds = new Element[2];
        tds[0] = new Element(ElementNames.TD);
        tds[0].setWhitespaceRelevant(true);
        tds[1] = new Element(ElementNames.TD);
        tds[1].setWhitespaceRelevant(true);
        
        if (margin0_1 > 0) {
            marginTds[0] = new Element(ElementNames.TD);
            marginTds[0].setWhitespaceRelevant(true);
            if ((orientation0_1 & VERTICAL) == 0) {
                marginTds[0].add(LayoutStrut.createElement(conn, margin0_1, 1));
            } else {
                marginTds[0].add(LayoutStrut.createElement(conn, 1, margin0_1));
            }
        }
                
        if ((orientation0_1 & VERTICAL) == 0) {
            // horizontally oriented
            Element tr = new Element(ElementNames.TR);
            if ((orientation0_1 & INVERTED) == 0) {
                // normal (left to right)
                addColumn(tr, tds[0]);
                addColumn(tr, marginTds[0]);
                addColumn(tr, tds[1]);
            } else {
                // inverted (right to left)
                addColumn(tr, tds[1]);
                addColumn(tr, marginTds[0]);
                addColumn(tr, tds[0]);
            }
            add(tr);
        } else {
            // vertically oriented
            if ((orientation0_1 & INVERTED) == 0) {
                // normal (top to bottom)
                addRow(tds[0]);
                addRow(marginTds[0]);
                addRow(tds[1]);
            } else {
                // inverted (bottom to top)
                addRow(tds[1]);
                addRow(marginTds[0]);
                addRow(tds[0]);
            }
        }
    }
    
    /**
     * Creates a three-celled <code>TriCellTable</code>.
     * 
     * @param conn The connection which is to render the underlying button 
     *        object.
     * @param orientation0_1 The orientation of Element 0 with respect to 
     *        Element 1, one of the following values:
     *        <ul>
     *        <li>LEFT_RIGHT (element 0 is to the left of element 1)</li>
     *        <li>RIGHT_LEFT (element 1 is to the left of element 0)</li>
     *        <li>TOP_BOTTOM (element 0 is above element 1)</li>
     *        <li>BOTTOM_TOP (element 1 is above element 0)</li>
     *        </ul>
     * @param margin0_1 The margin size (in pixels) between element 0 and
     *        element 1.
     * @param orientation01_2 The orientation of Elements 0 and 1 with 
     *        respect to Element 2, one of the following values:
     *        <ul>
     *        <li>LEFT_RIGHT (elements 0 and 1 are to the left of 
     *        element 1)</li>
     *        <li>RIGHT_LEFT (element 2 is to the left of elements 0 and 1)</li>
     *        <li>TOP_BOTTOM (elements 0 and 1 are above element 2)</li>
     *        <li>BOTTOM_TOP (element 2 is above elements 0 and 1)</li>
     *        </ul>
     * @param margin01_2 The margin size (in pixels) between the combination
     *        of elements 0 and 1 and element 2.
     *        element 1.
     */
    TriCellTable(Connection conn, int orientation0_1, int margin0_1, int orientation01_2, int margin01_2) {
        this();
        
        Element tr;
        
        marginTds = new Element[2];
        tds = new Element[3];
        tds[0] = new Element(ElementNames.TD);
        tds[0].setWhitespaceRelevant(true);
        tds[1] = new Element(ElementNames.TD);
        tds[1].setWhitespaceRelevant(true);
        tds[2] = new Element(ElementNames.TD);
        tds[2].setWhitespaceRelevant(true);

        // Create margin cells
        if (margin0_1 > 0 || margin01_2 > 0) {
            if (margin0_1 > 0) {
                marginTds[0] = new Element(ElementNames.TD);
                marginTds[0].setWhitespaceRelevant(true);
                if ((orientation0_1 & VERTICAL) == 0) {
                    marginTds[0].add(LayoutStrut.createElement(conn, margin0_1, 1));
                } else {
                    marginTds[0].add(LayoutStrut.createElement(conn, 1, margin0_1));
                }
            }
            if (margin01_2 > 0) {
                marginTds[1] = new Element(ElementNames.TD);
                marginTds[1].setWhitespaceRelevant(true);
                if ((orientation01_2 & VERTICAL) == 0) {
                    marginTds[1].add(LayoutStrut.createElement(conn, margin01_2, 1));
                } else {
                    marginTds[1].add(LayoutStrut.createElement(conn, 1, margin01_2));
                }
            }
        }
        
        if ((orientation0_1 & VERTICAL) == 0) {
            // horizontally oriented 0/1
            if ((orientation01_2 & VERTICAL) == 0) {
                // horizontally orientated 01/2
                tr = new Element(ElementNames.TR);
                if ((orientation01_2 & INVERTED) != 0) {
                    // 2 before 01: render #2 and margin at beginning of TR.
                    addColumn(tr, tds[2]);
                    addColumn(tr, marginTds[1]);
                }
                
                // Render 01
                if ((orientation0_1 & INVERTED) == 0) {
                    // normal (left to right)
                    addColumn(tr, tds[0]);
                    addColumn(tr, marginTds[0]);
                    addColumn(tr, tds[1]);
                } else {
                    // inverted (right to left)
                    addColumn(tr, tds[1]);
                    addColumn(tr, marginTds[0]);
                    addColumn(tr, tds[0]);
                }
                
                if ((orientation01_2 & INVERTED) == 0) {
                    addColumn(tr, marginTds[1]);
                    addColumn(tr, tds[2]);
                }
                
                add(tr);
            } else {
                // vertically oriented 01/2

                // determine and apply column span based on presence of margin between 0 and 1
                int columns = margin0_1 > 0 ? 3 : 2;
                tds[2].addAttribute("colspan", columns);
                if (marginTds[1] != null) {
                    marginTds[1].addAttribute("colspan", columns);
                }
                
                if ((orientation01_2 & INVERTED) != 0) {
                    // 2 before 01: render #2 and margin at beginning of TR.
                    addRow(tds[2]);
                    addRow(marginTds[1]);
                }
                
                // Render 01
                tr = new Element(ElementNames.TR);
                if ((orientation0_1 & INVERTED) == 0) {
                    // normal (left to right)
                    addColumn(tr, tds[0]);
                    addColumn(tr, marginTds[0]);
                    addColumn(tr, tds[1]);
                } else {
                    // inverted (right to left)
                    addColumn(tr, tds[1]);
                    addColumn(tr, marginTds[0]);
                    addColumn(tr, tds[0]);
                }
                add(tr);
                
                if ((orientation01_2 & INVERTED) == 0) {
                    // 01 before 2: render margin and #2 at end of TR.
                    addRow(marginTds[1]);
                    addRow(tds[2]);
                }
            }
        } else {
            // vertically oriented 0/1
            if ((orientation01_2 & VERTICAL) == 0) {
                // horizontally orientated 01/2

                // determine and apply row span based on presence of margin between 0 and 1
                int rows = margin0_1 > 0 ? 3 : 2;
                tds[2].addAttribute("rowspan", rows);
                if (marginTds[1] != null) {
                    marginTds[1].addAttribute("rowspan", rows);
                }
                
                tr = new Element(ElementNames.TR);
                if ((orientation01_2 & INVERTED) != 0) {
                    addColumn(tr, tds[2]);
                    addColumn(tr, marginTds[1]);
                    if ((orientation0_1 & INVERTED) == 0) {
                        addColumn(tr, tds[0]);
                    } else {
                        addColumn(tr, tds[1]);
                    }
                } else {
                    if ((orientation0_1 & INVERTED) == 0) {
                        addColumn(tr, tds[0]);
                    } else {
                        addColumn(tr, tds[1]);
                    }
                    addColumn(tr, marginTds[1]);
                    addColumn(tr, tds[2]);
                }
                add(tr);
                addRow(marginTds[0]);
                if ((orientation0_1 & INVERTED) == 0) {
                    addRow(tds[1]);
                } else {
                    addRow(tds[0]);
                }
            } else {
                // vertically oriented 01/2
                if ((orientation01_2 & INVERTED) != 0) {
                    // 2 before 01: render #2 and margin at beginning of TABLE.
                    addRow(tds[2]);
                    addRow(marginTds[1]);
                }
                
                // Render 01
                if ((orientation0_1 & INVERTED) == 0) {
                    // normal (top to bottom)
                    addRow(tds[0]);
                    addRow(marginTds[0]);
                    addRow(tds[1]);
                } else {
                    // inverted (bottom to top)
                    addRow(tds[1]);
                    addRow(marginTds[0]);
                    addRow(tds[0]);
                }
                
                if ((orientation01_2 & INVERTED) == 0) {
                    // 01 before 2: render margin and #2 at end of TABLE.
                    addRow(marginTds[1]);
                    addRow(tds[2]);
                }
            }
        }
    }
    
    /**
     * Adds an cell element to a table row.  The element will not be added
     * if null is provided for the value of <code>td</code>.
     * 
     * @param tr The table row to which the cell element is to be added.
     * @param td The cell element to be added (if not null).
     */
    private void addColumn(Element tr, Element td) {
        if (td != null) {
            tr.add(td);
        }
    }
    
    /**
     * Adds an attribute to every table cell in the rendered table.
     *
     * @param name The name of the attribute.
     * @param value The value of the attribute.
     */
    public void addGlobalAttribute(String name, String value) {
        for (int index = 0; index < tds.length; ++index) {
            tds[index].addAttribute(name, value);
        }
        if (marginTds != null) {
            for (int index = 0; index < marginTds.length; ++index) {
                if (marginTds[index] != null) {
                    marginTds[index].addAttribute(name, value);
                }
            }
        }
    }
    
    /**
     * Adds a row containing a single column element to a table.
     * The row will not be added if <code>td</code> is null.
     *
     * @param td The cell element to be added.
     */
    private void addRow(Element td) {
        if (td != null) {
            Element tr = new Element(ElementNames.TR);
            tr.add(td);
            add(tr);
        }
    }
    
    /**
     * Returns the specified container element.
     *
     * @param index The index of the table element to return.  For two-celled tables,
     *        legitimate values are 0 and 1.  For three-celled tables, 
     *        legitimate values are 0, 1, and 2.
     * @return The specified container element.
     */
    Element getTdElement(int index) {
        return tds[index];
    }
    
    /**
     * Returns the specified margin element.
     *
     * @param index The index of the table element to return.  Index 0 is the margin
     *        element between container cells 0 and 1.  Index 1 is the margin
     *        element between container cells 0/1 and 2.
     * @return The specified margin element.  Returns null if the margin is zero
     *         pixels.
     */
    Element getMarginTdElement(int index) {
        return marginTds[index];
    }
}
