/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.util;

import java.util.Collections;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

/**
 * Determines <code>ContentType</code> based on file extensions.
 */
public class Extensions {

    /** A PropertyMap mapping extensions to content types */
    private static final Map contentTypeMap;
    
    static {
        Map inputMap = new PropertyMap("nextapp.echoservlet.util.Extensions");
        Map outputMap = new HashMap();
        String key = null;
        String value = null;
        boolean binary = false;
        String mimeType = null;
        
        for (Iterator it = inputMap.keySet().iterator(); it.hasNext();) {
            key = (String) it.next();
            value = (String) inputMap.get(key);
            
            mimeType = value.substring(2);
            binary = value.charAt(0) == '1';
            outputMap.put(key, new ContentType(mimeType, binary));
        }
        
        contentTypeMap = Collections.unmodifiableMap(outputMap);
    }
    
    /**
     * Returns the content type associated with the specified extension.
     * If no content type is bound to the specified extension, null is 
     * returned.
     *
     * @param extension The extension whose content type is to be determined.
     * @return The content type which corresponds to the given extension.
     */
    public static ContentType getExtensionContentType(String extension) {
        return (ContentType) contentTypeMap.get(extension);
    }
    
    /** 
     * Returns the content type associated with the provided file name.
     * The file name is parsed and its extension is retrieved.
     *
     * @param fileName The file name whose content type is to be determined.
     * @return The content type which corresponds to the given file name.
     */
    public static ContentType getFileNameContentType(String fileName) {
        int dotPos = fileName.lastIndexOf(".");
        ContentType contentType = null;
        
        if (dotPos > -1) {
            contentType = getExtensionContentType(fileName.substring(dotPos + 1));
        }
        
        return contentType;
    }
    
    /** Non-instantiable class. */
    private Extensions() { }
}
