/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
 
package nextapp.echoservlet.util;

import java.io.Serializable;
import java.util.Map;
import java.util.MissingResourceException;

/**
 * Generates &quot;peer&quot; objects for objects based on associations.
 * Associations are stored in a properties files, where the keys are the fully
 * qualified class names of objects and the values are the fully qualified
 * class names of their peers.  When an object is passed to the createPeer()
 * method its peer will be returned based on the bindings in the properties file.
 * An objects superclass(es) will also be tested for available peers if none is
 * available for the derived class.  If no peer is found, a PeerFactoryException
 * is thrown.
 */
public class PeerFactory 
implements Serializable {

    /**
     * An exception used to describe a problem that was encountered loading
     * a peer object.
     */
    public class PeerFactoryException extends Exception {
    
        /**
         * Creates a <code>PeerFactoryException</code>.
         *
         * @param description A description of the problem which caused this
         *        exception to occur.
         */
        public PeerFactoryException(String description) {
            super(description);
        }
    }
    
    /** A map of associations */
    private Map associations = null;
    
    /**
     * Creates a new <code>PeerFactory</code>.
     */
    public PeerFactory() {
        super();
    }
    
    /** 
     * Creates a new <code>PeerFactory</code> with initial associations.
     *
     * @param propertiesFile The name for the properties file containing the 
     *        initial peer bindings.
     */ 
    public PeerFactory(String propertiesFile) 
    throws PeerFactoryException { 
        super();
        
        addAssociations(propertiesFile);
    }
    
    /**
     * Adds a single peer binding.
     * 
     * @param className The fully qualified name of the class.
     * @param peerClassName The fully qualified name of the peer class.
     */
    public void addAssociation(String className, String peerClassName) {
        associations.put(className, peerClassName);
    }
    
    /**
     * Loads peer bindings.  Invoking this method will result in the new
     * associations supplementing existing associations.  Existing associations
     * will not be removed.  If two associations have the same key, the most 
     * recently added one will be used.
     *
     * @param propertiesFile The name of a properties file containing the peer 
     *        bindings to be loaded. 
     */
    public void addAssociations(String propertiesFile)
    throws PeerFactoryException {
        try {
            if (associations == null) {
                associations = new PropertyMap(propertiesFile);
            } else {
                associations.putAll(new PropertyMap(propertiesFile));
            }
        } catch (MissingResourceException ex) {
            throw new PeerFactoryException("Unable to load associations file \"" + propertiesFile + "\":" + ex);
        }
    }

    /**
     * Instantiates a new peer for the specified object by invoking its
     * default constructor.
     *
     * @param object The object for which to create an peer instance.
     * @return A new instance of the peer object for <code>object</code>.
     * @throws PeerFactoryException if the peer cannot be instantiated.
     */
    public Object createPeer(Object object) 
    throws PeerFactoryException {
        Object peer = null;
        Class peerClass = getPeerClass(object.getClass());
        
        try {
            peer = peerClass.newInstance();
        } catch (InstantiationException ex) {
            throw new PeerFactoryException("Unable to instantiate peer class \"" + peerClass.getName()
                    + "\":" + ex);
        } catch (IllegalAccessException ex) {
            throw new PeerFactoryException("Unable to access peer class \"" + peerClass.getName()
                    + "\":" + ex);
        }
        
        return peer;
    }
    
    /**
     * Returns the peer class associated for the given object class.
     *
     * @param objectClass The <code>Class</code> of the object whose peer
     *        <code>Class</code> is to be determined.
     * @return The <code>Class</code> of the corresponding peer.
     * @throws PeerFactoryException if the peer class cannot be determined.
     */
    public Class getPeerClass(Class objectClass) 
    throws PeerFactoryException {
        String objectClassName = null;
        String originalObjectClassName = objectClass.getName();
        String peerClassName = null;
        Class peerClass;

        while (peerClassName == null) { 
            objectClassName = objectClass.getName();

            peerClassName = (String) associations.get(objectClassName);

            if (peerClassName == null) {
                objectClass = objectClass.getSuperclass();
                if (objectClass == null) {
                    throw new PeerFactoryException("No peer class is registered for \"" + originalObjectClassName + "\".");
                }
            }
        }
        
        try {
            peerClass = Thread.currentThread().getContextClassLoader().loadClass(peerClassName);
        } catch (ClassNotFoundException ex) {
            throw new PeerFactoryException("Unable to find peer class \"" + peerClassName 
                    + "\" for object \"" + objectClassName +  "\": ");
        }

        return peerClass;
    }
}
