/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.MissingResourceException;
import java.util.Properties;

/**
 * A <code>Map</code> that is generated from a properties file.
 * This class provides additional methods for returning properties
 * as <code>String</code>s or arrays of <code>String</code>s.
 */
public class PropertyMap extends HashMap 
implements Serializable {

    /**
     * Create a <code>PropertyMap</code>
     *
     * @param resourceName The name of the properties file to load.
     */
    public PropertyMap(String resourceName) {
        super();
        
        String propertiesFileName = "/" + resourceName.replace('.', '/') + ".properties";
        
        InputStream in = PropertyMap.class.getResourceAsStream(propertiesFileName);
        if (in == null) {
            throw new MissingResourceException("Cannot find resource", propertiesFileName, resourceName);
        }
        Properties properties = new Properties();
        try {
            properties.load(in);
        } catch (IOException ex) {
            throw new RuntimeException("IOException occurred during properties file load.");
        } finally {
            try { in.close(); } catch (IOException ex) { }
        }
        Enumeration enum = properties.propertyNames();
        String key = null;
        
        while (enum.hasMoreElements()) {
            key = (String) enum.nextElement();
            put(key, properties.getProperty(key));
        }
    }
    
    /**
     * Returns true if the value identified by <Code>key</code> is equal
     * to &quot;"1"&quot;.
     *
     * @param key the name of the property to return.
     * @return true if the value of the property is &quot;1&quot;.
     */
    public boolean getBoolean(String key) {
        return "1".equals(getString(key));
    }
    
    /**
     * Returns the value identified by <code>key</code> as an
     * int if it the value is numeric.  Zero is returned if the
     * value is null or otherwise not numeric. 
     *
     * @param key the name of the property to return.
     * @return The integer value of the property.
     */
    public int getInteger(String key) {
        String s = getString(key);
        int i;

        if (s == null) {
            i = 0;
        } else {
            try {
                i = Integer.parseInt(s);
            } catch (NumberFormatException ex) {
                i = 0;
            }
        }
        
        return i;
    }

    /**
     * Returns the value identified by <code>key</code> as a 
     * <code>String</code> if it is one, otherwise throws a 
     * <code>ClassCastException</code>.
     *
     * @param key The name of the property to return.
     * @return value The value of the property identified by <code>key</code> 
     *         as a <code>String</code>.
     */
    public String getString(String key) {
        Object o = get(key);
        String s = null;
        
        if (o == null) {
        } else if (o instanceof String) {
            s = (String) o;
        } else if (o instanceof String[]) {
            String[] sa = (String[]) o;
            if (sa.length > 0) {
                s = sa[0];
            }
        } else {
            throw new ClassCastException(key + " is not a String");
        }
        
        return s;
    }

    /**
     * Puts a boolean value.
     * 
     * @param key The key of the property to be set.
     * @param value The new value of the propert.
     */
    public void putBoolean(String key, boolean value) {
        put(key, value ? "1" : "0");
    }
    
    /**
     * Puts a integer value.
     * 
     * @param key The key of the property to be set.
     * @param value The new value of the propert.
     */
    public void putInteger(String key, int value) {
        put(key, Integer.toString(value));
    }
}
