/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.echoservlet.util;

import java.util.StringTokenizer;

/**
 * Utility class for comparing version numbers in String format.  This class
 * may be used to determine if a version number is greater than another to
 * make a hypothesis about whether the version is compatible with a
 * requirement.
 */
public final class Version {

    /**
     * Tests to see if a version number meets or exceeds a requirement.
     * For example, if version 1.4 is required, 1.4, 1.4.0, 1.4.1, 1.5, 
     * and 2.0 would meet the requirement.  0.7, 1, 1.0, 1.3, and 
     * 1.3.5 would not.  Warning: At this time all versions must be numeric and
     * separated by periods.  Any deviation will result in .0 versions, such as
     * 1.5.3b being calculated as 1.5.0.
     *
     * @param requiredVersion The version number that is required.
     * @param testVersion The version number that is available.
     * @return True if the tested version is equal to or higher than the 
     *         requirement.
     */ 
    public static final boolean meetsRequirement(String requiredVersion, String testVersion) {
        StringTokenizer reqTokenizer = new StringTokenizer(requiredVersion, ".");
        StringTokenizer testTokenizer = new StringTokenizer(testVersion, ".");
        boolean value = true;
        
        int reqValue;
        int testValue;
        
        while (value && (reqTokenizer.hasMoreTokens() || testTokenizer.hasMoreTokens())) {
            if (reqTokenizer.hasMoreTokens()) {
                reqValue = getTokenValue(reqTokenizer.nextToken());
            } else {
                reqValue = 0;
            }
            if (testTokenizer.hasMoreTokens()) {
                testValue = getTokenValue(testTokenizer.nextToken());
            } else {
                testValue = 0;
            }
            
            if (testValue > reqValue) {
                break;
            } else if (testValue < reqValue) {
                value = false;
            }
        }
        
        return value;
    }
    
    /**
     * Returns a best guess translation of a string to an integer.
     * If the string is not numeric, zero is returned.
     *
     * @param token A string that may or may not contain a number.
     * @return The string's value, forcibly converted into a number.
     */
    private static final int getTokenValue(String token) {
        int value;
        
        try {
            value = Integer.parseInt(token);
        } catch (NumberFormatException ex) {
            value = 0;
        }
        
        return value;
    }
    
    /** Non-instantiable class. */
    private Version() { }
}
