/* 
 * This file is part of the Echo Web Application Framework (hereinafter "Echo").
 * Copyright (C) 2002-2004 NextApp, Inc.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */

package nextapp.testforecho;

import nextapp.echo.Button;
import nextapp.echo.Color;
import nextapp.echo.Component;
import nextapp.echo.ContainerPane;
import nextapp.echo.ContentPane;
import nextapp.echo.EchoConstants;
import nextapp.echo.Filler;
import nextapp.echo.Label;
import nextapp.echo.Panel;
import nextapp.echo.SelectField;

import nextapp.echo.Table;
import nextapp.echo.table.*;

import nextapp.echo.event.ActionEvent;
import nextapp.echo.event.ActionListener;
import nextapp.echo.event.ItemEvent;
import nextapp.echo.event.ItemListener;
import nextapp.echo.event.TableColumnModelEvent;
import nextapp.echo.event.TableColumnModelListener;

public class TableTest extends ContainerPane {

    private static final String[] POSITIONS 
            = new String[]{"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"};

    private static Color randomColor(boolean light) {
        int red = ((int) Math.floor(Math.random() * 128)) 
                + (light ? 128 : 0);
        int green = ((int) Math.floor(Math.random() * 128)) 
                + (light ? 128 : 0);
        int blue = ((int) Math.floor(Math.random() * 128)) 
                + (light ? 128 : 0);
        return new Color(red, green, blue);
    }
    
    private ContentPane controlPane;
    private ContentPane tablePane;
    
    private Table table;
    
    private MultiplicationTableModel multiplicationTableModel = new MultiplicationTableModel();
    private DifferentTypesTableModel differentTypesTableModel = new DifferentTypesTableModel();
    
    private ColorfulRenderer colorfulRenderer = new ColorfulRenderer();
    
    private class ColorfulRenderer extends DefaultTableCellRenderer {
    
        public Component getTableCellRendererComponent(Table table, Object value, int column, int row) {
            Component c = super.getTableCellRendererComponent(table, value, column, row);
            if (c != null) {
                c.setBackground(randomColor(true));
            }
            return c;
        }
    }
    
    private class GradientRenderer extends DefaultTableCellRenderer {
    
        private Color leftColor;
        private Color rightColor;
        
        public GradientRenderer(Color leftColor, Color rightColor) {
            super();
            
            this.leftColor = leftColor;
            this.rightColor = rightColor;
        }
        
        public Component getTableCellRendererComponent(Table table, Object value, int column, int row) {
            Component c = super.getTableCellRendererComponent(table, value, column, row);
            if (c != null) {
                int columnCount = table.getModel().getColumnCount();
                int redDiff = rightColor.getRed() - leftColor.getRed();
                int greenDiff = rightColor.getGreen() - leftColor.getGreen();
                int blueDiff = rightColor.getBlue() - leftColor.getBlue();

                double blend = (double) column / (double) (columnCount - 1);
                Color color = new Color(leftColor.getRed() + ((int) (redDiff * blend)),
                        leftColor.getGreen() + ((int) (greenDiff * blend)),
                        leftColor.getBlue() + ((int) (blueDiff * blend)));
                c.setBackground(color);
                if (color.getRed() + color.getGreen() + color.getBlue() > 384) {
                    c.setForeground(Color.BLACK);
                } else {
                    c.setForeground(Color.WHITE);
                }
            }
            return c;
        }
    }
    
    private class MultiplicationTableModel extends AbstractTableModel {
        
        public Class getColumnClass(int column) {
            return Integer.class;
        }
        
        public int getColumnCount() {
            return 12;
        }
        
        public String getColumnName(int column) {
            return "x" + (column + 1);
        }
        
        public int getRowCount() {
            return 12;
        }
        
        public Object getValueAt(int column, int row) {
            return new Integer((column + 1) * (row + 1));
        }
    }
    
    private class DifferentTypesTableModel extends AbstractTableModel {
    
        private Object[][] data = new Object[][] {
                new Object[]{ "one", new Integer(2), "three",        null, "four", null,           null},
                new Object[]{ "one", new Integer(2), new Integer(3), null, null,   new Integer(5), null},
                new Object[]{ "one", new Integer(2), new Integer(3), null, null,   "five",         new Integer(6)},
                };
                
        public Class getColumnClass(int column) {
            if (column == 1) {
                return Integer.class;
            } else {
                return super.getColumnClass(column);
            }
        }
        
        public int getColumnCount() {
            return data[0].length;
        }
        
        public int getRowCount() {
            return data.length;
        }
        
        public Object getValueAt(int column, int row) {
            try {
                return data[row][column];
            } catch (ArrayIndexOutOfBoundsException ex) {
                TestForEcho.getApplicationInstance().getLogger().log("Attempt to get invalid cell value: " + ex);
                return null;
            }
        }
    }

    private class TablePane extends ContentPane {
        public TablePane() {
            TablePane.this.add(new HelpPanel("Table Test", "This test evaluates the functionality of the Table component."));
            TablePane.this.add(Filler.createVerticalStrut(20));

            table = new Table();
            TablePane.this.add(table);
        }
    }
    
    private static final Object NULL = new String("Null");

    private class ControlPane extends ContentPane 
    implements ActionListener, ItemListener, TableColumnModelListener {
    
        private static final int X = EchoConstants.UNDEFINED_SIZE;
        private static final int PX = Table.PIXEL_UNITS;
        private static final int PER = Table.PERCENT_UNITS;
        private static final int PIPT = Table.PERCENT_INDIVIDUAL_PIXEL_TOTAL_UNITS;
        
        private ListElement[] sizeOptions = new ListElement[] {
                new ListElement(new int[]{PX,     X,   X,   X,   X,   X,   X}, "All UNK total=UNK"),
                new ListElement(new int[]{PX,   600,   X,   X,   X,   X,   X}, "All UNK total=600"),
                new ListElement(new int[]{PER,   80,   X,   X,   X,   X,   X}, "All UNK total=80%"),
                new ListElement(new int[]{PER,  100,   X,   X,   X,   X,   X}, "All UNK total=100%"),
                new ListElement(new int[]{PX,     X,  80,  60,  40,  90, 120}, "80,60,40,90,120 total=UNK"),
                new ListElement(new int[]{PX,   600,  80,  60,  40,  90, 120}, "80,60,40,90,120 total=600"),
                new ListElement(new int[]{PX,     X,  80,   X,  40,   X, 120}, "80,UNK,40,UNK,120 total=UNK"),
                new ListElement(new int[]{PX,   600,  80,   X,  40,   X, 120}, "80,UNK,40,UNK,120 total=600"),
                new ListElement(new int[]{PER,    X,  10,   5,  20,  40,  25}, "10%,5%,20%,40%,25% total=UNK"),
                new ListElement(new int[]{PIPT, 600,  10,   5,  20,  40,  25}, "10%,5%,20%,40%,25% total=600"),
                new ListElement(new int[]{PER,   80,  10,   5,  20,  40,  25}, "10%,5%,20%,40%,25% total=80%"),
                new ListElement(new int[]{PER,  100,  10,   5,  20,  40,  25}, "10%,5%,20%,40%,25% total=100%"),
                new ListElement(new int[]{PER,    X,  10,   5,   X,  40,  25}, "10%,5%,UNK,40%,25% total=UNK"),
                new ListElement(new int[]{PIPT, 600,  10,   5,   X,  40,  25}, "10%,5%,UNK,40%,25% total=600"),
                new ListElement(new int[]{PER,   80,  10,   5,   X,  40,  25}, "10%,5%,UNK,40%,25% total=80%"),
                new ListElement(new int[]{PER,  100,  10,   5,   X,  40,  25}, "10%,5%,UNK,40%,25% total=100%")};

        private SelectField modelSelect;
        private SelectField objectRendererSelect;
        private SelectField integerRendererSelect;
        private SelectField headerRendererSelect;
        private SelectField backgroundSelect;
        private SelectField foregroundSelect;
        private SelectField borderColorSelect;
        private SelectField borderSizeSelect;
        private SelectField cellMarginSelect;
        private SelectField columnModelSelect;
        
        private SelectField moveFromSelect;
        private SelectField moveToSelect;
        private Button moveColumnButton;
        
        private SelectField addColumnSelect;
        private Button addColumnButton;
        
        private SelectField removeColumnSelect;
        private Button removeColumnButton;

        private SelectField columnWidthSelect;
        private SelectField rowHeightSelect;

        private Button applyButton;

        public ControlPane() {
            this.setResizable(true);

            Panel panel;
            TestPanel testPanel = new TestPanel();
            Object[] rendererOptions = new Object[]{
                    new ListElement(null, "Null"),
                    new ListElement(Table.DEFAULT_TABLE_CELL_RENDERER, "Default Renderer"),
                    new ListElement(colorfulRenderer, "Colorful Renderer")};
            Object[] numericOptions = new Object[] {new Integer( 0), new Integer( 1), new Integer( 2), new Integer( 3), 
                                                    new Integer( 4), new Integer( 5), new Integer( 6), new Integer( 7),
                                                    new Integer( 8), new Integer( 9), new Integer(10), new Integer(11),
                                                    new Integer(12), new Integer(13), new Integer(14), new Integer(15)};
            Object[] colorOptions = new Object[] {NULL, Color.WHITE, Color.RED, Color.YELLOW, Color.GREEN, Color.BLUE, Color.BLACK};
            
            // Model Properties
            testPanel.addHeader("Model Properties");
            
            modelSelect = new SelectField(new Object[]{
                    new ListElement(new DefaultTableModel(), "Default"),
                    new ListElement(differentTypesTableModel, "Table w/ Different Types"),
                    new ListElement(multiplicationTableModel, "Multiplication Table")});
            modelSelect.addItemListener(this);
            testPanel.add("Model", modelSelect);

            // Renderer Properties
            testPanel.addHeader("Renderer Properties");

            objectRendererSelect = new SelectField(rendererOptions);
            objectRendererSelect.addItemListener(this);
            testPanel.add("Cell Renderer (Object)", objectRendererSelect);

            integerRendererSelect = new SelectField(rendererOptions);
            integerRendererSelect.addItemListener(this);
            testPanel.add("Cell Renderer (Integer)", integerRendererSelect);
            
            headerRendererSelect = new SelectField(rendererOptions);
            headerRendererSelect.addItemListener(this);
            testPanel.add("Header Renderer", headerRendererSelect);
            
            testPanel.addHeader("Column Model");
            
            columnModelSelect = new SelectField(new String[]{"Default", "Custom"});
            columnModelSelect.addItemListener(this);
            testPanel.add("Column Model", columnModelSelect);
            
            // Aesthetic Properties
            testPanel.addHeader("Aesthetic Properties");

            backgroundSelect = new SelectField(colorOptions);
            backgroundSelect.addItemListener(this);
            testPanel.add("Background Color", backgroundSelect);

            foregroundSelect = new SelectField(colorOptions);
            foregroundSelect.addItemListener(this);
            testPanel.add("Foreground Color", foregroundSelect);

            borderColorSelect = new SelectField(colorOptions);
            borderColorSelect.addItemListener(this);
            testPanel.add("Border Color", borderColorSelect);

            borderSizeSelect = new SelectField(numericOptions);
            borderSizeSelect.addItemListener(this);
            testPanel.add("Border Size", borderSizeSelect);
            
            cellMarginSelect = new SelectField(numericOptions);
            cellMarginSelect.addItemListener(this);
            testPanel.add("Cell Margin", cellMarginSelect);

            // Column Tests
            testPanel.addHeader("Column Tests");
            
            // Add Column
            panel = new Panel();
            addColumnSelect = new SelectField(POSITIONS);
            addColumnButton = new Button("[ADD]");
            addColumnButton.addActionListener(this);
            panel.add(new Label("Select Model Index:"));
            panel.add(addColumnSelect);
            panel.add(Filler.createHorizontalStrut(25));
            panel.add(addColumnButton);
            testPanel.add("Add Column", panel);

            // Move Column
            panel = new Panel();
            moveFromSelect = new SelectField(POSITIONS);
            moveToSelect = new SelectField(POSITIONS);
            moveColumnButton = new Button("[MOVE]");
            moveColumnButton.addActionListener(this);
            panel.add(new Label("Current Position:"));
            panel.add(moveFromSelect);
            panel.add(Filler.createHorizontalStrut(25));
            panel.add(new Label("New Position:"));
            panel.add(moveToSelect);
            panel.add(Filler.createHorizontalStrut(25));
            panel.add(moveColumnButton);
            testPanel.add("Move Column", panel);
            
            // Remove Column
            panel = new Panel();
            removeColumnSelect = new SelectField(POSITIONS);
            removeColumnButton = new Button("[REMOVE]");
            removeColumnButton.addActionListener(this);
            panel.add(new Label("Remove Column:"));
            panel.add(removeColumnSelect);
            panel.add(Filler.createHorizontalStrut(25));
            panel.add(removeColumnButton);
            testPanel.add("Remove Column", panel);

            // Widths and Heights
            testPanel.addHeader("Widths and Heights");
            
            columnWidthSelect = new SelectField(sizeOptions);
            columnWidthSelect.addActionListener(this);
            testPanel.add("Column Widths", columnWidthSelect);

            rowHeightSelect = new SelectField(sizeOptions);
            rowHeightSelect.addActionListener(this);
            testPanel.add("Row Heights (First value is header (row -1) height)", rowHeightSelect);


            this.add(testPanel);

            this.add(Filler.createVerticalStrut(10));
            applyButton = new Button("Apply Changes");
            this.add(applyButton);
        }
        
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == addColumnButton) {
                int index = Integer.parseInt((String) addColumnSelect.getSelectedItem());
                TableColumn column = new TableColumn(index);
                table.getColumnModel().addColumn(column);
            } else if (e.getSource() == moveColumnButton) {
                int from = Integer.parseInt((String) moveFromSelect.getSelectedItem());
                int to = Integer.parseInt((String) moveToSelect.getSelectedItem());
                try {
                    table.getColumnModel().moveColumn(from, to);
                } catch (IllegalArgumentException ex) {
                    TestForEcho.getApplicationInstance().getLogger().log(ex.toString());
                }
            } else if (e.getSource() == removeColumnButton) {
                int index = Integer.parseInt((String) removeColumnSelect.getSelectedItem());
                TableColumn column = table.getColumnModel().getColumn(index);
                table.getColumnModel().removeColumn(column);
            } else if (e.getSource() == columnWidthSelect) {
                ListElement elem = (ListElement) columnWidthSelect.getSelectedItem();
                int[] data = (int[]) elem.getObject();
                table.setWidthUnits(data[0]);
                table.setWidth(data[1]);
                for (int index = 0; index < 5; ++index) {
                    TableColumn column = table.getColumnModel().getColumn(index);
                    if (column != null) {
                        column.setWidth(data[index + 2]);
                    }
                }
            } else if (e.getSource() == rowHeightSelect) {
                ListElement elem = (ListElement) rowHeightSelect.getSelectedItem();
                int[] data = (int[]) elem.getObject();
                table.setHeightUnits(data[0]);
                table.setHeight(data[1]);
                for (int index = 0; index < 5; ++index) {
                    table.setRowHeight(index - 1, data[index + 2]);
                }
            }
        }
        
        public void columnAdded(TableColumnModelEvent e) {
            TestForEcho app = TestForEcho.getApplicationInstance();
            app.getLogger().log("Column added: source=" + e.getSource() + " from=" + e.getFromIndex() + " to=" + e.getToIndex());
        }

        public void columnMoved(TableColumnModelEvent e) {
            TestForEcho app = TestForEcho.getApplicationInstance();
            app.getLogger().log("Column moved: source=" + e.getSource() + " from=" + e.getFromIndex() + " to=" + e.getToIndex());
        }

        public void columnRemoved(TableColumnModelEvent e) {
            TestForEcho app = TestForEcho.getApplicationInstance();
            app.getLogger().log("Column removed: source=" + e.getSource() + " from=" + e.getFromIndex() + " to=" + e.getToIndex());
        }

        private TableColumnModel createCustomTableColumnModel() {
            TableColumnModel cm = new DefaultTableColumnModel();
            
            TableModel model = table.getModel();
            int columnCount = model.getColumnCount();
            
            TableCellRenderer headerRenderer = new GradientRenderer(randomColor(false), randomColor(true));
            TableCellRenderer cellRenderer = new GradientRenderer(randomColor(true), randomColor(false));
            
            for (int index = 0; index < columnCount; ++index) {
                TableColumn column = new TableColumn(index);
                column.setHeaderRenderer(headerRenderer);
                column.setCellRenderer(cellRenderer);
                cm.addColumn(column);
            }
            
            return cm;
        }

        public void itemStateChanged(ItemEvent e) {
            if (e.getSource() == modelSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.setModel((TableModel) ((ListElement) e.getItem()).getObject());
            } else if (e.getSource() == objectRendererSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.setDefaultRenderer(Object.class, (TableCellRenderer) ((ListElement) e.getItem()).getObject());
            } else if (e.getSource() == integerRendererSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.setDefaultRenderer(Integer.class, (TableCellRenderer) ((ListElement) e.getItem()).getObject());
            } else if (e.getSource() == headerRendererSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.getTableHeader().setDefaultRenderer((TableCellRenderer) ((ListElement) e.getItem()).getObject());
            } else if (e.getSource() == columnModelSelect && e.getStateChange() == ItemEvent.SELECTED) {
                if (columnModelSelect.getSelectedItem().equals("Custom")) {
                    table.setAutoCreateColumnsFromModel(false);
                    table.setColumnModel(createCustomTableColumnModel());
                    table.getColumnModel().addColumnModelListener(this);
                } else {
                    table.setAutoCreateColumnsFromModel(true);
                    table.getColumnModel().addColumnModelListener(this);
                }
            } else if (e.getSource() == foregroundSelect && e.getStateChange() == ItemEvent.SELECTED) {
                if (e.getItem() == NULL) {
                    table.setForeground(null);
                } else {
                    table.setForeground((Color) e.getItem());
                }
            } else if (e.getSource() == backgroundSelect && e.getStateChange() == ItemEvent.SELECTED) {
                if (e.getItem() == NULL) {
                    table.setBackground(null);
                } else {
                    table.setBackground((Color) e.getItem());
                }
            } else if (e.getSource() == borderColorSelect && e.getStateChange() == ItemEvent.SELECTED) {
                if (e.getItem() == NULL) {
                    table.setBorderColor(null);
                } else {
                    table.setBorderColor((Color) e.getItem());
                }
            } else if (e.getSource() == borderSizeSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.setBorderSize(((Integer) e.getItem()).intValue());
            } else if (e.getSource() == cellMarginSelect && e.getStateChange() == ItemEvent.SELECTED) {
                table.setCellMargin(((Integer) e.getItem()).intValue());
            }
        }
    }
    
    public TableTest() {
        setOrientation(ORIENTATION_VERTICAL);

        controlPane = new ControlPane();
        tablePane = new TablePane();
        
        add(tablePane);
        add(controlPane);
    }
}
